// BattlePreviewWnd.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "FF3Doc.h"
#include "BattlePreviewWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CBattlePreviewWnd

CBattlePreviewWnd::CBattlePreviewWnd()
{
	for(int i = 0; i < 6; i++)
		visible[i] = FALSE;

	active = -1;
}

CBattlePreviewWnd::~CBattlePreviewWnd()
{
}


BEGIN_MESSAGE_MAP(CBattlePreviewWnd, CWnd)
	//{{AFX_MSG_MAP(CBattlePreviewWnd)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_SETCURSOR()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CBattlePreviewWnd message handlers

void CBattlePreviewWnd::OnPaint() 
{
	CPaintDC pdc(this);
	CRect clientRect;  GetClientRect(clientRect);
	CMemDC dc(&pdc, &clientRect);

	CDC bmpDC;
	bmpDC.CreateCompatibleDC(&dc);
	CBitmap bmp;
	bmp.LoadBitmap(IDB_BATTLEBG);
	CBitmap* oldBitmap = bmpDC.SelectObject(&bmp);
	dc.BitBlt(0, 0, 240, 147, &bmpDC, 0, 0, SRCCOPY);
	bmpDC.SelectObject(oldBitmap);

	int nHeight = -((dc.GetDeviceCaps(LOGPIXELSY) * 8.5) / 72);
	CFont font;  font.CreateFont(nHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_CHARACTER_PRECIS, CLIP_CHARACTER_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, "Tahoma");
	CFont* pOldFont = dc.SelectObject(&font);
	dc.SetBkMode(TRANSPARENT);

	for(int i = 0; i < 6; i++)
	{
		CString s;  s.Format("%d", i + 1);
		CRect rect;  GetItemRect(i, &rect);

		if(visible[i] && (active != i))
		{
			DrawMonster(&dc, i);
		}
		else if(visible[i] && (active == i))
		{
			CRect hiliteRect = rect;
			hiliteRect.InflateRect(2, 2, 2, 2);

			CPen hilitePen(PS_SOLID | PS_INSIDEFRAME, 2, GetSysColor(COLOR_HIGHLIGHT));
			CPen* oldPen = dc.SelectObject(&hilitePen);
			CBrush* oldBrush = (CBrush*)dc.SelectStockObject(NULL_BRUSH);
			dc.Rectangle(hiliteRect);
			dc.SelectObject(oldBrush);
			dc.SelectObject(oldPen);

			DrawMonster(&dc, i);
		}
	}

	dc.SelectObject(pOldFont);
}

void CBattlePreviewWnd::GetItemRect(int nIndex, LPRECT lpRect)
{
	lpRect->left = positions[nIndex].x * 8 - 8;
	lpRect->top = positions[nIndex].y * 8 - 4;
	lpRect->right = lpRect->left + (sizes[nIndex].cx * 8);
	lpRect->bottom = lpRect->top + (sizes[nIndex].cy * 8);
}

void CBattlePreviewWnd::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if(active == -1) return;

	CRect rect;  GetItemRect(active, &rect);
	if(!rect.PtInRect(point)) return;

	SetCapture();

	anchor = point - CPoint(positions[active].x * 8, positions[active].y * 8);

	int x, y;
	x = (point.x - anchor.x) / 8;  if(x < 0) x = 0; else if(x > 15) x = 15;
	y = (point.y - anchor.y) / 8;  if(y < 0) y = 0; else if(y > 15) y = 15;
	positions[active].x = x;
	positions[active].y = y;
	Invalidate(FALSE);

	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), BPN_POSITIONCHANGED),
		(LPARAM)GetSafeHwnd());
}

void CBattlePreviewWnd::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(active == -1 || GetCapture()->GetSafeHwnd() != GetSafeHwnd()) return;

	int x, y;
	x = (point.x - anchor.x) / 8;  if(x < 0) x = 0; else if(x > 15) x = 15;
	y = (point.y - anchor.y) / 8;  if(y < 0) y = 0; else if(y > 15) y = 15;
	positions[active].x = x;
	positions[active].y = y;
	Invalidate(FALSE);

	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), BPN_POSITIONCHANGED),
		(LPARAM)GetSafeHwnd());
}

void CBattlePreviewWnd::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if(GetCapture()->GetSafeHwnd() == GetSafeHwnd()) ReleaseCapture();
}

void CBattlePreviewWnd::DrawMonster(CDC* pDC, int nMonst)
{
	MonsterVisuals* pVis = &m_pDoc->monsterVisuals[monsters[nMonst]];

	MonsterComposition* pComp;

	if(pVis->extra & 0x80)
		pComp = &m_pDoc->monsterCompositions[pVis->composition + 128];
	else
		pComp = &m_pDoc->monsterCompositions[pVis->composition];

	CRect rect;  GetItemRect(nMonst, &rect);

	int offset = 0;

	if(pComp->length == 8)
	{
		for(int y = 0; y < 8; y++)
		{
			int x = 0;

			for(int bit = 7; bit >= 0; bit--, x += 8)
			{
				if(pComp->mask[y] & (1 << bit))
				{
					if(pVis->Is3Bitplane())
					{
						Put8x8x3Tile(pDC, rect.left + x, rect.top + (y * 8), pVis, offset);
						offset += 0x18;
					}
					else
					{
						Put8x8x4Tile(pDC, rect.left + x, rect.top + (y * 8), pVis, offset);
						offset += 0x20;
					}
				}
			}
		}
	}
	else if(pComp->length == 32)
	{
		for(int y = 0; y < 16; y++)
		{
			int x = 0;

			for(int bit = 15; bit >= 0; bit--, x += 8)
			{
				WORD wMask = MAKEWORD(pComp->mask[y * 2 + 1], pComp->mask[y * 2]);

				if(wMask & (1 << bit))
				{
					if(pVis->Is3Bitplane())
					{
						Put8x8x3Tile(pDC, rect.left + x, rect.top + (y * 8), pVis, offset);
						offset += 0x18;
					}
					else
					{
						Put8x8x4Tile(pDC, rect.left + x, rect.top + (y * 8), pVis, offset);
						offset += 0x20;
					}
				}
			}
		}
	}
}

void CBattlePreviewWnd::Put8x8x4Tile(CDC *pDC, int dx, int dy, MonsterVisuals* pVis, int offset)
{
	BYTE aTile[64];
	BYTE bPlanes[4];

	BYTE* pTile = &m_pDoc->monsterGraphics[(pVis->gfx & 0x7FFF) * 8 + offset];

	for(int row = 0; row < 8; row++)
	{
		bPlanes[0] = pTile[row * 2];
		bPlanes[1] = pTile[row * 2 + 1];
		bPlanes[2] = pTile[row * 2 + 16];
		bPlanes[3] = pTile[row * 2 + 17];

		for(int bit = 7; bit >= 0; bit--)
		{
			BYTE mask = 1 << bit;

			// Get a color index by separating the bits on each plane,
			// shifting them the correct amount depending on the plane,
			// and ORing them together.
			aTile[row * 8 + (7 - bit)] = ((bPlanes[0] & mask) >> bit) |
								   (((bPlanes[1] & mask) >> bit) << 1) |
								   (((bPlanes[2] & mask) >> bit) << 2) |
								   (((bPlanes[3] & mask) >> bit) << 3);
								   
		}
	}

	for(int y = 0; y < 8; y++)
	{
		for(int x = 0; x < 8; x++)
		{
			BYTE bCol = aTile[y * 8 + x];
			
			if(bCol != 0)
			{
				WORD wPal = MAKEWORD(pVis->palette, pVis->extra) & 0x03FF;
				WORD wCol = m_pDoc->monsterPalettes[wPal * 8 + bCol];
				COLORREF cCol = RGB(R555(wCol) * 8, G555(wCol) * 8, B555(wCol) * 8);
				pDC->SetPixel(dx + x, dy + y, cCol);
			}
		}
	}
}

void CBattlePreviewWnd::Put8x8x3Tile(CDC *pDC, int dx, int dy, MonsterVisuals* pVis, int offset)
{
	BYTE aTile[64];
	BYTE bPlanes[3];

	BYTE* pTile = &m_pDoc->monsterGraphics[(pVis->gfx & 0x7FFF) * 8 + offset];

	for(int row = 0; row < 8; row++)
	{
		bPlanes[0] = pTile[row * 2];
		bPlanes[1] = pTile[row * 2 + 1];
		bPlanes[2] = pTile[row + 16];

		for(int bit = 7; bit >= 0; bit--)
		{
			BYTE mask = 1 << bit;

			// Get a color index by separating the bits on each plane,
			// shifting them the correct amount depending on the plane,
			// and ORing them together.
			aTile[row * 8 + (7 - bit)] = ((bPlanes[0] & mask) >> bit) |
								   (((bPlanes[1] & mask) >> bit) << 1) |
								   (((bPlanes[2] & mask) >> bit) << 2);
								   
		}
	}

	for(int y = 0; y < 8; y++)
	{
		for(int x = 0; x < 8; x++)
		{
			BYTE bCol = aTile[y * 8 + x];
			
			if(bCol != 0)
			{
				WORD wPal = MAKEWORD(pVis->palette, pVis->extra) & 0x03FF;
				WORD wCol = m_pDoc->monsterPalettes[wPal * 8 + bCol];
				COLORREF cCol = RGB(R555(wCol) * 8, G555(wCol) * 8, B555(wCol) * 8);
				pDC->SetPixel(dx + x, dy + y, cCol);
			}
		}
	}
}

BOOL CBattlePreviewWnd::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	CPoint point;  GetCursorPos(&point);
	ScreenToClient(&point);

	CRect rect;  GetItemRect(active, &rect);
	if(!rect.PtInRect(point)) return CWnd::OnSetCursor(pWnd, nHitTest, message);

	::SetCursor(AfxGetApp()->LoadStandardCursor(IDC_SIZEALL));

	return TRUE;
}
