// ColorSelector.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "ColorSelector.h"

#include "PaletteEditDlg.h"
#include "MemDC.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CColorSelector

CColorSelector::CColorSelector()
{
	m_nFGSel = 0;
	m_nBGSel = 1;

	m_nRows = 2;
}

CColorSelector::~CColorSelector()
{
}


BEGIN_MESSAGE_MAP(CColorSelector, CWnd)
	//{{AFX_MSG_MAP(CColorSelector)
	ON_WM_SIZE()
	ON_WM_RBUTTONDOWN()
	ON_WM_LBUTTONDOWN()
	ON_WM_PAINT()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_RBUTTONDBLCLK()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CColorSelector message handlers

void CColorSelector::OnPaint() 
{
	CPaintDC pdc(this); // device context for painting
	CRect rect;  GetClientRect(rect);

	CMemDC dc(&pdc, &rect);
	dc.FillSolidRect(&rect, GetSysColor(COLOR_BTNFACE));

	CRect boxRect(rect.left, rect.top, rect.left + 31, rect.bottom);
	dc.DrawFrameControl(&boxRect, DFC_BUTTON, DFCS_BUTTONPUSH | DFCS_CHECKED);
	dc.DrawEdge(&boxRect, EDGE_SUNKEN, BF_RECT);

	CRect bgBoxRect(rect.left + 11, rect.top + 12, rect.left + 26, rect.top + 27);
	CRect bgColorRect(rect.left + 13, rect.top + 14, rect.left + 24, rect.top + 25);
	dc.FillSolidRect(&bgBoxRect, GetSysColor(COLOR_BTNFACE));
	dc.DrawEdge(&bgBoxRect, BDR_RAISEDINNER, BF_RECT);
	dc.FillSolidRect(&bgColorRect, RGB(R555(m_colors[m_nBGSel]) * 8, G555(m_colors[m_nBGSel]) * 8,
		B555(m_colors[m_nBGSel]) * 8));

	CRect fgBoxRect(rect.left + 4, rect.top + 5, rect.left + 19, rect.top + 20);
	CRect fgColorRect(rect.left + 6, rect.top + 7, rect.left + 17, rect.top + 18);
	dc.FillSolidRect(&fgBoxRect, GetSysColor(COLOR_BTNFACE));
	dc.DrawEdge(&fgBoxRect, BDR_RAISEDINNER, BF_RECT);
	dc.FillSolidRect(&fgColorRect, RGB(R555(m_colors[m_nFGSel]) * 8, G555(m_colors[m_nFGSel]) * 8,
		B555(m_colors[m_nFGSel]) * 8));

	CRect colorRect;
	int index;
	for(int y = 0; y < m_nRows; y++)
	{
		for(int x = 0; x < m_colors.GetSize() / m_nRows; x++)
		{
			index = y * (m_colors.GetSize() / m_nRows) + x;

			if(index < m_colors.GetSize())
			{
				GetItemRect(index, &colorRect);
				dc.DrawEdge(&colorRect, EDGE_SUNKEN, BF_RECT);
			
				colorRect.DeflateRect(2, 2, 2, 2);
				dc.FillSolidRect(&colorRect, RGB(R555(m_colors[index]) * 8, G555(m_colors[index]) * 8,
					B555(m_colors[index]) * 8));
			}
		}
	}
}

void CColorSelector::AddColors(WORD* aColors, int nColors)
{
	CRect clientRect;
	GetClientRect(&clientRect);

	m_nItemHeight = 16;
	m_nItemWidth = 16;

	for(int i = 0; i < nColors; i++)
		m_colors.Add(aColors[i]);
}

void CColorSelector::PreSubclassWindow() 
{
	CWnd::PreSubclassWindow();

	CRect rect;  GetClientRect(rect);
	SetWindowPos(NULL, 0, 0, rect.Width(), 32, SWP_NOZORDER | SWP_NOMOVE);
}

void CColorSelector::OnSize(UINT nType, int cx, int cy) 
{
	m_nItemHeight = 16;
	m_nItemWidth = 16;

	CWnd::OnSize(nType, cx, 32);
}

void CColorSelector::OnLButtonDown(UINT nFlags, CPoint point) 
{
	int index = ItemFromPoint(point);
	if(index == -1) return;

	CRect rect;  GetClientRect(rect);
	CRect fgColorRect(rect.left + 6, rect.top + 7, rect.left + 17, rect.top + 18);

	m_nFGSel = index;
	InvalidateRect(fgColorRect, FALSE);

	NMCOLORSELECTOR nmcs;
	nmcs.hdr.idFrom = GetDlgCtrlID();
	nmcs.hdr.hwndFrom = GetSafeHwnd();
	nmcs.hdr.code = CSN_SELCHANGED;
	nmcs.nColorChanged = CSNC_FOREGROUND;
	nmcs.nIndex = index;
	GetOwner()->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)&nmcs);
}

void CColorSelector::OnRButtonDown(UINT nFlags, CPoint point) 
{
	int index = ItemFromPoint(point);
	if(index == -1) return;

	CRect rect;  GetClientRect(rect);
	CRect bgColorRect(rect.left + 13, rect.top + 14, rect.left + 24, rect.top + 25);

	m_nBGSel = index;
	InvalidateRect(bgColorRect, FALSE);

	NMCOLORSELECTOR nmcs;
	nmcs.hdr.idFrom = GetDlgCtrlID();
	nmcs.hdr.hwndFrom = GetSafeHwnd();
	nmcs.hdr.code = CSN_SELCHANGED;
	nmcs.nColorChanged = CSNC_BACKGROUND;
	nmcs.nIndex = index;
	GetOwner()->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)&nmcs);
}

void CColorSelector::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	int index = ItemFromPoint(point);
	if(index == -1) return;

	CPaletteEditDlg dlg(this);
	dlg.m_wColor = m_colors[index];

	if(dlg.DoModal() == IDOK)
	{
		m_colors[index] = dlg.m_wColor;

		CRect rect;  GetClientRect(rect);
		CRect fgColorRect(rect.left + 6, rect.top + 7, rect.left + 17, rect.top + 18);
		CRect colorRect;  GetItemRect(index, &colorRect);

		InvalidateRect(fgColorRect, FALSE);
		InvalidateRect(colorRect, FALSE);

		NMCOLORSELECTOR nmcs;
		nmcs.hdr.idFrom = GetDlgCtrlID();
		nmcs.hdr.hwndFrom = GetSafeHwnd();
		nmcs.hdr.code = CSN_COLOREDITED;
		nmcs.nColorChanged = 0;
		nmcs.nIndex = index;
		nmcs.wRGB = dlg.m_wColor;
		GetOwner()->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)&nmcs);
	}
}

void CColorSelector::OnRButtonDblClk(UINT nFlags, CPoint point) 
{
	int index = ItemFromPoint(point);
	if(index == -1) return;

	CPaletteEditDlg dlg(this);
	dlg.m_wColor = m_colors[index];

	if(dlg.DoModal() == IDOK)
	{
		m_colors[index] = dlg.m_wColor;

		CRect rect;  GetClientRect(rect);
		CRect bgColorRect(rect.left + 6, rect.top + 7, rect.left + 17, rect.top + 18);
		CRect colorRect;  GetItemRect(index, &colorRect);

		InvalidateRect(bgColorRect, FALSE);
		InvalidateRect(colorRect, FALSE);

		NMCOLORSELECTOR nmcs;
		nmcs.hdr.idFrom = GetDlgCtrlID();
		nmcs.hdr.hwndFrom = GetSafeHwnd();
		nmcs.hdr.code = CSN_COLOREDITED;
		nmcs.nColorChanged = 0;
		nmcs.nIndex = index;
		nmcs.wRGB = dlg.m_wColor;
		GetOwner()->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)&nmcs);
	}
}

void CColorSelector::SetForegroundColor(int nColor)
{
	CRect rect;  GetClientRect(rect);
	CRect fgColorRect(rect.left + 6, rect.top + 7, rect.left + 17, rect.top + 18);

	m_nFGSel = nColor;
	InvalidateRect(fgColorRect, FALSE);
}

void CColorSelector::SetBackgroundColor(int nColor)
{
	CRect rect;  GetClientRect(rect);
	CRect bgColorRect(rect.left + 13, rect.top + 14, rect.left + 24, rect.top + 25);

	m_nBGSel = nColor;
	InvalidateRect(bgColorRect, FALSE);
}

int CColorSelector::GetForegroundColor()
{
	return m_nFGSel;
}

int CColorSelector::GetBackgroundColor()
{
	return m_nBGSel;
}

void CColorSelector::GetItemRect(int nIndex, LPRECT lpRect)
{
	lpRect->left = (nIndex % (m_colors.GetSize() / m_nRows)) * m_nItemWidth + 31;
	lpRect->top = nIndex / (m_colors.GetSize() / m_nRows) * m_nItemHeight;
	lpRect->right = (nIndex % (m_colors.GetSize() / m_nRows)) * m_nItemWidth + 31 + m_nItemWidth;
	lpRect->bottom = nIndex / (m_colors.GetSize() / m_nRows) * m_nItemHeight + m_nItemHeight;
}

void CColorSelector::ResetContent()
{
	m_colors.RemoveAll();
	Invalidate(FALSE);
}

int CColorSelector::ItemFromPoint(CPoint pt)
{
	for(int i = 0; i < m_colors.GetSize(); i++)
	{
		CRect rect;  GetItemRect(i, &rect);
		if(rect.PtInRect(pt)) return i;
	}

	return -1;
}
