// ColorWheel.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "ColorWheel.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CColorWheel

CColorWheel::CColorWheel()
{
	oldWheelBmp = NULL;
	m_ptMouse = CPoint(50, 50);
	m_bTracking = FALSE;

	m_Hue = m_Sat = 0.0;
}

CColorWheel::~CColorWheel()
{
}


BEGIN_MESSAGE_MAP(CColorWheel, CWnd)
	//{{AFX_MSG_MAP(CColorWheel)
	ON_WM_PAINT()
	ON_WM_DESTROY()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

#define DEFAULT_LUMINANCE				0.50f

/////////////////////////////////////////////////////////////////////////////
// CColorWheel message handlers

void CColorWheel::OnPaint() 
{
	CPaintDC pdc(this); // device context for painting

	CRect rect;  GetClientRect(rect);

	CMemDC dc(&pdc, &rect);

	dc.BitBlt(0, 0, rect.Width(), rect.Height(), &dcWheel, 0, 0, SRCCOPY);

	dc.IntersectClipRect(&rect);

//	m_Hue = (double)(point.x + 1) / (double)rect.Width();
//	m_Sat = (double)(rect.Height() - point.y - 1) / (double)rect.Height();

	// Draw the cross.
	dc.FillSolidRect(m_ptMouse.x + 5, m_ptMouse.y - 1, 5, 3, RGB(0, 0, 0));
	dc.FillSolidRect(m_ptMouse.x - 1, m_ptMouse.y + 5, 3, 5, RGB(0, 0, 0));
	dc.FillSolidRect(m_ptMouse.x - 9, m_ptMouse.y - 1, 5, 3, RGB(0, 0, 0));
	dc.FillSolidRect(m_ptMouse.x - 1, m_ptMouse.y - 9, 3, 5, RGB(0, 0, 0));
}

void CColorWheel::OnDestroy() 
{
	dcWheel.SelectObject(oldWheelBmp);

	CWnd::OnDestroy();
}

void CColorWheel::PreSubclassWindow() 
{
	CWnd::PreSubclassWindow();

	CRect rect;  GetClientRect(rect);

	CDC* pDC = GetDC();

	dcWheel.CreateCompatibleDC(pDC);
	bmpWheel.CreateCompatibleBitmap(pDC, rect.Width(), rect.Height());
	oldWheelBmp = dcWheel.SelectObject(&bmpWheel);

	WORD ref;
	COLORREF tcol;
	for(int i = 0; i < rect.Width(); i++)
	{
		for(int j = 0; j < rect.Height(); j++)
		{
			ref = HLStoRGB555((double)i / (double)rect.Width(), DEFAULT_LUMINANCE,
				(double)j / (double)rect.Height());
			tcol = RGB(R555(ref) * 8, G555(ref) * 8, B555(ref) * 8);
			dcWheel.SetPixelV(i, rect.Height() - j - 1, tcol);
		}
	}

	ReleaseDC(pDC);
}

void CColorWheel::OnLButtonDown(UINT nFlags, CPoint point) 
{
	SetCapture();
	m_bTracking = TRUE;

	CRect rect;  GetWindowRect(rect);
	::ClipCursor(&rect);
	GetClientRect(rect);

	COLORREF ref = dcWheel.GetPixel(point);
	
	if(nFlags & MK_CONTROL) point.x = m_ptMouse.x;
	if(nFlags & MK_SHIFT) point.y = m_ptMouse.y;

	point.x /= rect.Width() / 30;  point.x *= rect.Width() / 30;
	point.y /= rect.Height() / 30;  point.y *= rect.Height() / 30;

	m_Hue = (double)(point.x) / (double)rect.Width();
	m_Sat = (double)(rect.Height() - point.y) / (double)rect.Height();

	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), BN_CLICKED), (LPARAM)GetSafeHwnd());

	CWnd::OnLButtonDown(nFlags, point);
	m_ptMouse = point;

	Invalidate();
}

void CColorWheel::OnLButtonUp(UINT nFlags, CPoint point) 
{
	ReleaseCapture();
	m_bTracking = FALSE;
	::ClipCursor(NULL);

	CWnd::OnLButtonUp(nFlags, point);
}

void CColorWheel::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(!m_bTracking) return;

	CRect rect;  GetClientRect(rect);
	rect.DeflateRect(0, 0, 4, 0);

	COLORREF ref = dcWheel.GetPixel(point);
	
	if(nFlags & MK_CONTROL) point.x = m_ptMouse.x;
	if(nFlags & MK_SHIFT) point.y = m_ptMouse.y;

	point.x /= rect.Width() / 30;  point.x *= rect.Width() / 30;
	point.y /= rect.Height() / 30;  point.y *= rect.Height() / 30;

	m_Hue = (double)(point.x + 1) / (double)rect.Width();
	m_Sat = (double)(rect.Height() - point.y) / (double)rect.Height();

	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), BN_CLICKED), (LPARAM)GetSafeHwnd());

	CWnd::OnMouseMove(nFlags, point);
	m_ptMouse = point;

	Invalidate();
}

BOOL CColorWheel::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}

void CColorWheel::SetHueSat(double h, double s)
{
	CRect rect;  GetClientRect(rect);

	m_Hue = h;
	m_Sat = s;

	m_ptMouse.x = h * (rect.Width()) - 1;
	m_ptMouse.y = -rect.Height() * (s - 1) - 1;
	Invalidate();
}
