// CommandBar.h : header file
//
// You are free to use, modify and distribute this source, as long as
// it is not sold for profit, and this HEADER stays intact. This source is
// supplied "AS-IS", and without WARRANTY OF ANY KIND. The user
// agrees to hold the author(s) blameless from any or all problems that
// may arise from the use or distribution of this code.
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __COMMANDBAR_H__
#define __COMMANDBAR_H__

class CCommandBar;
class CCommandBarControlInfo;
class CResourceCollection;
class CCommandBarDockContext;
class CCommandBarList;
class CCommandBarHitTestInfo;

#include "CommandBarPopup.h"

// ToolBarEx creation styles, replace some of the unused CBRS_ styles
#define CBCS_MENUBAR		0x0080		// Sets this as the app's menu bar.  Stretches it when docked on top.

/////////////////////////////////////////////////////////////////////////////
// CCommandBar class

class CCommandBar : public CControlBar
{
protected:
#ifdef _DEBUG
	void AssertValid() const;
#endif

	void SetBarStyle(DWORD dwStyle);

public:
	CCommandBar();
	virtual ~CCommandBar();
	DECLARE_DYNAMIC(CCommandBar)

// Construction
	void GetItemRect(int nIndex, LPRECT lpRect);

	void SetControlState(int nIndex, WORD nState);
	WORD GetControlState(int nIndex);
	void ModifyControlState(int nIndex, WORD dwRemove, WORD dwAdd);

	void GetControl(int nIndex, CCommandBarControlInfo* lptbec);

	int CommandToIndex(UINT nIDFind, int nStart = 0);
	UINT GetItemID(int nIndex);
	int GetControlCount() const;

	CEdit* GetEditControl(int nIndex);

	BOOL AddControl(CCommandBarControlInfo* lptbec);
	BOOL AddControls(DWORD* aControls);
	BOOL AddControl(int nIndex, DWORD nCmd);

	void SetControlSize(int nIndex, CSize size);
	CSize GetControlSize(int nIndex);

	void SetControlImage(int nIndex, int nImage);
	int GetControlImage(int nIndex);

	void SetControlStyle(int nIndex, int nStyle);
	int GetControlStyle(int nIndex);

	void SetControlText(int nIndex, LPCTSTR lpszText);
	CString GetControlText(int nIndex);
	void GetControlText(int nIndex, CString& rString);

	void RedrawControl(int nIndex);
	void ShadeRect(CDC* pDC, LPRECT lpRect);

	// The nControls argument allows you to specify how many controls to load from the
	// toolbar image.  This is useful for situations with dropswitches when all the
	// commands and images can be stored at the end of the toolbar and then only the
	// ones before it can be loaded, so the needed images are still in the toolbar,
	// but not shown until necessary.  Specifying 0 loads all images.
	BOOL LoadToolBar(UINT nIDToolbar, UINT nControls = 0);

// Attributes
public:
	virtual BOOL Create(CWnd* pParentWnd, LPCTSTR lpszName, CResourceCollection* pRes,
		DWORD dwStyle = WS_VISIBLE | WS_CHILD | CBRS_ALIGN_TOP | CBRS_SIZE_DYNAMIC,
		UINT nID = AFX_IDW_TOOLBAR);

protected:
	COLORREF m_clrGrayed;
	COLORREF m_clrHilite;
	COLORREF m_clrShadow;
	COLORREF m_clrDkShadow;
	COLORREF m_clrButton;
	COLORREF m_clrWindow;
	COLORREF m_clrChecked;
	COLORREF m_clrText;

	int m_horzLeftMargin;	// Space to leave at the left for grippers when horiz.
	int m_vertTopMargin;	// Space to leave at the top for grippers when vert.

	CMenu m_menu;

	CResourceCollection* m_pResources;
	int m_iComboCapture;

	BOOL m_bBarHit;

	CCommandBarList* m_wndList;

	CCommandBarDockContext* m_pDockContextEx;
	CSize m_sizeBar;
	DWORD m_dwExStyle;
	BOOL m_bDelayedControlLayout;
	int m_iLastHot;
	void HitTest(CCommandBarHitTestInfo* tbehti);

	int m_iControlCapture;

	BOOL m_bMenuDropped;

	virtual void _DrawBasic(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl);
	virtual void _DrawText(CDC* pDC, CRect* rect, CCommandBarControlInfo* pCtrl);
	virtual void _DrawImage(CDC* pDC, CRect* rect, CCommandBarControlInfo* pCtrl);
	virtual void _DrawArrow(CDC* pDC, CRect* rect, CCommandBarControlInfo* pCtrl, UINT direction);
	virtual void _DrawInsertBar(CDC* pDC, CRect* rect);

	enum arrowDirection
	{
		arrowUp, arrowDown, arrowLeft, arrowRight
	};

	virtual void _DrawSeparator(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl);
	virtual void _DrawLabel(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl);
	virtual void _DrawButton(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl);
	virtual void _DrawCaptionButton(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl);
	virtual void _DrawButtonDropDown(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl);
	virtual void _DrawComboOrDropDown(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl);
	virtual void _DrawEdit(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl);
	virtual void _DrawPopup(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl);
	virtual void _DrawSysMenu(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl);
	void __ReleaseDroppedControl();
	void __EnableEditControl(int nIndex, BOOL bOn);

	virtual void CalcOptimalControlSize(CDC* pDC, CCommandBarControlInfo* pCtrl);
	void FilterAmpersand(CString& original, CString& before, CString& ampChar, CString& after);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CCommandBar)
	//}}AFX_VIRTUAL
	int OnToolHitTest(CPoint point, TOOLINFO* pTI) const;

// Implementation
public:
	virtual CSize CalcFixedLayout(BOOL bStretch, BOOL bHorz);
	virtual CSize CalcDynamicLayout(int nLength, DWORD dwMode);
	virtual CSize CalcLayout(DWORD dwMode, int nLength = -1);
	virtual CSize CalcSize(int nCount);
	virtual void SizeToolBar(int nCount, int nLength, BOOL bVert = FALSE);
	virtual int WrapToolBar(int nCount, int nWidth);
	virtual void EnableDocking(DWORD dwDockStyle);

	inline BOOL IsOrientVert() const { return m_dwStyle & CBRS_ORIENT_VERT; } 
	inline BOOL IsOrientHorz() const { return m_dwStyle & CBRS_ORIENT_HORZ; } 

protected:
	virtual void DrawBorders(CDC* pDC);
	virtual void DrawGripper(CDC& dc) const;

// Generated message map functions
protected:
	//{{AFX_MSG(CCommandBar)
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnPaint();
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnDestroy();
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnCancelMode();
	afx_msg void OnWindowPosChanging(WINDOWPOS FAR* lpwndpos);
	afx_msg void OnLButtonDblClk(UINT nFlags, CPoint point);
	afx_msg void OnSysColorChange();
	afx_msg void OnRButtonUp(UINT nFlags, CPoint point);
	//}}AFX_MSG
	afx_msg void OnUpdateCmdUI(CFrameWnd* pTarget, BOOL bDisableIfNoHndler);
	afx_msg UINT OnNcHitTest(CPoint point);
	afx_msg void OnCaptureChanged(CWnd* pWnd);

	DECLARE_MESSAGE_MAP()

protected:
	CArray<UINT, UINT> m_aRowHeights;
	CTypedPtrArray<CPtrArray, CCommandBarControlInfo*> m_aControls;
	CFont m_horizFont, m_vertFont, m_horizUFont, m_vertUFont;

	friend class CMiniDockFrameWndEx;
	friend class CDockBarEx;
	friend class CCommandBarEdit;
	friend class CCommandBarList;
	friend class CResourceCollection;
	friend class CCommandBarCmdUI;
};

#include "CommandBarCommon.h"
#include "CommandBarDockContext.h"
#include "DockBarEx.h"

/////////////////////////////////////////////////////////////////////////////

#endif // __COMMANDBAR_H__

