#include "stdafx.h"
#include "CommandBar.h"
#include "CommandBarChildren.h"

///////////////////////////////////////////////////////////
//
//  CCommandBarEdit class -- implementation for
//    embedded edit controls in the toolbars
//

BEGIN_MESSAGE_MAP(CCommandBarEdit, CEdit)
	ON_CONTROL_REFLECT(EN_KILLFOCUS, OnKillFocus)
	ON_CONTROL_REFLECT(EN_SETFOCUS, OnSetFocus)
	ON_WM_MOUSEMOVE()
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCommandBarEdit message handlers

BOOL CCommandBarEdit::Create(CWnd* pParentWnd, UINT nID, CResourceCollection* pRes)
{
	m_pResources = pRes;

	return CEdit::Create(WS_CHILD | WS_VISIBLE | ES_AUTOHSCROLL,
		CRect(0, 0, 0, 0), pParentWnd, nID);
}

void CCommandBarEdit::OnMouseMove(UINT nFlags, CPoint point) 
{
	// Convert the point from edit-relative to toolbar-relative and forward the
	// message to the toolbar as well as the base class implementation.  By
	// forwarding to the parent, we make sure the hot effect is still produced
	// when the mouse is moved quickly to the edit control without touching the
	// zone around it.  (DHTML's event bubbling would be mighty handy here, eh?)

	CPoint ptTBRel = point;
	ClientToScreen(&ptTBRel);
	GetParent()->ScreenToClient(&ptTBRel);	// The point is now toolbar-relative.
	
	LPARAM dwPoint = MAKELONG(ptTBRel.x, ptTBRel.y);
	GetParent()->SendMessage(WM_MOUSEMOVE, (WPARAM)nFlags, (LPARAM)dwPoint);

	CEdit::OnMouseMove(nFlags, point);
}

void CCommandBarEdit::OnSetFocus()
{
	m_bAccept = FALSE;
	m_pResources->SetLockState(TRUE);
	GetWindowText(m_strText);
	SetSel(0, -1, TRUE);
}

void CCommandBarEdit::OnKillFocus() 
{
	// Check to see if we are a stand-alone edit control or if we belong to
	// a combobox, then pass the appropriate notification.
	DWORD dwType = m_pResources->GetControlType(GetDlgCtrlID());

	CString oldText = m_strText;
	CString str;  GetWindowText(str);

	m_pResources->SetLockState(FALSE);

	if(m_bAccept)
	{
		if(dwType == cBarControlEdit)
		{
			NMCBCEDIT nmcbce;
			nmcbce.hdr.hwndFrom = GetSafeHwnd();
			nmcbce.hdr.idFrom = GetDlgCtrlID();
			nmcbce.hdr.code = CBCN_EDITCHANGED;
			nmcbce.strText = str;			// The user can change this if needed.
			LRESULT res = GetOwner()->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)&nmcbce);
			if(res) nmcbce.strText = oldText;

			m_pResources->SetControlEntry(GetDlgCtrlID(), nmcbce.strText);
			m_strText = nmcbce.strText;
		}
		else if(dwType == cBarControlComboBox || dwType == cBarControlGraphicCombo)
		{
			NMCBCCOMBO nmcbcc;
			nmcbcc.hdr.hwndFrom = GetSafeHwnd();
			nmcbcc.hdr.idFrom = GetDlgCtrlID();
			nmcbcc.hdr.code = CBCN_COMBOSELCHANGED;
			nmcbcc.nIndex = -1;				// No index for an edit change
			nmcbcc.strText = str;			// The user can change this if needed.
			LRESULT res = GetOwner()->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)&nmcbcc);
			if(res) nmcbcc.strText = oldText;

			m_pResources->SetControlEntry(GetDlgCtrlID(), nmcbcc.strText);
			m_strText = nmcbcc.strText;
		}
	}
	else
	{
		if(dwType == cBarControlEdit)
		{
			NMCBCEDIT nmcbce;
			nmcbce.hdr.hwndFrom = GetSafeHwnd();
			nmcbce.hdr.idFrom = GetDlgCtrlID();
			nmcbce.hdr.code = CBCN_EDITCANCEL;
			nmcbce.strText = str;			// The user can change this if needed.
			LRESULT res = GetOwner()->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)&nmcbce);

			m_pResources->SetControlEntry(GetDlgCtrlID(), m_strText);
		}
		else if(dwType == cBarControlComboBox || dwType == cBarControlGraphicCombo)
		{
			NMCBCCOMBO nmcbcc;
			nmcbcc.hdr.hwndFrom = GetSafeHwnd();
			nmcbcc.hdr.idFrom = GetDlgCtrlID();
			nmcbcc.hdr.code = CBCN_COMBOCANCEL;
			nmcbcc.nIndex = -1;
			nmcbcc.strText = str;			// The user can change this if needed.
			LRESULT res = GetOwner()->SendMessage(WM_NOTIFY, GetDlgCtrlID(), (LPARAM)&nmcbcc);

			m_pResources->SetControlEntry(GetDlgCtrlID(), m_strText);
		}
	}
}

BOOL CCommandBarEdit::PreTranslateMessage(MSG* pMsg) 
{
	if(pMsg->message == WM_KEYDOWN)
	{
		if(pMsg->wParam == VK_RETURN || pMsg->wParam == VK_TAB)
		{
			m_bAccept = TRUE;
			((CFrameWnd*)GetOwner())->GetActiveView()->SetFocus();
			return TRUE;
		}
		else if(pMsg->wParam == VK_ESCAPE)
		{
			m_bAccept = FALSE;
			SetWindowText(m_strText);
			((CFrameWnd*)GetOwner())->GetActiveView()->SetFocus();
			return TRUE;
		}
	}

	if(pMsg->message == WM_LBUTTONDOWN)
	{
		if(::GetFocus() != GetSafeHwnd())
		{
			::TranslateMessage(pMsg);
			::DispatchMessage(pMsg);
			return TRUE;
		}
	}

	return CEdit::PreTranslateMessage(pMsg);
}


///////////////////////////////////////////////////////////
//
//  CCommandBarList class -- implementation for
//    drop down list controls in the toolbars
//

BEGIN_MESSAGE_MAP(CCommandBarList, CMiniFrameWnd)
	ON_WM_CREATE()
	ON_WM_ACTIVATE()
	ON_MESSAGE(WM_SETFONT, OnSetFont)
	ON_WM_MEASUREITEM()
	ON_WM_DRAWITEM()
END_MESSAGE_MAP()

BOOL CCommandBarList::Create(CWnd* pParentWnd, UINT nID, CResourceCollection* pRes)
{
	m_pResources = pRes;

	LPCTSTR className = AfxRegisterWndClass(CS_SAVEBITS,
		AfxGetApp()->LoadStandardCursor(IDC_ARROW), (HBRUSH)(COLOR_WINDOW + 1), NULL);

	DWORD dwWndStyle = WS_POPUP | MFS_SYNCACTIVE;

	BOOL rval = CMiniFrameWnd::CreateEx(0, className, NULL, dwWndStyle, CRect(0, 0, 0, 0),
		pParentWnd);

	if(!rval) return FALSE;

	m_wndList.Create(WS_CHILD | WS_VISIBLE | WS_VSCROLL | LBS_NOTIFY | LBS_NOINTEGRALHEIGHT |
		LBS_HASSTRINGS | LBS_OWNERDRAWFIXED, CRect(0, 0, 0, 0), this, nID);

	return rval;
}

void CCommandBarList::Invoke(CPoint pt, int nWidth, BOOL bOwnerDraw)
{
	m_pResources->SetLockState(TRUE);

	m_bValid = FALSE;
	m_bCanceled = FALSE;

//	m_wndList.SetDlgCtrlID(nID);

	m_wndList.ResetContent();
	m_images.DeleteImageList();

	m_bOwnerDraw = bOwnerDraw;

	NMCBCINITLIST nmtbeil;
	nmtbeil.hdr.hwndFrom = m_wndList.GetSafeHwnd();
	nmtbeil.hdr.idFrom = m_wndList.GetDlgCtrlID();
	nmtbeil.hdr.code = CBCN_INITLIST;
	nmtbeil.pWnd = &m_wndList;
	nmtbeil.images = 0;				// The user will initialize this if needed.
	nmtbeil.imageWidth = 0;			// Ditto.
	GetOwner()->SendMessage(WM_NOTIFY, m_wndList.GetDlgCtrlID(), (LPARAM)&nmtbeil);

	if(nmtbeil.images) m_images.Create(nmtbeil.images, nmtbeil.imageWidth, 0, RGB(255, 0, 255));

	BOOL bScrollbar = FALSE;
	
	int nItemHeight = m_wndList.GetItemHeight(0);
	int nTotalHeight = m_wndList.GetCount() * nItemHeight;
	if(nTotalHeight > nItemHeight * 12)
	{
		nTotalHeight = nItemHeight * 12;
		bScrollbar = TRUE;
	}

	int nTotalWidth = 0;

	if(!m_bOwnerDraw)
	{
		for(int i = 0; i < m_wndList.GetCount(); i++)
		{
			CString strItem;  m_wndList.GetText(i, strItem);
			CSize extent = m_wndList.GetDC()->GetTextExtent(strItem);
			if(extent.cx > nTotalWidth) nTotalWidth = extent.cx;
		}

		if(nWidth > nTotalWidth) nTotalWidth = nWidth;
		nTotalWidth += nmtbeil.imageWidth;
	}
	else
	{
		nTotalWidth = m_sizeItem.cx;
	}

	if(bScrollbar) nTotalWidth += GetSystemMetrics(SM_CXVSCROLL);
	
	int cxScreen = GetSystemMetrics(SM_CXSCREEN);
	int cyScreen = GetSystemMetrics(SM_CYSCREEN);

	SetWindowPos(NULL, pt.x, pt.y,
		nTotalWidth, nTotalHeight + 4, SWP_SHOWWINDOW);
	m_wndList.SetWindowPos(NULL, 0, 0, nTotalWidth, nTotalHeight + 4, SWP_SHOWWINDOW);
}

void CCommandBarList::OnMeasureItem(int nID, LPMEASUREITEMSTRUCT lpmis)
{
	if(m_bOwnerDraw)
	{
		NMCBCMEASUREITEM nmtbeil;
		nmtbeil.hdr.hwndFrom = m_wndList.GetSafeHwnd();
		nmtbeil.hdr.idFrom = m_wndList.GetDlgCtrlID();
		nmtbeil.hdr.code = CBCN_MEASUREITEM;
		nmtbeil.pWnd = &m_wndList;
		nmtbeil.itemWidth = -1;				// The user will initialize this if needed.
		nmtbeil.itemHeight = 13;			// Ditto.
		GetOwner()->SendMessage(WM_NOTIFY, m_wndList.GetDlgCtrlID(), (LPARAM)&nmtbeil);

		m_sizeItem = CSize(nmtbeil.itemWidth, nmtbeil.itemHeight);
	}
	else
	{
		m_sizeItem = CSize(-1, 13);
	}

	lpmis->itemHeight = m_sizeItem.cy;
}

void CCommandBarList::OnDrawItem(int nID, LPDRAWITEMSTRUCT lpdis)
{
	// Only one control will ever need painting, and since its ID changes,
	// it's counterproductive to confirm this.

	CDC dc;
	dc.Attach(lpdis->hDC);

	CRect rect = lpdis->rcItem;
	UINT nIndex = lpdis->itemID;

	if(nIndex != (UINT)-1)
	{
		int nImage = m_wndList.GetItemData(nIndex);
		CString strText;  m_wndList.GetText(nIndex, strText);

		if(m_bOwnerDraw)
		{
			dc.FillSolidRect(rect, ::GetSysColor(COLOR_WINDOW));

			NMCBCDRAWITEM nm;
			nm.hdr.hwndFrom = m_wndList.GetSafeHwnd();
			nm.hdr.idFrom = m_wndList.GetDlgCtrlID();
			nm.hdr.code = CBCN_DRAWITEM;
			nm.pWnd = &m_wndList;
			nm.pDC = &dc;
			nm.nDrawType = cBarDrawTypeList;
			nm.nIndex = m_pResources->GetControlSelection(m_wndList.GetDlgCtrlID());
			nm.nState = lpdis->itemState;
			nm.strEntry = m_pResources->GetControlEntry(m_wndList.GetDlgCtrlID());
			nm.rect = rect;
			GetOwner()->SendMessage(WM_NOTIFY, m_wndList.GetDlgCtrlID(), (LPARAM)&nm);
		}
		else
		{
			dc.FillSolidRect(rect, ::GetSysColor((lpdis->itemState & ODS_SELECTED) ?
				COLOR_HIGHLIGHT : COLOR_WINDOW));

			CRect textRect = rect;

			if(m_images.GetSafeHandle())
			{
				IMAGEINFO ii;
				m_images.GetImageInfo(0, &ii);
				int nWidth = ii.rcImage.right - ii.rcImage.left;
				int nHeight = ii.rcImage.bottom - ii.rcImage.top;

				m_images.Draw(&dc, nImage,
					CPoint(rect.left, rect.top + (rect.Height() - nHeight) / 2), ILD_NORMAL);

				textRect.left += nWidth;
			}

			textRect.left += 2;

			CFont* oldFont = dc.SelectObject(m_wndList.GetFont());
			dc.SetBkMode(TRANSPARENT);
			dc.SetTextColor(::GetSysColor((lpdis->itemState & ODS_SELECTED) ?
				COLOR_HIGHLIGHTTEXT : COLOR_WINDOWTEXT));
			dc.DrawText(strText, &textRect, DT_LEFT | DT_VCENTER | DT_SINGLELINE | DT_NOPREFIX);
			dc.SelectObject(oldFont);
		}
	}

	dc.Detach();
}

void CCommandBarList::GetText(int nIndex, CString& rString)
{
	m_wndList.GetText(nIndex, rString);
}

void CCommandBarList::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized) 
{
	CMiniFrameWnd::OnActivate(nState, pWndOther, bMinimized);

	if(nState == WA_INACTIVE && !m_bValid)
	{
		m_wndList.SetCurSel(-1);
		if(!m_bCanceled) CancelWindow();
	}
}

void CCommandBarList::CancelWindow()
{
	m_bCanceled = TRUE;

	int sel = m_wndList.GetCurSel();

	if(sel != -1 && m_bValid)
	{
		CString str;
		m_wndList.GetText(sel, str);
				
		NMCBCCOMBO nmcbcc;
		nmcbcc.hdr.hwndFrom = m_wndList.GetSafeHwnd();
		nmcbcc.hdr.idFrom = m_wndList.GetDlgCtrlID();
		nmcbcc.hdr.code = CBCN_COMBOSELCHANGING;
		nmcbcc.nIndex = sel;
		nmcbcc.strText = str;		// The user can change this if needed.
		GetOwner()->SendMessage(WM_NOTIFY, m_wndList.GetDlgCtrlID(), (LPARAM)&nmcbcc);

		m_pResources->SetControlEntry(m_wndList.GetDlgCtrlID(), nmcbcc.strText);
	}

	ShowWindow(SW_HIDE);
	m_pResources->SetLockState(FALSE);

	m_pBar->__ReleaseDroppedControl();

	if(sel != -1 && m_bValid)
	{
		CString str = m_pResources->GetControlEntry(m_wndList.GetDlgCtrlID());

		NMCBCCOMBO nmcbcc;
		nmcbcc.hdr.hwndFrom = m_wndList.GetSafeHwnd();
		nmcbcc.hdr.idFrom = m_wndList.GetDlgCtrlID();
		nmcbcc.hdr.code = CBCN_COMBOSELCHANGED;
		nmcbcc.nIndex = sel;
		nmcbcc.strText = str;
		GetOwner()->SendMessage(WM_NOTIFY, m_wndList.GetDlgCtrlID(), (LPARAM)&nmcbcc);
	}
	else
	{
		NMCBCCOMBO nmcbcc;
		nmcbcc.hdr.hwndFrom = m_wndList.GetSafeHwnd();
		nmcbcc.hdr.idFrom = m_wndList.GetDlgCtrlID();
		nmcbcc.hdr.code = CBCN_COMBOCANCEL;
		nmcbcc.nIndex = -1;
		nmcbcc.strText.Empty();
		GetOwner()->SendMessage(WM_NOTIFY, m_wndList.GetDlgCtrlID(), (LPARAM)&nmcbcc);
	}

	//DestroyWindow();
}

LONG CCommandBarList::OnSetFont(UINT hFont, LONG fRedraw)
{
	m_wndList.SendMessage(WM_SETFONT, hFont, fRedraw);
	return NULL;
}

BOOL CCommandBarList::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	if(HIWORD(wParam) == LBN_SELCHANGE)
	{
		m_bValid = TRUE;
		CancelWindow();
	}

	return CMiniFrameWnd::OnCommand(wParam, lParam);
}

///////////////////////////////////////////////////////////
//
//  XCommandBarListBox class -- implementation for
//    drop down list controls in the toolbars
//

BEGIN_MESSAGE_MAP(XCommandBarListBox, CListBox)
	ON_WM_MOUSEMOVE()
	ON_WM_NCPAINT()
	ON_WM_NCCALCSIZE()
END_MESSAGE_MAP()

void XCommandBarListBox::OnMouseMove(UINT nFlags, CPoint point) 
{
	CListBox::OnMouseMove(nFlags, point);

	BOOL bOutside;
	int nSel = (int)ItemFromPoint(point, bOutside);
	if(bOutside) nSel = -1;

	if(GetCurSel() != nSel) SetCurSel(nSel);
}

int XCommandBarListBox::AddItem(LPCTSTR lpszText, int nImage)
{
	int nIndex = AddString(lpszText);  SetItemData(nIndex, nImage);
	return nIndex;
}

void XCommandBarListBox::OnNcCalcSize(BOOL bCalcValidRects, NCCALCSIZE_PARAMS* lpncsp)
{
	CListBox::OnNcCalcSize(bCalcValidRects, lpncsp);

	if(bCalcValidRects)
	{
		lpncsp->rgrc[0].left += 2;
		lpncsp->rgrc[0].top += 2;
		lpncsp->rgrc[0].right -= 2;
		lpncsp->rgrc[0].bottom -= 2;
	}
}

void XCommandBarListBox::OnNcPaint()
{
	CListBox::OnNcPaint();

	CWindowDC dc(this);
	CRect rect;  GetWindowRect(&rect);  ScreenToClient(&rect);
	rect.OffsetRect(2, 2);

	dc.DrawEdge(&rect, EDGE_RAISED, BF_RECT);
}
