#ifndef __COMMANDBARCOMMON_H__
#define __COMMANDBARCOMMON_H__

#include "ResourceCollection.h"
#include "CommandBarChildren.h"


// Notification messages
#define CBCN_FIRST				(0U-700U)
#define CBCN_INITLIST			(CBCN_FIRST - 1)
#define CBCN_MEASUREITEM		(CBCN_FIRST - 2)
#define CBCN_DRAWITEM			(CBCN_FIRST - 3)
#define CBCN_COMBOSELCHANGING	(CBCN_FIRST - 4)
#define CBCN_COMBOSELCHANGED	(CBCN_FIRST - 5)
#define CBCN_COMBOCANCEL		(CBCN_FIRST - 7)
#define CBCN_EDITCHANGED		(CBCN_FIRST - 8)
#define CBCN_EDITCANCEL			(CBCN_FIRST - 9)
#define CBCN_DROPDOWN			(CBCN_FIRST - 10)


// Message crackers for simplicity
#define ON_CBCN_INITLIST(id, memberFxn)			ON_NOTIFY(CBCN_INITLIST, id, memberFxn)
#define ON_CBCN_MEASUREITEM(id, memberFxn)		ON_NOTIFY(CBCN_MEASUREITEM, id, memberFxn)
#define ON_CBCN_DRAWITEM(id, memberFxn)			ON_NOTIFY(CBCN_DRAWITEM, id, memberFxn)
#define ON_CBCN_COMBOSELCHANGING(id, memberFxn)	ON_NOTIFY(CBCN_COMBOSELCHANGING, id, memberFxn)
#define ON_CBCN_COMBOSELCHANGED(id, memberFxn)	ON_NOTIFY(CBCN_COMBOSELCHANGED, id, memberFxn)
#define ON_CBCN_COMBOEDITCHANGED(id, memberFxn)	ON_NOTIFY(CBCN_COMBOEDITCHANGED, id, memberFxn)
#define ON_CBCN_COMBOCANCEL(id, memberFxn)		ON_NOTIFY(CBCN_COMBOCANCEL, id, memberFxn)
#define ON_CBCN_EDITCHANGED(id, memberFxn)		ON_NOTIFY(CBCN_EDITCHANGED, id, memberFxn)
#define ON_CBCN_EDITCANCEL(id, memberFxn)		ON_NOTIFY(CBCN_EDITCANCEL, id, memberFxn)
#define ON_CBCN_DROPDOWN(id, memberFxn)			ON_NOTIFY(CBCN_DROPDOWN, id, memberFxn)


typedef struct
{
    NMHDR				hdr;
	XCommandBarListBox*	pWnd;
	UINT				images;
	int					imageWidth;
} NMCBCINITLIST, FAR *LPNMCBCINITLIST;

typedef struct
{
    NMHDR				hdr;
	CWnd*				pWnd;
	int					itemWidth;
	int					itemHeight;
} NMCBCMEASUREITEM, FAR *LPNMCBCMEASUREITEM;

typedef struct
{
    NMHDR       hdr;
	CWnd*		pWnd;
	CDC*		pDC;
	int			nDrawType;
	int			nIndex;
	CString		strEntry;
	UINT		nState;
	CRect		rect;
} NMCBCDRAWITEM, FAR *LPNMCBCDRAWITEM;

typedef enum {
	cBarDrawTypeNone = 0,
	cBarDrawTypeBar = 1,
	cBarDrawTypeList = 2,
} CmdBarDrawType;

typedef struct
{
	NMHDR		hdr;
	int			nIndex;
	CString		strText;
} NMCBCCOMBO, FAR *LPNMCBCCOMBO;

typedef struct
{
	NMHDR		hdr;
	CString		strText;
} NMCBCEDIT, FAR *LPNMCBCEDIT;

typedef struct
{
	NMHDR		hdr;
	CPoint		ptMenu;
	UINT		nCmd;
} NMCBCDROPDOWN, FAR *LPNMCBCDROPDOWN;


class CCommandBarControlInfo : public CObject
{
public:
	CCommandBarControlInfo()
	{
		index = 0;  command = 0;  row = 0;  bWrap = FALSE;
		pEdit = NULL;  style = 0;  bHot = FALSE;  bHidden = FALSE;  bCompact = FALSE;
		bCustomInsert = FALSE;  bCustomMarked = FALSE;  bPressed = FALSE;  bDropped = FALSE;
		image = -1;  style = 0;
	}

	~CCommandBarControlInfo() { if(pEdit) delete pEdit; }

	DWORD index;
	UINT command;
	
	DWORD style;
	int image;
	CString label;

	CSize requestedSize;

protected:
	BOOL bPressed;
	BOOL bDropped;
	BOOL bHot;
	BOOL bHidden;
	BOOL bCompact;
	
	BOOL bCustomInsert;
	BOOL bCustomMarked;

	CSize size;

	// Used internally only.
	CPoint point;		// Top left of the "zone" the control will appear in.  This may or may not be the true top left of the control.
	CSize textExtent;	// The size of the text in the control.
	UINT row;			// The row the control is in.
	BOOL bWrap;			// Wrap after this control?

	CCommandBarEdit* pEdit;

//	CMenu dropDown;

	friend class CCommandBar;
	friend class CCommandBarPopup;
	friend class CResourceCollection;
};

class CCommandBarHitTestInfo
{
public:
	POINT pt;
	int flags;
	int item;
};

typedef enum {
	cBarHitNowhere = 0,
	cBarHitOnContent = 1,
	cBarHitOnArrow = 2,
} CmdBarHitTest;

#endif