// CommandBarPopup.cpp : implementation file
//
// You are free to use, modify and distribute this source, as long as
// it is not sold for profit, and this HEADER stays intact. This source is
// supplied "AS-IS", and without WARRANTY OF ANY KIND. The user
// agrees to hold the author(s) blameless from any or all problems that
// may arise from the use or distribution of this code.
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <..\src\afximpl.h>
#include "CommandBarPopup.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

// Some useful constants (makes things easy to update)
static const CString g_strCommandBarPopupClass = "MFCCommandBarPopup";

#define DROPDOWNWIDTH			9		// Width of the dropdown button on a dropdown control
#define DROPDOWNSPACE			2		// Space between dropdown button and next control
#define SEPARATORDIM			10		// Width of separator
#define IMAGEWIDTH				16		// Standard image width
#define IMAGEHEIGHT				16		// Standard image height
#define BUTTONWIDTH				19		// Standard button width
#define BUTTONHEIGHT			19		// Standard button height
#define HORZFLOATMARGIN			3		// Margins when floating
#define VERTFLOATMARGIN			3		// Margins when floating
#define SEPARATORMARGIN			6		// Margins for separators

#define IMAGESIZE				CSize(IMAGEWIDTH, IMAGEHEIGHT)	// Useful in one or two places


/////////////////////////////////////////////////////////////////////////////
// CCommandBarPopup idle update through CCommandBarPopupCmdUI class

class CCommandBarPopupCmdUI : public CCmdUI        // class private to this file !
{
public: // re-implementations only
	virtual void Enable(BOOL bOn);
	virtual void SetCheck(int nCheck);
	virtual void SetText(LPCTSTR lpszText);

	friend class CCommandBarPopup;
};

/////////////////////////////////////////////////////////////////////////////
// CCommandBarPopup

CCommandBarPopup::CCommandBarPopup()
{
	m_pDockContext = NULL;

	m_iControlCapture = -1;
	m_iLastHot = -1;
	m_bDelayedControlLayout = FALSE;

	m_clrHilite = ::GetSysColor(COLOR_3DHIGHLIGHT);
	m_clrShadow = ::GetSysColor(COLOR_3DSHADOW);
	m_clrDkShadow = ::GetSysColor(COLOR_3DDKSHADOW);
	m_clrButton = ::GetSysColor(COLOR_MENU);
	m_clrWindow = ::GetSysColor(COLOR_WINDOW);
	m_clrChecked = ::GetSysColor(COLOR_3DLIGHT);
	m_clrText = ::GetSysColor(COLOR_MENUTEXT);
	m_clrGrayed = ::GetSysColor(COLOR_GRAYTEXT);
	m_clrMenuHilite = ::GetSysColor(COLOR_HIGHLIGHT);
	m_clrMenuHiliteText = ::GetSysColor(COLOR_HIGHLIGHT);

	m_dwExStyle = 0;
	m_iComboCapture = -1;

	m_cxLeftBorder = m_cxRightBorder = 0;
	m_cyTopBorder = m_cyBottomBorder = 0;

	m_horzLeftMargin = m_vertTopMargin = 8;
}

CCommandBarPopup::~CCommandBarPopup()
{
	for(int i = 0; i < m_aControls.GetSize(); i++)
		delete m_aControls[i];
}

IMPLEMENT_DYNAMIC(CCommandBarPopup, CControlBar)

BEGIN_MESSAGE_MAP(CCommandBarPopup, CControlBar)
	//{{AFX_MSG_MAP(CCommandBarPopup)
	ON_WM_CREATE()
	ON_WM_PAINT()
	ON_WM_ERASEBKGND()
	ON_WM_MOUSEMOVE()
	ON_WM_TIMER()
	ON_WM_DESTROY()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_CANCELMODE()
	ON_WM_WINDOWPOSCHANGING()
	ON_WM_LBUTTONDBLCLK()
	ON_WM_SYSCOLORCHANGE()
	ON_WM_RBUTTONUP()
	ON_WM_ACTIVATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CCommandBarPopup::Create(CWnd* pParentWnd, LPCTSTR lpszName, const RECT& rect,
							  CResourceCollection* pRes, DWORD dwStyle, UINT nID)
{
	ASSERT_VALID(pParentWnd);   // must have a parent

	// save the style, sans styles that don't apply
	dwStyle &= ~(CBRS_BORDER_LEFT | CBRS_BORDER_RIGHT | CBRS_BORDER_TOP | CBRS_BORDER_BOTTOM |
		CBRS_FLOAT_MULTI);
	m_dwStyle = dwStyle;
	dwStyle &= ~CBRS_ALL;

	// Register the class for the CCommandBarPopup.
	WNDCLASS wc;  ZeroMemory(&wc, sizeof(WNDCLASS));
	wc.style = CS_SAVEBITS;
	wc.lpfnWndProc = AfxWndProc;
	wc.hCursor = AfxGetApp()->LoadStandardCursor(IDC_ARROW);
	wc.hbrBackground = ::GetSysColorBrush(COLOR_3DFACE);
	wc.lpszClassName = g_strCommandBarPopupClass;
	AfxRegisterClass(&wc);

	m_pResources = pRes;

	// create the HWND
	if (!CWnd::CreateEx(0, g_strCommandBarPopupClass, lpszName, dwStyle,
		rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top,
		pParentWnd->GetSafeHwnd(), NULL))
		return FALSE;

	// Note: Parent must resize itself for control bar to be resized
	return TRUE;
}

/////////////////////////////////////////////////////////////////////////////
// CCommandBarPopup message handlers

void CCommandBarPopup::DrawBorders(CDC* pDC)
{
	CRect rect;
	GetWindowRect(rect);
	ScreenToClient(rect);
	rect.OffsetRect(-rect.left, -rect.top );

	pDC->DrawEdge(rect, EDGE_RAISED, BF_RECT);
}

int CCommandBarPopup::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CControlBar::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	NONCLIENTMETRICS ncm;
	ncm.cbSize = sizeof(ncm);
	::SystemParametersInfo(SPI_GETNONCLIENTMETRICS, NULL, (PVOID)&ncm, NULL);

    // Define the font to use
	CWindowDC dc(this);
	int nHeight = ncm.lfMenuFont.lfHeight;

	m_horizFont.CreateFont(nHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET,
		OUT_CHARACTER_PRECIS, CLIP_CHARACTER_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE,
		ncm.lfMenuFont.lfFaceName);

	m_vertFont.CreateFont(nHeight, 0, -900, -900, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET,
		OUT_CHARACTER_PRECIS, CLIP_CHARACTER_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE,
		ncm.lfMenuFont.lfFaceName);

	m_horizUFont.CreateFont(nHeight, 0, 0, 0, FW_NORMAL, 0, TRUE, 0, DEFAULT_CHARSET,
		OUT_CHARACTER_PRECIS, CLIP_CHARACTER_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE,
		ncm.lfMenuFont.lfFaceName);

	m_vertUFont.CreateFont(nHeight, 0, -900, -900, FW_NORMAL, 0, TRUE, 0, DEFAULT_CHARSET,
		OUT_CHARACTER_PRECIS, CLIP_CHARACTER_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE,
		ncm.lfMenuFont.lfFaceName);

    SetFont(&m_horizFont);
	SetTimer(1, 10, NULL);

	return 0;
}

void CCommandBarPopup::OnDestroy() 
{
	// Don't use "delete" on a frame window.  Use DestroyWindow() instead.
	// The C++ object is automatically deleted by PostNcDestroy().
//	m_wndList->DestroyWindow();
//	m_wndList = NULL;

	KillTimer(1);

//	m_pResources->RemoveCommandBar(this);

	CControlBar::OnDestroy();
}

CSize CCommandBarPopup::CalcFixedLayout(BOOL bStretch, BOOL bHorz)
{
	DWORD dwMode = bStretch ? LM_STRETCH : 0;
	dwMode |= bHorz ? LM_HORZ : 0;

	return CalcLayout(dwMode);
}

CSize CCommandBarPopup::CalcDynamicLayout(int nLength, DWORD dwMode)
{
	if ((nLength == -1) && !(dwMode & LM_MRUWIDTH) && !(dwMode & LM_COMMIT) &&
		((dwMode & LM_HORZDOCK) || (dwMode & LM_VERTDOCK)))
	{
		return CalcFixedLayout(dwMode & LM_STRETCH, dwMode & LM_HORZDOCK);
	}
	
	return CalcLayout(dwMode, nLength);
}

void CCommandBarPopup::OnPaint()
{
	CPaintDC dc(this);

	CRect client;  GetClientRect(client);
	DrawBorders(&dc);

	dc.SetBkMode(TRANSPARENT);
	CFont* oldFont = dc.SelectObject(&m_horizFont);
	dc.SetTextAlign(TA_LEFT | TA_TOP);

	for(int i = 0; i < m_aControls.GetSize(); i++)
	{
		dc.SetTextColor(m_clrText);

		CCommandBarControlInfo* pCtrl = m_aControls[i];
		CRect rect;  GetItemRect(i, rect);

		switch(m_pResources->GetControlType(pCtrl->command))
		{
			case cBarControlButton:
			case cBarControlButtonPopup:
				_DrawButton(&dc, &rect, i, pCtrl);
				break;

			case cBarControlPopup:
				_DrawPopup(&dc, &rect, i, pCtrl);
				break;

			case cBarControlButtonDropDown:
				_DrawButtonDropDown(&dc, &rect, i, pCtrl);
				break;

			case cBarControlComboBox:
			case cBarControlDropDown:
			case cBarControlGraphicCombo:
			case cBarControlGraphicDropDown:
				_DrawComboOrDropDown(&dc, &rect, i, pCtrl);
				break;

			case cBarControlEdit:
				_DrawEdit(&dc, &rect, i, pCtrl);
				break;

			case cBarControlSeparator:
				_DrawSeparator(&dc, &rect, i, pCtrl);
				break;
		}
	}

	dc.SelectObject(oldFont);
}

CSize CCommandBarPopup::CalcSize(int nCount)
{
	CWindowDC dc(this);		// for text extent purposes
	CFont* oldFont = dc.SelectObject(&m_horizFont);

	CPoint cur(HORZFLOATMARGIN, VERTFLOATMARGIN);
	CSize sizeResult(BUTTONWIDTH, BUTTONHEIGHT);
	int cyRow = 0;
	int nRow = 0;
	m_aRowHeights.RemoveAll();

	if(nCount == 0)
	{
		m_sizeBar = sizeResult;
		return sizeResult;
	}

	for(int i = 0; i < nCount; i++)
	{
		CCommandBarControlInfo* pCtrl = m_aControls[i];

		DWORD dwType = m_pResources->GetControlType(pCtrl->command);

		if(pCtrl->bHidden) continue;

		if(dwType == cBarControlSeparator)
		{
			// A separator represents either a height or width.
			sizeResult.cx = max(cur.x + pCtrl->size.cx, sizeResult.cx);
			pCtrl->point = CPoint(cur.x, cur.y);
			cur.x += pCtrl->size.cx;
			pCtrl->row = nRow;
		}
		else
		{
			CalcOptimalControlSize(&dc, pCtrl);

			cyRow = max(pCtrl->size.cy, cyRow);
			pCtrl->row = nRow;

			sizeResult.cx = max(cur.x + pCtrl->size.cx, sizeResult.cx);
			sizeResult.cy = max(cur.y + cyRow, sizeResult.cy);

			pCtrl->point = CPoint(cur.x, cur.y);
			cur.x += pCtrl->size.cx;
		}

		cur.x = HORZFLOATMARGIN;
		cur.y += cyRow;

		m_aRowHeights.SetAtGrow(nRow, cyRow);
		cyRow = 0;  nRow++;

		if(dwType == cBarControlSeparator) cur.y += SEPARATORDIM;
	}

	m_aRowHeights.SetAtGrow(nRow, cyRow);
	cyRow = 0;  nRow++;

	for(i = 0; i < nCount; i++)
	{
		CCommandBarControlInfo* pCtrl = m_aControls[i];

		if(pCtrl->bHidden) continue;

		DWORD dwType = m_pResources->GetControlType(pCtrl->command);

		if(dwType == cBarControlComboBox || dwType == cBarControlGraphicCombo)
		{
			// When the control is oriented horizontally, show the edit control
			// and resize it to fit the area.
			int ecx = pCtrl->size.cy / 2 - 3;
					
			CRect rect;  GetItemRect(i, &rect);

			pCtrl->pEdit->SetWindowPos(NULL, rect.left + 3, rect.top + 2,
				rect.Width() - ecx - 11, rect.Height() - 4, SWP_SHOWWINDOW);
		}

		if(dwType == cBarControlEdit)
		{
			// When the control is oriented horizontally, show the edit control
			// and resize it to fit the area.
			CRect rect;  GetItemRect(i, &rect);

			pCtrl->pEdit->SetWindowPos(NULL, rect.left + 2, rect.top + 2,
				rect.Width() - 4, rect.Height() - 4, SWP_SHOWWINDOW);
		}
	}

	dc.SelectObject(oldFont);

	sizeResult += CSize(27, 3);

	m_sizeBar = sizeResult;
	return sizeResult;
}

void CCommandBarPopup::SizeToolBar(int nCount, int nLength, BOOL bVert)
{
	CSize size = CalcSize(nCount);
}

CSize CCommandBarPopup::CalcLayout(DWORD dwMode, int nLength)
{
	ASSERT_VALID(this);
	ASSERT(::IsWindow(m_hWnd));
	if (dwMode & LM_HORZDOCK) ASSERT(dwMode & LM_HORZ);

	int nCount;
	CSize sizeResult(0, 0);

	// Load Buttons
	nCount = m_aControls.GetSize();

	if (!(m_dwStyle & CBRS_SIZE_FIXED))
	{
		BOOL bDynamic = m_dwStyle & CBRS_SIZE_DYNAMIC;

		if (bDynamic && (dwMode & LM_MRUWIDTH))
			SizeToolBar(nCount, m_nMRUWidth);
		else if (bDynamic && (dwMode & LM_HORZDOCK))
			SizeToolBar(nCount, 32767);
		else if (bDynamic && (dwMode & LM_VERTDOCK))
			SizeToolBar(nCount, 0);
		else if (bDynamic && (nLength != -1))
		{
			CRect rect; rect.SetRectEmpty();
			CalcInsideRect(rect, (dwMode & LM_HORZ));
			BOOL bVert = (dwMode & LM_LENGTHY);
			int nLen = nLength + (bVert ? rect.Height() : rect.Width());

			SizeToolBar(nCount, nLen, bVert);
		}
		else if (bDynamic && IsFloating())
			SizeToolBar(nCount, m_nMRUWidth);
		else
			SizeToolBar(nCount, (dwMode & LM_HORZ) ? 32767 : 0);
	}

	sizeResult = CalcSize(nCount);

	if (dwMode & LM_COMMIT)
	{
		BOOL bIsDelayed = m_bDelayedControlLayout;
		m_bDelayedControlLayout = FALSE;

		if(IsFloating() && (m_dwStyle & CBRS_SIZE_DYNAMIC))
			m_nMRUWidth = sizeResult.cx;

		m_bDelayedControlLayout = bIsDelayed;
	}

	// Adjust Margins
	{
		CRect rect; rect.SetRectEmpty();
		CalcInsideRect(rect, (dwMode & LM_HORZ));
		sizeResult.cy -= rect.Height();
		sizeResult.cx -= rect.Width();

		CSize size = CControlBar::CalcFixedLayout((dwMode & LM_STRETCH), (dwMode & LM_HORZ));
		sizeResult.cx = max(sizeResult.cx, size.cx);
		sizeResult.cy = max(sizeResult.cy, size.cy);
	}

	SetWindowPos(NULL, 0, 0, sizeResult.cx, sizeResult.cy, SWP_NOZORDER | SWP_NOMOVE);

	return sizeResult;
}

// Structure local to this function.
struct CToolBarData
{
	WORD wVersion;
	WORD wWidth;
	WORD wHeight;
	WORD wItemCount;
	WORD* items() { return (WORD*)(this + 1); }	// WORD aItems[wItemCount]
};

BOOL CCommandBarPopup::LoadToolBar(UINT nIDToolbar, UINT nControls)
{
	// Remove all previous controls and images
	for(int j = 0; j < m_aControls.GetSize(); j++) delete m_aControls[j];
	m_aControls.RemoveAll();

	// load toolbar
	LPTSTR lpResName = MAKEINTRESOURCE(nIDToolbar);
	HINSTANCE hInst;
	HRSRC hRsrc;
	CToolBarData* ptbd;

	if((hInst= AfxFindResourceHandle(lpResName, RT_TOOLBAR)) == NULL ||
	   (hRsrc= FindResource(hInst, lpResName, RT_TOOLBAR)) == NULL ||
	   (ptbd = (CToolBarData*)LoadResource(hInst, hRsrc)) == NULL)
	{
		TRACE(_T("Can't load toolbar %d!\n"), nIDToolbar);
		return FALSE;
	}

	ASSERT(ptbd->wVersion == 1);

	int nLoadCtrls = 0;
	if(nControls == 0) nLoadCtrls = ptbd->wItemCount;
	else nLoadCtrls = nControls;
	if(nLoadCtrls > ptbd->wItemCount) nLoadCtrls = ptbd->wItemCount;

	for(WORD i = 0; i < nLoadCtrls; i++)
		AddControl(i, ptbd->items()[i]);

	// Force the buttons to be updated before we redraw the window (gets rid of an
	// annoying flicker between the time that the toolbar is loaded and the update
	// handler is called)
	OnUpdateCmdUI(GetDockingFrame(), TRUE);
	CalcFixedLayout(FALSE, TRUE);

	return TRUE; // success!
}

BOOL CCommandBarPopup::AddControls(DWORD* anControls)
{
	for(int i = 0; anControls[i] != -1; i++)
		AddControl(GetControlCount() + i, anControls[i]);

	// Force the buttons to be updated before we redraw the window (gets rid of an
	// annoying flicker between the time that the toolbar is loaded and the update
	// handler is called)
	OnUpdateCmdUI(GetDockingFrame(), TRUE);
	CalcFixedLayout(FALSE, TRUE);

	return TRUE; // success!
}

BOOL CCommandBarPopup::AddControl(int nIndex, DWORD nCmd)
{
	CCommandBarControlInfo control;
	control.index = nIndex;
	control.command = nCmd;

	AddControl(&control);

	return TRUE; // success!
}

BOOL CCommandBarPopup::AddControl(CCommandBarControlInfo* lptbec)
{
	CCommandBarControlInfo* pCtrl = new CCommandBarControlInfo;

	if(lptbec->index == -1) pCtrl->index = m_aControls.GetSize();
	else pCtrl->index = lptbec->index;

	pCtrl->command = lptbec->command;
	pCtrl->requestedSize = m_pResources->GetControlSize(lptbec->command);
	pCtrl->size = m_pResources->GetControlSize(lptbec->command);
	pCtrl->style = m_pResources->GetControlStyle(lptbec->command);
	pCtrl->image = m_pResources->GetControlImage(lptbec->command);
	pCtrl->label = m_pResources->GetControlText(lptbec->command);

	DWORD dwType = m_pResources->GetControlType(lptbec->command);

	if(dwType == cBarControlEdit || dwType == cBarControlComboBox ||
		dwType == cBarControlGraphicCombo)
	{
		pCtrl->pEdit = new CCommandBarEdit;
		pCtrl->pEdit->Create(this, lptbec->command, m_pResources);
		pCtrl->pEdit->SetFont(&m_horizFont);
		CString strEntry = m_pResources->GetControlEntry(lptbec->command);
		pCtrl->pEdit->SetWindowText(strEntry);
	}

	// Insert the control and then update all the indices.
	m_aControls.InsertAt(pCtrl->index, pCtrl);
	for(int i = 0; i < m_aControls.GetSize(); i++) m_aControls[i]->index = i;

	return TRUE;
}

void CCommandBarPopup::LoadMenu(UINT nID)
{
	for(int j = 0; j < m_aControls.GetSize(); j++) delete m_aControls[j];
	m_aControls.RemoveAll();

	m_menu.LoadMenu(nID);

	for(UINT i = 0; i < m_menu.GetMenuItemCount(); i++)
	{
		TCHAR str[256];
		m_menu.GetMenuString(i, str, sizeof(str), MF_BYPOSITION);

		CCommandBarControlInfo* pCtrl = new CCommandBarControlInfo;
		pCtrl->index = i;
		pCtrl->command = 0;
//		pCtrl->type = cBarControlPopup;
//		pCtrl->text = str;
		m_aControls.InsertAt(pCtrl->index, pCtrl);
	}

	// Update indices.
	for(int k = 0; k < m_aControls.GetSize(); k++) m_aControls[k]->index = k;

	Invalidate();
	GetDockingFrame()->RecalcLayout();
}

void CCommandBarPopupCmdUI::Enable(BOOL bOn)
{
	m_bEnableChanged = TRUE;
	CCommandBarPopup* pCommandBar = (CCommandBarPopup*)m_pOther;
	ASSERT(pCommandBar != NULL);
	ASSERT_KINDOF(CCommandBarPopup, pCommandBar);
	ASSERT(m_nIndex < m_nIndexMax);

	UINT nOldState = pCommandBar->GetControlState(m_nIndex);
	UINT nNewState = nOldState & ~TBES_DISABLED;

	if(!bOn) nNewState |= TBES_DISABLED;

	if(nOldState != nNewState) pCommandBar->SetControlState(m_nIndex, nNewState);

	pCommandBar->__EnableEditControl(m_nIndex, !(nNewState & TBES_DISABLED));
}

void CCommandBarPopupCmdUI::SetCheck(int nCheck)
{
	ASSERT(nCheck >= 0 && nCheck <= 2); // 0=>off, 1=>on, 2=>indeterminate
	CCommandBarPopup* pCommandBar = (CCommandBarPopup*)m_pOther;
	ASSERT(pCommandBar != NULL);
	ASSERT_KINDOF(CCommandBarPopup, pCommandBar);
	ASSERT(m_nIndex < m_nIndexMax);

	UINT nOldState = pCommandBar->GetControlState(m_nIndex);
	UINT nNewState = nOldState & ~(TBES_CHECKED | TBES_INDETERMINATE);

	if(nCheck == 1) nNewState |= TBES_CHECKED;
	else if(nCheck == 2) nNewState |= TBES_INDETERMINATE;

	if(nNewState != nOldState) pCommandBar->SetControlState(m_nIndex, nNewState);
}

void CCommandBarPopupCmdUI::SetText(LPCTSTR lpszText)
{
	CCommandBarPopup* pCommandBar = (CCommandBarPopup*)m_pOther;
	ASSERT(pCommandBar != NULL);
	ASSERT_KINDOF(CCommandBarPopup, pCommandBar);
	ASSERT(m_nIndex < m_nIndexMax);

	pCommandBar->SetControlText(m_nIndex, lpszText);
}

WORD CCommandBarPopup::GetControlState(int nIndex)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound()) return 0;

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];
	return m_pResources->GetControlState(pCtrl->command);
}

void CCommandBarPopup::SetControlState(int nIndex, WORD nState)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound()) return;

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];

	// Label controls don't have states.
	if(m_pResources->GetControlType(pCtrl->command) == cBarControlLabel)
		return;

	m_pResources->SetControlState(pCtrl->command, nState);
	__EnableEditControl(nIndex, !(nState & TBES_DISABLED));
}

void CCommandBarPopup::__EnableEditControl(int nIndex, BOOL bOn)
{
	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];

	if(pCtrl->pEdit) pCtrl->pEdit->EnableWindow(bOn);
}

void CCommandBarPopup::SetControlSize(int nIndex, CSize size)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound()) return;

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];
	pCtrl->requestedSize = size;

	GetDockingFrame()->RecalcLayout();
}

CSize CCommandBarPopup::GetControlSize(int nIndex)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound()) return CSize(0, 0);

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];
	return pCtrl->requestedSize;
}

void CCommandBarPopup::SetControlImage(int nIndex, int nImage)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound()) return;

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];
	pCtrl->image = nImage;

	RedrawControl(nIndex);
}

int CCommandBarPopup::GetControlImage(int nIndex)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound()) return -1;

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];
	return pCtrl->image;
}

void CCommandBarPopup::SetControlStyle(int nIndex, int nStyle)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound()) return;

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];
	pCtrl->style = nStyle;

	GetDockingFrame()->RecalcLayout();
}

int CCommandBarPopup::GetControlStyle(int nIndex)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound()) return -1;

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];
	return pCtrl->style;
}

void CCommandBarPopup::ModifyControlState(int nIndex, WORD dwRemove, WORD dwAdd)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound()) return;

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];

	SetControlState(nIndex,
		(m_pResources->GetControlState(pCtrl->command) & ~dwRemove) | dwAdd);
}

CString CCommandBarPopup::GetControlText(int nIndex)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound())
	{
		CString empty;  empty.Empty();
		return empty;
	}

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];
	return pCtrl->label;
}

void CCommandBarPopup::GetControlText(int nIndex, CString& rString)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound())
	{
		rString.Empty();
		return;
	}

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];
	rString = pCtrl->label;
}

void CCommandBarPopup::SetControlText(int nIndex, LPCTSTR lpszText)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound()) return;

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];
	pCtrl->label = lpszText;

	// Now force a redraw of the button we just updated.
	CRect rect; GetItemRect(nIndex, rect);
	InvalidateRect(rect, FALSE);
}

void CCommandBarPopup::GetControl(int nIndex, CCommandBarControlInfo* lptbec)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound()) return;

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];
	
	lptbec->index = pCtrl->index;
	lptbec->command = pCtrl->command;
	lptbec->requestedSize = pCtrl->requestedSize;
	lptbec->label = pCtrl->label;
	lptbec->style = pCtrl->style;
}

void CCommandBarPopup::GetItemRect(int nIndex, LPRECT lpRect)
{
	if(nIndex < 0 || nIndex > m_aControls.GetUpperBound()) return;

	CCommandBarControlInfo* pCtrl = m_aControls[nIndex];
	
	if(!m_aRowHeights.GetSize()) return;

	if(m_pResources->GetControlType(pCtrl->command) == cBarControlSeparator)		// Special handling for separators
	{
		lpRect->left = pCtrl->point.x;
		lpRect->top = pCtrl->point.y;
		lpRect->right = pCtrl->point.x + SEPARATORDIM;
		lpRect->bottom = pCtrl->point.y + m_aRowHeights[pCtrl->row];
		return;
	}

	CRect clientRect;  GetClientRect(&clientRect);

	lpRect->left = pCtrl->point.x;
	lpRect->top = pCtrl->point.y + (m_aRowHeights[pCtrl->row] - pCtrl->size.cy) / 2;
	lpRect->right = lpRect->left + clientRect.Width() - 6;
	lpRect->bottom = lpRect->top + pCtrl->size.cy;
}

BOOL CCommandBarPopup::OnEraseBkgnd(CDC* pDC) { return CControlBar::OnEraseBkgnd(pDC); }

void CCommandBarPopup::OnUpdateCmdUI(CFrameWnd* pTarget, BOOL bDisableIfNoHndler)
{
	CCommandBarPopupCmdUI state;
	state.m_pOther = this;

	state.m_nIndexMax = m_aControls.GetSize();
	for (state.m_nIndex = 0; state.m_nIndex < state.m_nIndexMax; state.m_nIndex++)
	{
		// get buttons state
		CCommandBarControlInfo* pCtrl = m_aControls[state.m_nIndex];
		state.m_nID = pCtrl->command;

		// ignore separators
		if (!(m_pResources->GetControlType(pCtrl->command) == cBarControlSeparator))
		{
			// allow the toolbar itself to have update handlers
			if (CControlBar::OnCmdMsg(state.m_nID, CN_UPDATE_COMMAND_UI, &state, NULL))
				continue;

			// allow the owner to process the update
			state.DoUpdate(pTarget, FALSE); //bDisableIfNoHndler);
		}
	}
}

void CCommandBarPopup::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(m_pResources->GetLockState()) { CControlBar::OnMouseMove(nFlags, point); return; }

	CCommandBarControlInfo* pCtrl;

	if(m_iControlCapture != -1)
	{
		pCtrl = m_aControls[m_iControlCapture];
		ASSERT(!(m_pResources->GetControlType(pCtrl->command) == cBarControlSeparator));

		UINT nNewState = m_pResources->GetControlState(pCtrl->command);

		if(GetCapture() != this)
		{
			m_iControlCapture = -1; // lost capture
			GetOwner()->SendMessage(WM_SETMESSAGESTRING, (WPARAM)AFX_IDS_IDLEMESSAGE);
		}
		else
		{
			// should be pressed if still hitting the captured button
			CCommandBarHitTestInfo tbehti;  tbehti.pt = point;
			HitTest(&tbehti);

			if(tbehti.item != -1)
			{
				pCtrl = m_aControls[tbehti.item];

				if(tbehti.item == m_iControlCapture)
				{
					if(m_pResources->GetControlType(pCtrl->command) == cBarControlButtonDropDown)
					{
						if(!m_aControls[m_iControlCapture]->bPressed)
						{
							m_aControls[m_iControlCapture]->bPressed = TRUE;
							RedrawControl(m_iControlCapture);
						}
					}
					else
					{
						if(!m_aControls[m_iControlCapture]->bPressed)
						{
							m_aControls[m_iControlCapture]->bPressed = TRUE;
							RedrawControl(m_iControlCapture);
						}
					}
				}
				else
				{
					if(m_aControls[m_iControlCapture]->bPressed ||
						m_aControls[m_iControlCapture]->bDropped)
					{
						m_aControls[m_iControlCapture]->bPressed = FALSE;
						m_aControls[m_iControlCapture]->bDropped = FALSE;
						RedrawControl(m_iControlCapture);
					}
				}
			}
			else
			{
				if(m_aControls[m_iControlCapture]->bPressed ||
					m_aControls[m_iControlCapture]->bDropped)
				{
					m_aControls[m_iControlCapture]->bPressed = FALSE;
					m_aControls[m_iControlCapture]->bDropped = FALSE;
					RedrawControl(m_iControlCapture);
				}
			}
		}
	}

	// Get the hot button.
	if(m_iControlCapture < 0)
	{
		CCommandBarHitTestInfo tbehti;  tbehti.pt = point;
		HitTest(&tbehti);
		int hot = tbehti.item;

		if(hot != -1)
		{
			pCtrl = m_aControls[hot];
			if(m_pResources->GetControlState(pCtrl->command) & TBES_DISABLED)
			{
				// Prevent disabled controls from being hot
				hot = -1;
			}
			else
			{
				if(!(m_pResources->GetControlType(pCtrl->command) == cBarControlSeparator) &&
					!(m_pResources->GetControlType(pCtrl->command) == cBarControlLabel))
				{
					BOOL bWasHot = pCtrl->bHot;
					pCtrl->bHot = TRUE;
					if(bWasHot != pCtrl->bHot) RedrawControl(hot);
					GetOwner()->SendMessage(WM_SETMESSAGESTRING, (WPARAM)pCtrl->command);
				}
			}
		}
		else GetOwner()->SendMessage(WM_SETMESSAGESTRING, (WPARAM)AFX_IDS_IDLEMESSAGE);
		
		if(hot != m_iLastHot && m_iLastHot != -1)
		{
			BOOL bWasHot = m_aControls[m_iLastHot]->bHot;
			m_aControls[m_iLastHot]->bHot = FALSE;
			if(bWasHot != m_aControls[m_iLastHot]->bHot) RedrawControl(m_iLastHot);
		}
		
		m_iLastHot = hot;
	}

	CControlBar::OnMouseMove(nFlags, point);
}

void CCommandBarPopup::RedrawControl(int nIndex)
{
	CRect rect; GetItemRect(nIndex, rect);
	InvalidateRect(rect, FALSE);
}

void CCommandBarPopup::HitTest(CCommandBarHitTestInfo* tbehti)
{
	// If the window the toolbar belongs to isn't active, don't do anything.
	// This cancels "hot" events when the window is inactive.  When the window is
	// docked, we can test for activity in the frame it's docked in.  When it's floating,
	// since its docking frame is the mini-frame it's contained in, we have to test for
	// activity in the top-level frame instead.
	BOOL bInactive = FALSE;

/*	if(!IsFloating() &&
		GetDockingFrame()->GetSafeHwnd() != GetActiveWindow()->GetSafeHwnd()) bInactive = TRUE;

	if(IsFloating() &&
		GetTopLevelFrame()->GetSafeHwnd() != GetActiveWindow()->GetSafeHwnd()) bInactive = TRUE;
*/
	if(bInactive)
	{
		tbehti->flags = cBarHitNowhere;
		tbehti->item = -1;
		return;
	}

	CCommandBarControlInfo* pCtrl;

	for(int iButton = 0; iButton < m_aControls.GetSize(); iButton++)
	{
		pCtrl = m_aControls[iButton];
		
		DWORD dwType = m_pResources->GetControlType(m_aControls[iButton]->command);

		CRect rect; GetItemRect(iButton, rect);
		
		if(rect.PtInRect(tbehti->pt))
		{
			// Ok, so far, we know we're on a control.
			// Figure out where on the control was hit.
			if(dwType == cBarControlSeparator)
			{
				tbehti->flags = cBarHitNowhere;
				tbehti->item = -1;
				return;
			}
			if(dwType == cBarControlButton || dwType == cBarControlEdit ||
				dwType == cBarControlPopup || dwType == cBarControlLabel ||
				dwType == cBarControlButtonPopup ||
				(dwType == cBarControlEdit && pCtrl->bCompact) ||
				(dwType == cBarControlComboBox && pCtrl->bCompact) ||
				(dwType == cBarControlDropDown && pCtrl->bCompact) ||
				(dwType == cBarControlGraphicCombo && pCtrl->bCompact) ||
				(dwType == cBarControlGraphicDropDown && pCtrl->bCompact))
			{
				// Will always be cBarHitOnContent.
				tbehti->flags = cBarHitOnContent;
				tbehti->item = iButton;
				return;
			}
			if(dwType == cBarControlButtonDropDown)
			{
				tbehti->item = iButton;

				CRect btnRect = rect;
				btnRect.right -= DROPDOWNWIDTH - DROPDOWNSPACE;
				CRect dropRect = rect;
				dropRect.left = dropRect.right - DROPDOWNWIDTH - DROPDOWNSPACE;

				if(btnRect.PtInRect(tbehti->pt)) tbehti->flags = cBarHitOnContent;
				if(dropRect.PtInRect(tbehti->pt)) tbehti->flags = cBarHitOnArrow;

				return;
			}
			if((dwType == cBarControlComboBox && !pCtrl->bCompact) ||
				(dwType == cBarControlDropDown && !pCtrl->bCompact) ||
				(dwType == cBarControlGraphicCombo && !pCtrl->bCompact) ||
				(dwType == cBarControlGraphicDropDown && !pCtrl->bCompact))
			{
				tbehti->item = iButton;
				
				// A combo's drop arrow's width is half the control's height.
				CRect contRect = rect;  contRect.right -= rect.Height() / 2;
				CRect dropRect = rect;  contRect.left = contRect.right - rect.Height() / 2;

				if(contRect.PtInRect(tbehti->pt)) tbehti->flags = cBarHitOnContent;
				if(dropRect.PtInRect(tbehti->pt)) tbehti->flags = cBarHitOnArrow;

				return;
			}
		}
	}

	// If we made it this far, we know we didn't hit a control.  Check for other
	// non-control possibilities, such as the gripper or a tear-off bar.
	tbehti->flags = cBarHitNowhere;
	tbehti->item = -1;
	return;      // nothing hit
}

int CCommandBarPopup::OnToolHitTest(CPoint point, TOOLINFO* pTI)
{
//	if (!m_bShowTooltips) return -1;

	// Send message to CControlBar first
	int nHit = CControlBar::OnToolHitTest(point, pTI);
	if(nHit != -1) return nHit;

	// Nothing happened there, so check for our buttons now.
	CCommandBarHitTestInfo tbehti;
	((CCommandBarPopup*)this)->HitTest(&tbehti);
	nHit = tbehti.item;

	if(nHit != -1)
	{
		CCommandBarControlInfo* pCtrl = m_aControls[nHit];
		if(pTI != NULL)
		{
			CString strTip;

			if((pCtrl->command != 0) && !pCtrl->label.IsEmpty())
			{
				// Use button text as tooltip!
				strTip = pCtrl->label;

				// Remove the ampersand first.
				int iPos = strTip.Find(_T('&'));
				if (iPos != -1)
				{
					CString strLeft = strTip.Left(iPos);
					CString strRight = strTip.Mid(iPos + 1);
					strTip = strLeft + strRight;
				}
			}
			else
			{
				TCHAR szFullText[256];

				AfxLoadString(pCtrl->command, szFullText);
				AfxExtractSubString(strTip, szFullText, 1, '\n');
			}

			if(strTip.IsEmpty()) return -1;

/*			if(pCtrl->command != 0 && m_bShowShortcutKeys)
			{
				//--------------------
				// Add shortcut label:
				//--------------------
				CString strLabel;
				CFrameWnd* pParent = GetParentFrame ();

				if (pParent != NULL &&
					(CBCGKeyboardManager::FindDefaultAccelerator (
						pButton->m_nID, strLabel, pParent) ||
					CBCGKeyboardManager::FindDefaultAccelerator (
						pButton->m_nID, strLabel, pParent->GetActiveFrame ())))
				{
					strTipText += _T(" (");
					strTipText += strLabel;
					strTipText += ')';
				}
			}*/

			pTI->lpszText = (LPTSTR)malloc(strTip.GetLength() + 1);
			strcpy(pTI->lpszText, strTip);

			GetItemRect(nHit, &pTI->rect);
			pTI->uId = pCtrl->command;
			pTI->hwnd = GetSafeHwnd();
		}

		nHit = pCtrl->command;
	}

	return nHit;
}

void CCommandBarPopup::OnTimer(UINT nIDEvent) 
{
	if(m_pResources->GetLockState()) { CControlBar::OnTimer(nIDEvent); return; }

	CCommandBarControlInfo tbec;
	CPoint point;  ::GetCursorPos(&point);
	ScreenToClient(&point);
	CRect rect;  GetClientRect(&rect);

	if(m_iControlCapture < 0)
	{
		if(!rect.PtInRect(point))	// We lost the mouse, so get rid of any hot button.
		{
			if(m_iLastHot != -1)
			{
				BOOL bWasHot = m_aControls[m_iLastHot]->bHot;
				m_aControls[m_iLastHot]->bHot = FALSE;
				if(bWasHot != m_aControls[m_iLastHot]->bHot) RedrawControl(m_iLastHot);

				m_iLastHot = -1;
				GetOwner()->SendMessage(WM_SETMESSAGESTRING, (WPARAM)AFX_IDS_IDLEMESSAGE);
			}
		}
	}

	CControlBar::OnTimer(nIDEvent);
}

void CCommandBarPopup::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	CCommandBarHitTestInfo tbehti;  tbehti.pt = point;
	HitTest(&tbehti);

	if(tbehti.flags == cBarHitNowhere)
	{
		// Either nothing was hit, or we hit a separator.  That means it's
		// ok to drag the bar.
		if(m_pDockBar != NULL && OnToolHitTest(point, NULL) == -1)
		{
			// start the drag
//			ASSERT(m_pDockContextEx != NULL);
			ClientToScreen(&point);
//			m_pDockContextEx->ToggleDocking();
		}
		else
		{
			CControlBar::OnLButtonDblClk(nFlags, point);
		}
	}
}

void CCommandBarPopup::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CCommandBarHitTestInfo tbehti;  tbehti.pt = point;
	HitTest(&tbehti);
	m_iControlCapture = tbehti.item;

	((CFrameWnd*)AfxGetMainWnd())->GetActiveView()->SetFocus();

	if(tbehti.flags == cBarHitNowhere)
	{
		// Either nothing was hit, or we hit a separator.  That means it's
		// ok to drag the bar.
		if(m_pDockBar != NULL && OnToolHitTest(point, NULL) == -1)
		{
			// start the drag
//			ASSERT(m_pDockContextEx != NULL);
			ClientToScreen(&point);
//			m_pDockContextEx->StartDrag(point);
		}
		else
		{
			CControlBar::OnLButtonDown(nFlags, point);
		}

		return;
	}

	CCommandBarControlInfo* pCtrl = m_aControls[m_iControlCapture];

	ASSERT(!(m_pResources->GetControlType(pCtrl->command) == cBarControlSeparator));
	
	if(m_pResources->GetControlState(pCtrl->command) & TBES_DISABLED)
	{
		m_iControlCapture = -1;
		return;     // don't press it
	}

	BOOL bSendMessage = TRUE;

	if(m_pResources->GetControlType(pCtrl->command) == cBarControlComboBox ||
		m_pResources->GetControlType(pCtrl->command) == cBarControlDropDown ||
		m_pResources->GetControlType(pCtrl->command) == cBarControlGraphicCombo ||
		m_pResources->GetControlType(pCtrl->command) == cBarControlGraphicDropDown)
	{
		if(tbehti.flags == cBarHitOnContent)
		{
			SetCapture();
			pCtrl->bPressed = TRUE;
			RedrawControl(m_iControlCapture);
			GetOwner()->SendMessage(WM_SETMESSAGESTRING, (WPARAM)pCtrl->command);
		}
		else
		{
			int nControl = m_iControlCapture;

			pCtrl->bDropped = TRUE;
			GetOwner()->SendMessage(WM_SETMESSAGESTRING, (WPARAM)pCtrl->command);
			RedrawControl(m_iControlCapture);
			
			CRect rect;  GetItemRect(m_iControlCapture, &rect);
			CPoint pt(rect.left, rect.bottom);
			ClientToScreen(&pt);

			m_iComboCapture = m_iControlCapture;

			BOOL bOwnerDraw = FALSE;
			if(m_pResources->GetControlType(pCtrl->command) == cBarControlComboBox ||
				m_pResources->GetControlType(pCtrl->command) == cBarControlDropDown)
				bOwnerDraw = FALSE;
			if(m_pResources->GetControlType(pCtrl->command) == cBarControlGraphicCombo ||
				m_pResources->GetControlType(pCtrl->command) == cBarControlGraphicDropDown)
				bOwnerDraw = TRUE;

			m_wndList = new CCommandBarList;
			m_wndList->Create(GetOwner(), pCtrl->command, m_pResources);
			m_wndList->SetFont(&m_horizFont);
//			m_wndList->m_pBar = this;
			m_wndList->Invoke(pt, rect.Width(), bOwnerDraw);
		}
	}
	if(m_pResources->GetControlType(pCtrl->command) == cBarControlButtonDropDown)
	{
		if(tbehti.flags == cBarHitOnContent)
		{
			SetCapture();
			pCtrl->bPressed = TRUE;
			RedrawControl(m_iControlCapture);
			GetOwner()->SendMessage(WM_SETMESSAGESTRING, (WPARAM)pCtrl->command);
		}
		else
		{
			if(pCtrl->bDropped)
			{
				pCtrl->bDropped = FALSE;
			}
			else
			{
				pCtrl->bDropped = TRUE;
				GetOwner()->SendMessage(WM_SETMESSAGESTRING, (WPARAM)pCtrl->command);
				RedrawControl(m_iControlCapture);

				CRect rect;  GetItemRect(m_iControlCapture, &rect);
				CPoint pt;
				pt = CPoint(rect.left, rect.bottom);
				ClientToScreen(&pt);

				// Must do this for Windows to find our notify message valid o_O
				int nOldID = SetDlgCtrlID(pCtrl->command);

				NMCBCDROPDOWN nmcbcdd;
				nmcbcdd.hdr.hwndFrom = GetSafeHwnd();
				nmcbcdd.hdr.idFrom = pCtrl->command;
				nmcbcdd.hdr.code = CBCN_DROPDOWN;
				nmcbcdd.ptMenu = pt;
				nmcbcdd.nCmd = pCtrl->command;			// The user can change this if needed.
				GetOwner()->SendMessage(WM_NOTIFY, pCtrl->command, (LPARAM)&nmcbcdd);

				SetDlgCtrlID(nOldID);

				CCommandBarHitTestInfo tbehti;  tbehti.pt = point;
				HitTest(&tbehti);
				if(tbehti.item != m_iControlCapture)
				{
					pCtrl->command = nmcbcdd.nCmd;
					pCtrl->image = m_pResources->GetControlImage(pCtrl->command);
				}

				pCtrl->bDropped = FALSE;
				pCtrl->bHot = FALSE;
				RedrawControl(m_iControlCapture);
			}
		}
	}
	else if(m_pResources->GetControlType(pCtrl->command) == cBarControlEdit)
	{
		pCtrl->pEdit->SetFocus();
	}
	else if(m_pResources->GetControlType(pCtrl->command) == cBarControlPopup)
	{
		pCtrl->bPressed = TRUE;
		GetOwner()->SendMessage(WM_SETMESSAGESTRING, (WPARAM)pCtrl->command);
		pCtrl->bPressed = FALSE;
	}
	else if(m_pResources->GetControlType(pCtrl->command) == cBarControlButtonPopup)
	{
		if(pCtrl->bPressed)
		{
			pCtrl->bPressed = FALSE;
		}
		else
		{
			pCtrl->bPressed = TRUE;
			GetOwner()->SendMessage(WM_SETMESSAGESTRING, (WPARAM)pCtrl->command);
			RedrawControl(m_iControlCapture);

			CRect rect;  GetItemRect(m_iControlCapture, &rect);
			CPoint pt;
			pt = CPoint(rect.left, rect.bottom);
			ClientToScreen(&pt);

			// Must do this for Windows to find our notify message valid o_O
			int nOldID = SetDlgCtrlID(pCtrl->command);

			NMCBCDROPDOWN nmcbcdd;
			nmcbcdd.hdr.hwndFrom = GetSafeHwnd();
			nmcbcdd.hdr.idFrom = pCtrl->command;
			nmcbcdd.hdr.code = CBCN_DROPDOWN;
			nmcbcdd.ptMenu = pt;
			nmcbcdd.nCmd = pCtrl->command;			// The user can change this if needed.
			GetOwner()->SendMessage(WM_NOTIFY, pCtrl->command, (LPARAM)&nmcbcdd);

			SetDlgCtrlID(nOldID);

			pCtrl->bPressed = FALSE;
			pCtrl->bHot = FALSE;
			RedrawControl(m_iControlCapture);
		}
	}
	else
	{
		SetCapture();
		pCtrl->bPressed = TRUE;
		RedrawControl(m_iControlCapture);
		GetOwner()->SendMessage(WM_SETMESSAGESTRING, (WPARAM)pCtrl->command);
	}
}

void CCommandBarPopup::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CControlBar::OnLButtonUp(nFlags, point);

	if(m_iControlCapture < 0) return;     // not captured

	CCommandBarControlInfo* pCtrl = m_aControls[m_iControlCapture];

	UINT nIDCmd = 0;

	if(GetCapture() == this)
	{
		// we did not lose the capture
		ReleaseCapture();

		CCommandBarHitTestInfo tbehti;  tbehti.pt = point;
		HitTest(&tbehti);
		if(tbehti.item == m_iControlCapture &&
			!(m_pResources->GetControlState(pCtrl->command) & TBES_DISABLED))
		{
			// pressed, will send command notification
			nIDCmd = pCtrl->command;
		}
	}

	GetOwner()->SendMessage(WM_SETMESSAGESTRING, (WPARAM)AFX_IDS_IDLEMESSAGE);

	if(nIDCmd != 0 && m_pResources->GetControlType(nIDCmd) != cBarControlSeparator)
	{
		GetOwner()->SendMessage(WM_COMMAND, nIDCmd);    // send command

		if(!m_pResources->GetLockState())
		{
			pCtrl->bPressed = FALSE;
			pCtrl->bDropped = FALSE;
			RedrawControl(m_iControlCapture);
		}

		m_iControlCapture = -1;

		// Menus are destroyed on an LButtonUp event.
		DestroyWindow();
	}
}

void CCommandBarPopup::OnCancelMode() 
{
	if(m_iControlCapture < 0) return;     // not captured

	CCommandBarControlInfo* pCtrl = m_aControls[m_iControlCapture];

	ASSERT(!(m_pResources->GetControlType(pCtrl->command) == cBarControlSeparator));
	
	if(GetCapture() == this) ReleaseCapture();

	UINT nNewState = (m_pResources->GetControlState(pCtrl->command));
	pCtrl->bPressed = pCtrl->bDropped = FALSE;

	m_iControlCapture = -1;

	GetOwner()->SendMessage(WM_SETMESSAGESTRING, (WPARAM)AFX_IDS_IDLEMESSAGE);

	CControlBar::OnCancelMode();
}

void CCommandBarPopup::__ReleaseDroppedControl()
{
	if(m_iComboCapture != -1)
	{
		CCommandBarControlInfo* pCombo = m_aControls[m_iComboCapture];
		pCombo->bPressed = pCombo->bDropped = FALSE;
		RedrawControl(m_iComboCapture);
		m_iComboCapture = -1;
	}
}

void CCommandBarPopup::OnWindowPosChanging(WINDOWPOS FAR* lpwndpos) 
{
	CControlBar::OnWindowPosChanging(lpwndpos);
	
	// If we can resize while floating
	if(m_dwStyle & CBRS_SIZE_DYNAMIC)
	{
		// And we are resizing
		if(lpwndpos->flags & SWP_NOSIZE) return;

		// Then redraw the buttons
		Invalidate();
	}
}

void CCommandBarPopup::EnableDocking(DWORD dwDockStyle)
{
	// must be CBRS_ALIGN_XXX or CBRS_FLOAT_MULTI only
	ASSERT((dwDockStyle & ~(CBRS_ALIGN_ANY|CBRS_FLOAT_MULTI)) == 0);
	// CBRS_SIZE_DYNAMIC toolbar cannot have the CBRS_FLOAT_MULTI style
	ASSERT(((dwDockStyle & CBRS_FLOAT_MULTI) == 0) || ((m_dwStyle & CBRS_SIZE_DYNAMIC) == 0));

	m_dwDockStyle = dwDockStyle;
	if (m_pDockContext == NULL)
		m_pDockContext = new CDockContext(this);

	// permanently wire the bar's owner to its current parent
	if (m_hWndOwner == NULL)
		m_hWndOwner = ::GetParent(m_hWnd);
}

// Draw the basic borders and fill for all buttons.
void CCommandBarPopup::_DrawBasic(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl)
{
	// Get rid of any border that may already be there.
	pDC->FillSolidRect(rect, m_clrButton);

	CRect btnRect = rect;
	btnRect.right = btnRect.left + BUTTONWIDTH;
	CRect textRect = rect;
	if(pCtrl->image != -1) textRect.left += BUTTONWIDTH + 1;

	if(pCtrl->bCustomMarked)
	{
		CRect rect2 = rect;
		pDC->Draw3dRect(rect2, m_clrDkShadow, m_clrDkShadow);
		rect2.DeflateRect(1, 1);
		pDC->Draw3dRect(rect2, m_clrDkShadow, m_clrDkShadow);
	}

	if(pCtrl->bCustomInsert) _DrawInsertBar(pDC, rect);

	if(m_pResources->GetControlType(pCtrl->command) == cBarControlButton ||
		m_pResources->GetControlType(pCtrl->command) == cBarControlButtonPopup ||
		m_pResources->GetControlType(pCtrl->command) == cBarControlPopup ||
		m_pResources->GetControlType(pCtrl->command) == cBarControlButtonDropDown)
	{
		if(pCtrl->bHot &&
			!(m_pResources->GetControlState(pCtrl->command) & TBES_CHECKED))
		{
			if(pCtrl->image != -1) pDC->DrawEdge(btnRect, BDR_RAISEDINNER, BF_RECT);
			pDC->FillSolidRect(textRect, m_clrMenuHilite);
		}

		if(m_pResources->GetControlState(pCtrl->command) & TBES_CHECKED)
		{
			if(!pCtrl->bHot) pDC->FillSolidRect(rect, m_clrChecked);
			pDC->DrawEdge(btnRect, BDR_SUNKENOUTER, BF_RECT);
		}
	}

/*	if(m_pResources->GetControlType(pCtrl->command) == cBarControlButtonDropDown)
	{
		CRect btnRect = rect;  btnRect.right -= DROPDOWNWIDTH;
		CRect dropRect = rect;  dropRect.left = dropRect.right - DROPDOWNWIDTH;

		if(pCtrl->bHot &&
			!(pCtrl->bPressed) &&
			!(m_pResources->GetControlState(pCtrl->command) & TBES_CHECKED))
		{
			pDC->DrawEdge(btnRect, BDR_RAISEDINNER, BF_RECT);
			pDC->DrawEdge(dropRect, BDR_RAISEDINNER, BF_RECT);
		}
		
		if(pCtrl->bDropped)
		{
			pDC->DrawEdge(btnRect, BDR_RAISEDINNER, BF_RECT);
			pDC->DrawEdge(dropRect, BDR_SUNKENOUTER, BF_RECT);
		}
			
		if(pCtrl->bPressed)
		{
			pDC->DrawEdge(btnRect, BDR_SUNKENOUTER, BF_RECT);
			pDC->DrawEdge(dropRect, BDR_RAISEDINNER, BF_RECT);
		}
			
		if(m_pResources->GetControlState(pCtrl->command) & TBES_CHECKED)
		{
			if(!pCtrl->bHot) pDC->FillSolidRect(btnRect, m_clrChecked);

				pDC->DrawEdge(&btnRect, BDR_SUNKENOUTER, BF_RECT);
				if(pCtrl->bHot) pDC->DrawEdge(&dropRect, BDR_RAISEDINNER, BF_RECT);
				if(pCtrl->bDropped) pDC->DrawEdge(&dropRect, BDR_SUNKENOUTER, BF_RECT);
		}
		
		// Draw the arrow.
		CRect arrowRect = dropRect;
		arrowRect.left += 2; arrowRect.right -= 3;
		_DrawArrow(pDC, &arrowRect, pCtrl, arrowDown);
	}*/
}

void CCommandBarPopup::_DrawInsertBar(CDC* pDC, CRect* rect)
{
	CPen pen(PS_SOLID, 1, m_clrDkShadow);
	CPen* oldPen = pDC->SelectObject(&pen);

	CPoint pt(rect->left, rect->top);

		                  pDC->MoveTo(pt);  pDC->LineTo(pt.x + 6, pt.y);
		pt.x++;  pt.y++;  pDC->MoveTo(pt);  pDC->LineTo(pt.x + 4, pt.y);
		pt.x++;  pt.y++;  pDC->MoveTo(pt);  pDC->LineTo(pt.x, rect->bottom);
		pt.x++;           pDC->MoveTo(pt);  pDC->LineTo(pt.x, rect->bottom);
		pt.x -= 2;  pt.y = rect->bottom - 2;  pDC->MoveTo(pt);  pDC->LineTo(pt.x + 4, pt.y);
		pt.x--;  pt.y++;  pDC->MoveTo(pt);  pDC->LineTo(pt.x + 6, pt.y);

	pDC->SelectObject(oldPen);
}

void CCommandBarPopup::_DrawArrow(CDC* pDC, CRect* rect, CCommandBarControlInfo* pCtrl, UINT direction)
{
	CRect realRect = rect;
	if(((m_pResources->GetControlType(pCtrl->command) == cBarControlComboBox) ||
		(m_pResources->GetControlType(pCtrl->command) == cBarControlDropDown) ||
		(m_pResources->GetControlType(pCtrl->command) == cBarControlGraphicCombo) ||
		(m_pResources->GetControlType(pCtrl->command) == cBarControlGraphicDropDown)) &&
		(pCtrl->bDropped))
		realRect.OffsetRect(1, 1);

	// Ugh.
	if((( ( !(m_pResources->GetControlType(pCtrl->command) == cBarControlButtonDropDown) ) &&
			(m_pResources->GetControlState(pCtrl->command) & TBES_DISABLED) ) ||
		( (  (m_pResources->GetControlType(pCtrl->command) == cBarControlButtonDropDown) ) && 
			(m_pResources->GetControlState(pCtrl->command) & TBES_DISABLED) ) ) &&
		  ( !(m_pResources->GetControlType(pCtrl->command) == cBarControlComboBox) &&
		    !(m_pResources->GetControlType(pCtrl->command) == cBarControlDropDown) &&
		    !(m_pResources->GetControlType(pCtrl->command) == cBarControlGraphicCombo) &&
		    !(m_pResources->GetControlType(pCtrl->command) == cBarControlGraphicDropDown) ) )
	{
		if(direction == arrowDown)
		{
			CPoint firstSide = CPoint(realRect.left + 1, realRect.top + (realRect.Height() / 2));
			CPoint lastSide = CPoint(realRect.right + 1, realRect.top + (realRect.Height() / 2));
			int arrowSize = 1;
	
			CPen arrowPen(PS_SOLID, 1, m_clrHilite);
			CPen* oldPen = pDC->SelectObject(&arrowPen);

			for(;;)
			{
				pDC->MoveTo(firstSide);
				pDC->LineTo(firstSide + CPoint(0, arrowSize));

				pDC->MoveTo(lastSide);
				pDC->LineTo(lastSide + CPoint(0, arrowSize));

				firstSide.Offset(1, 0);
				lastSide.Offset(-1, 0);

				if(lastSide.x < firstSide.x) break;

				arrowSize++;
			}

			pDC->SelectObject(oldPen);

			firstSide = CPoint(realRect.left, realRect.top + (realRect.Height() / 2) - 1);
			lastSide = CPoint(realRect.right, realRect.top + (realRect.Height() / 2) - 1);
			arrowSize = 1;
	
			CPen arrowPen2(PS_SOLID, 1, m_clrShadow);
			oldPen = pDC->SelectObject(&arrowPen2);

			for(;;)
			{
				pDC->MoveTo(firstSide);
				pDC->LineTo(firstSide + CPoint(0, arrowSize));

				pDC->MoveTo(lastSide);
				pDC->LineTo(lastSide + CPoint(0, arrowSize));

				firstSide.Offset(1, 0);
				lastSide.Offset(-1, 0);

				if(lastSide.x < firstSide.x) break;

				arrowSize++;
			}
	
			pDC->SelectObject(oldPen);
		}
		if(direction == arrowRight)
		{
			CPoint firstSide = CPoint(realRect.left + 1, realRect.top + (realRect.Height() / 2) - 1);
			CPoint lastSide = CPoint(realRect.left + 1, realRect.bottom - (realRect.Height() / 2) + 3);
			int arrowSize = 1;
	
			CPen arrowPen(PS_SOLID, 1, m_clrHilite);
			CPen* oldPen = pDC->SelectObject(&arrowPen);

			for(;;)
			{
				pDC->MoveTo(firstSide);
				pDC->LineTo(firstSide + CPoint(arrowSize, 0));

				pDC->MoveTo(lastSide);
				pDC->LineTo(lastSide + CPoint(arrowSize, 0));

				firstSide.Offset(0, 1);
				lastSide.Offset(0, -1);

				if(lastSide.y < firstSide.y) break;

				arrowSize++;
			}

			pDC->SelectObject(oldPen);

			firstSide = CPoint(realRect.left, realRect.top + (realRect.Height() / 2) - 2);
			lastSide = CPoint(realRect.left, realRect.bottom - (realRect.Height() / 2) + 2);
			arrowSize = 1;
	
			CPen arrowPen2(PS_SOLID, 1, m_clrShadow);
			oldPen = pDC->SelectObject(&arrowPen2);

			for(;;)
			{
				pDC->MoveTo(firstSide);
				pDC->LineTo(firstSide + CPoint(arrowSize, 0));

				pDC->MoveTo(lastSide);
				pDC->LineTo(lastSide + CPoint(arrowSize, 0));

				firstSide.Offset(0, 1);
				lastSide.Offset(0, -1);

				if(lastSide.y < firstSide.y) break;

				arrowSize++;
			}
	
			pDC->SelectObject(oldPen);
		}
	}
	else
	{
		if(direction == arrowDown)
		{
			CPoint firstSide = CPoint(realRect.left, realRect.top + (realRect.Height() / 2) - 1);
			CPoint lastSide = CPoint(realRect.right, realRect.top + (realRect.Height() / 2) - 1);
			int arrowSize = 1;

			CPen arrowPen(PS_SOLID, 1, m_clrText);
			CPen* oldPen = pDC->SelectObject(&arrowPen);

			for(;;)
			{
				pDC->MoveTo(firstSide);
				pDC->LineTo(firstSide + CPoint(0, arrowSize));

				pDC->MoveTo(lastSide);
				pDC->LineTo(lastSide + CPoint(0, arrowSize));

				firstSide.Offset(1, 0);
				lastSide.Offset(-1, 0);

				if(lastSide.x < firstSide.x) break;

				arrowSize++;
			}

			pDC->SelectObject(oldPen);
		}
		else if(direction == arrowRight)
		{
			CPoint firstSide = CPoint(realRect.left, realRect.top + (realRect.Height() / 2) - 2);
			CPoint lastSide = CPoint(realRect.left, realRect.bottom - (realRect.Height() / 2) + 2);
			int arrowSize = 1;

			CPen arrowPen(PS_SOLID, 1, m_clrText);
			CPen* oldPen = pDC->SelectObject(&arrowPen);

			for(;;)
			{
				pDC->MoveTo(firstSide);
				pDC->LineTo(firstSide + CPoint(arrowSize, 0));

				pDC->MoveTo(lastSide);
				pDC->LineTo(lastSide + CPoint(arrowSize, 0));

				firstSide.Offset(0, 1);
				lastSide.Offset(0, -1);

				if(lastSide.y < firstSide.y) break;

				arrowSize++;
			}

			pDC->SelectObject(oldPen);
		}
	}
}

void CCommandBarPopup::_DrawText(CDC* pDC, CRect* rect, CCommandBarControlInfo* pCtrl)
{
	CString before, ampChar, after;
	CString original = pCtrl->label;
//	CString original = m_pResources->GetControlAcceleratorString(pCtrl->command);
	FilterAmpersand(original, before, ampChar, after);

	// If we're checked, offset the rect.
	if(m_pResources->GetControlState(pCtrl->command) & TBES_CHECKED) rect->OffsetRect(1, 1);

	if(m_pResources->GetControlState(pCtrl->command) & TBES_DISABLED)
	{
		rect->OffsetRect(1, 1);
		pDC->SetTextColor(m_clrHilite);

		// Position the cursor and print everything to the left of the underlined character.
		pDC->MoveTo(rect->left, rect->top);
		pDC->SetTextAlign(pDC->GetTextAlign() | TA_UPDATECP);
		pDC->ExtTextOut(NULL, NULL, NULL, NULL, before, NULL);

		// Select the underlined font and print the underlined character.
		CFont* oldFont = pDC->SelectObject(&m_horizUFont);
		pDC->ExtTextOut(NULL, NULL, NULL, NULL, ampChar, NULL);

		// Select the old font and print the rest.
		pDC->SelectObject(oldFont);
		pDC->ExtTextOut(NULL, NULL, NULL, NULL, after, NULL);
		pDC->SetTextAlign(pDC->GetTextAlign() & ~TA_UPDATECP);

		rect->OffsetRect(-1, -1);
		pDC->SetTextColor(m_clrGrayed);

		// Position the cursor and print everything to the left of the underlined character.
		pDC->MoveTo(rect->left, rect->top);
		pDC->SetTextAlign(pDC->GetTextAlign() | TA_UPDATECP);
		pDC->ExtTextOut(NULL, NULL, NULL, NULL, before, NULL);

		// Select the underlined font and print the underlined character.
		oldFont = pDC->SelectObject( &m_horizUFont);
		pDC->ExtTextOut(NULL, NULL, NULL, NULL, ampChar, NULL);

		// Select the old font and print the rest.
		pDC->SelectObject(oldFont);
		pDC->ExtTextOut(NULL, NULL, NULL, NULL, after, NULL);
		pDC->SetTextAlign(pDC->GetTextAlign() & ~TA_UPDATECP);
	}
	if(!(m_pResources->GetControlState(pCtrl->command) & TBES_DISABLED))
	{
		// Position the cursor and print everything to the left of the underlined character.
		pDC->MoveTo(rect->left, rect->top);
		pDC->SetTextAlign(pDC->GetTextAlign() | TA_UPDATECP);
		pDC->ExtTextOut(NULL, NULL, NULL, NULL, before, NULL);

		// Select the underlined font and print the underlined character.
		CFont* oldFont = pDC->SelectObject(&m_horizUFont);
		pDC->ExtTextOut(NULL, NULL, NULL, NULL, ampChar, NULL);

		// Select the old font and print the rest.
		pDC->SelectObject(oldFont);
		pDC->ExtTextOut(NULL, NULL, NULL, NULL, after, NULL);
		pDC->SetTextAlign(pDC->GetTextAlign() & ~TA_UPDATECP);
	}

	// If we decide to use the rect again, we need to change it back.
	if(m_pResources->GetControlState(pCtrl->command) & TBES_CHECKED) rect->OffsetRect(-1, -1);
}

void CCommandBarPopup::_DrawAccelerator(CDC* pDC, CRect* ctrlRect, CCommandBarControlInfo* pCtrl)
{
	CRect rect;  GetClientRect(&rect);
	CString strAccel = m_pResources->GetControlAcceleratorString(pCtrl->command);
	CSize extent;
	::GetTextExtentPoint32(pDC->GetSafeHdc(), strAccel, strAccel.GetLength(), &extent);

	rect.left = rect.right - 24 - 3 - extent.cx;
	rect.top = (ctrlRect->Height() - extent.cy) / 2 + ctrlRect->top;
	rect.bottom = rect.top + extent.cy;

	// If we're checked, offset the rect.
	if(m_pResources->GetControlState(pCtrl->command) & TBES_CHECKED) rect.OffsetRect(1, 1);

	if(m_pResources->GetControlState(pCtrl->command) & TBES_DISABLED)
	{
		rect.OffsetRect(1, 1);
		pDC->SetTextColor(m_clrHilite);

		// Position the cursor and print everything to the left of the underlined character.
		pDC->MoveTo(rect.left, rect.top);
		pDC->SetTextAlign(pDC->GetTextAlign() | TA_UPDATECP);
		pDC->ExtTextOut(NULL, NULL, NULL, NULL, strAccel, NULL);
		pDC->SetTextAlign(pDC->GetTextAlign() & ~TA_UPDATECP);

		rect.OffsetRect(-1, -1);
		pDC->SetTextColor(m_clrGrayed);

		// Position the cursor and print everything to the left of the underlined character.
		pDC->MoveTo(rect.left, rect.top);
		pDC->SetTextAlign(pDC->GetTextAlign() | TA_UPDATECP);
		pDC->ExtTextOut(NULL, NULL, NULL, NULL, strAccel, NULL);
		pDC->SetTextAlign(pDC->GetTextAlign() & ~TA_UPDATECP);
	}
	if(!(m_pResources->GetControlState(pCtrl->command) & TBES_DISABLED))
	{
		// Position the cursor and print everything to the left of the underlined character.
		pDC->MoveTo(rect.left, rect.top);
		pDC->SetTextAlign(pDC->GetTextAlign() | TA_UPDATECP);
		pDC->ExtTextOut(NULL, NULL, NULL, NULL, strAccel, NULL);
		pDC->SetTextAlign(pDC->GetTextAlign() & ~TA_UPDATECP);
	}

	// If we decide to use the rect again, we need to change it back.
	if(m_pResources->GetControlState(pCtrl->command) & TBES_CHECKED) rect.OffsetRect(-1, -1);
}

void CCommandBarPopup::_DrawImage(CDC* pDC, CRect* rect, CCommandBarControlInfo* pCtrl)
{
	CPoint ptImage(rect->left, rect->top + 2);
			
	if(m_pResources->GetControlState(pCtrl->command) & TBES_CHECKED) ptImage.Offset(1, 1);

	m_pResources->DrawImage(pDC, pCtrl->image, ptImage,
		(m_pResources->GetControlState(pCtrl->command) & TBES_DISABLED) ?
		imageDisabled : imageNormal);
}

void CCommandBarPopup::_DrawSeparator(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl)
{
	CRect client;  GetClientRect(client);

	CRect sepRect(SEPARATORMARGIN, rect->bottom + 4,
		client.right - SEPARATORMARGIN, rect->bottom + 5);
	pDC->DrawEdge(&sepRect, EDGE_ETCHED, BF_TOP);
}

void CCommandBarPopup::_DrawButton(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl)
{
	_DrawBasic(pDC, rect, index, pCtrl);

	CRect rcText;
	CRect rcImage;

	rcText = rect;  rcText.left += BUTTONWIDTH + 3;
	rcText.top = (rcText.Height() - pCtrl->textExtent.cy) / 2 + rcText.top;
	rcImage = rect; rcImage.right = rcImage.left + BUTTONWIDTH;
			
	_DrawImage(pDC, &rcImage, pCtrl);
	_DrawText(pDC, &rcText, pCtrl);
	_DrawAccelerator(pDC, rect, pCtrl);
}

void CCommandBarPopup::_DrawPopup(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl)
{
	_DrawBasic(pDC, rect, index, pCtrl);

	CRect rcText = rect;

	rcText.left = (rcText.Width() - pCtrl->textExtent.cx) / 2 + rcText.left;
	rcText.top = (rcText.Height() - pCtrl->textExtent.cy) / 2 + rcText.top;

	_DrawText(pDC, &rcText, pCtrl);
}

void CCommandBarPopup::_DrawButtonDropDown(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl)
{
	_DrawBasic(pDC, rect, index, pCtrl);

/*	CRect btnRect = wholeRect;  btnRect.right -= DROPDOWNWIDTH;
	_DrawImage(pDC, &btnRect, pCtrl);*/

	CRect rcText;
	CRect rcImage;

	rcText = rect;  rcText.left += BUTTONWIDTH + 3;
	rcText.top = (rcText.Height() - pCtrl->textExtent.cy) / 2 + rcText.top;
	rcImage = rect; rcImage.right = rcImage.left + BUTTONWIDTH;
			
	_DrawImage(pDC, &rcImage, pCtrl);
	_DrawText(pDC, &rcText, pCtrl);
}

void CCommandBarPopup::_DrawComboOrDropDown(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl)
{
	// Get rid of any border that may already be there.
	pDC->Draw3dRect(rect, m_clrButton, m_clrButton);

	CRect contentRect = rect;
	CRect outlineRect = rect;
	outlineRect.DeflateRect(1, 1);
	contentRect.DeflateRect(2, 2);

	CRect dropRect = rect;
	dropRect.left = dropRect.right - rect->Height() / 2 - 3;
	dropRect.right -= 3;
	dropRect.top += 3; dropRect.bottom -= 3;

	CRect arrowRect = dropRect;
	arrowRect.left += 2; arrowRect.right -= 3;

	int nType = m_pResources->GetControlType(pCtrl->command);

	if(pCtrl->bCustomMarked)
	{
		CRect rect2 = rect;
		pDC->Draw3dRect(rect2, m_clrDkShadow, m_clrDkShadow);
		rect2.DeflateRect(1, 1);
		pDC->Draw3dRect(rect2, m_clrDkShadow, m_clrDkShadow);
	}

	if(pCtrl->bCustomInsert) _DrawInsertBar(pDC, rect);

	if(m_pResources->GetControlState(pCtrl->command) & TBES_DISABLED)
	{
		pDC->Draw3dRect(outlineRect, m_clrWindow, m_clrWindow);
		_DrawArrow(pDC, &arrowRect, pCtrl, arrowDown);
//		return;
	}
	else
	{
		pDC->Draw3dRect(contentRect, m_clrWindow, m_clrWindow);

		if(nType == cBarControlDropDown || nType == cBarControlGraphicDropDown)
		{
			pDC->FillSolidRect(contentRect, m_clrWindow);
		}

		if(nType == cBarControlComboBox || nType == cBarControlGraphicCombo)
		{
			contentRect.left = contentRect.right - (rect->Height() / 2 + 3);
			pDC->FillSolidRect(contentRect, m_clrWindow);
		}

		pDC->FillSolidRect(dropRect, m_clrButton);

		_DrawArrow(pDC, &arrowRect, pCtrl, arrowDown);

		if(pCtrl->bHot)
		{
			pDC->DrawEdge(rect, BDR_SUNKENOUTER, BF_RECT);

			CRect borderRect = dropRect;
			borderRect.InflateRect(2, 2);
			dropRect.InflateRect(1, 1);
			pDC->Draw3dRect(borderRect, m_clrButton, m_clrButton);
			pDC->DrawEdge(dropRect, BDR_RAISEDINNER, BF_RECT);
		}

		if(pCtrl->bDropped)
		{
			pDC->DrawEdge(rect, BDR_SUNKENOUTER, BF_RECT);

			CRect borderRect = dropRect;
			borderRect.InflateRect(0, 0);
			pDC->DrawEdge(borderRect, BDR_SUNKENOUTER, BF_RECT);
		}
	}

	// Prepare to draw contents.
	contentRect.right -= (rect->Height() / 2 + 3);
	contentRect.bottom--;

	if(nType == cBarControlDropDown)
	{
		contentRect.left += 2;
		pDC->SetTextColor(m_clrText);
		
		CString str = m_pResources->GetControlEntry(pCtrl->command);
		pDC->DrawText(str, contentRect, DT_SINGLELINE | DT_NOPREFIX | DT_VCENTER);
	}
	else if(nType == cBarControlGraphicDropDown)
	{
		int nOldID = SetDlgCtrlID(pCtrl->command);

		NMCBCDRAWITEM nm;
		nm.hdr.hwndFrom = GetSafeHwnd();
		nm.hdr.idFrom = pCtrl->command;
		nm.hdr.code = CBCN_DRAWITEM;
		nm.pWnd = m_wndList;
		nm.pDC = pDC;
		nm.nDrawType = cBarDrawTypeBar;
		nm.nIndex = m_pResources->GetControlSelection(pCtrl->command);
		nm.nState = m_pResources->GetControlState(pCtrl->command);
		nm.strEntry = m_pResources->GetControlEntry(pCtrl->command);
		nm.rect = contentRect;
		GetOwner()->SendMessage(WM_NOTIFY, pCtrl->command, (LPARAM)&nm);

		SetDlgCtrlID(nOldID);
	}
}

void CCommandBarPopup::_DrawEdit(CDC* pDC, CRect* rect, int index, CCommandBarControlInfo* pCtrl)
{
	// Get rid of any border that may already be there.
	pDC->Draw3dRect(rect, m_clrButton, m_clrButton);

	CRect outlineRect = rect;
	outlineRect.DeflateRect(1, 1);

	if(m_pResources->GetControlState(pCtrl->command) & TBES_DISABLED)
	{
		pDC->Draw3dRect(outlineRect, m_clrWindow, m_clrWindow);
		return;
	}

	if(pCtrl->bHot)
	{
		pDC->DrawEdge(rect, BDR_SUNKENOUTER, BF_RECT);
	}
}

// Calculates the optimal size for a control, depending on its styles.  It will replace
// the control's size value.  Whether it replaces the width or height depends on how the
// bar is docked.  (This basically does a lot of the grunt work in sizing a toolbar,
// keeping CalcSize() all white and shiney.)
void CCommandBarPopup::CalcOptimalControlSize(CDC* pDC, CCommandBarControlInfo* pCtrl)
{
	// A convenient little way to make sure our measurement goes in the right place
	// avoiding a load of conditional code later.
	// "longSide" represents the longest side, meaning the side that will be extended
	// if space for text is required.  "shortSide" is the untouched side.
	long* longSide = &(pCtrl->size.cx);
	long* shortSide = &(pCtrl->size.cy);
	
	*shortSide = *longSide = 0;
	int buttonLong = BUTTONWIDTH;
	int buttonShort = BUTTONHEIGHT;

	pCtrl->textExtent = CSize(0, 0);

	CRect rect;
	CString strAccel;

	switch(m_pResources->GetControlType(pCtrl->command))
	{
		case cBarControlSeparator:
		// There is no optimal size for a separator--it is automatically determined
		// in the layout routines.
			return;

		case cBarControlEdit:
		case cBarControlComboBox:
		case cBarControlDropDown:
		case cBarControlGraphicCombo:
		case cBarControlGraphicDropDown:
			// No optimal width, must be set.  In this case, set the regular size
			// to the preferred size.  Account slightly for the border space as well.
			pCtrl->size = pCtrl->requestedSize + CSize(6, 0);
			return;

		case cBarControlButton:
		case cBarControlButtonPopup:
		case cBarControlButtonDropDown:
			*shortSide = buttonShort;
			*longSide += buttonLong;
			
			if(!pCtrl->label.IsEmpty())
			{
				// Filter out ampersands before getting the extent.
				CString before, ampChar, after;
				CString original = pCtrl->label;
				FilterAmpersand(original, before, ampChar, after);
				CString realString = before + ampChar + after;

				CSize extent;
				::GetTextExtentPoint32(pDC->GetSafeHdc(), realString, realString.GetLength(), &extent);
				pCtrl->textExtent = extent;
				*longSide += extent.cx + 6;
				*shortSide = max(*shortSide, extent.cy + 6);
			}
			else
				*longSide = buttonLong;

			strAccel = m_pResources->GetControlAcceleratorString(pCtrl->command);
			if(!strAccel.IsEmpty())
			{
				CSize extent;
				::GetTextExtentPoint32(pDC->GetSafeHdc(), strAccel, strAccel.GetLength(), &extent);
				*longSide += extent.cx + 12;
			}

			return;

/*		case cBarControlButtonDropDown:
			*longSide += buttonLong;		// A button with just an image has standard dimensions
			*shortSide = buttonShort;
			
			// Tack on the drop down, and some comfort space
			// (always on the right, so *longSide is not used)
			pCtrl->size.cx += DROPDOWNWIDTH + DROPDOWNSPACE;
			return;
*/
		case cBarControlPopup:
			CRect rect;
			if(!pCtrl->label.IsEmpty())
			{
				// Filter out ampersands before getting the extent.
				CString before, ampChar, after;
				CString original = pCtrl->label;
				FilterAmpersand(original, before, ampChar, after);
				CString realString = before + ampChar + after;

				CSize extent;
				::GetTextExtentPoint32(pDC->GetSafeHdc(), realString, realString.GetLength(), &extent);
				pCtrl->textExtent = extent;
				*longSide += extent.cx + 10;
				*shortSide = max(*shortSide, extent.cy + 6);
			}
			return;
	}
}

void CCommandBarPopup::FilterAmpersand(CString& original, CString& before, CString& ampChar, CString& after)
{
	before.Empty(); ampChar.Empty(); after.Empty();

	for(int i = 0; i < original.GetLength(); i++)
	{
		if(original[i] != '&')		// Not an ampersand
		{
			before += original[i];
		}
		else if(original[i] == '&' && original[i + 1] == '&')		// Two ampersands
		{
			before += '&';
			i++;			// Force us past the second ampersand
		}
		else						// An ampersand and something else
		{
			i++;
			ampChar = original[i];

			CString tempAfter = original.Mid(i + 1);
			// Now we still have to parse the rest for double ampersands.
			for(int j = 0; j < tempAfter.GetLength(); j++)
			{
				if(tempAfter[j] != '&')		// Not an ampersand
				{
					after += tempAfter[j];
				}
				else if(tempAfter[j] == '&' && tempAfter[j + 1] == '&')		// Two ampersands
				{
					after += '&';
					j++;			// Force us past the second ampersand
				}
				else
				{
					// Do nothing if we have an ampersand and another character.  The next
					// time through the loop will hit it.
				}
			}
			break;
		}
	}
}

int CCommandBarPopup::CommandToIndex(UINT nIDFind, int nStart)
{
	for(int i = nStart; i < m_aControls.GetSize(); i++)
		if(m_aControls[i]->command == nIDFind) return i;

	return -1;
}

UINT CCommandBarPopup::GetItemID(int nIndex)
{
	if(nIndex > m_aControls.GetSize()) return (UINT)-1;
	else return m_aControls[nIndex]->command;
}

int CCommandBarPopup::GetControlCount() const { return m_aControls.GetSize(); }

void CCommandBarPopup::OnSysColorChange() 
{
	CWnd::OnSysColorChange();
	
	m_clrHilite = ::GetSysColor(COLOR_BTNHIGHLIGHT);
	m_clrShadow = ::GetSysColor(COLOR_BTNSHADOW);
	m_clrDkShadow = ::GetSysColor(COLOR_3DDKSHADOW);
	m_clrButton = ::GetSysColor(COLOR_BTNFACE);
	m_clrWindow = ::GetSysColor(COLOR_WINDOW);
	m_clrChecked = ::GetSysColor(COLOR_3DLIGHT);
	m_clrText = ::GetSysColor(COLOR_BTNTEXT);
	m_clrGrayed = ::GetSysColor(COLOR_GRAYTEXT);
}

void CCommandBarPopup::OnRButtonUp(UINT nFlags, CPoint point) 
{
	CPoint pt = point;  ClientToScreen(&pt);
	m_pResources->PopupControlBarMenu((CFrameWnd*)GetOwner(), pt);
	
	CControlBar::OnRButtonUp(nFlags, point);
}

void CCommandBarPopup::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized) 
{
	CControlBar::OnActivate(nState, pWndOther, bMinimized);
	
	if(nState == WA_INACTIVE) DestroyWindow();
}
