////////////////////////////////////////////////////////////////////////
//
//  DDraw namespace definition
//
////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "FF3Edit.h"

////////////////////////////////////////////////////////////////////////
//
//  DDLoadBitmap
//
//  Create a DirectDrawSurface from a bitmap resource.
//
////////////////////////////////////////////////////////////////////////
LPDIRECTDRAWSURFACE4 DDrawUtil::LoadBitmap(LPDIRECTDRAW4 pdd, LPCSTR szBitmap, int dx, int dy)
{
    HBITMAP hbm;
    BITMAP bm;
    DDSURFACEDESC2 ddsd;
    LPDIRECTDRAWSURFACE4 pdds;

    //  try to load the bitmap as a resource, if that fails, try it as a file
    hbm = (HBITMAP)LoadImage(GetModuleHandle(NULL), szBitmap, IMAGE_BITMAP, dx, dy, LR_CREATEDIBSECTION);
    if(hbm == NULL) hbm = (HBITMAP)LoadImage(NULL, szBitmap, IMAGE_BITMAP, dx, dy, LR_LOADFROMFILE|LR_CREATEDIBSECTION);
    if(hbm == NULL) return NULL;

    // get size of the bitmap
    GetObject(hbm, sizeof(bm), &bm);

    // create a DirectDrawSurface for this bitmap
    ZeroMemory(&ddsd, sizeof(ddsd));
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT |DDSD_WIDTH;
    ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
    ddsd.dwWidth = bm.bmWidth;
    ddsd.dwHeight = bm.bmHeight;

    if(pdd->CreateSurface(&ddsd, &pdds, NULL) != DD_OK) return NULL;

    CopyBitmap(pdds, hbm, 0, 0, 0, 0);
    DeleteObject(hbm);

    return pdds;
}

LPDIRECTDRAWSURFACE4 DDrawUtil::LoadSizeBitmap(LPDIRECTDRAW4 pdd, LPCSTR szBitmap, int *dx, int *dy)
{
	HBITMAP hbm;
	BITMAP bm;
	DDSURFACEDESC2 ddsd;
	LPDIRECTDRAWSURFACE4 pdds;

	hbm = (HBITMAP)LoadImage(GetModuleHandle(NULL), szBitmap, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);
	if (hbm == NULL) hbm = (HBITMAP)LoadImage(NULL, szBitmap, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE|LR_CREATEDIBSECTION);
	if (hbm == NULL) return NULL;

	// get size of the bitmap
	GetObject(hbm, sizeof(bm), &bm);

	// create a DirectDrawSurface for this bitmap
	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS | DDSD_HEIGHT |DDSD_WIDTH;
	ddsd.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
	ddsd.dwWidth = bm.bmWidth;
	ddsd.dwHeight = bm.bmHeight;

	if (pdd->CreateSurface(&ddsd, &pdds, NULL) != DD_OK) return NULL;
	
	CopyBitmap(pdds, hbm, 0, 0, 0, 0);

	DeleteObject(hbm);

	*dx = ddsd.dwWidth;
	*dy = ddsd.dwHeight;

	return pdds;
}

////////////////////////////////////////////////////////////////////////
//
//  DDrawUtil::ReLoadBitmap
//
//  Load a bitmap from a file or resource into a directdraw surface.
//  Normaly used to re-load a surface after a restore.
//
////////////////////////////////////////////////////////////////////////
HRESULT DDrawUtil::ReLoadBitmap(LPDIRECTDRAWSURFACE4 pdds, LPCSTR szBitmap)
{
    HBITMAP hbm;
    HRESULT hr;

    //  try to load the bitmap as a resource, if that fails, try it as a file
    hbm = (HBITMAP)LoadImage(GetModuleHandle(NULL), szBitmap, IMAGE_BITMAP, 0, 0, LR_CREATEDIBSECTION);
    if (hbm == NULL) hbm = (HBITMAP)LoadImage(NULL, szBitmap, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE|LR_CREATEDIBSECTION);
    if (hbm == NULL)
	{
        OutputDebugString("handle is null\n");
        return E_FAIL;
    }

    hr = CopyBitmap(pdds, hbm, 0, 0, 0, 0);
    if (hr != DD_OK)
    {
        OutputDebugString("ddcopybitmap failed\n");
    }

    DeleteObject(hbm);
    return hr;
}

////////////////////////////////////////////////////////////////////////
//
//  DDCopyBitmap
//
//  Draw a bitmap into a DirectDrawSurface
//
////////////////////////////////////////////////////////////////////////
HRESULT DDrawUtil::CopyBitmap(LPDIRECTDRAWSURFACE4 pdds, HBITMAP hbm, int x, int y, int dx, int dy)
{
    HDC hdcImage;
    HDC hdc;
    BITMAP bm;
    DDSURFACEDESC2 ddsd;
    HRESULT hr;

    if (hbm == NULL || pdds == NULL) return E_FAIL;

    // make sure this surface is restored.
    pdds->Restore();

    //  select bitmap into a memoryDC so we can use it.
    hdcImage = CreateCompatibleDC(NULL);
    if (!hdcImage) OutputDebugString("createcompatible dc failed\n");
    SelectObject(hdcImage, hbm);

    // get size of the bitmap
    GetObject(hbm, sizeof(bm), &bm);
    dx = dx == 0 ? bm.bmWidth : dx;		// use the passed size, unless zero
    dy = dy == 0 ? bm.bmHeight : dy;

    // get size of surface.
    ddsd.dwSize = sizeof(ddsd);
    ddsd.dwFlags = DDSD_HEIGHT | DDSD_WIDTH;
    pdds->GetSurfaceDesc(&ddsd);

    if((hr = pdds->GetDC(&hdc)) == DD_OK)
    {
        StretchBlt(hdc, 0, 0, ddsd.dwWidth, ddsd.dwHeight, hdcImage, x, y, dx, dy, SRCCOPY);
        pdds->ReleaseDC(hdc);
    }

    DeleteDC(hdcImage);

    return hr;
}

////////////////////////////////////////////////////////////////////////
//
//  DDrawUtil::LoadPalette
//
//  Create a DirectDraw palette object from a bitmap resoure.
//
//  If the resource does not exist or NULL is passed create a
//  default 332 palette.
//
////////////////////////////////////////////////////////////////////////
LPDIRECTDRAWPALETTE DDrawUtil::LoadPalette(LPDIRECTDRAW4 pdd, LPCSTR szBitmap)
{
    LPDIRECTDRAWPALETTE ddpal;
    int i;
    int n;
    int fh;
    HRSRC h;
    LPBITMAPINFOHEADER lpbi;
    PALETTEENTRY  ape[256];
    RGBQUAD* prgb;

    // build a 332 palette as the default.
    for (i=0; i<256; i++)
    {
        ape[i].peRed   = (BYTE)(((i >> 5) & 0x07) * 255 / 7);
        ape[i].peGreen = (BYTE)(((i >> 2) & 0x07) * 255 / 7);
        ape[i].peBlue  = (BYTE)(((i >> 0) & 0x03) * 255 / 3);
        ape[i].peFlags = (BYTE)0;
    }

    // get a pointer to the bitmap resource.
    if (szBitmap && (h = FindResource(NULL, szBitmap, RT_BITMAP)))
    {
        lpbi = (LPBITMAPINFOHEADER)LockResource(LoadResource(NULL, h));
        if(!lpbi) OutputDebugString("lock resource failed\n");
        prgb = (RGBQUAD*)((BYTE*)lpbi + lpbi->biSize);

        if (lpbi == NULL || lpbi->biSize < sizeof(BITMAPINFOHEADER)) n = 0;
        else if (lpbi->biBitCount > 8) n = 0;
        else if (lpbi->biClrUsed == 0) n = 1 << lpbi->biBitCount;
        else n = lpbi->biClrUsed;

        //  a DIB color table has its colors stored BGR not RGB
        //  so flip them around.
        for(i=0; i<n; i++ )
        {
            ape[i].peRed   = prgb[i].rgbRed;
            ape[i].peGreen = prgb[i].rgbGreen;
            ape[i].peBlue  = prgb[i].rgbBlue;
            ape[i].peFlags = 0;
        }
    }
    else if (szBitmap && (fh = _lopen(szBitmap, OF_READ)) != -1)
    {
        BITMAPFILEHEADER bf;
        BITMAPINFOHEADER bi;

        _lread(fh, &bf, sizeof(bf));
        _lread(fh, &bi, sizeof(bi));
        _lread(fh, ape, sizeof(ape));
        _lclose(fh);

        if (bi.biSize != sizeof(BITMAPINFOHEADER)) n = 0;
        else if (bi.biBitCount > 8) n = 0;
        else if (bi.biClrUsed == 0) n = 1 << bi.biBitCount;
        else n = bi.biClrUsed;

        //  a DIB color table has its colors stored BGR not RGB
        //  so flip them around.
        for(i=0; i<n; i++ )
        {
            BYTE r = ape[i].peRed;
            ape[i].peRed  = ape[i].peBlue;
            ape[i].peBlue = r;
        }
    }

    pdd->CreatePalette(DDPCAPS_8BIT, ape, &ddpal, NULL);

    return ddpal;
}

/*
 * DDColorMatch
 *
 * convert a RGB color to a pysical color.
 *
 * we do this by leting GDI SetPixel() do the color matching
 * then we lock the memory and see what it got mapped to.
 */
DWORD DDrawUtil::ColorMatch(LPDIRECTDRAWSURFACE4 pdds, COLORREF rgb)
{
	return rgb;

	COLORREF rgbT;
	HDC hdc;
	DWORD dw = CLR_INVALID;
	DDSURFACEDESC2 ddsd;
	HRESULT hres;

	// use GDI SetPixel to color match for us
	if(rgb != CLR_INVALID && pdds->GetDC(&hdc) == DD_OK)
	{
		rgbT = GetPixel(hdc, 0, 0);             // save current pixel value
		SetPixel(hdc, 0, 0, rgb);               // set our value
		pdds->ReleaseDC(hdc);
    }

    // now lock the surface so we can read back the converted color
    ddsd.dwSize = sizeof(ddsd);
    while ((hres = pdds->Lock(NULL, &ddsd, 0, NULL)) == DDERR_WASSTILLDRAWING)
		;

    if(hres == DD_OK)
    {
        dw = *(DWORD*)ddsd.lpSurface;                     // get DWORD
        dw &= (1 << ddsd.ddpfPixelFormat.dwRGBBitCount) - 1;  // mask it to bpp
        pdds->Unlock(NULL);
    }

    //  now put the color that was there back.
    if(rgb != CLR_INVALID && pdds->GetDC(&hdc) == DD_OK)
    {
        SetPixel(hdc, 0, 0, rgbT);
        pdds->ReleaseDC(hdc);
    }

    return dw;
}

/*
 * DDSetColorKey
 *
 * set a color key for a surface, given a RGB.
 * if you pass CLR_INVALID as the color key, the pixel
 * in the upper-left corner will be used.
 */
HRESULT DDrawUtil::SetColorKey(LPDIRECTDRAWSURFACE4 pdds, COLORREF rgb)
{
    DDCOLORKEY ddck;

    ddck.dwColorSpaceLowValue  = ColorMatch(pdds, rgb);
    ddck.dwColorSpaceHighValue = ddck.dwColorSpaceLowValue;
    return pdds->SetColorKey(DDCKEY_SRCBLT, &ddck);
}

////////////////////////////////////////////////////////////////////////
// 
// Function     : DDError()
//
// Purpose      : Traces an error (DirectDraw)
//
////////////////////////////////////////////////////////////////////////

void DDrawUtil::Error(HRESULT hErr, HWND hWnd)
{
	char dderr[256];
	char err[1024];

	switch (hErr)
	{
		case DDERR_ALREADYINITIALIZED : sprintf(dderr, "DDERR_ALREADYINITIALIZED"); break;
		case DDERR_CANNOTATTACHSURFACE : sprintf(dderr, "DDERR_CANNOTATTACHSURFACE"); break;
		case DDERR_CANNOTDETACHSURFACE : sprintf(dderr, "DDERR_CANNOTDETACHSURFACE"); break;
		case DDERR_CURRENTLYNOTAVAIL : sprintf(dderr, "DDERR_CURRENTLYNOTAVAIL"); break;
		case DDERR_EXCEPTION : sprintf(dderr, "DDERR_EXCEPTION"); break;
		case DDERR_GENERIC : sprintf(dderr, "DDERR_GENERIC"); break;
		case DDERR_HEIGHTALIGN : sprintf(dderr, "DDERR_HEIGHTALIGN"); break;
		case DDERR_INCOMPATIBLEPRIMARY : sprintf(dderr, "DDERR_INCOMPATIBLEPRIMARY"); break;
		case DDERR_INVALIDCAPS : sprintf(dderr, "DDERR_INVALIDCAPS"); break;
		case DDERR_INVALIDCLIPLIST : sprintf(dderr, "DDERR_INVALIDCLIPLIST"); break;
		case DDERR_INVALIDMODE : sprintf(dderr, "DDERR_INVALIDMODE"); break;
		case DDERR_INVALIDOBJECT : sprintf(dderr, "DDERR_INVALIDOBJECT"); break;
		case DDERR_INVALIDPARAMS : sprintf(dderr, "DDERR_INVALIDPARAMS"); break;
		case DDERR_INVALIDPIXELFORMAT : sprintf(dderr, "DDERR_INVALIDPIXELFORMAT"); break;
		case DDERR_INVALIDRECT : sprintf(dderr, "DDERR_INVALIDRECT"); break;
		case DDERR_LOCKEDSURFACES : sprintf(dderr, "DDERR_LOCKEDSURFACES"); break;
		case DDERR_NO3D : sprintf(dderr, "DDERR_NO3D"); break;
		case DDERR_NOALPHAHW : sprintf(dderr, "DDERR_NOALPHAHW"); break;
		case DDERR_NOCLIPLIST : sprintf(dderr, "DDERR_NOCLIPLIST"); break;
		case DDERR_NOCOLORCONVHW : sprintf(dderr, "DDERR_NOCOLORCONVHW"); break;
		case DDERR_NOCOOPERATIVELEVELSET : sprintf(dderr, "DDERR_NOCOOPERATIVELEVELSET"); break;
		case DDERR_NOCOLORKEY : sprintf(dderr, "DDERR_NOCOLORKEY"); break;
		case DDERR_NOCOLORKEYHW : sprintf(dderr, "DDERR_NOCOLORKEYHW"); break;
		case DDERR_NODIRECTDRAWSUPPORT : sprintf(dderr, "DDERR_NODIRECTDRAWSUPPORT"); break;
		case DDERR_NOEXCLUSIVEMODE : sprintf(dderr, "DDERR_NOEXCLUSIVEMODE"); break;
		case DDERR_NOFLIPHW : sprintf(dderr, "DDERR_NOFLIPHW"); break;
		case DDERR_NOGDI : sprintf(dderr, "DDERR_NOGDI"); break;
		case DDERR_NOMIRRORHW : sprintf(dderr, "DDERR_NOMIRRORHW"); break;
		case DDERR_NOTFOUND : sprintf(dderr, "DDERR_NOTFOUND"); break;
		case DDERR_NOOVERLAYHW : sprintf(dderr, "DDERR_NOOVERLAYHW"); break;
		case DDERR_NORASTEROPHW : sprintf(dderr, "DDERR_NORASTEROPHW"); break;
		case DDERR_NOROTATIONHW : sprintf(dderr, "DDERR_NOROTATIONHW"); break;
		case DDERR_NOSTRETCHHW : sprintf(dderr, "DDERR_NOSTRETCHHW"); break;
		case DDERR_NOT4BITCOLOR : sprintf(dderr, "DDERR_NOT4BITCOLOR"); break;
		case DDERR_NOT4BITCOLORINDEX : sprintf(dderr, "DDERR_NOT4BITCOLORINDEX"); break;
		case DDERR_NOT8BITCOLOR : sprintf(dderr, "DDERR_NOT8BITCOLOR"); break;
		case DDERR_NOTEXTUREHW : sprintf(dderr, "DDERR_NOTEXTUREHW"); break;
		case DDERR_NOVSYNCHW : sprintf(dderr, "DDERR_NOVSYNCHW"); break;
		case DDERR_NOZBUFFERHW : sprintf(dderr, "DDERR_NOZBUFFERHW"); break;
		case DDERR_NOZOVERLAYHW : sprintf(dderr, "DDERR_NOZOVERLAYHW"); break;
		case DDERR_OUTOFCAPS : sprintf(dderr, "DDERR_OUTOFCAPS"); break;
		case DDERR_OUTOFMEMORY : sprintf(dderr, "DDERR_OUTOFMEMORY"); break;
		case DDERR_OUTOFVIDEOMEMORY : sprintf(dderr, "DDERR_OUTOFVIDEOMEMORY"); break;
		case DDERR_OVERLAYCANTCLIP : sprintf(dderr, "DDERR_OVERLAYCANTCLIP"); break;
		case DDERR_OVERLAYCOLORKEYONLYONEACTIVE : sprintf(dderr, "DDERR_OVERLAYCOLORKEYONLYONEACTIVE"); break;
		case DDERR_PALETTEBUSY : sprintf(dderr, "DDERR_PALETTEBUSY"); break;
		case DDERR_COLORKEYNOTSET : sprintf(dderr, "DDERR_COLORKEYNOTSET"); break;
		case DDERR_SURFACEALREADYATTACHED : sprintf(dderr, "DDERR_SURFACEALREADYATTACHED"); break;
		case DDERR_SURFACEALREADYDEPENDENT : sprintf(dderr, "DDERR_SURFACEALREADYDEPENDENT"); break;
		case DDERR_SURFACEBUSY : sprintf(dderr, "DDERR_SURFACEBUSY"); break;
		case DDERR_CANTLOCKSURFACE : sprintf(dderr, "DDERR_CANTLOCKSURFACE"); break;
		case DDERR_SURFACEISOBSCURED : sprintf(dderr, "DDERR_SURFACEISOBSCURED"); break;
		case DDERR_SURFACELOST : sprintf(dderr, "DDERR_SURFACELOST"); break;
		case DDERR_SURFACENOTATTACHED : sprintf(dderr, "DDERR_SURFACENOTATTACHED"); break;
		case DDERR_TOOBIGHEIGHT : sprintf(dderr, "DDERR_TOOBIGHEIGHT"); break;
		case DDERR_TOOBIGSIZE : sprintf(dderr, "DDERR_TOOBIGSIZE"); break;
		case DDERR_TOOBIGWIDTH : sprintf(dderr, "DDERR_TOOBIGWIDTH"); break;
		case DDERR_UNSUPPORTED : sprintf(dderr, "DDERR_UNSUPPORTED"); break;
		case DDERR_UNSUPPORTEDFORMAT : sprintf(dderr, "DDERR_UNSUPPORTEDFORMAT"); break;
		case DDERR_UNSUPPORTEDMASK : sprintf(dderr, "DDERR_UNSUPPORTEDMASK"); break;
		case DDERR_VERTICALBLANKINPROGRESS : sprintf(dderr, "DDERR_VERTICALBLANKINPROGRESS"); break;
		case DDERR_WASSTILLDRAWING : sprintf(dderr, "DDERR_WASSTILLDRAWING"); break;
		case DDERR_XALIGN : sprintf(dderr, "DDERR_XALIGN"); break;
		case DDERR_INVALIDDIRECTDRAWGUID : sprintf(dderr, "DDERR_INVALIDDIRECTDRAWGUID"); break;
		case DDERR_DIRECTDRAWALREADYCREATED : sprintf(dderr, "DDERR_DIRECTDRAWALREADYCREATED"); break;
		case DDERR_NODIRECTDRAWHW : sprintf(dderr, "DDERR_NODIRECTDRAWHW"); break;
		case DDERR_PRIMARYSURFACEALREADYEXISTS : sprintf(dderr, "DDERR_PRIMARYSURFACEALREADYEXISTS"); break;
		case DDERR_NOEMULATION : sprintf(dderr, "DDERR_NOEMULATION"); break;
		case DDERR_REGIONTOOSMALL : sprintf(dderr, "DDERR_REGIONTOOSMALL"); break;
		case DDERR_CLIPPERISUSINGHWND : sprintf(dderr, "DDERR_CLIPPERISUSINGHWND"); break;
		case DDERR_NOCLIPPERATTACHED : sprintf(dderr, "DDERR_NOCLIPPERATTACHED"); break;
		case DDERR_NOHWND : sprintf(dderr, "DDERR_NOHWND"); break;
		case DDERR_HWNDSUBCLASSED : sprintf(dderr, "DDERR_HWNDSUBCLASSED"); break;
		case DDERR_HWNDALREADYSET : sprintf(dderr, "DDERR_HWNDALREADYSET"); break;
		case DDERR_NOPALETTEATTACHED : sprintf(dderr, "DDERR_NOPALETTEATTACHED"); break;
		case DDERR_NOPALETTEHW : sprintf(dderr, "DDERR_NOPALETTEHW"); break;
		case DDERR_BLTFASTCANTCLIP : sprintf(dderr, "DDERR_BLTFASTCANTCLIP"); break;
		case DDERR_NOBLTHW : sprintf(dderr, "DDERR_NOBLTHW"); break;
		case DDERR_NODDROPSHW : sprintf(dderr, "DDERR_NODDROPSHW"); break;
		case DDERR_OVERLAYNOTVISIBLE : sprintf(dderr, "DDERR_OVERLAYNOTVISIBLE"); break;
		case DDERR_NOOVERLAYDEST : sprintf(dderr, "DDERR_NOOVERLAYDEST"); break;
		case DDERR_INVALIDPOSITION : sprintf(dderr, "DDERR_INVALIDPOSITION"); break;
		case DDERR_NOTAOVERLAYSURFACE : sprintf(dderr, "DDERR_NOTAOVERLAYSURFACE"); break;
		case DDERR_EXCLUSIVEMODEALREADYSET : sprintf(dderr, "DDERR_EXCLUSIVEMODEALREADYSET"); break;
		case DDERR_NOTFLIPPABLE : sprintf(dderr, "DDERR_NOTFLIPPABLE"); break;
		case DDERR_CANTDUPLICATE : sprintf(dderr, "DDERR_CANTDUPLICATE"); break;
		case DDERR_NOTLOCKED : sprintf(dderr, "DDERR_NOTLOCKED"); break;
		case DDERR_CANTCREATEDC : sprintf(dderr, "DDERR_CANTCREATEDC"); break;
		case DDERR_NODC : sprintf(dderr, "DDERR_NODC"); break;
		case DDERR_WRONGMODE : sprintf(dderr, "DDERR_WRONGMODE"); break;
		case DDERR_IMPLICITLYCREATED : sprintf(dderr, "DDERR_IMPLICITLYCREATED"); break;
		case DDERR_NOTPALETTIZED : sprintf(dderr, "DDERR_NOTPALETTIZED"); break;
		case DDERR_UNSUPPORTEDMODE : sprintf(dderr, "DDERR_UNSUPPORTEDMODE"); break;
		case DDERR_NOMIPMAPHW : sprintf(dderr, "DDERR_NOMIPMAPHW"); break;
		case DDERR_INVALIDSURFACETYPE : sprintf(dderr, "DDERR_INVALIDSURFACETYPE"); break;
		case DDERR_DCALREADYCREATED : sprintf(dderr, "DDERR_DCALREADYCREATED"); break;
		case DDERR_CANTPAGELOCK : sprintf(dderr, "DDERR_CANTPAGELOCK"); break;
		case DDERR_CANTPAGEUNLOCK : sprintf(dderr, "DDERR_CANTPAGEUNLOCK"); break;
		case DDERR_NOTPAGELOCKED : sprintf(dderr, "DDERR_NOTPAGELOCKED"); break;
		case DDERR_NOTINITIALIZED : sprintf(dderr, "DDERR_NOTINITIALIZED"); break;

		default : sprintf(dderr, "Unknown Error"); break;
	}

	sprintf(err, "DirectDraw Error %s", dderr);
	::MessageBox(hWnd, err, "Error", MB_OK);
	::PostQuitMessage(0);
}

static double HuetoRGB(double m1, double m2, double h )
{
   if(h < 0) h += 1.0;
   if(h > 1) h -= 1.0;
   
   if(6.0 * h < 1)
      return m1 + (m2 - m1) * h * 6.0;
   
   if(2.0 * h < 1)
      return m2;
   
   if(3.0 * h < 2.0)
      return m1 + (m2 - m1) * ((2.0 / 3.0) - h) * 6.0;
   
   return m1;
}


COLORREF HLStoRGB( double H, double L, double S )
{
  double r,g,b;
  double m1, m2;

  if(S == 0)
  {
     r = g = b = L;
  }
  else
  {
     if(L <= 0.5)
        m2 = L * (1.0 + S);
     else
        m2 = L + S - L * S;
     
	 m1 = 2.0 * L - m2;

     r = HuetoRGB(m1, m2, H + 1.0 / 3.0);
     g = HuetoRGB(m1, m2, H);
     b = HuetoRGB(m1, m2, H - 1.0 / 3.0);
  }
  
  return RGB((BYTE)(r * 255),(BYTE)(g * 255),(BYTE)(b * 255));
}

void RGBtoHSL( COLORREF rgb, double *H, double *S, double *L )
{   
	double delta;
	double r = (double)GetRValue(rgb) / 255;
	double g = (double)GetGValue(rgb) / 255;
	double b = (double)GetBValue(rgb) / 255;   
	double cmax = max(r, max(g, b));
	double cmin = min(r, min(g, b));   
	
	*L= (cmax + cmin) / 2.0;   
	
	if(cmax == cmin) 
	{
      *S = 0;      
	  *H = 0;		// it's really undefined   
	}
	else 
	{
      if(*L < 0.5) 
		  *S = (cmax - cmin)/(cmax + cmin);
	  else
         *S = (cmax-cmin) / (2.0 - cmax - cmin);      
	  
	  delta = cmax - cmin;
      
	  if(r == cmax) 
		  *H = (g-b) / delta;      
	  else if(g == cmax)
          *H = 2.0 +(b - r) / delta;
      else          
		  *H = 4.0 + (r - g) / delta;
      
	  *H /= 6.0; 
	  if(*H < 0.0)
          *H += 1;  
	}
}

WORD HLStoRGB555( double H, double L, double S )
{
  double r, g, b;
  double m1, m2;

  if(S == 0)
  {
     r = g = b = L;
  }
  else
  {
     if(L <= 0.5)
        m2 = L * (1.0 + S);
     else
        m2 = L + S - L * S;
     
	 m1 = 2.0 * L - m2;

     r = HuetoRGB(m1, m2, H + 1.0 / 3.0);
     g = HuetoRGB(m1, m2, H);
     b = HuetoRGB(m1, m2, H - 1.0 / 3.0);
  }
  
  return RGB555((BYTE)(r * 31),(BYTE)(g * 31),(BYTE)(b * 31));
}

void RGB555toHSL( WORD rgb, double *H, double *S, double *L )
{   
	double delta;
	double r = (double)R555(rgb)/31;
	double g = (double)G555(rgb)/31;
	double b = (double)B555(rgb)/31;   
	double cmax = max(r,max(g,b));
	double cmin = min(r,min(g,b));   
	*L=(cmax+cmin)/2.0;   
	
	if(cmax==cmin) 
	{
      *S = 0;      
	  *H = 0; // it's really undefined   
	} else 
	{
      if(*L < 0.5) 
		  *S = (cmax-cmin)/(cmax+cmin);      
	  else
         *S = (cmax-cmin)/(2.0-cmax-cmin);      
	  
	  delta = cmax - cmin;
      if(r==cmax) 
		  *H = (g-b)/delta;      
	  else if(g==cmax)
          *H = 2.0 +(b-r)/delta;
      else          
		  *H=4.0+(r-g)/delta;
      *H /= 6.0; 
	  if(*H < 0.0)
          *H += 1;  
	}
}
