// DialogueView.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "FF3Doc.h"
#include "DialogueView.h"
#include "FF3Data.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDialogueView

IMPLEMENT_DYNCREATE(CDialogueView, cdxCSizingFormView)

CDialogueView::CDialogueView()
	: cdxCSizingFormView(CDialogueView::IDD)
{
	//{{AFX_DATA_INIT(CDialogueView)
	//}}AFX_DATA_INIT

	int nHeight = -((GetDesktopWindow()->GetWindowDC()->GetDeviceCaps(LOGPIXELSY) * 9) / 72);
	m_dlgFont.CreateFont(nHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_CHARACTER_PRECIS,
		CLIP_CHARACTER_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, "Courier New");

	m_nDirection = 0;

	m_pFindReplace = NULL;
}

CDialogueView::~CDialogueView()
{
	if(m_pFindReplace) m_pFindReplace->DestroyWindow();
}

void CDialogueView::DoDataExchange(CDataExchange* pDX)
{
	cdxCSizingFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDialogueView)
	DDX_Control(pDX, IDC_APPLY, m_wndApply);
	DDX_Control(pDX, IDC_MINBOUNDS, m_wndMinBounds);
	DDX_Control(pDX, IDC_EDIT, m_wndEdit);
	DDX_Control(pDX, IDC_DIALOGUEPAGES, m_wndPages);
	DDX_Control(pDX, IDC_DIALOGUELIST, m_wndDialogueList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDialogueView, cdxCSizingFormView)
	//{{AFX_MSG_MAP(CDialogueView)
	ON_LBN_SELCHANGE(IDC_DIALOGUELIST, OnSelChangeDialogueList)
	ON_BN_CLICKED(IDC_APPLY, OnApply)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditClipboard)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditClipboard)
	ON_COMMAND(ID_EDIT_GOTO, OnProperties)//OnEditGoto)
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_DIALOGUESWITCH, OnDialogueSwitch)
	ON_MESSAGE(WM_COMPAREVIEW, OnCompareView)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDialogueView diagnostics

#ifdef _DEBUG
void CDialogueView::AssertValid() const
{
	cdxCSizingFormView::AssertValid();
}

void CDialogueView::Dump(CDumpContext& dc) const
{
	cdxCSizingFormView::Dump(dc);
}

CFF3Doc* CDialogueView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CFF3Doc)));
	return (CFF3Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDialogueView message handlers

void CDialogueView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CFF3Doc* pDoc = GetDocument();

	m_wndDialogueList.SetFont(&m_dlgFont);
	m_wndEdit.SetFont(&m_dlgFont);

	m_wndDialogueList.ResetContent();
	m_wndDialogueList.m_pDoc = pDoc;
	m_wndDialogueList.SetRedraw(FALSE);

	CDC* pDC = m_wndDialogueList.GetDC();
	CRect rect, cRect;
	m_wndDialogueList.GetClientRect(rect);
	CFont* pOldFont = pDC->SelectObject(GetFont());

	pDC->DrawText("Loading dialogue...", &rect, DT_CENTER | DT_VCENTER | DT_NOPREFIX | DT_SINGLELINE);

	pDC->SelectObject(pOldFont);
	m_wndDialogueList.ReleaseDC(pDC);

	m_wndDialogueList.SetHorizontalExtent(4096);
	m_wndPages.m_pDoc = pDoc;

	for(int i = 0; i < pDoc->dialogue.GetSize() - 1; i++)
	{
		int index = m_wndDialogueList.AddString(NULL);
		m_wndDialogueList.SetItemData(index, (DWORD)pDoc->dialogue[i]);
	}

	m_wndDialogueList.SetRedraw(TRUE);
	m_wndDialogueList.Invalidate();

	m_wndPages.ResetContent();

	m_wndDialogueList.SetCurSel(0);
}

void CDialogueView::OnInitialUpdate() 
{
	cdxCSizingFormView::OnInitialUpdate(m_wndMinBounds);

	AddSzControl(m_wndDialogueList, mdResize, mdResize);
	AddSzControl(m_wndPages, mdResize, mdRepos);
	AddSzControl(m_wndEdit, mdResize, mdRepos);
	AddSzControl(m_wndApply, mdRepos, mdRepos);

	ReorganizeControls();
}

void CDialogueView::OnSelChangeDialogueList() 
{
	CFF3Doc* pDoc = GetDocument();
	int nSel = m_wndDialogueList.GetCurSel();

	m_wndPages.ResetContent();
	m_wndPages.SetColumnWidth(248);

	DialogueItem* pDlg = pDoc->dialogue[nSel];
	BYTE* pOrigPtr = pDlg->text;
	BYTE* pCurPtr = pOrigPtr;
	BYTE* pPagePtr = pOrigPtr;
	
	while(pCurPtr < (pOrigPtr + pDlg->length))
	{
		BYTE bChar = *(pCurPtr++);

		if(bChar == 0x00 || bChar == 0x13)
		{
			int nCur = m_wndPages.AddString(NULL);
			m_wndPages.SetItemData(nCur, (DWORD)pPagePtr);

			pPagePtr = pCurPtr;
		}
	}

	m_wndPages.SetItemHeight(0, 96);

	CString s;
	pDoc->dialogue[nSel]->ConvertToASCII(s, DI_ConvertMode, DICM_Edit, TAG_END);

	m_wndEdit.SetWindowText(s);
	m_wndEdit.SetSel(0, -1);
	m_wndEdit.SetFocus();
}

void CDialogueView::OnApply() 
{
	CFF3Doc* pDoc = GetDocument();
	CString str;  m_wndEdit.GetWindowText(str);

	int nSel = m_wndDialogueList.GetCurSel();
	DWORD result = pDoc->dialogue[nSel]->ConvertFromASCII(str, TAG_END, NULL);

	if(result == DICERROR_Malformed)
	{
		MessageBox("The string entered is malformed; there may\nbe mismatched brackets or the text can \notherwise not be parsed correctly.  Please\ncorrect any errors and reapply.", "Error", MB_ICONEXCLAMATION | MB_OK);
		return;
	}

	OnSelChangeDialogueList();

	m_wndDialogueList.SetCurSel(nSel + m_nDirection);
	m_nDirection = 0;

	OnSelChangeDialogueList();
	
	m_wndEdit.SetSel(0, -1);
	m_wndEdit.SetFocus();
}


void CDialogueView::OnUpdateEditClipboard(CCmdUI* pCmdUI) 
{
	if(::IsWindow(m_wndEdit.m_hWnd))
	{
		int nBegin = 0, nEnd = 0;
		m_wndEdit.GetSel(nBegin, nEnd);

		if(nBegin == nEnd) pCmdUI->Enable(FALSE);
		else pCmdUI->Enable(TRUE);
	}
}

void CDialogueView::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	COleDataObject obj;

	if(obj.AttachClipboard())
	{
		if(obj.IsDataAvailable(CF_TEXT))
		{
			pCmdUI->Enable(TRUE);
			return;
		}
	}

	pCmdUI->Enable(FALSE);
}

void CDialogueView::OnUpdateEditUndo(CCmdUI* pCmdUI) 
{
	if(::IsWindow(m_wndEdit.m_hWnd))
		pCmdUI->Enable(m_wndEdit.CanUndo());
}


void CDialogueView::OnEditCut() 
{
	m_wndEdit.Cut();
}

void CDialogueView::OnEditCopy() 
{
	m_wndEdit.Copy();
}

void CDialogueView::OnEditPaste() 
{
	m_wndEdit.Paste();
}

void CDialogueView::OnEditUndo() 
{
	m_wndEdit.Undo();
}

LRESULT CDialogueView::OnDialogueSwitch(WPARAM wParam, LPARAM lParam)
{
	m_nDirection = (int)wParam;
	OnApply();

	return 0;
}

void CDialogueView::OnEditGoto() 
{
/*	CFF3Doc* pDoc = GetDocument();

	CGotoDlg dlg;

	if(dlg.DoModal() == IDOK)
	{
		int entry = dlg.m_nEntry;

		if(0x000 < entry && entry < 0xCFD)
		{
			CString str;  m_wndEdit.GetWindowText(str);

			int nSel = m_wndDialogueList.GetCurSel();
			DWORD result = pDoc->dialogue[nSel]->ConvertFromASCII(str, TAG_END, NULL);

			if(result == DICERROR_Malformed)
			{
				AfxMessageBox("The string entered is malformed; there may\nbe mismatched brackets or the text can \notherwise not be parsed correctly.  Please\ncorrect any errors and reapply.");
				return;
			}

			OnSelChangeDialogueList();

			CString s;
			pDoc->dialogue[nSel]->ConvertToASCII(s, DI_ConvertMode, DICM_List, TAG_END);
			m_wndDialogueList.DeleteString(nSel);
			m_wndDialogueList.InsertString(nSel, s);
			m_wndDialogueList.SetCurSel(entry);

			m_nDirection = 0;

			OnSelChangeDialogueList();
	
			m_wndEdit.SetSel(0, -1);
			m_wndEdit.SetFocus();
		}
	}*/
}

void CDialogueView::OnProperties()
{
	// TODO: The property sheet attached to your project
	// via this function is not hooked up to any message
	// handler.  In order to actually use the property sheet,
	// you will need to associate this function with a control
	// in your project such as a menu item or tool bar button.

	if(!m_pFindReplace)
	{
		m_pFindReplace = new CFindReplaceSheet;
		m_pFindReplace->Create();
	}

	// This is where you would retrieve information from the property
	// sheet if propSheet.DoModal() returned IDOK.  We aren't doing
	// anything for simplicity.
}

LRESULT CDialogueView::OnCompareView(WPARAM wParam, LPARAM lParam)
{
	if(lParam == 'DLG ') return TRUE;
	else return FALSE;
}
