// DirectDrawListBox.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "DirectDrawListBox.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDirectDrawListBox

CDirectDrawListBox::CDirectDrawListBox()
{
	m_lpSurfaceBack = NULL;
	m_lpSurfaceClipper = NULL;
}

CDirectDrawListBox::~CDirectDrawListBox()
{
}


BEGIN_MESSAGE_MAP(CDirectDrawListBox, CListBox)
	//{{AFX_MSG_MAP(CDirectDrawListBox)
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDirectDrawListBox message handlers

void CDirectDrawListBox::BeginDraw(LPRECT lpWorkRect, BOOL bErase /* = TRUE */)
{
	if(m_rectOffset != lpWorkRect)
	{
		m_rectOffset = lpWorkRect;

		CRect rect(0, 0, m_rectOffset.Width(), m_rectOffset.Height());

		// Detach the clipper from the surface
		HRESULT rval = m_lpSurfaceBack->SetClipper(NULL);
		if(rval != DD_OK) DDraw.Error(rval, NULL);

		// Release the surface, because we'll need to create it again.
		RELEASE(m_lpSurfaceBack);

		// Create the surface.
		ZeroMemory(&m_surfaceDesc, sizeof(m_surfaceDesc));
		m_surfaceDesc.dwSize = sizeof(m_surfaceDesc);
		m_surfaceDesc.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
		m_surfaceDesc.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
		m_surfaceDesc.dwWidth = m_rectOffset.Width();
		m_surfaceDesc.dwHeight = m_rectOffset.Height();

		rval = DDraw.lpDD->CreateSurface(&m_surfaceDesc, &m_lpSurfaceBack, NULL);
		if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());

		m_clipList.hdr.dwSize = sizeof(RGNDATAHEADER);
		m_clipList.hdr.iType = RDH_RECTANGLES;
		m_clipList.hdr.nCount = 1;
		m_clipList.hdr.nRgnSize = 0;
		m_clipList.hdr.rcBound = rect;
		m_clipList.rgndata[0] = rect;

		rval = m_lpSurfaceClipper->SetClipList((LPRGNDATA)&m_clipList, 0);
		if(rval != DD_OK) DDraw.Error(rval, NULL);

		rval = m_lpSurfaceBack->SetClipper(m_lpSurfaceClipper);
		if(rval != DD_OK) DDraw.Error(rval, NULL);

		ZeroMemory(&m_pixelFormat, sizeof(DDPIXELFORMAT));
		m_pixelFormat.dwSize = sizeof(DDPIXELFORMAT);
		rval = m_lpSurfaceBack->GetPixelFormat(&m_pixelFormat);
		if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());
	}

	// Point the clipper to our client area (we do this here so derived classes
	// don't have to worry about this extra step).
	HRESULT rval = DDraw.lpClipper->SetHWnd(0, GetSafeHwnd());
	if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());

	// Now erase the surface, if requested.
	if(bErase) ClearSurface(::GetSysColor(COLOR_WINDOW));
}

void CDirectDrawListBox::EndDraw()
{
	// Flip the surfaces so the newly drawn area is revealed.
	Flip();
}

HRESULT CDirectDrawListBox::Flip()
{
	CRect window;

	GetClientRect(&window);
	ClientToScreen(&window);
	window.left += m_rectOffset.left;
	window.top += m_rectOffset.top;
	window.right = window.left + m_rectOffset.Width();
	window.bottom = window.top + m_rectOffset.Height();

	HRESULT rval = DDraw.lpSurfFront->Blt(&window, m_lpSurfaceBack,
		CRect(0, 0, window.Width(), window.Height()), DDBLT_WAIT, NULL);
	if(rval == DDERR_SURFACELOST) RestoreSurface();

	return rval;
}

void CDirectDrawListBox::RestoreSurface()
{
	if(DDraw.lpSurfFront != NULL && DDraw.lpSurfFront->IsLost() != DD_OK)
		DDraw.lpSurfFront->Restore();
}

void CDirectDrawListBox::OnDestroy() 
{
	RELEASE(m_lpSurfaceClipper);
	RELEASE(m_lpSurfaceBack);

	CListBox::OnDestroy();
}

CDC* CDirectDrawListBox::GetSurfaceDC()
{
	HDC hDC;
	HRESULT rval = m_lpSurfaceBack->GetDC(&hDC);
	if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());

	CDC* pDC = new CDC;  pDC->Attach(hDC);
	return pDC;
}

void CDirectDrawListBox::ReleaseSurfaceDC(CDC* pDC)
{
	HDC hDC = pDC->GetSafeHdc();
	pDC->Detach();

	HRESULT rval = m_lpSurfaceBack->ReleaseDC(hDC);
	if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());

	delete pDC;
}

void CDirectDrawListBox::ClearSurface(COLORREF clr)
{
	CRect rect(0, 0, m_surfaceDesc.dwWidth, m_surfaceDesc.dwHeight);
	DDBLTFX ddbltfx;
	ddbltfx.dwSize = sizeof(ddbltfx);
	ddbltfx.dwFillColor = DDraw.ColorMatch(m_lpSurfaceBack, clr);

	HRESULT rval = m_lpSurfaceBack->Blt(&rect, NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL,
		&ddbltfx);
	if(rval == DDERR_SURFACELOST) RestoreSurface();
}

void CDirectDrawListBox::SetPixel(int x, int y, COLORREF clr)
{
	DDBLTFX ddbltfx;
	ddbltfx.dwSize = sizeof(ddbltfx);
	ddbltfx.dwFillColor = DDraw.ColorMatch(m_lpSurfaceBack, clr);

	HRESULT rval = m_lpSurfaceBack->Blt(CRect(x, y, x + 1, y + 1),
		NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL, &ddbltfx);
	if(rval == DDERR_SURFACELOST) RestoreSurface();
}

void CDirectDrawListBox::HorizontalLine(int xStart, int yStart, int xEnd, COLORREF clr)
{
	DDBLTFX ddbltfx;
	ddbltfx.dwSize = sizeof(ddbltfx);
	ddbltfx.dwFillColor = clr;//DDraw.ColorMatch(m_lpSurfaceBack, clr);

	HRESULT rval = m_lpSurfaceBack->Blt(CRect(xStart, yStart, xEnd, yStart + 1),
		NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL, &ddbltfx);
	if(rval == DDERR_SURFACELOST) RestoreSurface();
}

void CDirectDrawListBox::VerticalLine(int xStart, int yStart, int yEnd, COLORREF clr)
{
	DDBLTFX ddbltfx;
	ddbltfx.dwSize = sizeof(ddbltfx);
	ddbltfx.dwFillColor = clr;//DDraw.ColorMatch(m_lpSurfaceBack, clr);

	HRESULT rval = m_lpSurfaceBack->Blt(CRect(xStart, yStart, xStart + 1, yEnd),
		NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL, &ddbltfx);
	if(rval == DDERR_SURFACELOST) RestoreSurface();
}

void CDirectDrawListBox::Line(int xStart, int yStart, int xEnd, int yEnd, CPen* pPen)
{
	CDC* pDC = GetSurfaceDC();

	CPen* oldPen = pDC->SelectObject(pPen);
	pDC->MoveTo(xStart, yStart);
	pDC->LineTo(xEnd, yEnd);
	pDC->SelectObject(oldPen);

	ReleaseSurfaceDC(pDC);
}

void CDirectDrawListBox::Rectangle(LPRECT lpRect, CPen* pPen, CBrush* pBrush)
{
	CDC* pDC = GetSurfaceDC();

	CPen* oldPen = pDC->SelectObject(pPen);
	CBrush* oldBrush;
	if(pBrush) oldBrush = pDC->SelectObject(pBrush);
	else oldBrush = (CBrush*)pDC->SelectStockObject(NULL_BRUSH);
	pDC->Rectangle(lpRect);
	pDC->SelectObject(oldBrush);
	pDC->SelectObject(oldPen);

	ReleaseSurfaceDC(pDC);
}

void CDirectDrawListBox::Rectangle(int x, int y, int cx, int cy, CPen* pPen, CBrush* pBrush)
{
	CRect rect(x, y, cx, cy);
	Rectangle(&rect, pPen, pBrush);
}

void CDirectDrawListBox::FillSolidRect(LPRECT lpRect, COLORREF clr)
{
	DDBLTFX ddbltfx;
	ddbltfx.dwSize = sizeof(ddbltfx);
	ddbltfx.dwFillColor = clr; //DDraw.ColorMatch(m_lpSurfaceBack, clr);

	HRESULT rval = m_lpSurfaceBack->Blt(lpRect, NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL,
		&ddbltfx);
	if(rval == DDERR_SURFACELOST) RestoreSurface();
}

void CDirectDrawListBox::FillSolidRect(int x, int y, int cx, int cy, COLORREF clr)
{
	CRect rect(x, y, cx, cy);
	FillSolidRect(&rect, clr);
}

void CDirectDrawListBox::TextOut(int x, int y, LPCTSTR lpszString, int nCount, CFont* pFont,
							  COLORREF clr)
{
	CDC* pDC = GetSurfaceDC();

	CFont* oldFont = pDC->SelectObject(pFont);
	pDC->SetTextColor(clr);
	pDC->SetBkMode(TRANSPARENT);
	pDC->TextOut(x, y, lpszString, nCount);
	pDC->SelectObject(oldFont);

	ReleaseSurfaceDC(pDC);
}

void CDirectDrawListBox::TextOut(int x, int y, const CString& str, CFont* pFont, COLORREF clr)
{
	TextOut(x, y, str, str.GetLength(), pFont, clr);
}

void CDirectDrawListBox::ShadeRect(DDSURFACEDESC2* ddsd, int dx, int dy, int cx, int cy,
								WORD wShadeColor, float fPercent)
{
	DWORD* lpDest = (DWORD*)ddsd->lpSurface + (ddsd->lPitch / 4) * dy;

	for(int y = dy; y < dy + cy; y++)
	{
		for(int x = dx / 2; x < (dx + cx) / 2; x++)
		{
			DWORD dwColors = lpDest[x];

			WORD wDestColor1 = SubtractDDColor(HIWORD(dwColors), wShadeColor, (100 - fPercent) / 100);
			WORD wDestColor2 = SubtractDDColor(LOWORD(dwColors), wShadeColor, (100 - fPercent) / 100);

			lpDest[x] = MAKELONG(wDestColor2, wDestColor1);
		}

		lpDest += ddsd->lPitch / 4;
	}
}


void CDirectDrawListBox::PreSubclassWindow() 
{
	CListBox::PreSubclassWindow();

	// Create the back surface initially here.  This may not be necessary, as it'll be
	// recreated when the window is sized before creation anyways, but it's better to
	// be on the safe side.
	ZeroMemory(&m_surfaceDesc, sizeof(m_surfaceDesc));
	m_surfaceDesc.dwSize = sizeof(m_surfaceDesc);
	m_surfaceDesc.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
	m_surfaceDesc.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
	m_surfaceDesc.dwWidth = 2;
	m_surfaceDesc.dwHeight = 2;

	HRESULT	rval = DDraw.lpDD->CreateSurface(&m_surfaceDesc, &m_lpSurfaceBack, NULL);
	if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());

	rval = DDraw.lpDD->CreateClipper(0, &m_lpSurfaceClipper, NULL);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	rval = m_lpSurfaceBack->SetClipper(m_lpSurfaceClipper);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	ZeroMemory(&m_pixelFormat, sizeof(DDPIXELFORMAT));
	m_pixelFormat.dwSize = sizeof(DDPIXELFORMAT);
	rval = m_lpSurfaceBack->GetPixelFormat(&m_pixelFormat);
	if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());
}

