// DirectDrawView.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "DirectDrawView.h"
#include "mfxWhlPan.h"
#include <..\src\afximpl.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDirectDrawView

IMPLEMENT_DYNCREATE(CDirectDrawView, CView)

CDirectDrawView::CDirectDrawView()
{
	m_lpSurfaceBack = NULL;
	m_pDC = NULL;
	m_nRoundDown = 1;
}

CDirectDrawView::~CDirectDrawView()
{
}


BEGIN_MESSAGE_MAP(CDirectDrawView, CView)
	//{{AFX_MSG_MAP(CDirectDrawView)
	ON_WM_ERASEBKGND()
	ON_WM_SIZE()
	ON_WM_CREATE()
	ON_WM_DESTROY()
	ON_WM_PAINT()
	ON_WM_HSCROLL()
	ON_WM_VSCROLL()
	ON_WM_MOUSEWHEEL()
	ON_WM_MBUTTONDOWN()
	ON_WM_KEYDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDirectDrawView drawing

void CDirectDrawView::OnDraw(CDC* pDC)
{
	CDocument* pDoc = GetDocument();

	BeginDraw();
	
	// This will be overridden in a derived class and drawing code will be put
	// between the Begin/EndPaint() functions.

	EndDraw();
}

void CDirectDrawView::BeginDraw(BOOL bErase /* = TRUE */)
{
	// Point the clipper to our client area (we do this here so derived classes
	// don't have to worry about this extra step).
	HRESULT rval = DDraw.lpClipper->SetHWnd(0, GetSafeHwnd());
	if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());

	// Now erase the surface, if requested.
	if(bErase) ClearSurface(::GetSysColor(COLOR_WINDOW));
}

void CDirectDrawView::EndDraw()
{
	// Flip the surfaces so the newly drawn area is revealed.
	Flip();
}


/////////////////////////////////////////////////////////////////////////////
// CDirectDrawView diagnostics

#ifdef _DEBUG
void CDirectDrawView::AssertValid() const
{
	CView::AssertValid();
}

void CDirectDrawView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CDirectDrawView message handlers

void CDirectDrawView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);
	
	// Remake the back surface to conform to the new window size.
	RemakeSurfaces(cx, cy);

	SCROLLINFO si;

	// First the horizontal.
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_PAGE;
    si.nPage = cx / m_nRoundDown + 1;
	SetScrollInfo(SB_HORZ, &si);

	// Now the vertical.
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_PAGE;
    si.nPage = cy / m_nRoundDown + 1;
	SetScrollInfo(SB_VERT, &si);
}

void CDirectDrawView::RemakeSurfaces(int cx, int cy)
{
	CRect clientRect;  GetClientRect(clientRect);
	if(!clientRect.Width() || !clientRect.Height()) clientRect.SetRectEmpty();

	clientRect.right += 32;  clientRect.bottom += 32;

	// Detach the clipper from the surface
	HRESULT rval = m_lpSurfaceBack->SetClipper(NULL);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	// Release the surface, because we'll need to create it again.
	RELEASE(m_lpSurfaceBack);

	// Create the surface.
	ZeroMemory(&m_surfaceDesc, sizeof(m_surfaceDesc));
	m_surfaceDesc.dwSize = sizeof(m_surfaceDesc);
	m_surfaceDesc.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
	m_surfaceDesc.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
	m_surfaceDesc.dwWidth = clientRect.Width();
	m_surfaceDesc.dwHeight = clientRect.Height();

	rval = DDraw.lpDD->CreateSurface(&m_surfaceDesc, &m_lpSurfaceBack, NULL);
	if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());

	m_clipList.hdr.dwSize = sizeof(RGNDATAHEADER);
	m_clipList.hdr.iType = RDH_RECTANGLES;
	m_clipList.hdr.nCount = 1;
	m_clipList.hdr.nRgnSize = 0;
	m_clipList.hdr.rcBound = clientRect;
	m_clipList.rgndata[0] = clientRect;

	rval = m_lpSurfaceClipper->SetClipList((LPRGNDATA)&m_clipList, 0);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	rval = m_lpSurfaceBack->SetClipper(m_lpSurfaceClipper);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	ZeroMemory(&m_pixelFormat, sizeof(DDPIXELFORMAT));
	m_pixelFormat.dwSize = sizeof(DDPIXELFORMAT);
	rval = m_lpSurfaceBack->GetPixelFormat(&m_pixelFormat);
	if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());
}

void CDirectDrawView::ReleaseObjects()
{
	RELEASE(m_lpSurfaceClipper);
	RELEASE(m_lpSurfaceBack);
}

int CDirectDrawView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	CRect clientRect;  GetClientRect(clientRect);

	// Create the back surface initially here.  This may not be necessary, as it'll be
	// recreated when the window is sized before creation anyways, but it's better to
	// be on the safe side.
	ZeroMemory(&m_surfaceDesc, sizeof(m_surfaceDesc));
	m_surfaceDesc.dwSize = sizeof(m_surfaceDesc);
	m_surfaceDesc.dwFlags = DDSD_CAPS | DDSD_WIDTH | DDSD_HEIGHT;
	m_surfaceDesc.ddsCaps.dwCaps = DDSCAPS_OFFSCREENPLAIN;
	m_surfaceDesc.dwWidth = 2;
	m_surfaceDesc.dwHeight = 2;

	HRESULT	rval = DDraw.lpDD->CreateSurface(&m_surfaceDesc, &m_lpSurfaceBack, NULL);
	if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());

	rval = DDraw.lpDD->CreateClipper(0, &m_lpSurfaceClipper, NULL);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	rval = m_lpSurfaceBack->SetClipper(m_lpSurfaceClipper);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	ZeroMemory(&m_pixelFormat, sizeof(DDPIXELFORMAT));
	m_pixelFormat.dwSize = sizeof(DDPIXELFORMAT);
	rval = m_lpSurfaceBack->GetPixelFormat(&m_pixelFormat);
	if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());

	return 0;
}

HRESULT CDirectDrawView::Flip()
{
	CRect window;
	CPoint pt;

	GetWindowRect(&window);

	if(GetExStyle() & WS_EX_CLIENTEDGE) window.DeflateRect(2, 2, 2, 2);

	window.OffsetRect(m_rcUpdate.left, m_rcUpdate.top);
	window.right = window.left + m_rcUpdate.Width();
	window.bottom = window.top + m_rcUpdate.Height();

	HRESULT rval = DDraw.lpSurfFront->Blt(&window, m_lpSurfaceBack,
		CRect(0, 0, window.Width(), window.Height()), DDBLT_WAIT, NULL);
	if(rval == DDERR_SURFACELOST) RestoreSurface();

	return rval;
}

void CDirectDrawView::RestoreSurface()
{
	HRESULT hres = DDraw.lpSurfFront->Restore();

	if(hres == DDERR_WRONGMODE)
	{
		ReleaseObjects();
		RemakeSurfaces(0, 0);
	}
	else if(FAILED(hres))
	{
		AfxMessageBox("Could not recreate surfaces.");
		AfxGetMainWnd()->SendMessage(WM_CLOSE);
	}
	else
	{
		m_lpSurfaceBack->Restore();
	}
}

void CDirectDrawView::OnDestroy() 
{
	CView::OnDestroy();

	ReleaseObjects();
}

/////////////////////////////////////////////////////////////////////////////
// CDirectDrawView drawing helpers

CDC* CDirectDrawView::GetSurfaceDC()
{
	HDC hDC;
	HRESULT rval = m_lpSurfaceBack->GetDC(&hDC);
	if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());

	CDC* pDC = new CDC;  pDC->Attach(hDC);
	return pDC;
}

void CDirectDrawView::ReleaseSurfaceDC(CDC* pDC)
{
	HDC hDC = pDC->GetSafeHdc();
	pDC->Detach();

	HRESULT rval = m_lpSurfaceBack->ReleaseDC(hDC);
	if(rval != DD_OK) DDraw.Error(rval, GetSafeHwnd());

	delete pDC;
}

void CDirectDrawView::ClearSurface(COLORREF clr)
{
	CRect rect(0, 0, m_surfaceDesc.dwWidth, m_surfaceDesc.dwHeight);
	DDBLTFX ddbltfx;
	ddbltfx.dwSize = sizeof(ddbltfx);
	ddbltfx.dwFillColor = DDraw.ColorMatch(m_lpSurfaceBack, clr);

	HRESULT rval = m_lpSurfaceBack->Blt(&rect, NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL,
		&ddbltfx);
	if(rval == DDERR_SURFACELOST) RestoreSurface();
}

void CDirectDrawView::SetPixel(int x, int y, COLORREF clr)
{
	DDBLTFX ddbltfx;
	ddbltfx.dwSize = sizeof(ddbltfx);
	ddbltfx.dwFillColor = DDraw.ColorMatch(m_lpSurfaceBack, clr);

	HRESULT rval = m_lpSurfaceBack->Blt(CRect(x, y, x + 1, y + 1),
		NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL, &ddbltfx);
	if(rval == DDERR_SURFACELOST) RestoreSurface();
}

void CDirectDrawView::HorizontalLine(int xStart, int yStart, int xEnd, COLORREF clr)
{
	DDBLTFX ddbltfx;
	ddbltfx.dwSize = sizeof(ddbltfx);
	ddbltfx.dwFillColor = clr;//DDraw.ColorMatch(m_lpSurfaceBack, clr);

	HRESULT rval = m_lpSurfaceBack->Blt(CRect(xStart, yStart, xEnd, yStart + 1),
		NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL, &ddbltfx);
	if(rval == DDERR_SURFACELOST) RestoreSurface();
}

void CDirectDrawView::VerticalLine(int xStart, int yStart, int yEnd, COLORREF clr)
{
	DDBLTFX ddbltfx;
	ddbltfx.dwSize = sizeof(ddbltfx);
	ddbltfx.dwFillColor = clr;//DDraw.ColorMatch(m_lpSurfaceBack, clr);

	HRESULT rval = m_lpSurfaceBack->Blt(CRect(xStart, yStart, xStart + 1, yEnd),
		NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL, &ddbltfx);
	if(rval == DDERR_SURFACELOST) RestoreSurface();
}

void CDirectDrawView::Line(int xStart, int yStart, int xEnd, int yEnd, CPen* pPen)
{
	CDC* pDC = GetSurfaceDC();

	CPen* oldPen = pDC->SelectObject(pPen);
	pDC->MoveTo(xStart, yStart);
	pDC->LineTo(xEnd, yEnd);
	pDC->SelectObject(oldPen);

	ReleaseSurfaceDC(pDC);
}

void CDirectDrawView::Rectangle(LPRECT lpRect, CPen* pPen, CBrush* pBrush)
{
	CDC* pDC = GetSurfaceDC();

	CPen* oldPen = pDC->SelectObject(pPen);
	CBrush* oldBrush;
	if(pBrush) oldBrush = pDC->SelectObject(pBrush);
	else oldBrush = (CBrush*)pDC->SelectStockObject(NULL_BRUSH);
	pDC->Rectangle(lpRect);
	pDC->SelectObject(oldBrush);
	pDC->SelectObject(oldPen);

	ReleaseSurfaceDC(pDC);
}

void CDirectDrawView::Rectangle(int x, int y, int cx, int cy, CPen* pPen, CBrush* pBrush)
{
	CRect rect(x, y, cx, cy);
	Rectangle(&rect, pPen, pBrush);
}

void CDirectDrawView::FillSolidRect(LPRECT lpRect, COLORREF clr)
{
	DDBLTFX ddbltfx;
	ddbltfx.dwSize = sizeof(ddbltfx);
	ddbltfx.dwFillColor = clr; //DDraw.ColorMatch(m_lpSurfaceBack, clr);

	HRESULT rval = m_lpSurfaceBack->Blt(lpRect, NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL,
		&ddbltfx);
	if(rval == DDERR_SURFACELOST) RestoreSurface();
}

void CDirectDrawView::FillSolidRect(int x, int y, int cx, int cy, COLORREF clr)
{
	CRect rect(x, y, cx, cy);
	FillSolidRect(&rect, clr);
}

void CDirectDrawView::TextOut(int x, int y, LPCTSTR lpszString, int nCount, CFont* pFont,
							  COLORREF clr)
{
	CDC* pDC = GetSurfaceDC();

	CFont* oldFont = pDC->SelectObject(pFont);
	pDC->SetTextColor(clr);
	pDC->SetBkMode(TRANSPARENT);
	pDC->TextOut(x, y, lpszString, nCount);
	pDC->SelectObject(oldFont);

	ReleaseSurfaceDC(pDC);
}

void CDirectDrawView::TextOut(int x, int y, const CString& str, CFont* pFont, COLORREF clr)
{
	TextOut(x, y, str, str.GetLength(), pFont, clr);
}

void CDirectDrawView::ShadeRect(DDSURFACEDESC2* ddsd, int dx, int dy, int cx, int cy,
								DWORD dwShadeColor, char fPercent)
{
	if(DDraw.ddpfPrimary.dwRGBBitCount == 16)
	{
		DWORD* lpDest = (DWORD*)ddsd->lpSurface + (ddsd->lPitch / 4) * dy;

		for(int y = dy; y < dy + cy; y++)
		{
			for(int x = dx / 2; x < (dx + cx) / 2; x++)
			{
				DWORD dwColors = lpDest[x];

				WORD wDestColor1 = SubtractDDColor(HIWORD(dwColors), dwShadeColor, (100 - fPercent) / 100.0);
				WORD wDestColor2 = SubtractDDColor(LOWORD(dwColors), dwShadeColor, (100 - fPercent) / 100.0);

				lpDest[x] = MAKELONG(wDestColor2, wDestColor1);
			}

			lpDest += ddsd->lPitch / 4;
		}
	}
	else if(DDraw.ddpfPrimary.dwRGBBitCount == 24)
	{
		BYTE* lpDest = (BYTE*)ddsd->lpSurface + ddsd->lPitch * dy;

		for(int y = dy; y < dy + cy; y++)
		{
			int sx = dx * 3;

			for(int x = dx; x < dx + cx; x++)
			{
				DWORD dwColor = (BYTE)(lpDest[sx]) | ((BYTE)(lpDest[sx + 1]) << 8) | ((BYTE)(lpDest[sx + 2]) << 16);
				DWORD dwDestColor = SubtractDDColor(dwColor, dwShadeColor, (100 - fPercent) / 100.0);
				lpDest[sx + 2] = ((dwDestColor >> 16) & 0xFF);
				lpDest[sx + 1] = ((dwDestColor >> 8) & 0xFF);
				lpDest[sx] = (dwDestColor & 0xFF);

				sx += 3;
			}

			lpDest += ddsd->lPitch;
		}
	}
	else if(DDraw.ddpfPrimary.dwRGBBitCount == 32)
	{
		DWORD* lpDest = (DWORD*)ddsd->lpSurface + (ddsd->lPitch / 4) * dy;

		for(int y = dy; y < dy + cy; y++)
		{
			for(int x = dx; x < dx + cx; x++)
			{
				DWORD dwColor = lpDest[x];
				DWORD dwDestColor = SubtractDDColor(dwColor, dwShadeColor, (100 - fPercent) / 100);
				lpDest[x] = dwDestColor;
			}

			lpDest += ddsd->lPitch / 4;
		}
	}
}

void CDirectDrawView::OnPaint() 
{
	GetUpdateRect(&m_rcUpdate);
	
	m_rcUpdate.left -= m_rcUpdate.left % m_nRoundDown;
	m_rcUpdate.top -= m_rcUpdate.top % m_nRoundDown;
	m_rcUpdate.right += m_nRoundDown - m_rcUpdate.right % m_nRoundDown;
	m_rcUpdate.bottom += m_nRoundDown - m_rcUpdate.bottom % m_nRoundDown;

	CView::OnPaint();
}

void CDirectDrawView::SetScrollSizes(SIZE sizeTotal, const SIZE& sizePage)
{
	CRect rect;  GetClientRect(rect);

	SCROLLINFO si;

	// First the horizontal.
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_RANGE | SIF_PAGE;
	si.nMin = 0;
	si.nMax = sizeTotal.cx;
    si.nPage = rect.Width() / m_nRoundDown;
	SetScrollInfo(SB_HORZ, &si);

	// Now the vertical.
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_RANGE | SIF_PAGE;
	si.nMin = 0;
	si.nMax = sizeTotal.cy;
    si.nPage = rect.Height() / m_nRoundDown;
	SetScrollInfo(SB_VERT, &si);

	Invalidate();
}

void CDirectDrawView::ScrollToPosition(POINT pt)
{
	SCROLLINFO si;

	// First the horizontal.
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_POS;
	si.nPos = pt.x;
	SetScrollInfo(SB_HORZ, &si);

	// Now the vertical.
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_POS;
	si.nPos = pt.y;
	SetScrollInfo(SB_VERT, &si);

	Invalidate();
}

CPoint CDirectDrawView::GetScrollPosition()
{
	SCROLLINFO si;
	CPoint pt;

	// First the horizontal.
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_POS;
	GetScrollInfo(SB_HORZ, &si);
	pt.x = si.nPos;

	// Now the vertical.
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_POS;
	GetScrollInfo(SB_VERT, &si);
	pt.y = si.nPos;

	return pt;
}

void CDirectDrawView::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	if (pScrollBar != NULL && pScrollBar->SendChildNotifyLastMsg())
		return;     // eat it

	// ignore scroll bar msgs from other controls
	if (pScrollBar != GetScrollBarCtrl(SB_HORZ))
		return;

	OnScroll(MAKEWORD(nSBCode, -1), nPos);
}

void CDirectDrawView::OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
	if (pScrollBar != NULL && pScrollBar->SendChildNotifyLastMsg())
		return;     // eat it

	// ignore scroll bar msgs from other controls
	if (pScrollBar != GetScrollBarCtrl(SB_VERT))
		return;

	OnScroll(MAKEWORD(-1, nSBCode), nPos);
}

BOOL CDirectDrawView::OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll)
{
	SCROLLINFO si;

	// calc new x position
	int x = GetScrollPos(SB_HORZ);
	int xOrig = x;

	int xLimit, yLimit;

	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_PAGE | SIF_RANGE;
	GetScrollInfo(SB_HORZ, &si);
	xLimit = GetScrollLimit(SB_HORZ);

	switch (LOBYTE(nScrollCode))
	{
	case SB_TOP:
		x = 0;
		break;
	case SB_BOTTOM:
		x = INT_MAX;
		break;
	case SB_LINEUP:
		x -= 1;
		break;
	case SB_LINEDOWN:
		x += 1;
		break;
	case SB_PAGEUP:
		x -= si.nPage;
		break;
	case SB_PAGEDOWN:
		x += si.nPage;
		break;
	case SB_THUMBTRACK:
		x = nPos;
		break;
	}

	// calc new y position
	int y = GetScrollPos(SB_VERT);
	int yOrig = y;

	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_PAGE | SIF_RANGE;
	GetScrollInfo(SB_VERT, &si);
	yLimit = GetScrollLimit(SB_VERT);

	switch (HIBYTE(nScrollCode))
	{
	case SB_TOP:
		y = 0;
		break;
	case SB_BOTTOM:
		y = INT_MAX;
		break;
	case SB_LINEUP:
		y -= 1;
		break;
	case SB_LINEDOWN:
		y += 1;
		break;
	case SB_PAGEUP:
		y -= si.nPage;
		break;
	case SB_PAGEDOWN:
		y += si.nPage;
		break;
	case SB_THUMBTRACK:
		y = nPos;
		break;
	}

	if(x < 0) x = 0;
	else if(x > xLimit) x = xLimit;
	
	if(y < 0) y = 0;
	else if(y > yLimit) y = yLimit;

	if(x != xOrig || y != yOrig)
	{
		SetScrollPos(SB_HORZ, x);
		SetScrollPos(SB_VERT, y);

		ScrollWindow((xOrig - x) * m_nRoundDown, (yOrig - y) * m_nRoundDown);
		UpdateWindow();
	}

	return TRUE;
}

BOOL _afxDDGotScrollLines;

UINT PASCAL _AfxDDGetMouseScrollLines()
{
	static UINT uCachedScrollLines;

	// if we've already got it and we're not refreshing,
	// return what we've already got

	if (_afxDDGotScrollLines)
		return uCachedScrollLines;

	// see if we can find the mouse window

	_afxDDGotScrollLines = TRUE;

	static UINT msgGetScrollLines;
	static WORD nRegisteredMessage;

	if (nRegisteredMessage == 0)
	{
		msgGetScrollLines = ::RegisterWindowMessage(MSH_SCROLL_LINES);
		if (msgGetScrollLines == 0)
			nRegisteredMessage = 1;     // couldn't register!  never try again
		else
			nRegisteredMessage = 2;     // it worked: use it
	}

	if (nRegisteredMessage == 2)
	{
		HWND hwMouseWheel = NULL;
		hwMouseWheel = FindWindow(MSH_WHEELMODULE_CLASS, MSH_WHEELMODULE_TITLE);
		if (hwMouseWheel && msgGetScrollLines)
		{
			uCachedScrollLines = (UINT)
				::SendMessage(hwMouseWheel, msgGetScrollLines, 0, 0);
			return uCachedScrollLines;
		}
	}

	// couldn't use the window -- try system settings
	uCachedScrollLines = 3; // reasonable default
	if (!afxData.bWin4)
	{
		HKEY hKey;
		if (RegOpenKeyEx(HKEY_CURRENT_USER,  _T("Control Panel\\Desktop"),
				0, KEY_QUERY_VALUE, &hKey) == ERROR_SUCCESS)
		{
			TCHAR szData[128];
			DWORD dwKeyDataType;
			DWORD dwDataBufSize = _countof(szData);

			if (RegQueryValueEx(hKey, _T("WheelScrollLines"), NULL, &dwKeyDataType,
					(LPBYTE) &szData, &dwDataBufSize) == ERROR_SUCCESS)
			{
				uCachedScrollLines = _tcstoul(szData, NULL, 10);
			}
			RegCloseKey(hKey);
		}
	}
	else if (!afxData.bWin95)
	{
		::SystemParametersInfo(SPI_GETWHEELSCROLLLINES, 0, &uCachedScrollLines, 0);
	}

	return uCachedScrollLines;
}

BOOL CDirectDrawView::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
	// if we have a vertical scroll bar, the wheel scrolls that
	// if we have _only_ a horizontal scroll bar, the wheel scrolls that
	// otherwise, don't do any work at all

	DWORD dwStyle = GetStyle();
	CScrollBar* pBar = GetScrollBarCtrl(SB_VERT);
	BOOL bHasVertBar = ((pBar != NULL) && pBar->IsWindowEnabled()) ||
					(dwStyle & WS_VSCROLL);

	pBar = GetScrollBarCtrl(SB_HORZ);
	BOOL bHasHorzBar = ((pBar != NULL) && pBar->IsWindowEnabled()) ||
					(dwStyle & WS_HSCROLL);

	if (!bHasVertBar && !bHasHorzBar)
		return FALSE;

	BOOL bRedraw = FALSE;
	UINT uWheelScrollLines = _AfxDDGetMouseScrollLines();
	int nToScroll;
	int nDisplacement;

	SCROLLINFO si;

	if (bHasVertBar)
	{
		si.cbSize = sizeof(SCROLLINFO);
		si.fMask = SIF_PAGE | SIF_POS;
		GetScrollInfo(SB_VERT, &si);

		nToScroll = ::MulDiv(-zDelta, uWheelScrollLines, WHEEL_DELTA);
		if (nToScroll == -1 || uWheelScrollLines == WHEEL_PAGESCROLL)
		{
			nDisplacement = si.nPage;
			if (zDelta > 0)
				nDisplacement = -nDisplacement;
		}
		else
		{
			nDisplacement = nToScroll;
		}

		OnScroll(MAKEWORD(-1, SB_THUMBTRACK), si.nPos + nDisplacement);
	}
	else if (bHasHorzBar)
	{
		si.cbSize = sizeof(SCROLLINFO);
		si.fMask = SIF_PAGE;
		GetScrollInfo(SB_HORZ, &si);

		nToScroll = ::MulDiv(-zDelta, uWheelScrollLines, WHEEL_DELTA);
		if (nToScroll == -1 || uWheelScrollLines == WHEEL_PAGESCROLL)
		{
			nDisplacement = si.nPage;
			if (zDelta > 0)
				nDisplacement = -nDisplacement;
		}
		else
		{
			nDisplacement = nToScroll;
		}

		OnScroll(MAKEWORD(SB_THUMBTRACK, -1), si.nPos + nDisplacement);
	}

	return CView::OnMouseWheel(nFlags, zDelta, pt);
}

void CDirectDrawView::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	WORD wHScrollNotify = 0xFFFF;
	WORD wVScrollNotify = 0xFFFF;

	switch(nChar) 
	{ 
		case VK_UP: 
			if(::GetKeyState(VK_CONTROL) & 0x8000)
				wVScrollNotify = SB_PAGEUP;
			else
				wVScrollNotify = SB_LINEUP;
			break; 
 
		case VK_DOWN: 
			if(::GetKeyState(VK_CONTROL) & 0x8000)
				wVScrollNotify = SB_PAGEDOWN;
			else
				wVScrollNotify = SB_LINEDOWN;
			break; 
 
		case VK_PRIOR: 
			wVScrollNotify = SB_PAGEUP; 
			break; 
 
		case VK_NEXT: 
			wVScrollNotify = SB_PAGEDOWN; 
			break; 

		case VK_HOME: 
			wVScrollNotify = SB_TOP; 
			wHScrollNotify = SB_LEFT;
			break; 
 
		case VK_END: 
			wVScrollNotify = SB_BOTTOM; 
			wHScrollNotify = SB_RIGHT; 
			break; 

		case VK_LEFT: 
			if(::GetKeyState(VK_CONTROL) & 0x8000)
				wHScrollNotify = SB_PAGELEFT;
			else
				wHScrollNotify = SB_LINELEFT;
			break; 
 
		case VK_RIGHT: 
			if(::GetKeyState(VK_CONTROL) & 0x8000)
				wHScrollNotify = SB_PAGERIGHT;
			else
				wHScrollNotify = SB_LINERIGHT;
			break; 
	}
 
	if(wVScrollNotify != 0xFFFF) 
		SendMessage(WM_VSCROLL, MAKELONG(wVScrollNotify, 0), 0L);

	if(wHScrollNotify != 0xFFFF) 
		SendMessage(WM_HSCROLL, MAKELONG(wHScrollNotify, 0), 0L);

	CView::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CDirectDrawView::OnMButtonDown(UINT nFlags, CPoint point) 
{
	BOOL bCtl = GetKeyState(VK_CONTROL) & 0x8000;

	if(!bCtl && nFlags == MK_MBUTTON)
		MfxTrackAutoPan(this);
	else
		CView::OnMButtonDown(nFlags, point);
}
