// DirectDrawView.h : header file
//

/////////////////////////////////////////////////////////////////////////////
// CDirectDrawView view

#ifndef __DIRECTDRAWVIEW_H__
#define __DIRECTDRAWVIEW_H__

#include "FF3Data.h"

class CDirectDrawView : public CView
{
typedef struct 
{
	RGNDATAHEADER hdr;
	RECT rgndata[4];
} CLIPLIST, *LPCLIPLIST;

protected:
	CDirectDrawView();           // protected constructor used by dynamic creation
	DECLARE_DYNCREATE(CDirectDrawView)

	LPDIRECTDRAWSURFACE4 m_lpSurfaceBack;
	LPDIRECTDRAWCLIPPER m_lpSurfaceClipper;
	DDSURFACEDESC2 m_surfaceDesc;
	CLIPLIST m_clipList;
	DDPIXELFORMAT m_pixelFormat;

	void RemakeSurfaces(int cx, int cy);
	void ReleaseObjects();
	void RestoreSurface();
	HRESULT Flip();

	void BeginDraw(BOOL bErase = TRUE);
	void EndDraw();

	void SetScrollSizes(SIZE sizeTotal, const SIZE& sizePage);
	void ScrollToPosition(POINT pt);
	CPoint GetScrollPosition();

// Attributes
public:

// Operations
public:
protected:
	CDC* m_pDC;
	CRect m_rcUpdate;
	int m_nRoundDown;

	CDC* GetSurfaceDC();
	void ReleaseSurfaceDC(CDC* pDC);

	void BeginGDI();
	void EndGDI();

	void ClearSurface(COLORREF clr);
	void SetPixel(int x, int y, COLORREF clr);
	void HorizontalLine(int xStart, int yStart, int xEnd, COLORREF clr);
	void VerticalLine(int xStart, int yStart, int yEnd, COLORREF clr);
	void Line(int xStart, int yStart, int xEnd, int yEnd, CPen* pPen);
	void FillSolidRect(LPRECT lpRect, COLORREF clr);
	void FillSolidRect(int x, int y, int cx, int cy, COLORREF clr);
	void Rectangle(LPRECT lpRect, CPen* pPen, CBrush* pBrush = NULL);
	void Rectangle(int x, int y, int cx, int cy, CPen* pPen, CBrush* pBrush = NULL);
	void TextOut(int x, int y, LPCTSTR lpszString, int nCount, CFont* pFont, COLORREF clr);
	void TextOut(int x, int y, const CString& str, CFont* pFont, COLORREF clr);

	void ShadeRect(DDSURFACEDESC2* ddsd, int dx, int dy, int cx, int cy, DWORD wShadeColor, char fPercent);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDirectDrawView)
	protected:
	virtual void OnDraw(CDC* pDC);      // overridden to draw this view
	//}}AFX_VIRTUAL

// Implementation
protected:
	virtual ~CDirectDrawView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

	// Generated message map functions
	//{{AFX_MSG(CDirectDrawView)
	afx_msg BOOL OnEraseBkgnd(CDC* pDC) { return FALSE; };
	afx_msg void OnSize(UINT nType, int cx, int cy);
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDestroy();
	afx_msg void OnPaint();
	afx_msg void OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg void OnVScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar);
	afx_msg BOOL OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll = TRUE);
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg BOOL OnMouseWheel(UINT nFlags, short zDelta, CPoint pt);
	afx_msg void OnMButtonDown(UINT nFlags, CPoint point);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

/////////////////////////////////////////////////////////////////////////////

#endif