// EsperEditView.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "EsperEditView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CEsperEditView

IMPLEMENT_DYNCREATE(CEsperEditView, CSizingFormView)

CEsperEditView::CEsperEditView()
	: CSizingFormView(CEsperEditView::IDD)
{
	//{{AFX_DATA_INIT(CEsperEditView)
	//}}AFX_DATA_INIT
}

CEsperEditView::~CEsperEditView()
{
}

void CEsperEditView::DoDataExchange(CDataExchange* pDX)
{
	CSizingFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEsperEditView)
	DDX_Control(pDX, IDC_PROPS, m_wndProps);
	DDX_Control(pDX, IDC_LIST, m_wndList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CEsperEditView, CSizingFormView)
	//{{AFX_MSG_MAP(CEsperEditView)
	ON_LBN_SELCHANGE(IDC_LIST, OnSelChangeItemList)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
	ON_NOTIFY(PROPTN_PROPCHANGED, IDC_PROPS, OnChangeProperties)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CEsperEditView diagnostics

#ifdef _DEBUG
void CEsperEditView::AssertValid() const
{
	CSizingFormView::AssertValid();
}

void CEsperEditView::Dump(CDumpContext& dc) const
{
	CSizingFormView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CEsperEditView message handlers

int CEsperEditView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CSizingFormView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	SetControlInfo(IDC_MINBOUNDS, MINIMUM_BOUND);
	SetControlInfo(IDC_LIST, RESIZE_VERT);
	SetControlInfo(IDC_PROPS, RESIZE_HORZ | RESIZE_VERT);
	
	return 0;
}

void CEsperEditView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();

	m_wndProps.m_pDoc = pDoc;

	m_wndList.ResetContent();
	m_wndList.m_pDoc = pDoc;
	m_wndList.m_dwListStyle = FF3LBS_NOCHANGE;

	m_esperArray.RemoveAll();
	m_magicArray.RemoveAll();

	for(int i = 0; i < 27; i++)
		m_esperArray.Add(pDoc->espers[i].name);

	m_wndList.m_pArray = &m_esperArray;

	for(i = 0; i < 27; i++)
	{
		m_wndList.AddItem(i, 8);
	}

	for(i = 0; i < 0x36; i++)
		m_magicArray.Add(pDoc->magic[i].name);

	m_wndList.SetCurSel(0);
	UpdateTree();

}

PROPERTYITEM g_strEsperBonuses[] =
{
	{ 0, "HP + 10%", NULL },
	{ 1, "HP + 30%", NULL },
	{ 2, "HP + 50%", NULL },
	{ 3, "MP + 10%", NULL },
	{ 4, "MP + 30%", NULL },
	{ 5, "MP + 50%", NULL },
	{ 6, "HP + 100%", NULL },
	{ 7, "LV + 30%", NULL },
	{ 8, "LV + 50%", NULL },
	{ 9, "Strength + 1", NULL },
	{ 10, "Strength + 2", NULL },
	{ 11, "Speed + 1", NULL },
	{ 12, "Speed + 2", NULL },
	{ 13, "Stamina + 1", NULL },
	{ 14, "Stamina + 2", NULL },
	{ 15, "Mag. Pwr. + 1", NULL },
	{ 16, "Mag. Pwr. + 2", NULL },
	{ 0xFF, "No bonus", NULL },
	{ -1, NULL, NULL },
};

void CEsperEditView::UpdateTree()
{
	int nSel = m_wndList.GetCurSel();

	m_wndProps.SetRedraw(FALSE);

	// Store the expanded nodes.
	LRESULT bExpanded;
	bExpanded = m_wndProps.IsItemExpanded(1000);  if(bExpanded != 0x80000001) m_expanded[1000] = bExpanded; 
	bExpanded = m_wndProps.IsItemExpanded(2000);  if(bExpanded != 0x80000001) m_expanded[2000] = bExpanded; 

	// Update the property tree.
	m_wndProps.DeleteAllItems();
	
	if(nSel == -1)
	{
		m_wndProps.SetRedraw(TRUE);
		m_wndProps.Invalidate();
		return;
	}

	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();
	BYTE* pProps = pDoc->items[nSel].props;

	m_wndProps.SetTrueFalseText(_T("Yes"), _T("No"));

	m_wndProps.AddGroup(1000, "General");
		m_wndProps.AddItem(1000, 1001, "Name", PROPT_FF3EDIT, 8, (LPARAM)&pDoc->espers[nSel].name);
		m_wndProps.AddItem(1000, 1002, "Level Up Bonus", PROPT_COMBO, 0, (LPARAM)&g_strEsperBonuses, (LPARAM)pDoc->espers[nSel].bonus);

	m_wndProps.AddGroup(2000, "Spells");
		m_wndProps.AddItem(2000, 2001, "Spell 1", PROPT_FF3COMBO, 0x10000 | 7, (LPARAM)&m_magicArray, (LPARAM)pDoc->espers[nSel].spell1);
		m_wndProps.AddItem(2000, 2002, "Spell 1 Learn Rate", PROPT_NUMEDIT, 0, (LPARAM)pDoc->espers[nSel].spell1Rate);
		m_wndProps.AddItem(2000, 2003, "Spell 2", PROPT_FF3COMBO, 0x10000 | 7, (LPARAM)&m_magicArray, (LPARAM)pDoc->espers[nSel].spell2);
		m_wndProps.AddItem(2000, 2004, "Spell 2 Learn Rate", PROPT_NUMEDIT, 0, (LPARAM)pDoc->espers[nSel].spell2Rate);
		m_wndProps.AddItem(2000, 2005, "Spell 3", PROPT_FF3COMBO, 0x10000 | 7, (LPARAM)&m_magicArray, (LPARAM)pDoc->espers[nSel].spell3);
		m_wndProps.AddItem(2000, 2006, "Spell 3 Learn Rate", PROPT_NUMEDIT, 0, (LPARAM)pDoc->espers[nSel].spell3Rate);
		m_wndProps.AddItem(2000, 2007, "Spell 4", PROPT_FF3COMBO, 0x10000 | 7, (LPARAM)&m_magicArray, (LPARAM)pDoc->espers[nSel].spell4);
		m_wndProps.AddItem(2000, 2008, "Spell 4 Learn Rate", PROPT_NUMEDIT, 0, (LPARAM)pDoc->espers[nSel].spell4Rate);
		m_wndProps.AddItem(2000, 2009, "Spell 5", PROPT_FF3COMBO, 0x10000 | 7, (LPARAM)&m_magicArray, (LPARAM)pDoc->espers[nSel].spell5);
		m_wndProps.AddItem(2000, 2010, "Spell 5 Learn Rate", PROPT_NUMEDIT, 0, (LPARAM)pDoc->espers[nSel].spell5Rate);

	// Re-expand the nodes by expanded them all and then closing the ones
	// that should remain closed.
	m_wndProps.ExpandAll();

	UINT nID;  BOOL bExpand;
	POSITION pos = m_expanded.GetStartPosition(); 
	while(pos != NULL)
	{
		m_expanded.GetNextAssoc(pos, nID, bExpand);

		if(bExpand == FALSE)
			m_wndProps.Expand(m_wndProps.IndexToItem(nID), TVE_COLLAPSE);
	}

	m_wndProps.SetRedraw(TRUE);
	m_wndProps.Invalidate();
}

void CEsperEditView::OnSelChangeItemList() 
{
	UpdateTree();
}

void CEsperEditView::OnChangeProperties(NMHDR* pNMHDR, LRESULT* pResult) 
{
	int nSel = m_wndList.GetCurSel();

	NMPROPTREE* pNMPropTree = (NMPROPTREE*)pNMHDR;
	LPPROPERTYNODE node = pNMPropTree->node;

	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();

	if(node->id == 1001)
	{
		CRect rect;
		m_wndList.GetItemRect(nSel, &rect);
		m_wndList.InvalidateRect(&rect);
	}
	else if(node->id == 1002) pDoc->espers[nSel].bonus = (BYTE)node->lData2;
	else if(node->id == 2001) pDoc->espers[nSel].spell1 = (BYTE)node->lData2;
	else if(node->id == 2002) pDoc->espers[nSel].spell1Rate = (BYTE)node->lData;
	else if(node->id == 2003) pDoc->espers[nSel].spell2 = (BYTE)node->lData2;
	else if(node->id == 2004) pDoc->espers[nSel].spell2Rate = (BYTE)node->lData;
	else if(node->id == 2005) pDoc->espers[nSel].spell3 = (BYTE)node->lData2;
	else if(node->id == 2006) pDoc->espers[nSel].spell3Rate = (BYTE)node->lData;
	else if(node->id == 2007) pDoc->espers[nSel].spell4 = (BYTE)node->lData2;
	else if(node->id == 2008) pDoc->espers[nSel].spell4Rate = (BYTE)node->lData;
	else if(node->id == 2009) pDoc->espers[nSel].spell5 = (BYTE)node->lData2;
	else if(node->id == 2010) pDoc->espers[nSel].spell5Rate = (BYTE)node->lData;

	*pResult = 0;
}
