#ifndef __FF4DATA_H__
#define __FF4DATA_H__

#include "FF3Edit.h"

CString FixedTextToCString(BYTE* pText, int nLength, DWORD tag, DWORD value, ... );
BYTE ASCIIToFixedChar(char ch);
char VariableCharToASCII(BYTE fixed);
char FixedCharToASCII(BYTE fixed);

#define TTCS_BASE					(0x81000000)
#define TTCS_TrimTrailingSpaces		(TTCS_BASE + 1)

///////////////////////////////////////////////////////////

class CFF3Doc;
class WorldData;

enum ObjectTypes
{
	ObjWorldData = 1,
	ObjLocationData
};

class FF3Object
{
public:
	FF3Object() { modified = FALSE; }
	int objectType;

	BOOL modified;
};

class EventCommand
{
public:
	BYTE command;
	BOOL queueCommand;
	BYTE paramCount;
	BYTE params[24];
};

class DialogueItem
{
public:
	DialogueItem();
	~DialogueItem();

	int length;
	BYTE* text;
	
	void ConvertToASCII(CString& str, DWORD tag, DWORD value, ... );
	DWORD ConvertFromASCII(CString str, DWORD tag, DWORD value, ... );
};

// DialogueItem::ConvertToASCII tags
#define DI_Base					(0x83000000)
#define DI_ConvertMode			(DI_Base + 1)
#define DI_Offset				(DI_Base + 2)

// DI_ConvertMode
#define DICM_List				0x00000001
#define DICM_Edit				0x00000002

// DialogueItem::ConvertFromASCII errors
#define DICERROR_Malformed		-1

/*
class Event
{
public:
	Event();
	~Event();

	WORD size;
	CTypedPtrArray<CPtrArray, EventCommand*> code;
};

class LocationTilePart
{
public:
	BYTE tilePart;
	BYTE palette;
};
*/

enum TriggerTypes
{
	triggerInvalid = 0,
	triggerEntrance,
	triggerEvent,
};

#define TRIGGER_MINIMAP		0x00000001

class Trigger
{
public:
	TriggerTypes type;
	DWORD flags;
	BYTE srcX;
	BYTE srcY;
};

class EntranceTrigger : public Trigger
{
public:
	WORD destination;
	BYTE destX;
	BYTE destY;
};

class EventTrigger : public Trigger
{
public:
	DWORD event;
};

/*
class GlobalLocationData : public FF3Object
{
public:
	GlobalLocationData();
	~GlobalLocationData();

	CFile* pFile;

	BYTE* tiles[16];
	BYTE* maps[256];
	CSize mapSize;
	WORD palettes[32][64];
	LocationTilePart tileFormations[16][512];
	WORD tileProperties[16][128];
};

class LocationData : public FF3Object
{
public:
	LocationData();
	~LocationData();

	CFF4Doc* pDoc;
	CFile* pFile;
	GlobalLocationData* gLocationData;
	CString name;

	BYTE battleBackground;
	BYTE mapIndex;
	BYTE tileSet;
	BYTE NPCIndex;
	BYTE borderTile;
	BYTE paletteIndex;
	BYTE NPCPaletteIndex;
	BYTE musicIndex;
	BYTE backgroundTile;
	BYTE scrollSpecs;
	BYTE UK2;
	BYTE titleIndex;
	BYTE UK3;

	CTypedPtrArray<CPtrArray, Entrance*> entrances;

	Entrance* FindEntranceFromSource(BYTE x, BYTE y);

	BYTE GetMapTile(int x, int y) { return gLocationData->maps[mapIndex][y * gLocationData->mapSize.cx + x]; }
	void SetMapTile(int x, int y, BYTE bTile) { gLocationData->maps[mapIndex][y * gLocationData->mapSize.cx + x] = bTile; }

	// Program persistent data
	CPoint ppdLastScrollPos;
};

#define LD_BATTLEBACKGROUND		0x0F
#define LD_BATTLEALTPALETTE		0x40
#define LD_BATTLEMAGNETIC		0x80

#define SSF_TRANSPARENT			0x01
#define SSF_VERTSTICKY			0x02
#define SSF_HORZSTICKY			0x04
#define SSF_DIRECTION			0x30
#define SSF_SPEED				0xC0

#define DIRECTION_UP			0x00
#define DIRECTION_RIGHT			0x01
#define DIRECTION_DOWN			0x02
#define DIRECTION_LEFT			0x03

#define DIRECTION_BACKWARD		DIRECTION_UP
#define DIRECTION_FORWARD		DIRECTION_DOWN

#define LDF_WALK				0x0001
#define LDF_UNKNOWN1			0x0006
#define LDF_SAVEPOINT			0x0008
#define LDF_UNKNOWN2			0x00F0

#define LDF_WALKDAMAGE			0x0100
#define LDF_UNKNOWN3			0x0200
#define LDF_HIDECHARACTER		0x0400
#define LDF_TOPHALFCHARACTER	0x0800
#define LDF_EXIT				0x1000
#define LDF_UNKNOWN4			0x6000
#define LDF_ENTRANCE			0x8000
*/
#define MAP_BALANCE 0
#define MAP_RUIN 1
#define MAP_TRENCH 2

class WorldData : public FF3Object
{
public:
	WorldData();
	~WorldData();

public:
	CFF3Doc* pDoc;

	CString name;
	CSize size;
	BYTE* map;
	BYTE* tiles;
	BYTE* miniMap;
	BYTE tileFormations[1024];
	WORD tileProperties[256];
	BYTE tilePalettes[256];
	WORD palette[128];

//	BYTE enemyLayout[64];
//	BYTE enemyOdds[64];

	CTypedPtrArray<CPtrArray, Trigger*> triggers;

	BOOL ReadPalettes(DWORD dwOffset);
	BOOL ReadMapData(DWORD dwOffset);
	BOOL ReadMiniMapData(DWORD dwOffset);
	BOOL ReadTileData(DWORD dwOffset);
	BOOL ReadTileProperties(DWORD dwOffset);
	BOOL ReadTriggers(DWORD dwEntranceOffset, DWORD dwEventOffset);
/*	BOOL ReadEnemyLayout(DWORD dwOffset, DWORD dwLength);
	BOOL ReadEnemyOdds(DWORD dwOffset, DWORD dwLength);
*/
	BOOL StorePalettes(DWORD dwOffset);
	BOOL StoreMapData(DWORD dwDest, DWORD dwOrigSize);
	BOOL StoreMiniMapData(DWORD dwDest, DWORD dwOrigSize);
	BOOL StoreTileData(DWORD dwOffset, DWORD dwOrigSize);
	BOOL StoreTileProperties(DWORD dwOffset);
/*	BOOL StoreEntrances(DWORD dwOffset, BOOL bExpand);
	BOOL StoreEnemyLayout(DWORD dwOffset, DWORD dwLength);
	BOOL StoreEnemyOdds(DWORD dwOffset, DWORD dwLength);
*/
	CDWordArray mapPtrs;
	CDWordArray miniMapPtrs;
	CDWordArray tilePtrs;
	CDWordArray triggerPtrs;

	Trigger* GetTrigger(BYTE x, BYTE y);

	inline BYTE GetMapTile(int x, int y) { return map[y * size.cx + x]; }
	inline void SetMapTile(int x, int y, BYTE bTile) { modified = TRUE; map[y * size.cx + x] = bTile; }

	// Program persistent data
	CPoint ppdLastScrollPos;
};

#define WDF_CHOCODISABLE		0x0001
#define WDF_AIRSHIPDISABLE		0x0002
#define WDF_AIRSHIPSHADOW		0x000C
#define WDF_WALKDISABLE			0x0010
#define WDF_CHARACTERTRANS		0x0020
#define WDF_ENEMYENCOUNTERS		0x0040
#define WDF_UNKNOWN1			0x0080
#define WDF_BATTLEBACKGROUND	0x0700
#define WDF_UNKNOWN2			0x0800
#define WDF_UNKNOWN3			0x1000
#define WDF_ALTERNATEMUSIC		0x2000
#define WDF_UNKNOWN5			0x4000
#define WDF_UNKNOWN6			0x8000

class Character
{
public:
	BYTE initialHP;
	BYTE initialMP;
	BYTE battleCmds[4];
	BYTE vigor;
	BYTE speed;
	BYTE stamina;
	BYTE magicPower;
	BYTE battlePower;
	BYTE defense;
	BYTE magicDefense;
	BYTE evade;
	BYTE magicBlock;
	BYTE weapon;
	BYTE shield;
	BYTE hat;
	BYTE armor;
	BYTE relic1;
	BYTE relic2;
	BYTE unknown;

	BYTE name[6];

	BYTE gfx[0x16A0];
	BYTE portrait[0x80];
	BYTE palette;
};

class CharacterGraphic
{
public:
	WORD offset[6];
};

class Item
{
public:
	BYTE name[13];
	CString description;

	BYTE props[30];		// Since parameters vary per item type
};

class Esper
{
public:
	BYTE name[8];

	BYTE spell1Rate;
	BYTE spell1;
	BYTE spell2Rate;
	BYTE spell2;
	BYTE spell3Rate;
	BYTE spell3;
	BYTE spell4Rate;
	BYTE spell4;
	BYTE spell5Rate;
	BYTE spell5;
	BYTE bonus;
};

class Magic
{
public:
	BYTE name[12];

	BYTE target;
	BYTE elemProps;
	BYTE effect1;
	BYTE damageType;
	BYTE effect2;
	BYTE mp;
	BYTE power;
	BYTE unknown1;
	BYTE successRate;
	BYTE special;
	BYTE cond1, cond2, cond3, cond4;
};

class BattleCommand
{
public:
	BYTE name[7];
};

class ShopData
{
public:
	BYTE title;
	BYTE items[8];
};

/////////////////////////////////////////////////////////////////////
//
//  MonsterFormation class
//  Stores data about monster formations and positioning.
//
class MonsterFormation
{
protected:
	BYTE sizeTemplate;
	BYTE monstersPresent;
	BYTE monsters[6];
	BYTE monsterPositions[6];
	BYTE bosses;

public:
// Auxiliary data
	BYTE aux[4];

	inline BOOL IsPresent(int nMonst) { return (monstersPresent & (1 << nMonst)) ? TRUE : FALSE; }
	inline BOOL IsBoss(int nMonst) { return (bosses & (1 << nMonst)) ? TRUE : FALSE; }

	inline void SetPresent(int nMonst, BOOL bSet) { monstersPresent = monstersPresent & ~(1 << nMonst) | (bSet ? (1 << nMonst) : 0x00); }

	inline int GetMonster(int nMonst)
	{
		if(IsBoss(nMonst)) return monsters[nMonst] + 256;
		else return monsters[nMonst];
	}

	inline void SetMonster(int nMonst, int bMonst)
	{
		if(bMonst >= 256)
		{
			monsters[nMonst] = bMonst & 0xFF;
			bosses |= (1 << nMonst);
		}
		else
		{
			monsters[nMonst] = bMonst;
			bosses &= ~(1 << nMonst);
		}
	}

	inline int GetAppearance() { return aux[0] & 0x0F; }
	inline void SetAppearance(int nAppr) { aux[0] = (aux[0] & ~0x0F) | nAppr; }

	inline int GetSizeTemplate() { return sizeTemplate >> 4; }
	inline void SetSizeTemplate(int nTempl) { sizeTemplate = nTempl << 4; }

	inline int GetXPos(int nMonster) { return (monsterPositions[nMonster] & 0xF0) >> 4; }
	inline int GetYPos(int nMonster) { return monsterPositions[nMonster] & 0x0F; }

	inline void SetXPos(int nMonster, int nPos) { monsterPositions[nMonster] = (monsterPositions[nMonster] & 0x0F) | (nPos << 4); }
	inline void SetYPos(int nMonster, int nPos) { monsterPositions[nMonster] = (monsterPositions[nMonster] & 0xF0) | nPos; }
};

class MonsterTemplate
{
public:
	WORD unknown;
	BYTE width;
	BYTE height;

	inline int GetWidth() { return width; }
	inline int GetHeight() { return height; }

	inline void SetWidth(int n) { width = n; }
	inline void SetHeight(int n) { height = n; }

	inline WORD GetUnknown() { return unknown; }
	inline void SetUnknown(WORD w) { unknown = w; }
};

class MonsterVisuals
{
public:
	WORD gfx;
	BYTE extra;
	BYTE palette;
	BYTE composition;

	inline DWORD GetGfx() { return gfx & 0x7FFF; }
	inline void SetGfx(WORD wGfx, BOOL is3BP) { gfx = (is3BP ? 0x8000 : 0x0000) | wGfx; }

	inline BOOL Is3Bitplane() { return (gfx & 0x8000) ? TRUE : FALSE; }

	inline int GetPalette() { return palette; }
	inline void SetPalette(int nPalette) { palette = nPalette; }

	inline int GetComposition() { return composition; }
	inline void SetComposition(int nComp) { composition = nComp; }
};

class MonsterComposition
{
public:
	BYTE length;
	BYTE mask[32];
};

class Monster
{
public:
	BYTE name[10];
	BYTE attackName[10];

	BYTE itemsStolen[2];
	BYTE itemsDropped[2];

	BYTE strength;
	BYTE speed;
	BYTE hitRate;
	BYTE evadeRate;
	BYTE magBlockRate;
	BYTE defense;
	BYTE magicDefense;
	BYTE magicPower;
	WORD hp;
	WORD mp;
	WORD exp;
	WORD gold;
	BYTE level;
	BYTE morph;
	BYTE flags1;
	BYTE flags2;
	BYTE unknown1;
	BYTE unknown2;
	BYTE unknown3;
	BYTE elemAbsorb;
	BYTE elemNoEffect;
	BYTE elemWeak;
	BYTE attackType;
	BYTE conditions[4];
	BYTE specialAttack;

	int GetMorphPackage() { return morph & 0x1F; }
	int GetMorphSuccess() { return (morph & 0xE0) >> 5; }
};

#endif