// FF3Doc.cpp : implementation of the CFF3Doc class
//

#include "stdafx.h"
#include "FF3Edit.h"

#include "FF3Doc.h"
#include "MainFrame.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFF3Doc

IMPLEMENT_DYNCREATE(CFF3Doc, CDocument)

BEGIN_MESSAGE_MAP(CFF3Doc, CDocument)
	//{{AFX_MSG_MAP(CFF3Doc)
	ON_UPDATE_COMMAND_UI(ID_FILE_SAVE, OnUpdateFileSave)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFF3Doc construction/destruction

CFF3Doc::CFF3Doc()
{
	worldData = NULL;
}

CFF3Doc::~CFF3Doc()
{
}

BOOL CFF3Doc::OnNewDocument()
{
	if (!CDocument::OnNewDocument())
		return FALSE;

	// TODO: add reinitialization code here
	// (SDI documents will reuse this document)

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CFF3Doc serialization

void CFF3Doc::Serialize(CArchive& ar)
{
	if (ar.IsStoring())
	{
		// TODO: add storing code here
	}
	else
	{
		// TODO: add loading code here
	}
}

/////////////////////////////////////////////////////////////////////////////
// CFF3Doc diagnostics

#ifdef _DEBUG
void CFF3Doc::AssertValid() const
{
	CDocument::AssertValid();
}

void CFF3Doc::Dump(CDumpContext& dc) const
{
	CDocument::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CFF3Doc commands

DWORD CFF3Doc::GetPointer(CFile* pFile, DWORD dwOffset)
{
	DWORD dwPtr = 0;

	dwOffset = HiROMToSMC(dwOffset);

	pFile->Seek(dwOffset, CFile::begin);
	pFile->Read(&dwPtr, 3);

	return dwPtr;
}

void CFF3Doc::DeleteContents() 
{
	m_bValid = FALSE;
	
	if(worldData) { delete [] worldData; worldData = NULL; }

	for(int i = 0; i < dialogue.GetSize(); i++)
		delete dialogue[i];
	dialogue.RemoveAll();

//	for(i = 0; i < expRanges.GetSize(); i++)
//		delete expRanges[i];
//	expRanges.RemoveAll();

	CDocument::DeleteContents();
}

// ROM Offsets
// Key:
//		FF3_xxx			A standard offset
//		FF3_P_xxx		A pointer, uses GetPointer() to retrieve the value
//		FF3_PF_xxx		A function pointer, takes a parameter and calculates a pointer

DWORD FF3_ROMTitle				= 0xC0FFC0;
DWORD FF3_ROMVersion			= 0xC0FFDB;

DWORD FF3_EventCode				= 0xCA0000;

DWORD FF3_DialoguePtrs			= 0xCCE602;
DWORD FF3_Dialogue				= 0xCD0000;

DWORD FF3_WoB_Palettes			= 0xD2EC00;
#define FF3_P_WoB_MapData		GetPointer(m_pFile, 0xEEB20F)
#define FF3_P_WoB_MiniMapData	GetPointer(m_pFile, 0xEEB24B)
#define FF3_P_WoB_TileData		GetPointer(m_pFile, 0xEEB212)
DWORD FF3_WoB_TileProperties	= 0xEE9B14;

DWORD FF3_WoR_Palettes			= 0xD2ED00;
#define FF3_P_WoR_MapData		GetPointer(m_pFile, 0xEEB221)
#define FF3_P_WoR_MiniMapData	GetPointer(m_pFile, 0xEEB24E)
#define FF3_P_WoR_TileData		GetPointer(m_pFile, 0xEEB21E)
DWORD FF3_WoR_TileProperties	= 0xEE9D14;

DWORD FF3_Entrances				= 0xDFBB00;
DWORD FF3_EventTriggers			= 0xC40000;
DWORD FF3_LocationData			= 0xED8F00;

#define FF3_PF_Entrances(x)		(FF3_Entrances + x * 2)
#define FF3_PF_EventTriggers(x)	(FF3_EventTriggers + x * 2)
#define FF3_PF_Location(x)		(FF3_LocationData + x * 33)

// Changes made in the ROM
#define FF3C_DialogueRoutine	0x01, 0x01				// Dialogue routine replaced

BOOL CFF3Doc::OnOpenDocument(LPCTSTR lpszPathName) 
{
	if(IsModified())
		TRACE0("Warning: OnOpenDocument replaces an unsaved document.\n");

	CFileException fe;
	m_pFile = GetFile(lpszPathName,
		CFile::modeRead|CFile::shareDenyWrite, &fe);
	if (m_pFile == NULL)
	{
		ReportSaveLoadException(lpszPathName, &fe,
			FALSE, AFX_IDP_FAILED_TO_OPEN_DOC);

		m_bValid = FALSE;
		return FALSE;
	}

	DeleteContents();
	SetModifiedFlag();  // dirty during de-serialize

	CWaitCursor wait;
	if(m_pFile->GetLength() == 0)
	{
		ReportSaveLoadException(lpszPathName, &fe,
			FALSE, AFX_IDP_FAILED_TO_OPEN_DOC);

		m_bValid = FALSE;
		return FALSE;
	}

	// Load the changes stored in the header into memory.
	LoadChanges();

	// Check to see if the file is valid (ie, the American FF3 ROM and version 1.0).
	char szV10Title[22] = "FINAL FANTASY 3      ";
	char szROMTitle[22];
	Seek(FF3_ROMTitle, CFile::begin);
	Read(szROMTitle, 21);
	
	if(memcmp(szV10Title, szROMTitle, 21) != 0)
	{
		AfxMessageBox("The ROM you are attempting to load\nis not the American FF3 ROM.");
		m_bValid = FALSE;
		return FALSE;
	}

	BYTE bVersion = 0;
	Seek(FF3_ROMVersion, CFile::begin);
	Read(&bVersion, 1);
	
	if(bVersion != 0)
	{
		AfxMessageBox("The ROM you are attempting to load\nis not version 1.0 of the American\nFF3 ROM.  Let me know if you see this, because\nI was not aware of other versions of\nFF3 being available.");
		m_bValid = FALSE;
		return FALSE;
	}
	
	// Load the actual file.
	worldData = new WorldData[3];

	worldData[MAP_BALANCE].pDoc = this;
	worldData[MAP_BALANCE].size = CSize(256, 256);
	worldData[MAP_BALANCE].name = "World of Balance";
	worldData[MAP_BALANCE].mapPtrs.Add(0xEEB20F);
	worldData[MAP_BALANCE].miniMapPtrs.Add(0xEEB24B);
	worldData[MAP_BALANCE].tilePtrs.Add(0xEEB212);
	worldData[MAP_BALANCE].ReadMapData(FF3_P_WoB_MapData);
	worldData[MAP_BALANCE].ReadMiniMapData(FF3_P_WoB_MiniMapData);
	worldData[MAP_BALANCE].ReadPalettes(FF3_WoB_Palettes);
	worldData[MAP_BALANCE].ReadTileData(FF3_P_WoB_TileData);
	worldData[MAP_BALANCE].ReadTileProperties(FF3_WoB_TileProperties);
	worldData[MAP_BALANCE].ReadTriggers(FF3_PF_Entrances(0x000), FF3_PF_EventTriggers(0x000));
//	worldData[MAP_BALANCE].ReadEnemyLayout(0x74742, 64);
//	worldData[MAP_BALANCE].ReadEnemyOdds(0x74500, 64);

	worldData[MAP_RUIN].pDoc = this;
	worldData[MAP_RUIN].size = CSize(256, 256);
	worldData[MAP_RUIN].name = "World of Ruin";
	worldData[MAP_RUIN].mapPtrs.Add(0xEEB221);
	worldData[MAP_RUIN].mapPtrs.Add(0xEEB224);
	worldData[MAP_RUIN].miniMapPtrs.Add(0xEEB24E);
	worldData[MAP_RUIN].tilePtrs.Add(0xEEB21E);
	worldData[MAP_RUIN].ReadMapData(FF3_P_WoR_MapData);
	worldData[MAP_RUIN].ReadMiniMapData(FF3_P_WoR_MiniMapData);
	worldData[MAP_RUIN].ReadPalettes(FF3_WoR_Palettes);
	worldData[MAP_RUIN].ReadTileData(FF3_P_WoR_TileData);
	worldData[MAP_RUIN].ReadTileProperties(FF3_WoR_TileProperties);
	worldData[MAP_RUIN].ReadTriggers(FF3_PF_Entrances(0x001), FF3_PF_EventTriggers(0x001));
//	worldData[MAP_RUIN].ReadEnemyLayout(0x74742, 64);
//	worldData[MAP_RUIN].ReadEnemyOdds(0x74500, 64);

	// BEGIN: Read global palettes.
	Seek(0xE68000);
	Read(characterPalettes, 256);

	Seek(0xED5860);
	Read(portraitPalettes, 512);

	Seek(0xC0CE3A);
	Read(characterGraphic, 12 * 40);

	Seek(0xC48FC0);
	Read(charWidths, 128);

	Seek(0xC490C0);
	Read(&variableFont, 11 * 128 * sizeof(WORD));

	Seek(0xC487C0);
	Read(&fixedFont, 16 * 128);

	int i = 0;

	// BEGIN: Read character data.
	Seek(0xED7CA0);
	for(int n = 0; n < 64; n++)
		Read(&characters[n], 22);

	Seek(0xC478C0);
	for(n = 0; n < 64; n++)
		Read(characters[n].name, 6);

	for(n = 0; n < 32; n++)
	{
		WORD wLowPtr = 0;
		Seek(0xC0D0F2 + n * 2);
		Read(&wLowPtr, 2);

		BYTE bHiPtr = 0;
		Seek(0xC0D23C + n * 2);
		Read(&bHiPtr, 1);

		DWORD dwPtr = bHiPtr << 16 | wLowPtr;
		Seek(dwPtr);
		Read(characters[n].gfx, 0x16A0);

		Seek(0xED1D00);
		Read(characters[n].portrait, 0x80);
	}

	Seek(0xC2CE2B);
	for(n = 0; n < 32; n++)
		Read(&characters[n].palette, 1);

	Seek(0xE6F4A0);
	Read(hpProgression, 98);
	Read(mpProgression, 98);

	// END: Read character data.

	// Read monster formation data.
	Seek(0xCF6200);
	for(n = 0; n < 576; n++)
		Read(&monsterFormations[n], 15);

	// Read monster formation auxiliary data.
	Seek(0xCF5900);
	for(n = 0; n < 576; n++)
		Read(&monsterFormations[n].aux, 4);

	// Read monster data.
	Seek(0xCF0000);
	for(n = 0; n < 384; n++)
		Read(&monsters[n].strength, 32);

	Seek(0xCFC050);
	for(n = 0; n < 384; n++)
		Read(&monsters[n].name, 10);

	Seek(0xCFD0D0);
	for(n = 0; n < 384; n++)
		Read(&monsters[n].attackName, 10);

	Seek(0xCF3000);
	for(n = 0; n < 384; n++)
		Read(&monsters[n].itemsStolen, 4);

	Seek(0xC47F40);
	Read(morphPackages, 26 * 4);

	for(n = 0; n < 13; n++)
	{
		int m = 0;
		BOOL bDone = FALSE;

		Seek(0xC2D01A + n * 2);
		WORD wPtr, w1;
		BYTE b1, b2;
		Read(&wPtr, 2);

		Seek(0xC20000 + wPtr);
		Read(&w1, 2);
		Read(&b1, 1);
		Read(&b2, 1);
		monsterTemplates[n][m].SetUnknown(w1);
		monsterTemplates[n][m].SetWidth(b1);
		monsterTemplates[n][m].SetHeight(b2);
		m++;
		Read(&w1, 2);
		Read(&b1, 1);
		Read(&b2, 1);

		while(w1 != 0x0000 && m < 6)
		{
			monsterTemplates[n][m].SetUnknown(w1);
			monsterTemplates[n][m].SetWidth(b1);
			monsterTemplates[n][m].SetHeight(b2);
			m++;
			Read(&w1, 2);
			Read(&b1, 1);
			Read(&b2, 1);
		}

		monsterTemplateCount[n] = m;
	}

	// Read monster palettes.
	Seek(0xD27820);
	Read(monsterPalettes, 0x3000);

	// Read monster visuals.
	Seek(0xD27000);
	for(n = 0; n < 384; n++)
		Read(&monsterVisuals[n], 5);

	// Read monster compositions.
	WORD wPtr;
	Seek(0xD2A820);
	Read(&wPtr, 2);
	Seek(0xD20000 + wPtr);
	for(n = 0; n < 128; n++)
	{
		monsterCompositions[n].length = 8;
		Read(monsterCompositions[n].mask, 8);
	}

	Seek(0xD2A822);
	Read(&wPtr, 2);
	Seek(0xD20000 + wPtr);
	for(n = 128; n < 176; n++)
	{
		monsterCompositions[n].length = 32;
		Read(monsterCompositions[n].mask, 32);
	}

	// Read monster gfx.
	Seek(0xE97000);
	Read(monsterGraphics, 0x38000);

	// Read item data.
	Seek(0xD2B300);
	for(n = 0; n < 256; n++)
		Read(items[n].name, 13);

	Seek(0xD85000);
	for(n = 0; n < 256; n++)
		Read(items[n].props, 30);

	// Read shop data.
	Seek(0xC47AC0);
	Read(shops, 9 * 128);

	// Read esper data.
	Seek(0xE6F6E1);
	for(n = 0; n < 27; n++)
		Read(espers[n].name, 8);

	Seek(0xD86E00);
	for(n = 0; n < 27; n++)
		Read(&espers[n].spell1Rate, 11);

	// Read magic data.
	Seek(0xE6F567);
	for(n = 0; n < 0x36; n++)
		Read(magic[n].name, 7);

	for(n = 0x36; n < 0x51; n++)
		Read(magic[n].name, 8);

	for(n = 0x51; n < 0x55; n++)
		Read(magic[n].name, 10);

	Seek(0xCF3C40);
	for(n = 0x56; n < 0x5E; n++)
		Read(magic[n - 1].name, 12);

	Seek(0xE6F831);
	for(n = 0x5E; n < 0x100; n++)
		Read(magic[n - 1].name, 10);

	Seek(0xC46AC0);
	for(n = 0; n < 256; n++)
		Read(&magic[n].target, 14);

	// Read the battle command data.
	Seek(0xD8CEA0);
	for(n = 0; n < 32; n++)
		Read(battleCommands[n].name, 7);

	m_pFile->Seek(0x029B1D + FF3_EventCode, CFile::begin);
	m_pFile->Read(&event, 8192);

	// Find out if the dialogue routine has been changed.
	BYTE opcode = 0;
	Seek(0xC07FBF);
	Read(&opcode, 1);

	if(opcode == 0xA9)
	{
		WORD wPageBound = 0;
		m_pFile->Seek(0xCE800, CFile::begin);
		m_pFile->Read(&wPageBound, 2);

		BYTE bTemp[1024];

		for(DWORD i = 0xCE802; i < 0xD01FE; i += 2)
		{
			int nPos = 0;

			DialogueItem* pDlg = new DialogueItem;
			dialogue.Add(pDlg);

			WORD wBegin = 0, wEnd = 0;
			m_pFile->Seek(i, CFile::begin);
			m_pFile->Read(&wBegin, 2);
			m_pFile->Read(&wEnd, 2);

			DWORD dwBegin = wBegin, dwEnd = wEnd;

			if((i - 0xCE802) / 2 + 1 >= wPageBound) dwEnd += 0x10000;
			if((i - 0xCE802) / 2 >= wPageBound) dwBegin += 0x10000;

			DWORD dwLength = dwEnd - dwBegin;

			if(dwLength)
			{
				BYTE bChar;
				m_pFile->Seek(dwBegin + 0xD0200, CFile::begin);
				m_pFile->Read(&bChar, 1);

				while(bChar)
				{
					bTemp[nPos++] = bChar;
					m_pFile->Read(&bChar, 1);
				}

				bTemp[nPos++] = 0x00;

				pDlg->text = new BYTE[nPos];
				pDlg->length = nPos;
				CopyMemory(pDlg->text, bTemp, nPos);
			}
			else
			{
				pDlg->text = NULL;
				pDlg->length = 0;
			}
		}
	}
	else if(opcode == 0x7B)
	{
		WORD ptrs[10];
		BYTE pages[10];
	
		Seek(0xC0DF80);
		i = 0;

		Read(&ptrs[i], 2);
		Read(&pages[i], 1);

		while(ptrs[i] != 0xFFFF)
		{
			i++;
			Read(&ptrs[i], 2);
			Read(&pages[i], 1);
		}

		i++;
		int nNumPages = i;

		WORD wItems;
		Read(&wItems, 2);
		dialogue.SetSize(wItems);

		WORD* dlgPtrs = new WORD[wItems];

		Seek(0xCCE602);
		Read(dlgPtrs, dialogue.GetSize() * sizeof(WORD));

		BYTE bTemp[1024];

		int i = 0;
		for(int n = 0; n < nNumPages; n++)
		{
			while(i < ptrs[n] && i < dialogue.GetSize())
			{
				int nPos = 0;

				DialogueItem* pDlg = new DialogueItem;
				dialogue[i] = pDlg;

				if((i < dialogue.GetSize()) && (dlgPtrs[i] != dlgPtrs[i + 1]))
				{
					BYTE bChar;
					Seek(pages[n] << 16 | dlgPtrs[i]);
					Read(&bChar, 1);

					while(bChar)
					{
						bTemp[nPos++] = bChar;
						Read(&bChar, 1);
					}

					bTemp[nPos++] = 0x00;

					pDlg->text = new BYTE[nPos];
					pDlg->length = nPos;
					CopyMemory(pDlg->text, bTemp, nPos);
				}
				else
				{
					pDlg->text = NULL;
					pDlg->length = 0;
				}

				i++;
			}
		}

		delete [] dlgPtrs;
	}

	m_bValid = TRUE;

	ReleaseFile(m_pFile, FALSE);
	SetModifiedFlag(FALSE);     // start off with unmodified

	m_pFile = NULL;

	return TRUE;
}

BOOL CFF3Doc::OnSaveDocument(LPCTSTR lpszPathName) 
{
	CFileException fe;
	m_pFile = GetFile(lpszPathName, CFile::modeReadWrite | CFile::shareExclusive, &fe);

	if(m_pFile == NULL)
	{
		ReportSaveLoadException(lpszPathName, &fe,
			TRUE, AFX_IDP_INVALID_FILENAME);
		return FALSE;
	}

	CWaitCursor wait;

	m_dwROMEnd = 0xF00000;

	int nCurDlg = 0;
	BYTE nCurPage = 0xCD;
	BYTE pages[10];
	WORD ptrs[10];
	DWORD dwLength = 0, dwPtr = 0;
	WORD wOffset = 0;

	WORD* dlgPtrs = new WORD[dialogue.GetSize()];

	Seek(0xCD0000);
	BYTE* dummy = new BYTE[0x1F100];
	FillMemory(dummy, 0x1F100, 0xFF);
	Write(dummy, 0x1F100);
	delete dummy;

	for(int i = 0; i < dialogue.GetSize(); i++)
	{
		if((dwLength + dialogue[i]->length) >= 0x1F000)
		{
			dwLength = 0;
			dwPtr = 0;
			pages[nCurDlg] = nCurPage;
			ptrs[nCurDlg] = i;
			nCurDlg++;  nCurPage = 0xF0;
		}

		if((dwPtr + dialogue[i]->length) >= 0x10000)
		{
			dwPtr = 0;
			pages[nCurDlg] = nCurPage;
			ptrs[nCurDlg] = i;
			nCurDlg++;  nCurPage++;
		}

		WORD wPtr = (WORD)(dwPtr & 0xFFFF);
		dlgPtrs[i] = wPtr;

		dwPtr += dialogue[i]->length;
		dwLength += dialogue[i]->length;
	}

	pages[nCurDlg] = nCurPage;
	ptrs[nCurDlg] = 0xFFFF;

	Seek(0xCCE602);
	Write(dlgPtrs, dialogue.GetSize() * sizeof(WORD));

	i = 0;
	for(int n = 0; n <= nCurDlg; n++)
	{
		while(i < ptrs[n] && i < dialogue.GetSize())
		{
			if((i < dialogue.GetSize()) && (dlgPtrs[i] != dlgPtrs[i + 1]))
			{
				Seek((pages[n] << 16) | dlgPtrs[i]);
				Write(dialogue[i]->text, dialogue[i]->length);
			}
			
			i++;
		}
	}

	if(SMCToHiROM(m_pFile->GetPosition()) > m_dwROMEnd)
		m_dwROMEnd = SMCToHiROM(m_pFile->GetPosition());

	Seek(0xC0DF80);
	for(i = 0; i <= nCurDlg; i++)
	{
		Write(&ptrs[i], 2);
		Write(&pages[i], 1);
	}

	WORD wItems = dialogue.GetSize();
	Write(&wItems, 2);

	delete [] dlgPtrs;

	PatchDialogueRoutine(m_pFile);

	Seek(0xC48FC0);
	Write(charWidths, 128);

	Seek(0xC490C0);
	Write(&variableFont, 11 * 128 * sizeof(WORD));

	Seek(0xC487C0);
	Write(&fixedFont, 16 * 128);

	// Write item data.
	Seek(0xD2B300);
	for(i = 0; i < 256; i++)
		Write(items[i].name, 13);

	Seek(0xD85000);
	for(i = 0; i < 256; i++)
		Write(items[i].props, 30);

	// Write shop data.
	Seek(0xC47AC0);
	Write(shops, 9 * 128);

	// Write esper data.
	Seek(0xE6F6E1);
	for(i = 0; i < 27; i++)
		Write(espers[i].name, 8);

	Seek(0xD86E00);
	for(i = 0; i < 27; i++)
		Write(&espers[i].spell1Rate, 11);

	// Maps
	worldData[MAP_BALANCE].StoreMapData(FF3_P_WoB_MapData, 0x3D1B);
	worldData[MAP_BALANCE].StoreMiniMapData(FF3_P_WoB_MiniMapData, 0x0418);
	worldData[MAP_BALANCE].StoreTileData(FF3_P_WoB_TileData, 0x2101);
	worldData[MAP_BALANCE].StorePalettes(FF3_WoB_Palettes);
	worldData[MAP_BALANCE].StoreTileProperties(FF3_WoB_TileProperties);
//	worldData[MAP_BALANCE].ReadTriggers(FF3_PF_Entrances(0x000), FF3_PF_EventTriggers(0x000));

	worldData[MAP_RUIN].StoreMapData(FF3_P_WoR_MapData, 0x32C1);
	worldData[MAP_RUIN].StoreMiniMapData(FF3_P_WoR_MiniMapData, 0x0473);
	worldData[MAP_RUIN].StoreTileData(FF3_P_WoR_TileData, 0x2010);
	worldData[MAP_RUIN].StorePalettes(FF3_WoR_Palettes);
	worldData[MAP_RUIN].StoreTileProperties(FF3_WoR_TileProperties);

	// Set the expansion offset to the end of the ROM.  Various save routines
	// will use this if they need to expand, then add to it once they've done
	// so.  This makes the optimal use of space by not fixing data at static
	// locations in the ROM.
/*	m_dwExpansionOffset = 0x100200;

	// Check to see if there are more triggers than can fit in
	// the space allowed.  It is of no importance if one map has
	// more triggers -as long as- another has had enough removed
	// that they will still fit.  We check for -total- number of
	// triggers here.
	BOOL bExpandEntr = FALSE;
	if(worldData[MAP_OVERWORLD].entrances.GetSize() + worldData[MAP_UNDERGROUND].entrances.GetSize() +
		worldData[MAP_MOON].entrances.GetSize() > 82) bExpandEntr = TRUE;

	// Data requiring expansion.
	worldData[MAP_OVERWORLD].StoreMapData(MAP_OVERWORLD);
	worldData[MAP_UNDERGROUND].StoreMapData(MAP_UNDERGROUND);
	worldData[MAP_MOON].StoreMapData(MAP_MOON);

	worldData[MAP_OVERWORLD].StoreEntrances(0xD0060, bExpandEntr);
	worldData[MAP_UNDERGROUND].StoreEntrances(0xD0062, bExpandEntr);
	worldData[MAP_MOON].StoreEntrances(0xD0064, bExpandEntr);

	// Data not requiring expansion.
	worldData[MAP_OVERWORLD].StoreTileData(0xE8200);
	worldData[MAP_UNDERGROUND].StoreTileData(0xEA200);
	worldData[MAP_MOON].StoreTileData(0xEC200);

	worldData[MAP_OVERWORLD].StoreTileFormations(0xA0200);
	worldData[MAP_UNDERGROUND].StoreTileFormations(0xA0400);
	worldData[MAP_MOON].StoreTileFormations(0xA0600);

	worldData[MAP_OVERWORLD].StoreTileProperties(0xA0C80);
	worldData[MAP_UNDERGROUND].StoreTileProperties(0xA0D80);
	worldData[MAP_MOON].StoreTileProperties(0xA0E80);

	worldData[MAP_OVERWORLD].StoreTilePalettes(0xA0800);
	worldData[MAP_UNDERGROUND].StoreTilePalettes(0xA0900);
	worldData[MAP_MOON].StoreTilePalettes(0xA0A00);

	worldData[MAP_OVERWORLD].StorePalettes(0xA0B00);
	worldData[MAP_UNDERGROUND].StorePalettes(0xA0B80);
	worldData[MAP_MOON].StorePalettes(0xA0C00);

	worldData[MAP_OVERWORLD].StoreEnemyLayout(0x74742, 64);

	worldData[MAP_OVERWORLD].StoreEnemyOdds(0x74500, 64);

	StoreLocations();
*/
	// Round the ROM size up to the nearest MB.
	DWORD dwBound = 0;
	if(m_dwROMEnd & 0x0FFFFF)
	{
		dwBound = (m_dwROMEnd & 0xF00000) + 0x100000;
		dummy = new BYTE[dwBound - m_dwROMEnd];
		FillMemory(dummy, dwBound - m_dwROMEnd, 0x00);
		Seek(m_dwROMEnd);
		Write(dummy, dwBound - m_dwROMEnd);
		delete dummy;
		Seek(dwBound);
		::SetEndOfFile((HANDLE)m_pFile->m_hFile);
	}

	// Save any of the changes made to the header.
	SaveChanges();

	ReleaseFile(m_pFile, FALSE);

	SetModifiedFlag(FALSE);     // back to unmodified
	m_pFile = NULL;

	return TRUE;        // success
}

#define MAX_FILESIZE 100000

LPVOID CFF3Doc::ReadCompressed(CFile* pFile, DWORD dwSource)
{
	BYTE* pTemp = new BYTE[1048576];		// 1MB should be enough to hold any
											// decompressed data in a 3MB ROM.
	BYTE* pTempPtr = pTemp;

	dwSource = HiROMToSMC(dwSource);
	pFile->Seek(dwSource, CFile::begin);

	CFile file;
	BYTE buf[MAX_FILESIZE] = { 0 };
	BYTE buf2[2048] = { 0 };
	BYTE n, x, b;
	UINT size, w, num, i;
	ULONG bpos, bpos2;
	UINT finalCount = 0;

	pFile->Read(&size, 2);
	size = (size - 2) & 0xFFFF;
	pFile->Read(buf, size);

	bpos = 0; bpos2 = 2014;

	do
	{
		n = buf[bpos]; bpos++;

		for(x = 0; x < 8; x++)
		{
			if((n >> x) & 1 == 1)
			{
				b = buf[bpos]; bpos++;
				*(pTemp++) = b;
				finalCount++;
				buf2[bpos2&2047] = b; bpos2++;
			}
			else
			{
				w = buf[bpos] + (buf[bpos + 1] << 8);
				bpos += 2;
				num = (w >> 11) + 3;
				w = w & 2047;
				
				for(i = 0; i < num; i++)
				{
					b = buf2[(w + i) & 2047];
					*(pTemp++) = b;
					finalCount++;
					buf2[bpos2 & 2047] = b; bpos2++;
				}
			}
			if(bpos >= size)
				x = 8;
		}
    }
	while(bpos < size);

	BYTE* pData = new BYTE[finalCount];
	memcpy(pData, pTempPtr, finalCount);
	delete [] pTempPtr;

	return pData;
}

LPVOID CFF3Doc::CompressData(LPVOID pData, DWORD& dwSize)
{
	BYTE buf[MAX_FILESIZE] = { 0 };
	BYTE b[16] = { 0 };
	BYTE p, n, run, bp;
	UINT maxx, maxrun, x, w, start;
	ULONG bpos, bpos2, fsize, size;

	fsize = dwSize;
	memcpy(buf, pData, dwSize);

	BYTE* pTemp = new BYTE[1048576];		// 1MB should be enough to hold any
											// decompressed data in a 3MB ROM.
	BYTE* pTempPtr = pTemp;

	*(pTemp++) = 0;
	*(pTemp++) = 0;

	size = 0;
	bpos = 0; bp = 0;
	bpos2 = 2014;
	n = 0; p = 0;
	while (bpos < fsize)
	{
		maxrun = 0;
		if(bpos < 2048)
			start = bpos;
		else
			start = 2048;
		for(x = 1; x <= start; x++)
		{
			run = 0;
			while((run < 31 + 3) &&
				(buf[bpos - x + run] == buf[bpos + run]) &&
				(bpos + run < fsize))
			{
				run++;
			}

			if(run > maxrun)
			{
				maxrun = run;
				maxx = (bpos2 - x) & 2047;
			}
		}
		if(maxrun >= 3)
		{
			w = ((maxrun - 3) << 11) + maxx;
			b[bp] = w & 255;
			b[bp + 1] = w >> 8;
			bp += 2;
			bpos += maxrun;
			bpos2 = (bpos2 + maxrun) & 2047;
		}
		else
		{
			n = n | (1 << p);
			b[bp] = buf[bpos];
			bp++; bpos++;
			bpos2 = (bpos2 + 1) & 2047;
		}
		p = (p+1) & 7;
		if(p == 0)
		{
			*(pTemp++) = n;
			for(int tc = 0; tc < bp; tc++)
				*(pTemp++) = b[tc];

			size += bp + 1;
			n = 0; bp = 0;
		}
	}
	if(p != 0)
	{
		*(pTemp++) = n;
		for(int tc = 0; tc < bp; tc++)
			*(pTemp++) = b[tc];

		size += bp + 1;
		n = 0; bp = 0;
	}

	size += 2;

	pTempPtr[0] = LOBYTE(size & 0xFFFF);
	pTempPtr[1] = HIBYTE(size & 0xFFFF);

	dwSize = size;
	return pTempPtr;
}

void CFF3Doc::PatchDialogueRoutine(CFile* pFile)
{
	BYTE dlgRoutine[] = {
		0x7B,								// TDC
		0xAA,								// TAX
		0xC2, 0x20,							// REP #$20
		0xA5, 0xD0,							// LDA $D0
		
/* - */	0xE8,								// INX
		0xE8,								// INX
		0xE8,								// INX
		
		0xDF, 0x7D, 0xDF, 0xC0,				// CMP $C0DF7D,X			//$CCFEFD,X
		0xB0, 0xF7,							// BCS -
		
		0x0A,								// ASL
		0x48,								// PHA
		
		0xBF, 0x7E, 0xDF, 0xC0,				// LDA $C0DF7E,X
		0x85, 0xCA,							// STA $CA
		
		0xFA,								// PLX
		0xBF, 0x02, 0xE6, 0xCC,				// LDA $CCE602,X
		0x85, 0xC9,							// STA $C9
		
		0xE2, 0x20,							// SEP #$20
		0x7B,								// TDC
		0x1A,								// INC
		0x8D, 0x68, 0x05,					// STA $0568
		
		0x60								// RTS
	};

	Seek(0xC07FBF);
	Write(dlgRoutine, sizeof(dlgRoutine));

	SetChange(FF3C_DialogueRoutine, TRUE);
}


void CFF3Doc::OnUpdateFileSave(CCmdUI* pCmdUI) 
{
	pCmdUI->Enable(m_bValid);
}

CFrameWnd* CFF3Doc::CreateFrame(CRuntimeClass* pFrameClass, CRuntimeClass* pViewClass,
								int nID, LPVOID lpData, DWORD dwStyle)
{
	// Walk the list of views associated with the document, and ask each view to
	// "compare" itself with the given data pointer.  A return value of FALSE means
	// the frame should be created--anything else means that the views are the same
	// and it should be activated instead of creating a new one.
	CView* pTempView;
	POSITION pos = GetFirstViewPosition();
	while(pos != NULL)
	{
		pTempView = GetNextView(pos);

		// Don't check the view if it's inside the CMainFrame.  That way, the user
		// can implement a preview pane if desired, but a new frame can be created as
		// well.
		if(!pTempView->GetParentFrame()->IsKindOf(RUNTIME_CLASS(CMainFrame)) &&
			pTempView->SendMessage(WM_COMPAREVIEW, NULL, (LPARAM)lpData))
		{
			// They're the same, activate the frame.
			pTempView->GetParentFrame()->ActivateFrame();
			return NULL;
		}
	}

	CCreateContext context;
	context.m_pCurrentFrame = NULL;
	context.m_pCurrentDoc = this;
	context.m_pNewViewClass = pViewClass;
	context.m_pNewDocTemplate = NULL;

	CFrameWnd* pFrame = (CFrameWnd*)pFrameClass->CreateObject();
	if (pFrame == NULL)
	{
		TRACE1("Warning: Dynamic create of frame %hs failed.\n",
			pFrameClass->m_lpszClassName);
		return NULL;
	}
	ASSERT_KINDOF(CFrameWnd, pFrame);

	// create new from resource
	if (!pFrame->LoadFrame(nID, dwStyle, NULL, &context))
	{
		TRACE0("Warning: CDocTemplate couldn't create a frame.\n");
		// frame will be deleted in PostNcDestroy cleanup
		return NULL;
	}
	pFrame->ModifyStyleEx(WS_EX_CLIENTEDGE, 0, SWP_FRAMECHANGED);

	return pFrame;
}

CFrameWnd* CFF3Doc::CreateFrameAlways(CRuntimeClass* pFrameClass, CRuntimeClass* pViewClass,
									  int nID, DWORD dwStyle)
{
	// Does the same thing as CreateFrame, but doesn't check to see if a view of the
	// same type and comparable content already exists.  Use this to force-create a 
	// frame for whatever reason.

	CCreateContext context;
	context.m_pCurrentFrame = NULL;
	context.m_pCurrentDoc = this;
	context.m_pNewViewClass = pViewClass;
	context.m_pNewDocTemplate = NULL;

	CFrameWnd* pFrame = (CFrameWnd*)pFrameClass->CreateObject();
	if (pFrame == NULL)
	{
		TRACE1("Warning: Dynamic create of frame %hs failed.\n",
			pFrameClass->m_lpszClassName);
		return NULL;
	}
	ASSERT_KINDOF(CFrameWnd, pFrame);

	// create new from resource
	if (!pFrame->LoadFrame(nID, dwStyle, NULL, &context))
	{
		TRACE0("Warning: CDocument couldn't create a frame.\n");
		// frame will be deleted in PostNcDestroy cleanup
		return NULL;
	}
	pFrame->ModifyStyleEx(WS_EX_CLIENTEDGE, 0, SWP_FRAMECHANGED);

	return pFrame;
}