// FF3Edit.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "FF3Edit.h"

#include "MainFrame.h"
#include "FF3Doc.h"
#include "WorkspaceView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// AppSettings

AppSettings::AppSettings()
{
	// Read all the settings at construction so they'll be ready
	// throughout the run of the program.

	bAdvanced = AfxGetApp()->GetProfileInt("Settings", "AdvancedOptions", FALSE);
	bMapGridlines = AfxGetApp()->GetProfileInt("Settings", "MapGridlines", FALSE);
	
	if(!ExtractWindowSpecs(AfxGetApp()->GetProfileString("Settings", "MainWindowSpecs"), &wsMainWindow))
	{
		wsMainWindow.rect = CRect(10, 10, 500, 400);
		wsMainWindow.bVisible = TRUE;
		wsMainWindow.bMaximized = FALSE;
		wsMainWindow.bMinimized = FALSE;
	}

	if(!ExtractWindowSpecs(AfxGetApp()->GetProfileString("Settings", "TileDialogSpecs"), &wsTileDlg))
	{
		wsTileDlg.rect = CRect(CW_USEDEFAULT, CW_USEDEFAULT, 0, 0);
		wsTileDlg.bVisible = FALSE;
		wsTileDlg.bMaximized = FALSE;
		wsTileDlg.bMinimized = FALSE;
	}

	if(!ExtractWindowSpecs(AfxGetApp()->GetProfileString("Settings", "TileWindowSpecs"), &wsTileWnd))
	{
		wsTileWnd.rect = CRect(CW_USEDEFAULT, CW_USEDEFAULT, 0, 0);
		wsTileWnd.bVisible = TRUE;
		wsTileWnd.bMaximized = FALSE;
		wsTileWnd.bMinimized = FALSE;
	}

	if(!ExtractWindowSpecs(AfxGetApp()->GetProfileString("Settings", "TriggerWindowSpecs"), &wsTriggerWnd))
	{
		wsTriggerWnd.rect = CRect(CW_USEDEFAULT, CW_USEDEFAULT, 0, 0);
		wsTriggerWnd.bVisible = FALSE;
		wsTriggerWnd.bMaximized = FALSE;
		wsTriggerWnd.bMinimized = FALSE;
	}

	if(!ExtractWindowSpecs(AfxGetApp()->GetProfileString("Settings", "GenPropWindowSpecs"), &wsGenericPropWnd))
	{
		wsGenericPropWnd.rect = CRect(CW_USEDEFAULT, CW_USEDEFAULT, 0, 0);
		wsGenericPropWnd.bVisible = TRUE;
		wsGenericPropWnd.bMaximized = FALSE;
		wsGenericPropWnd.bMinimized = FALSE;
	}
}

AppSettings::~AppSettings()
{
	// Likewise, store everything at destruction, when the app
	// is closing.

	AfxGetApp()->WriteProfileInt("Settings", "AdvancedOptions", bAdvanced);
	AfxGetApp()->WriteProfileInt("Settings", "MapGridlines", bMapGridlines);

	AfxGetApp()->WriteProfileString("Settings", "MainWindowSpecs", FormatWindowSpecs(&wsMainWindow));
	AfxGetApp()->WriteProfileString("Settings", "TileDialogSpecs", FormatWindowSpecs(&wsTileDlg));
	AfxGetApp()->WriteProfileString("Settings", "TileWindowSpecs", FormatWindowSpecs(&wsTileWnd));
	AfxGetApp()->WriteProfileString("Settings", "TriggerWindowSpecs", FormatWindowSpecs(&wsTriggerWnd));
	AfxGetApp()->WriteProfileString("Settings", "GenPropWindowSpecs", FormatWindowSpecs(&wsGenericPropWnd));
}

CString AppSettings::FormatWindowSpecs(WINDOWSPECS* ws)
{
	CString str;
	str.Format("%04d %04d %04d %04d %1d %1d %1d",
		ws->rect.left, ws->rect.top, ws->rect.right, ws->rect.bottom, ws->bVisible,
		ws->bMaximized, ws->bMinimized);

	return str;
}

BOOL AppSettings::ExtractWindowSpecs(CString& str, WINDOWSPECS* ws)
{
	if(str.IsEmpty())
	{
		return FALSE;
	}
	else
	{
		ws->rect.left = atoi((const char*)str);
		ws->rect.top = atoi((const char*)str + 5);
		ws->rect.right = atoi((const char*)str + 10);
		ws->rect.bottom = atoi((const char*)str + 15);
		ws->bVisible = atoi((const char*)str + 20);
		ws->bMaximized = atoi((const char*)str + 22);
		ws->bMinimized = atoi((const char*)str + 24);

		return TRUE;
	}
}

/////////////////////////////////////////////////////////////////////////////
// CFF3EditApp

BEGIN_MESSAGE_MAP(CFF3EditApp, CWinApp)
	//{{AFX_MSG_MAP(CFF3EditApp)
	ON_COMMAND(ID_APP_ABOUT, OnAppAbout)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code!
	//}}AFX_MSG_MAP
	// Standard file based document commands
	ON_COMMAND(ID_FILE_NEW, CWinApp::OnFileNew)
	ON_COMMAND(ID_FILE_OPEN, CWinApp::OnFileOpen)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFF3EditApp construction

CFF3EditApp::CFF3EditApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CFF3EditApp object

CFF3EditApp theApp;

/////////////////////////////////////////////////////////////////////////////
// CFF3EditApp initialization

BOOL CFF3EditApp::InitInstance()
{
	// Standard initialization
	// If you are not using these features and wish to reduce the size
	//  of your final executable, you should remove from the following
	//  the specific initialization routines you do not need.

	AfxOleInit();
	AfxInitRichEdit();

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif

	// Change the registry key under which our settings are stored.
	// TODO: You should modify this string to be something appropriate
	// such as the name of your company or organization.
	SetRegistryKey(_T("Yousei"));

	LoadStdProfileSettings();  // Load standard INI file options (including MRU)

	// Create our settings object here.  It will be destroyed in ExitInstance().
	_Settings = new AppSettings;

	// Register clipboard formats.
	clipboardWorldMap = ::RegisterClipboardFormat(_T("FF3ED_WORLDMAP"));

	// Initialize common controls.
//	INITCOMMONCONTROLSEX iccex;
//	iccex.dwSize = sizeof(INITCOMMONCONTROLSEX);
//	iccex.dwICC = ICC_PAGESCROLLER_CLASS;
//	InitCommonControlsEx(&iccex);

	// Initialize command bar resources.
	m_pResources = new CResourceCollection(IDR_RESOURCES, NULL);
	m_pResources->LoadResources(IDR_MAINFRAME);
	m_pResources->LoadResources(IDR_WORLDMAP);
	m_pResources->LoadResources(IDR_DIALOGUE);

	m_pResources->SetControlType(ID_CURSOR_POSITION, cBarControlLabel);
	m_pResources->SetControlStyle(ID_CURSOR_POSITION, cBarControlStyleIconAndCaption);
	m_pResources->SetControlSize(ID_CURSOR_POSITION, CSize(96, 21));
	m_pResources->SetControlType(ID_DRAG_SIZE, cBarControlLabel);
	m_pResources->SetControlStyle(ID_DRAG_SIZE, cBarControlStyleIconAndCaption);
	m_pResources->SetControlSize(ID_DRAG_SIZE, CSize(96, 21));
	m_pResources->SetControlStyle(ID_WINDOW_CLOSE, cBarControlStyleIconAndCaption);

	// Initialize DirectX.
	HRESULT rval;
	LPDIRECTDRAW lpDD;

	rval = DirectDrawCreate(NULL, &lpDD, NULL);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	rval = lpDD->QueryInterface(IID_IDirectDraw4, (LPVOID*)&DDraw.lpDD);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	RELEASE(lpDD);

	rval = DDraw.lpDD->SetCooperativeLevel(NULL, DDSCL_NORMAL);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	// Create the primary surface.  We do it here because in a windowed application, one
	// primary surface covers all our windows--the primary surface is just a DirectDraw-ized
	// pointer to the GDI for the entire screen.
	DDSURFACEDESC2 ddsd;

	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);
	ddsd.dwFlags = DDSD_CAPS;
	ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;

	rval = DDraw.lpDD->CreateSurface(&ddsd, &DDraw.lpSurfFront, NULL);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	DDraw.ddpfPrimary.dwSize = sizeof(DDPIXELFORMAT);
	rval = DDraw.lpSurfFront->GetPixelFormat(&DDraw.ddpfPrimary);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	// Check for a proper screenmode (if running in 256 colors, quit).
	if(DDraw.ddpfPrimary.dwRGBBitCount < 16)
	{
		AfxMessageBox("This program cannot currently be run in\na 256-color screenmode.  Please increase the\nnumber of colors to 16-bit or higher.", MB_OK | MB_ICONSTOP);
		return FALSE;
	}

	rval = DDraw.lpDD->CreateClipper(0, &DDraw.lpClipper, NULL);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	rval = DDraw.lpSurfFront->SetClipper(DDraw.lpClipper);
	if(rval != DD_OK) DDraw.Error(rval, NULL);

	// Register the application's document templates.  Document templates
	//  serve as the connection between documents, frame windows and views.
	CSingleDocTemplate* pDocTemplate;
	pDocTemplate = new CSingleDocTemplate(
		IDR_MAINFRAME,
		RUNTIME_CLASS(CFF3Doc),
		RUNTIME_CLASS(CMainFrame),       // main SDI frame window
		RUNTIME_CLASS(CWorkspaceView));
	AddDocTemplate(pDocTemplate);

	// Parse command line for standard shell commands, DDE, file open
	CCommandLineInfo cmdInfo;
	ParseCommandLine(cmdInfo);

	// Dispatch commands specified on the command line
	if (!ProcessShellCommand(cmdInfo))
		return FALSE;

	// The one and only window has been initialized, so show and update it.
	m_pMainWnd->ShowWindow(SW_SHOW);
	m_pMainWnd->UpdateWindow();

	return TRUE;
}

int CFF3EditApp::ExitInstance() 
{
	RELEASE(DDraw.lpClipper);
	RELEASE(DDraw.lpSurfFront);
	RELEASE(DDraw.lpDD);
	
	if(m_pResources) delete m_pResources;
	if(_Settings) delete _Settings;
	
	return CWinApp::ExitInstance();
}

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	CHyperLink	m_wndLink;
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	virtual BOOL OnInitDialog();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	DDX_Control(pDX, IDC_ABOUT_LINK, m_wndLink);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CAboutDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();
	
	m_wndLink.SetURL(_T("http://yousei.lfx.org"));

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

// App command to run the dialog
void CFF3EditApp::OnAppAbout()
{
	CAboutDlg aboutDlg;
	aboutDlg.DoModal();
}

/////////////////////////////////////////////////////////////////////////////
// CFF3EditApp message handlers

#pragma warning( disable : 4035 ) 

DWORD DDColor(BYTE r, BYTE g, BYTE b)
{
	DWORD dwDepth = DDraw.ddpfPrimary.dwRGBBitCount;

	__asm
	{
			cmp dwDepth, 16
			jne TEST2432

			// Composition
			__asm mov al, r  __asm shl eax, 5
			__asm mov al, g  __asm shl eax, 6
			__asm mov al, b

			jmp DONE

		TEST2432:
			cmp dwDepth, 24
			jne TEST32
			jmp DO32

		TEST32:
			cmp dwDepth, 32
			jne DONE

		DO32:
			// Composition
			__asm mov al, r  __asm shl eax, 8
			__asm mov al, g  __asm shl eax, 8
			__asm mov al, b  __asm shl eax, 3

		DONE:
			//	impled return eax here;
	}
}

#pragma warning( default : 4035 ) 

BYTE DDGetR(DWORD color)
{
	BYTE r;

	if(DDraw.ddpfPrimary.dwRGBBitCount == 16)
		r = (BYTE)((color >> 11) & 0x1F);
	else if(DDraw.ddpfPrimary.dwRGBBitCount == 24 || DDraw.ddpfPrimary.dwRGBBitCount == 32)
		r = (BYTE)(((color >> 16) & 0xFF) / 8);
	
	return r;
}

BYTE DDGetG(DWORD color)
{
	BYTE g;

	if(DDraw.ddpfPrimary.dwRGBBitCount == 16)
		g = (BYTE)((color >> 6) & 0x1F);
	else if(DDraw.ddpfPrimary.dwRGBBitCount == 24 || DDraw.ddpfPrimary.dwRGBBitCount == 32)
		g = (BYTE)(((color >> 8) & 0xFF) / 8);
	
	return g;
}

BYTE DDGetB(DWORD color)
{
	BYTE b;

	if(DDraw.ddpfPrimary.dwRGBBitCount == 16)
		b = (BYTE)(color & 0x1F);
	else if(DDraw.ddpfPrimary.dwRGBBitCount == 24 || DDraw.ddpfPrimary.dwRGBBitCount == 32)
		b = (BYTE)((color & 0xFF) / 8);
	
	return b;
}

DWORD Convert555ToDDColor(WORD color)
{
	return DDColor(DDGetR(color), DDGetG(color), DDGetB(color));
}

#pragma warning( disable : 4035 ) 

DWORD SubtractDDColor(DWORD src, DWORD dest, BYTE opacity)
{
	char dr, dg, db;
	char sr, sg, sb;

	DWORD dwDepth = DDraw.ddpfPrimary.dwRGBBitCount;

	__asm
	{
			cmp dwDepth, 16
			jne TEST2432

			// Source RGB
			__asm mov eax, src  __asm and eax, 0xF800  __asm shr eax, 11  __asm mov sr, al
			__asm mov eax, src  __asm and eax, 0x07E0  __asm shr eax, 6   __asm mov sg, al
			__asm mov eax, src  __asm and eax, 0x001F  __asm mov sb, al

			// Destination RGB
			__asm mov eax, dest  __asm and eax, 0xF800  __asm shr eax, 11  __asm mov dr, al
			__asm mov eax, dest  __asm and eax, 0x07E0  __asm shr eax, 6   __asm mov dg, al
			__asm mov eax, dest  __asm and eax, 0x001F  __asm mov db, al

			// Subtraction
			__asm mov al, sr  __asm sub al, dr  __asm div opacity  __asm add al, dr  __asm mov sr, al
			__asm mov al, sg  __asm sub al, dg  __asm div opacity  __asm add al, dg  __asm mov sg, al
			__asm mov al, sb  __asm sub al, db  __asm div opacity  __asm add al, db  __asm mov sb, al

			// Composition
			__asm mov al, sr  __asm shl eax, 5
			__asm mov al, sg  __asm shl eax, 6
			__asm mov al, sb

			jmp DONE

		TEST2432:
			cmp dwDepth, 24
			jne TEST32
			jmp DO32

		TEST32:
			cmp dwDepth, 32
			jne DONE

		DO32:
			// Source RGB
			__asm mov eax, src  __asm and eax, 0xFF0000  __asm shr eax, 19  __asm mov sr, al
			__asm mov eax, src  __asm and eax, 0x00FF00  __asm shr eax, 11  __asm mov sg, al
			__asm mov eax, src  __asm and eax, 0x0000FF  __asm shr eax, 3   __asm mov sb, al

			// Destination RGB
			__asm mov eax, dest  __asm and eax, 0xFF0000  __asm shr eax, 19  __asm mov dr, al
			__asm mov eax, dest  __asm and eax, 0x00FF00  __asm shr eax, 11  __asm mov dg, al
			__asm mov eax, dest  __asm and eax, 0x0000FF  __asm shr eax, 3   __asm mov db, al

			// Subtraction
			__asm mov al, sr  __asm sub al, dr  __asm div opacity  __asm add al, dr  __asm mov sr, al
			__asm mov al, sg  __asm sub al, dg  __asm div opacity  __asm add al, dg  __asm mov sg, al
			__asm mov al, sb  __asm sub al, db  __asm div opacity  __asm add al, db  __asm mov sb, al

			// Composition
			__asm mov al, sr  __asm shl eax, 8
			__asm mov al, sg  __asm shl eax, 8
			__asm mov al, sb  __asm shl eax, 3

		DONE:
			//	impled return eax here;
	}
}

#pragma warning(default : 4035) 

#pragma warning(default : 4244)

int OutputVariableFontChar(CDC* pDC, CPoint pt, CFF3Doc* pDoc, BYTE bChar, BOOL bShadow)
{
	WORD* pChar = &pDoc->variableFont[(bChar - 0x20) * 11];

	for(int y = 0; y < 11; y++)
		for(int x = 0; x < pDoc->charWidths[bChar]; x++)
			if(pChar[y] & (1 << (11 - x)))
			{
				if(bShadow)
				{
					pDC->SetPixelV(pt.x + x + 1, pt.y + y + 1, RGB(0, 0, 0));
					pDC->SetPixelV(pt.x + x, pt.y + y, RGB(255, 255, 255));
				}
				else
					pDC->SetPixelV(pt.x + x, pt.y + y, RGB(0, 0, 0));
			}
	
	return pDoc->charWidths[bChar];
}

int OutputFixedFontChar(CDC* pDC, CPoint pt, CFF3Doc* pDoc, BYTE bChar, BOOL bShadow)
{
	BYTE* pChar = &pDoc->fixedFont[(bChar - 0x80) * 16];

	BYTE aTile[64];
	BYTE bPlanes[2];
	BYTE mask;

	for(int row = 0; row < 8; row++)
	{
		bPlanes[0] = pChar[row * 2];
		bPlanes[1] = pChar[row * 2 + 1];

		for(int bit = 7; bit >= 0; bit--)
		{
			mask = 1 << bit;
		
			aTile[row * 8 + (7 - bit)] = ((bPlanes[0] & mask) >> bit) |
								   (((bPlanes[1] & mask) >> bit) << 1);
		}
	}

	COLORREF colors[4] = {
		RGB(112, 120, 192), RGB(0, 0, 0), RGB(112, 112, 112), RGB(255, 255, 255) };

	for(int y = 0; y < 8; y++)
		for(int x = 0; x < 8; x++)
			if(aTile[y * 8 + x] != 0)
				if(bShadow)
					if(aTile[y * 8 + x] == 3)
						pDC->SetPixelV(pt.x + x, pt.y + y, colors[aTile[y * 8 + x]]);
				else
					pDC->SetPixelV(pt.x + x, pt.y + y, colors[aTile[y * 8 + x]]);
	
	return pDoc->charWidths[bChar];
}
