// FF3EditCtrl.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "FF3EditCtrl.h"
#include "MemDC.h"
#include "FF3Data.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFF3EditCtrl

CFF3EditCtrl::CFF3EditCtrl()
{
	m_nMaxLength = 8;
	m_nCaret = 0;
	m_bTracking = FALSE;
	m_bValid = TRUE;

	m_pText = new BYTE[m_nMaxLength];
	for(int i = 0; i < m_nMaxLength; i++) m_pText[i] = 0xFF;
}

CFF3EditCtrl::~CFF3EditCtrl()
{
	if(m_pText) delete m_pText;
}


BEGIN_MESSAGE_MAP(CFF3EditCtrl, CWnd)
	//{{AFX_MSG_MAP(CFF3EditCtrl)
	ON_WM_PAINT()
	ON_WM_SETFOCUS()
	ON_WM_KILLFOCUS()
	ON_WM_SETCURSOR()
	ON_WM_KEYDOWN()
	ON_WM_CHAR()
	ON_WM_ERASEBKGND()
	ON_WM_CONTEXTMENU()
	ON_WM_LBUTTONDOWN()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CFF3EditCtrl message handlers

void CFF3EditCtrl::OnPaint() 
{
	CPaintDC pdc(this); // device context for painting

	CRect rect;  GetClientRect(rect);
	CRect textRect = rect;  textRect.DeflateRect(1, 1, 1, 1);

	CMemDC dc(&pdc, rect);

	CFont* pOldFont = dc.SelectObject(GetFont());

	dc.FillSolidRect(rect, RGB(112, 120, 192));
	dc.SetBkMode(TRANSPARENT);

	for(int i = 0; i < m_nMaxLength; i++)
	{
		CPen linePen(PS_SOLID, 1, RGB(255, 255, 255));
		CPen* oldPen = dc.SelectObject(&linePen);

		dc.MoveTo(textRect.left, textRect.bottom - 1);
		dc.LineTo(textRect.left + 7, textRect.bottom - 1);

		dc.SelectObject(oldPen);

		OutputFixedFontChar(&dc, CPoint(textRect.left, textRect.top + 2), m_pDoc, m_pText[i]);

		textRect.left += 8;
	}

	dc.SelectObject(pOldFont);
}

void CFF3EditCtrl::OnSetFocus(CWnd* pOldWnd) 
{
	CWnd::OnSetFocus(pOldWnd);

	CreateSolidCaret(2, 12);

	SetCaretPos(CPoint(m_nCaret * 8 + 1, 3));
	ShowCaret();
}

void CFF3EditCtrl::OnLButtonDown(UINT nFlags, CPoint pt) 
{
	pt.x -= 3;
	m_nCaret = pt.x / 8;
	if(pt.x % 8 > 4) m_nCaret++;
	m_nCaret = min(m_nCaret, m_nMaxLength);

	SetCaretPos(CPoint(m_nCaret * 8 + 1, 3));
	
	CWnd::OnLButtonDown(nFlags, pt);
}

void CFF3EditCtrl::OnKillFocus(CWnd* pNewWnd) 
{
	HideCaret();
	DestroyCaret();

	CWnd::OnKillFocus(pNewWnd);
}

BOOL CFF3EditCtrl::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	char* nCursor = IDC_ARROW;

	if(!m_bTracking)
		nCursor = IDC_IBEAM;
		
	::SetCursor(AfxGetApp()->LoadStandardCursor(MAKEINTRESOURCE(nCursor)));

	return TRUE;
}

BOOL CFF3EditCtrl::PreTranslateMessage(MSG* pMsg) 
{
	switch(pMsg->message)
	{
		case WM_KEYDOWN:
		case WM_KEYUP:
			switch(pMsg->wParam)
			{
				case VK_UP:
				case VK_DOWN:
				case VK_LEFT:
				case VK_RIGHT:
				case VK_RETURN:

				// Bad Windows!  Don't eat my messages before I get them.
				SendMessage(pMsg->message, pMsg->wParam, pMsg->lParam);
				return TRUE;
			}
			break;

		case WM_CHAR:
			SendMessage(pMsg->message, pMsg->wParam, pMsg->lParam);
			return TRUE;
	}	

	return CWnd::PreTranslateMessage(pMsg);
}

void CFF3EditCtrl::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	switch(nChar)
	{ 
		case VK_LEFT:
			if(::GetKeyState(VK_CONTROL) & 0x8000)
				m_nCaret = 0;
			else
				m_nCaret = max(0, m_nCaret - 1);
			break;
 
		case VK_RIGHT:
			if(::GetKeyState(VK_CONTROL) & 0x8000)
				m_nCaret = m_nMaxLength;
			else
				m_nCaret = min(m_nCaret + 1, m_nMaxLength);
			break;

		case VK_HOME:
			m_nCaret = 0;
			break;
 
		case VK_END:
			m_nCaret = m_nMaxLength;
			break;

		case VK_DELETE:
			if(m_nCaret < m_nMaxLength)
			{
				memmove(m_pText + m_nCaret, m_pText + m_nCaret + 1, m_nMaxLength - (m_nCaret + 1));
				m_pText[m_nMaxLength - 1] = 0xFF;

				Invalidate();
			}
			break;			

		case VK_BACK:
			if(m_nCaret > 0)
			{
				if(m_nCaret == m_nMaxLength)
					m_pText[m_nMaxLength - 1] = 0xFF;
				else
				{
					memmove(m_pText + m_nCaret - 1, m_pText + m_nCaret, m_nMaxLength - m_nCaret);
					m_pText[m_nMaxLength - 1] = 0xFF;
				}

				m_nCaret--;
				Invalidate();
			}
			break;
	}

	SetCaretPos(CPoint(m_nCaret * 8 + 1, 3));

	CWnd::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CFF3EditCtrl::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if(nChar == 0x08) return;

	if(m_nCaret >= m_nMaxLength)
	{
		MessageBeep(-1);
		return;
	}

	BYTE ch = ASCIIToFixedChar((char)nChar);

	memmove(m_pText + m_nCaret + 1, m_pText + m_nCaret,
		m_nMaxLength - m_nCaret - 1);
	m_pText[m_nCaret] = ch;

	BOOL bOtherChar = FALSE;

	for(int i = m_nMaxLength - 1; i >= 0; i--)
		if(m_pText[i] == 0xFF)
			m_pText[i] = 0xFE;

	for(i = m_nMaxLength - 1; i >= 0; i--)
		if(m_pText[i] == 0xFE && !bOtherChar)
			m_pText[i] = 0xFF;
		else
			bOtherChar = TRUE;

	m_nCaret++;
	SetCaretPos(CPoint(m_nCaret * 8 + 1, 3));

	Invalidate();

	CWnd::OnChar(nChar, nRepCnt, nFlags);
}

BOOL CFF3EditCtrl::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}

void CFF3EditCtrl::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	SetFocus();

	CMenu menu;
	menu.LoadMenu(IDR_FIXEDTEXTICONS);

	CMenu* pMenu = menu.GetSubMenu(0);

	m_bTracking = TRUE;
	int cmd = (int)pMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON | TPM_NONOTIFY | TPM_RETURNCMD,
		point.x, point.y, pWnd, NULL);
	m_bTracking = FALSE;

	BYTE chIns = 0;

	switch(cmd)
	{
		case ID_POPUP_DAGGER:		chIns = 0xA0;  break;
		case ID_POPUP_SWORD:		chIns = 0xA1;  break;
		case ID_POPUP_SPEAR:		chIns = 0xA2;  break;
		case ID_POPUP_KATANA:		chIns = 0xA3;  break;
		case ID_POPUP_STAFF:		chIns = 0xA4;  break;
		case ID_POPUP_ROD:			chIns = 0xA5;  break;
		case ID_POPUP_STAR:			chIns = 0xA6;  break;
		case ID_POPUP_DIAMOND:		chIns = 0xA7;  break;
		case ID_POPUP_CARD:			chIns = 0xA8;  break;
		case ID_POPUP_CLAW:			chIns = 0xA9;  break;
		case ID_POPUP_SHIELD:		chIns = 0xAA;  break;
		case ID_POPUP_HELMET:		chIns = 0xAB;  break;
		case ID_POPUP_ARMOR:		chIns = 0xAC;  break;
		case ID_POPUP_BRUSH:		chIns = 0xAD;  break;
		case ID_POPUP_SKEAN:		chIns = 0xAE;  break;
		case ID_POPUP_RELIC:		chIns = 0xAF;  break;
		case ID_POPUP_WHITEMAGIC:	chIns = 0xB0;  break;
		case ID_POPUP_BLACKMAGIC:	chIns = 0xB1;  break;
		case ID_POPUP_EFFECTMAGIC:	chIns = 0xB2;  break;
	}

	if(chIns) OnChar((UINT)chIns, 0, 0);
}

void CFF3EditCtrl::SizeToContent()
{
	SetWindowPos(NULL, 0, 0, m_nMaxLength * 8 + 8, 24, SWP_NOMOVE | SWP_NOZORDER);
}

void CFF3EditCtrl::SetText(BYTE* pText, int len)
{
	if(m_pText) delete m_pText;
	
	m_nMaxLength = len;
	m_pText = new BYTE[m_nMaxLength];
	
	for(int i = 0; i < m_nMaxLength; i++) m_pText[i] = pText[i];

	Invalidate();
}
