// FF3ListBox.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "FF3ListBox.h"
#include "FF3Data.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFF3ListBox

CFF3ListBox::CFF3ListBox()
{
	m_nActiveButton = -1;
	m_dwListStyle = 0;
}

CFF3ListBox::~CFF3ListBox()
{
}


BEGIN_MESSAGE_MAP(CFF3ListBox, CListBox)
	//{{AFX_MSG_MAP(CFF3ListBox)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFF3ListBox message handlers

int CFF3ListBox::AddItem(int nDataIndex, int nLength)
{
	int index = AddString(NULL);
	SetItemData(index, MAKELONG(nDataIndex, nLength));
	return index;
}

int CFF3ListBox::GetItem(int nIndex)
{
	return LOWORD(GetItemData(nIndex));
}

void CFF3ListBox::MeasureItem(LPMEASUREITEMSTRUCT lpmis) 
{
	lpmis->itemHeight = 18;
}

void CFF3ListBox::DrawItem(LPDRAWITEMSTRUCT lpdis) 
{
	CDC dc;  dc.Attach(lpdis->hDC);

	CRect rect = lpdis->rcItem;
	UINT nIndex = lpdis->itemID;

	if(nIndex != (UINT)-1)
	{
		int nLength = (int)HIWORD(GetItemData(nIndex));
		int nText = (int)LOWORD(GetItemData(nIndex));

		CRect textRect = rect;
		if(!(m_dwListStyle & FF3LBS_NOCHANGE))
			textRect.right -= ::GetSystemMetrics(SM_CXHSCROLL);
		textRect.DeflateRect(1, 1, 1, 1);
		
		CRect focusRect = textRect;  focusRect.bottom--;

		CRect dropRect = rect;  dropRect.left = dropRect.right - ::GetSystemMetrics(SM_CXHSCROLL);
		dropRect.bottom--;

		if((m_dwListStyle & FF3LBS_FFEMPTY) && (nText == 0xFF))
		{
			dc.FillSolidRect(rect, GetSysColor(COLOR_WINDOW));
		}
		else
		{
			dc.FillSolidRect(rect, RGB(112, 120, 192));

			if((m_dwListStyle & FF3LBS_NOCHANGE) && (lpdis->itemState & ODS_SELECTED))
				dc.InvertRect(rect);
		}

		if((nText == 0xFF) && (m_dwListStyle & FF3LBS_FFEMPTY))
		{
			textRect.left += 2;
			dc.SetTextColor(GetSysColor(COLOR_WINDOWTEXT));
			dc.DrawText("None", &textRect, DT_SINGLELINE);
		}
		else if(nText != -1)
		{
			textRect.left += 2;

			for(int i = 0; i < nLength; i++)
			{
				OutputFixedFontChar(&dc, CPoint(textRect.left, textRect.top + 4), m_pDoc,
					((BYTE*)m_pArray->GetAt(nText))[i]);

				textRect.left += 8;
			}
		}

		if(!(m_dwListStyle & FF3LBS_NOCHANGE))
		{
			if(lpdis->itemState & ODS_SELECTED)
				dc.DrawFrameControl(dropRect, DFC_SCROLL, DFCS_SCROLLCOMBOBOX | ((m_nActiveButton == 0) ? (DFCS_PUSHED | DFCS_FLAT) : 0));
			else
				dc.DrawFrameControl(dropRect, DFC_SCROLL, DFCS_SCROLLCOMBOBOX);
		}

		CPen linePen(PS_SOLID, 1, ::GetSysColor(COLOR_BTNFACE));
		CPen* oldPen = dc.SelectObject(&linePen);

		dc.MoveTo(rect.left, rect.bottom - 1);
		dc.LineTo(rect.right, rect.bottom - 1);

		dc.SelectObject(oldPen);
	}

	dc.Detach();
}

void CFF3ListBox::OnLButtonDown(UINT nFlags, CPoint point) 
{
	CListBox::OnLButtonDown(nFlags, point);

	if(m_dwListStyle & FF3LBS_NOCHANGE) return;

	int nIndex = GetCurSel();
	int nLength = (int)HIWORD(GetItemData(nIndex));
	int nText = (int)LOWORD(GetItemData(nIndex));

	CRect rect; GetItemRect(nIndex, rect);
	CPoint ptPopup(rect.left, rect.bottom);
	ClientToScreen(&ptPopup);

	if(m_dwListStyle & FF3LBS_FFEMPTY) nLength |= 0x10000;

	m_wndList->Invoke(NULL, m_pDoc, ptPopup, rect.Width(), m_pArray, nLength, nText);

	if(m_wndList->m_nSel != LB_ERR)
	{
		SetItemData(nIndex, MAKELONG(m_wndList->m_nSel, nLength));
		InvalidateRect(&rect);
	}
}

void CFF3ListBox::OnLButtonUp(UINT nFlags, CPoint point) 
{
	// TODO: Add your message handler code here and/or call default
	
	CListBox::OnLButtonUp(nFlags, point);
}

void CFF3ListBox::PreSubclassWindow() 
{
	CListBox::PreSubclassWindow();

	if(m_dwListStyle & FF3LBS_NOCHANGE) return;

	m_wndList = new CFF3ListBoxPopup;
	m_wndList->Create(this);
}

///////////////////////////////////////////////////////////
//
//  CFF3ListBoxPopup class -- implementation for
//    drop down list controls in the toolbars
//

BEGIN_MESSAGE_MAP(CFF3ListBoxPopup, CMiniFrameWnd)
	ON_WM_CREATE()
	ON_WM_ACTIVATE()
	ON_MESSAGE(WM_SETFONT, OnSetFont)
	ON_WM_MEASUREITEM()
	ON_WM_DRAWITEM()
END_MESSAGE_MAP()

BOOL CFF3ListBoxPopup::Create(CWnd* pParentWnd)
{
	LPCTSTR className = AfxRegisterWndClass(CS_SAVEBITS,
		AfxGetApp()->LoadStandardCursor(IDC_ARROW), (HBRUSH)(COLOR_WINDOW + 1), NULL);

	DWORD dwWndStyle = WS_POPUP | MFS_SYNCACTIVE;

	BOOL rval = CMiniFrameWnd::CreateEx(WS_EX_TOOLWINDOW, className, NULL, dwWndStyle,
		CRect(0, 0, 0, 0), pParentWnd);

	if(!rval) return FALSE;

	m_wndList.Create(WS_CHILD | WS_VISIBLE | WS_VSCROLL | LBS_NOTIFY | LBS_NOINTEGRALHEIGHT |
		LBS_HASSTRINGS | LBS_OWNERDRAWFIXED | WS_BORDER, CRect(0, 0, 0, 0), this, 0);

	m_modal = 0;
	m_nItemToChange = -1;
	m_bFFEmpty = FALSE;

	return rval;
}

void CFF3ListBoxPopup::Invoke(CFF3ListBox* pParent, CFF3Doc* pDoc, CPoint pt, int nWidth, CPtrArray* aStrings, int nLength, int nSel)
{
	m_pParent = pParent;
	m_pDoc = pDoc;

	m_bValid = FALSE;
	m_bCanceled = FALSE;
	
	m_pArray = aStrings;
	m_nLength = nLength & 0xFFFF;
	m_bFFEmpty = (nLength & 0x10000) ? TRUE : FALSE;

	m_wndList.ResetContent();

	if(m_bFFEmpty)
	{
		int index = m_wndList.AddString(NULL);
		m_wndList.SetItemData(index, (DWORD)0xFF);
		nSel++;
	}

	for(int i = 0; i < aStrings->GetSize(); i++)
	{
		int index = m_wndList.AddString(NULL);
		m_wndList.SetItemData(index, (DWORD)i);
	}

	m_wndList.SetCurSel(nSel);
	m_nSel = m_wndList.GetItemData(nSel);
	m_wndList.SetTopIndex(nSel);

	BOOL bScrollbar = FALSE;
	
	int nItemHeight = m_wndList.GetItemHeight(0);
	int nTotalHeight = m_wndList.GetCount() * nItemHeight;
	if(nTotalHeight > nItemHeight * 12)
	{
		nTotalHeight = nItemHeight * 12;
		bScrollbar = TRUE;
	}

	int nTotalWidth = nWidth;
	if(bScrollbar) nTotalWidth += GetSystemMetrics(SM_CXVSCROLL);
	
	int cxScreen = GetSystemMetrics(SM_CXSCREEN);
	int cyScreen = GetSystemMetrics(SM_CYSCREEN);

	SetWindowPos(NULL, pt.x, pt.y,
		nTotalWidth, nTotalHeight + 2, SWP_SHOWWINDOW);
	m_wndList.SetWindowPos(NULL, 0, 0, nTotalWidth, nTotalHeight + 2, SWP_SHOWWINDOW);

	if(!(m_nFlags & WF_MODALLOOP)) m_modal = RunModalLoop();
}

void CFF3ListBoxPopup::OnActivate(UINT nState, CWnd* pWndOther, BOOL bMinimized) 
{
	CMiniFrameWnd::OnActivate(nState, pWndOther, bMinimized);

	if(nState == WA_INACTIVE)
	{
		CancelWindow(FALSE);
	}
}

void CFF3ListBoxPopup::CancelWindow(BOOL bSelected)
{
	if(bSelected) m_nSel = m_wndList.GetItemData(m_wndList.GetCurSel());

	if(bSelected && m_nItemToChange != -1)
	{
		CRect rect;  m_pParent->GetItemRect(m_nItemToChange, &rect);

		m_pParent->SetItemData(m_nItemToChange, m_nSel);
		m_pParent->m_nActiveButton = -1;
		m_pParent->InvalidateRect(rect);
	}

	ShowWindow(SW_HIDE);
	EndModalLoop(m_modal);
	m_modal = 0;

	if(bSelected && m_nItemToChange != -1)
		m_pParent->GetOwner()->SendMessage(WM_COMMAND,
			MAKELONG(m_pParent->GetDlgCtrlID(), FF3LB_ITEMCHANGED),
			(DWORD)m_pParent->GetSafeHwnd());
}

LONG CFF3ListBoxPopup::OnSetFont(UINT hFont, LONG fRedraw)
{
	m_wndList.SendMessage(WM_SETFONT, hFont, fRedraw);
	return NULL;
}

BOOL CFF3ListBoxPopup::OnCommand(WPARAM wParam, LPARAM lParam) 
{
	if(HIWORD(wParam) == LBN_SELCHANGE)
	{
		CancelWindow(TRUE);
	}

	return CMiniFrameWnd::OnCommand(wParam, lParam);
}

void CFF3ListBoxPopup::OnMeasureItem(int nID, LPMEASUREITEMSTRUCT lpmis)
{
	lpmis->itemHeight = 15;
}

void CFF3ListBoxPopup::OnDrawItem(int nID, LPDRAWITEMSTRUCT lpdis)
{
	// Only one control will ever need painting, and since its ID changes,
	// it's counterproductive to confirm this.

	CDC dc;
	dc.Attach(lpdis->hDC);

	CRect rect = lpdis->rcItem;
	UINT nIndex = lpdis->itemID;

	if(nIndex != (UINT)-1)
	{
		int nLength = m_nLength;
		int nText = (int)m_wndList.GetItemData(nIndex);

		CRect textRect = rect;
		CRect focusRect = textRect;

		if(m_bFFEmpty && (nText == 0xFF))
		{
			CFont font;
			int nHeight = -((dc.GetDeviceCaps(LOGPIXELSY) * 8.5) / 72);
			font.CreateFont(nHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_CHARACTER_PRECIS,
				CLIP_CHARACTER_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, "Tahoma");

			CFont* oldFont = dc.SelectObject(&font);
			
			dc.FillSolidRect(rect, GetSysColor((lpdis->itemState & ODS_SELECTED) ? COLOR_HIGHLIGHT : COLOR_WINDOW));
			dc.SetTextColor(GetSysColor((lpdis->itemState & ODS_SELECTED) ? COLOR_HIGHLIGHTTEXT : COLOR_WINDOWTEXT));

			textRect.left += 2;

			dc.DrawText("None", &textRect, DT_VCENTER);

			dc.SelectObject(oldFont);
		}
		else
		{
			dc.FillSolidRect(rect, RGB(112, 120, 192));
			if(lpdis->itemState & ODS_SELECTED) dc.InvertRect(rect);
		
			textRect.left += 2;

			for(int i = 0; i < nLength; i++)
			{
				OutputFixedFontChar(&dc, CPoint(textRect.left, textRect.top + 4),
					m_pDoc, ((BYTE*)m_pArray->GetAt(nText))[i]);

				textRect.left += 8;
			}
		}
	}

	dc.Detach();
}

///////////////////////////////////////////////////////////
//
//  XFF3ListBoxPopup class -- implementation for
//    drop down list controls in the toolbars
//

BEGIN_MESSAGE_MAP(XFF3ListBoxPopup, CListBox)
	ON_WM_MOUSEMOVE()
END_MESSAGE_MAP()

void XFF3ListBoxPopup::OnMouseMove(UINT nFlags, CPoint point) 
{
	CListBox::OnMouseMove(nFlags, point);

	BOOL bOutside;
	int nSel = (int)ItemFromPoint(point, bOutside);
	if(bOutside) nSel = -1;

	if(GetCurSel() != nSel) SetCurSel(nSel);
}
