// MainFrame.cpp : implementation of the CFrameWndEx class
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "FrameWndEx.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CFrameWndEx

IMPLEMENT_DYNCREATE(CFrameWndEx, CFrameWnd)

BEGIN_MESSAGE_MAP(CFrameWndEx, CFrameWnd)
	//{{AFX_MSG_MAP(CFrameWndEx)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
	ON_NOTIFY_RANGE(CBCN_DROPDOWN, 1, 20, OnMenuPopup)
	ON_COMMAND(ID_WINDOW_CLOSE, OnWindowClose)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFrameWndEx construction/destruction

CFrameWndEx::CFrameWndEx()
{
	m_pResources = App.GetResources();
}

CFrameWndEx::~CFrameWndEx()
{
}

void CFrameWndEx::RegisterUIElement(LPCTSTR lpszClass, UINT nID)
{
	RegisteredUIElement* element = new RegisteredUIElement;

	element->nID = nID;
	element->menu.LoadMenu(nID);

	m_uiElements[lpszClass] = element;
}

RegisteredUIElement* CFrameWndEx::GetUIElement(LPCTSTR lpszClass)
{
	RegisteredUIElement* element = NULL;
	BOOL res = m_uiElements.Lookup(lpszClass, element);
	if(!res) element = NULL;

	return element;
}

void CFrameWndEx::EnableDocking(DWORD dwDockStyle)
{
	// must be CBRS_ALIGN_XXX or CBRS_FLOAT_MULTI only
	ASSERT((dwDockStyle & ~(CBRS_ALIGN_ANY|CBRS_FLOAT_MULTI)) == 0);
	CFrameWnd::EnableDocking(dwDockStyle);
	m_pFloatingFrameClass = RUNTIME_CLASS(CMiniDockFrameWndEx);
	
	for (int i = 0; i < 4; i++) {
		if (nDockBarMap[i][1] & dwDockStyle & CBRS_ALIGN_ANY) {
			CDockBarEx* pDock = (CDockBarEx*)GetControlBar(nDockBarMap[i][0]);
			
			// make sure the dock bar is of correct type
			if( pDock == 0 || ! pDock->IsKindOf(RUNTIME_CLASS(CDockBarEx)) ) {
				BOOL bNeedDelete = ! pDock->m_bAutoDelete;
				pDock->m_pDockSite->RemoveControlBar(pDock);
				pDock->m_pDockSite = 0;	// avoid problems in destroying the dockbar
				pDock->DestroyWindow();
				if( bNeedDelete )
					delete pDock;
				pDock = 0;
			}
			
			if( pDock == 0 ) {
				pDock = new CDockBarEx;
				if (!pDock->Create(this,
					WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_CHILD | WS_VISIBLE |
					nDockBarMap[i][1], nDockBarMap[i][0])) {
					AfxThrowResourceException();
				}
				pDock->SetBarStyle(pDock->GetBarStyle() & ~CBRS_BORDER_ANY);
			}
		}
	}
}

void CFrameWndEx::FloatControlBar(CControlBar* pBar, CPoint point, DWORD dwStyle)
{
	ASSERT(pBar != NULL);

	// if the bar is already floating and the dock bar only contains this
	// bar and same orientation then move the window rather than recreating
	// the frame
	if (pBar->m_pDockSite != NULL && pBar->m_pDockBar != NULL)
	{
		CDockBarEx* pDockBar = (CDockBarEx*)pBar->m_pDockBar;
		ASSERT_KINDOF(CDockBarEx, pDockBar);
		if (pDockBar->m_bFloating && pDockBar->GetDockedCount() == 1 &&
			(dwStyle & pDockBar->m_dwStyle & CBRS_ALIGN_ANY) != 0)
		{
			CMiniDockFrameWndEx* pDockFrame =
				(CMiniDockFrameWndEx*)pDockBar->GetParent();
			ASSERT(pDockFrame != NULL);
			ASSERT_KINDOF(CMiniDockFrameWndEx, pDockFrame);
			pDockFrame->SetWindowPos(NULL, point.x, point.y, 0, 0,
				SWP_NOSIZE|SWP_NOZORDER|SWP_NOACTIVATE);
			pDockFrame->RecalcLayout(TRUE);
			pDockFrame->UpdateWindow();
			return;
		}
	}

	if (pBar->m_dwStyle & CBRS_SIZE_DYNAMIC)
	{
		dwStyle |= CBRS_SIZE_DYNAMIC;
		if (dwStyle & CBRS_ORIENT_VERT)
		{
			dwStyle &= ~CBRS_ALIGN_ANY;
			dwStyle |= CBRS_ALIGN_TOP;
		}
	}

	CMiniDockFrameWndEx* pDockFrame = CreateFloatingFrame(dwStyle);
	ASSERT(pDockFrame != NULL);
	pDockFrame->SetWindowPos(NULL, point.x, point.y, 0, 0,
		SWP_NOSIZE|SWP_NOZORDER|SWP_NOACTIVATE);
	if (pDockFrame->m_hWndOwner == NULL)
		pDockFrame->m_hWndOwner = pBar->m_hWnd;

	CDockBarEx* pDockBar = (CDockBarEx*)pDockFrame->GetDlgItem(AFX_IDW_DOCKBAR_FLOAT);
	ASSERT(pDockBar != NULL);
	ASSERT_KINDOF(CDockBarEx, pDockBar);

	ASSERT(pBar->m_pDockSite == this);
	// if this assertion occurred it is because the parent of pBar was not
	//  initially this CFrameWnd when pBar's OnCreate was called
	// (this control bar should have been created with a different
	//  parent initially)

	pDockBar->DockControlBar((CCommandBar*)pBar);
	pDockFrame->RecalcLayout(TRUE);
	pDockFrame->ShowWindow(SW_SHOWNA);
	pDockFrame->UpdateWindow();
}

CMiniDockFrameWndEx* CFrameWndEx::CreateFloatingFrame(DWORD dwStyle)
{
	CMiniDockFrameWndEx* pFrame = NULL;
	if(m_pFloatingFrameClass == NULL) m_pFloatingFrameClass = RUNTIME_CLASS(CMiniDockFrameWndEx);

	pFrame = (CMiniDockFrameWndEx*)m_pFloatingFrameClass->CreateObject();
	if (pFrame == NULL) AfxThrowMemoryException();
	ASSERT_KINDOF(CMiniDockFrameWndEx, pFrame);
	
	if (!pFrame->Create(this, dwStyle)) AfxThrowResourceException();
	
	return pFrame;
}

int CFrameWndEx::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	EnableDocking(CBRS_ALIGN_ANY);

/*	m_cBarMenu.Create(this, "Menu Bar", m_pResources,
		WS_VISIBLE | WS_CHILD | CBRS_SIZE_DYNAMIC | CBRS_ALIGN_TOP | CBRS_TOOLTIPS |
		CBCS_MENUBAR, IDCB_MENUBAR);
	m_cBarMenu.EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_cBarMenu, AFX_IDW_DOCKBAR_TOP);

	CMenu menu;  menu.LoadMenu(m_nIDHelp);
	for(int i = 0; i < menu.GetMenuItemCount(); i++)
	{
		CString str;  menu.GetMenuString(i, str, MF_BYPOSITION);
		m_pResources->AddResource(i + 1, cBarControlPopup, cBarControlStyleCaption, str);
		m_cBarMenu.AddControl(-1, i + 1);
	}

	menu.DestroyMenu();
*/
	m_menuManager.Install((CFrameWnd*)this);
	m_menuManager.LoadToolbar(IDR_RESOURCES);
	m_menuManager.m_bAutoAccel = FALSE;

//	SetMenu(NULL);

	m_cBarStandard.Create(this, "Standard", m_pResources,
		WS_VISIBLE | WS_CHILD | CBRS_SIZE_DYNAMIC | CBRS_ALIGN_TOP | CBRS_TOOLTIPS,
		m_nIDHelp);
	m_cBarStandard.LoadToolBar(m_nIDHelp);
	m_cBarStandard.EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_cBarStandard, AFX_IDW_DOCKBAR_TOP);

	return 0;
}

void CFrameWndEx::OnMenuPopup(UINT id, NMHDR* pNotifyStruct, LRESULT* result)
{
	LPNMCBCDROPDOWN lpNM = (LPNMCBCDROPDOWN)pNotifyStruct;

	CMenu menu;  menu.LoadMenu(m_nIDHelp);
	CMenu* pMenu;  pMenu = menu.GetSubMenu(id - 1);
	pMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_LEFTBUTTON, lpNM->ptMenu.x,
		lpNM->ptMenu.y, this);

	*result = NULL;
}

void CFrameWndEx::OnUpdateFrameTitle(BOOL bAddToTitle)
{
	if((GetStyle() & FWS_ADDTOTITLE) == 0)
		return;     // leave it alone!

//#ifndef _AFX_NO_OLE_SUPPORT
	// allow hook to set the title (used for OLE support)
//	if (m_pNotifyHook != NULL && m_pNotifyHook->OnUpdateFrameTitle())
//		return;
//#endif

	CDocument* pDocument = GetActiveDocument();
	if (bAddToTitle && pDocument != NULL)
		UpdateFrameTitleForDocument(pDocument->GetTitle());
	else
		UpdateFrameTitleForDocument(NULL);
}

void CFrameWndEx::UpdateFrameTitleForDocument(LPCTSTR lpszDocName)
{
	// copy first part of title loaded at time of frame creation
	TCHAR szText[256+_MAX_PATH];

	if (GetStyle() & FWS_PREFIXTITLE)
	{
		szText[0] = '\0';   // start with nothing

		// get name of currently active view
		if (lpszDocName != NULL)
		{
			lstrcpy(szText, lpszDocName);
			// add current window # if needed
//			if (m_nWindow > 0)
//				wsprintf(szText + lstrlen(szText), _T(":%d"), m_nWindow);
			lstrcat(szText, _T(" - "));
		}
		lstrcat(szText, m_strTitle);
	}
	else
	{
		// get name of currently active view
		lstrcpy(szText, m_strTitle);
		if (lpszDocName != NULL)
		{
			lstrcat(szText, _T(" - "));
			lstrcat(szText, lpszDocName);
			// add current window # if needed
//			if (m_nWindow > 0)
//				wsprintf(szText + lstrlen(szText), _T(":%d"), m_nWindow);
		}
	}

	// set title if changed, but don't remove completely
	// Note: will be excessive for MDI Frame with maximized child
	AfxSetWindowText(m_hWnd, szText);
}

const DWORD CFrameWndEx::dwDockBarMap[4][2] =
{
	{ AFX_IDW_DOCKBAR_TOP,      CBRS_ALIGN_TOP    },
	{ AFX_IDW_DOCKBAR_BOTTOM,   CBRS_ALIGN_BOTTOM },
	{ AFX_IDW_DOCKBAR_LEFT,     CBRS_ALIGN_LEFT   },
	{ AFX_IDW_DOCKBAR_RIGHT,    CBRS_ALIGN_RIGHT  },
};

void CFrameWndEx::DockControlBar(CCommandBar* pBar, UINT nDockBarID, LPCRECT lpRect)
{
	CDockBarEx* pDockBar = (nDockBarID == 0) ? NULL :
		(CDockBarEx*)GetControlBar(nDockBarID);
	DockControlBar(pBar, pDockBar, lpRect);
}

void CFrameWndEx::DockControlBar(CCommandBar* pBar, CDockBarEx* pDockBar, LPCRECT lpRect)
{
	ASSERT(pBar != NULL);
	// make sure CControlBar::EnableDocking has been called
	ASSERT(pBar->m_pDockContext != NULL);

	if (pDockBar == NULL)
	{
		for (int i = 0; i < 4; i++)
		{
			if ((dwDockBarMap[i][1] & CBRS_ALIGN_ANY) ==
				(pBar->m_dwStyle & CBRS_ALIGN_ANY))
			{
				pDockBar = (CDockBarEx*)GetControlBar(dwDockBarMap[i][0]);
				ASSERT(pDockBar != NULL);
				// assert fails when initial CBRS_ of bar does not
				// match available docking sites, as set by EnableDocking()
				break;
			}
		}
	}
	ASSERT(pDockBar != NULL);
	ASSERT(m_listControlBars.Find(pBar) != NULL);
	ASSERT(pBar->m_pDockSite == this);
	// if this assertion occurred it is because the parent of pBar was not initially
	// this CFrameWnd when pBar's OnCreate was called
	// i.e. this control bar should have been created with a different parent initially

	pDockBar->DockControlBar(pBar, lpRect);
}

void CFrameWndEx::ReDockControlBar(CCommandBar* pBar, CDockBarEx* pDockBar, LPCRECT lpRect)
{
	ASSERT(pBar != NULL);
	// make sure CControlBar::EnableDocking has been called
	ASSERT(pBar->m_pDockContext != NULL);

	if (pDockBar == NULL)
	{
		// Search for the place holder.

		// In case we don't find a place holder, find a bar with the correct alignment
		// and keep it in pPossibleBar.
		CDockBar* pPossibleBar = NULL;
		for (int i = 0; i < 4; i++)
		{
			CDockBar* pTempBar = (CDockBarEx*)GetControlBar(dwDockBarMap[i][0]);
			if (pTempBar != NULL)
			{
				// Is this the same bar we docked with before?
				if (pTempBar->FindBar((CCommandBar*)pBar->GetDlgCtrlID()) > 0)
				{
					pDockBar = (CDockBarEx*)pTempBar;
					break;
				}
			}

			if ((dwDockBarMap[i][1] & CBRS_ALIGN_ANY) ==
				(pBar->m_dwStyle & CBRS_ALIGN_ANY))
			{
				pPossibleBar = (CDockBarEx*)GetControlBar(dwDockBarMap[i][0]);
				ASSERT(pPossibleBar != NULL);
				// assert fails when initial CBRS_ of bar does not
				// match available docking sites, as set by EnableDocking()
			}
		}

		// Did we find the place holder?
		if (pDockBar == NULL)
			pDockBar = (CDockBarEx*)pPossibleBar;
	}
	ASSERT(pDockBar != NULL);
	ASSERT(m_listControlBars.Find(pBar) != NULL);
	ASSERT(pBar->m_pDockSite == this);
	// if this assertion occurred it is because the parent of pBar was not initially
	// this CFrameWnd when pBar's OnCreate was called
	// i.e. this control bar should have been created with a different parent initially

	pDockBar->ReDockControlBar(pBar, lpRect);
}

void CFrameWndEx::OnWindowClose()
{
	DestroyWindow();
}
