// LuminanceSlider.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "LuminanceSlider.h"
#include <math.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLuminanceSlider

CLuminanceSlider::CLuminanceSlider()
{
	oldSliderBmp = NULL;
	m_nPos = 0;
	m_bTracking = FALSE;

	m_Hue = m_Sat = m_Lum = 0.0;
}

CLuminanceSlider::~CLuminanceSlider()
{
}


BEGIN_MESSAGE_MAP(CLuminanceSlider, CWnd)
	//{{AFX_MSG_MAP(CLuminanceSlider)
	ON_WM_DESTROY()
	ON_WM_PAINT()
	ON_WM_ERASEBKGND()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CLuminanceSlider message handlers

void CLuminanceSlider::PreSubclassWindow() 
{
	CWnd::PreSubclassWindow();

	ModifyStyleEx(WS_EX_NOPARENTNOTIFY, 0);

	CRect rect;  GetClientRect(rect);

	CDC* pDC = GetDC();

	dcSlider.CreateCompatibleDC(pDC);
	bmpSlider.CreateCompatibleBitmap(pDC, rect.Width(), rect.Height());
	oldSliderBmp = dcSlider.SelectObject(&bmpSlider);
}

void CLuminanceSlider::OnDestroy() 
{
	dcSlider.SelectObject(oldSliderBmp);

	CWnd::OnDestroy();	
}

void CLuminanceSlider::OnPaint() 
{
	CPaintDC dc(this); // device context for painting

	CRect rect;  GetClientRect(rect);

	WORD col;
	COLORREF tcol;
	for(int j = 0; j < rect.Height(); j++)
	{
		col = HLStoRGB555(m_Hue, (double)j / (double)rect.Height(), m_Sat);
		tcol = RGB(R555(col) * 8, G555(col) * 8, B555(col) * 8);
		dcSlider.FillSolidRect(0, rect.Height() - j - 1, rect.Width(), 1, tcol);
	}

	dc.BitBlt(0, 0, rect.Width(), rect.Height(), &dcSlider, 0, 0, SRCCOPY);

	rect.left = rect.right + 2;
	rect.top -= 5;
	rect.right = rect.left + 5;
	rect.bottom += 5;

	dc.FillSolidRect(&rect, GetSysColor(COLOR_BTNFACE));

	int w = 0;
	for(int i = 0; i < 5; i++)
	{
		w++;
		dc.MoveTo(rect.left + (5 - w), rect.top + m_nPos + i + 1);
		dc.LineTo(rect.left + 5, rect.top + m_nPos + i + 1);
	}
	for(i = 5; i < 10; i++)
	{
		w--;
		dc.MoveTo(rect.left + (5 - w), rect.top + m_nPos + i + 1);
		dc.LineTo(rect.left + 5, rect.top + m_nPos + i + 1);
	}
}

BOOL CLuminanceSlider::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}

void CLuminanceSlider::OnLButtonDown(UINT nFlags, CPoint point) 
{
	SetCapture();
	m_bTracking = TRUE;

	CRect rect;  GetWindowRect(rect);

	point.y /= rect.Height() / 30;  point.y *= rect.Height() / 30;

	m_Lum = (double)(rect.Height() - point.y) / (double)rect.Height();
	m_nPos = point.y;

	rect.left = rect.right - 2;
	rect.right = rect.left + 8;
	::ClipCursor(&rect);

	CWnd::OnLButtonDown(nFlags, point);
	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), BN_CLICKED), (LPARAM)GetSafeHwnd());

	Invalidate();
}

void CLuminanceSlider::OnLButtonUp(UINT nFlags, CPoint point) 
{
	ReleaseCapture();
	m_bTracking = FALSE;
	::ClipCursor(NULL);
	
	CWnd::OnLButtonUp(nFlags, point);
}

void CLuminanceSlider::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(!m_bTracking) return;

	CRect rect;  GetWindowRect(rect);

	point.y /= rect.Height() / 30;  point.y *= rect.Height() / 30;

	m_Lum = (double)(rect.Height() - point.y) / (double)rect.Height();
	m_nPos = point.y;

	CWnd::OnMouseMove(nFlags, point);
	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), BN_CLICKED), (LPARAM)GetSafeHwnd());

	Invalidate();
}

void CLuminanceSlider::SetLuminance(double l)
{
	CRect rect;  GetClientRect(rect);

	m_Lum = l;
	m_nPos = -rect.Height() * (l - 1);
	Invalidate();
}
