// MagicEditView.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "MagicEditView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMagicEditView

IMPLEMENT_DYNCREATE(CMagicEditView, CSizingFormView)

CMagicEditView::CMagicEditView()
	: CSizingFormView(CMagicEditView::IDD)
{
	//{{AFX_DATA_INIT(CMagicEditView)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

CMagicEditView::~CMagicEditView()
{
}

void CMagicEditView::DoDataExchange(CDataExchange* pDX)
{
	CSizingFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMagicEditView)
	DDX_Control(pDX, IDC_PROPS, m_wndProps);
	DDX_Control(pDX, IDC_LIST, m_wndList);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMagicEditView, CSizingFormView)
	//{{AFX_MSG_MAP(CMagicEditView)
	ON_LBN_SELCHANGE(IDC_LIST, OnSelChangeItemList)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
	ON_NOTIFY(PROPTN_PROPCHANGED, IDC_PROPS, OnChangeProperties)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMagicEditView diagnostics

#ifdef _DEBUG
void CMagicEditView::AssertValid() const
{
	CSizingFormView::AssertValid();
}

void CMagicEditView::Dump(CDumpContext& dc) const
{
	CSizingFormView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMagicEditView message handlers

int CMagicEditView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CSizingFormView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
	SetControlInfo(IDC_MINBOUNDS, MINIMUM_BOUND);
	SetControlInfo(IDC_LIST, RESIZE_VERT);
	SetControlInfo(IDC_PROPS, RESIZE_HORZ | RESIZE_VERT);
	
	return 0;
}

void CMagicEditView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();

	m_wndProps.m_pDoc = pDoc;

	m_wndList.ResetContent();
	m_wndList.m_pDoc = pDoc;
	m_wndList.m_dwListStyle = FF3LBS_NOCHANGE;

	m_magicArray.RemoveAll();
	m_wndList.m_pArray = &m_magicArray;

	for(int i = 0; i < 0x100; i++)
		m_magicArray.Add(pDoc->magic[i].name);

	for(i = 0; i < 0x36; i++)
		m_wndList.AddItem(i, 7);

	for(i = 0x36; i < 0x51; i++)
		m_wndList.AddItem(i, 8);

	for(i = 0x51; i < 0x55; i++)
		m_wndList.AddItem(i, 10);

	for(i = 0x55; i < 0x5D; i++)
		m_wndList.AddItem(i, 12);

	for(i = 0x5D; i < 0xFF; i++)
		m_wndList.AddItem(i, 10);

//	m_wndList.SetCurSel(0);
	UpdateTree();

}

void CMagicEditView::UpdateTree()
{
	int nSel = m_wndList.GetCurSel();

	m_wndProps.SetRedraw(FALSE);

	// Store the expanded nodes.
	LRESULT bExpanded;
	bExpanded = m_wndProps.IsItemExpanded(1000);  if(bExpanded != 0x80000001) m_expanded[1000] = bExpanded; 
//	bExpanded = m_wndProps.IsItemExpanded(2000);  if(bExpanded != 0x80000001) m_expanded[2000] = bExpanded; 

	// Update the property tree.
	m_wndProps.DeleteAllItems();
	
	if(nSel == -1)
	{
		m_wndProps.SetRedraw(TRUE);
		m_wndProps.Invalidate();
		return;
	}

	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();
	BYTE* pProps = pDoc->items[nSel].props;

	m_wndProps.SetTrueFalseText(_T("Yes"), _T("No"));

	m_wndProps.AddGroup(1000, "General");

		if(nSel >= 0 && nSel < 0x36)
			m_wndProps.AddItem(1000, 1001, "Name", PROPT_FF3EDIT, 7, (LPARAM)&pDoc->magic[nSel].name);
		else if(nSel >= 0x36 && nSel < 0x51)
			m_wndProps.AddItem(1000, 1001, "Name", PROPT_FF3EDIT, 8, (LPARAM)&pDoc->magic[nSel].name);
		else if(nSel >= 0x51 && nSel < 0x55)
			m_wndProps.AddItem(1000, 1001, "Name", PROPT_FF3EDIT, 10, (LPARAM)&pDoc->magic[nSel].name);
		else if(nSel >= 0x55 && nSel < 0x5D)
			m_wndProps.AddItem(1000, 1001, "Name", PROPT_FF3EDIT, 12, (LPARAM)&pDoc->magic[nSel].name);
		else if(nSel >= 0x5D && nSel < 0xFF)
			m_wndProps.AddItem(1000, 1001, "Name", PROPT_FF3EDIT, 10, (LPARAM)&pDoc->magic[nSel].name);

		m_wndProps.AddItem(1000, 1002, "MP Cost", PROPT_NUMEDIT, 0, (LPARAM)pDoc->magic[nSel].mp);
		m_wndProps.AddItem(1000, 1003, "Power", PROPT_NUMEDIT, 0, (LPARAM)pDoc->magic[nSel].power);
		m_wndProps.AddItem(1000, 1004, "Success Rate", PROPT_NUMEDIT, 0, (LPARAM)pDoc->magic[nSel].successRate);

	m_wndProps.AddGroup(2000, "Targetting");
		m_wndProps.AddItem(2000, 2001, "Affects single ally or enemy", PROPT_BOOLEAN, NULL, pDoc->magic[nSel].target & 0x01);
		m_wndProps.AddItem(2000, 2002, "Affects either enemies or allies only", PROPT_BOOLEAN, NULL, pDoc->magic[nSel].target & 0x02);
		m_wndProps.AddItem(2000, 2003, "Affects all allies and all enemies", PROPT_BOOLEAN, NULL, pDoc->magic[nSel].target & 0x04);
		m_wndProps.AddItem(2000, 2004, "Affects either all allies or all enemies", PROPT_BOOLEAN, NULL, pDoc->magic[nSel].target & 0x08);
		m_wndProps.AddItem(2000, 2005, "Auto-accept default selection", PROPT_BOOLEAN, NULL, pDoc->magic[nSel].target & 0x10);
		m_wndProps.AddItem(2000, 2006, "Multiple selection possible", PROPT_BOOLEAN, NULL, pDoc->magic[nSel].target & 0x20);
		m_wndProps.AddItem(2000, 2007, "Enemy is selected by default", PROPT_BOOLEAN, NULL, pDoc->magic[nSel].target & 0x40);
		m_wndProps.AddItem(2000, 2008, "Random selection among all enemies and allies", PROPT_BOOLEAN, NULL, pDoc->magic[nSel].target & 0x80);

	m_wndProps.AddGroup(4000, "Elemental Properties");
		m_wndProps.AddItem(4000, 4001, "Fire", PROPT_BOOLEAN, 0, pDoc->magic[nSel].elemProps & 0x01);
		m_wndProps.AddItem(4000, 4002, "Ice", PROPT_BOOLEAN, 0, pDoc->magic[nSel].elemProps & 0x02);
		m_wndProps.AddItem(4000, 4003, "Lightning", PROPT_BOOLEAN, 0, pDoc->magic[nSel].elemProps & 0x04);
		m_wndProps.AddItem(4000, 4004, "Poison", PROPT_BOOLEAN, 0, pDoc->magic[nSel].elemProps & 0x08);
		m_wndProps.AddItem(4000, 4005, "Wind", PROPT_BOOLEAN, 0, pDoc->magic[nSel].elemProps & 0x10);
		m_wndProps.AddItem(4000, 4006, "Holy", PROPT_BOOLEAN, 0, pDoc->magic[nSel].elemProps & 0x20);
		m_wndProps.AddItem(4000, 4007, "Earth", PROPT_BOOLEAN, 0, pDoc->magic[nSel].elemProps & 0x40);
		m_wndProps.AddItem(4000, 4008, "Water", PROPT_BOOLEAN, 0, pDoc->magic[nSel].elemProps & 0x80);

		m_wndProps.AddGroup(9200, "Status Conditions Affected when Used");
			m_wndProps.AddItem(9200, 9201, "Blindness", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond1 & 0x01);
			m_wndProps.AddItem(9200, 9202, "Zombie", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond1 & 0x02);
			m_wndProps.AddItem(9200, 9203, "Poison", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond1 & 0x04);
			m_wndProps.AddItem(9200, 9204, "Invisibility", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond1 & 0x10);
			m_wndProps.AddItem(9200, 9205, "Imp", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond1 & 0x20);
			m_wndProps.AddItem(9200, 9206, "Petrification", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond1 & 0x40);
			m_wndProps.AddItem(9200, 9207, "Death", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond1 & 0x80);
			m_wndProps.AddItem(9200, 9208, "Condemned", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond2 & 0x01);
			m_wndProps.AddItem(9200, 9209, "Kneeling", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond2 & 0x02);
			m_wndProps.AddItem(9200, 9210, "Blink", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond2 & 0x04);
			m_wndProps.AddItem(9200, 9211, "Silence", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond2 & 0x08);
			m_wndProps.AddItem(9200, 9212, "Berserk", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond2 & 0x10);
			m_wndProps.AddItem(9200, 9213, "Confusion", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond2 & 0x20);
			m_wndProps.AddItem(9200, 9214, "HP Drain", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond2 & 0x40);
			m_wndProps.AddItem(9200, 9215, "Sleep", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond2 & 0x80);
			m_wndProps.AddItem(9200, 9216, "Dance", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond3 & 0x01);
			m_wndProps.AddItem(9200, 9217, "Regenerate", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond3 & 0x02);
			m_wndProps.AddItem(9200, 9218, "Slow", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond3 & 0x04);
			m_wndProps.AddItem(9200, 9219, "Haste", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond3 & 0x08);
			m_wndProps.AddItem(9200, 9220, "Stop", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond3 & 0x10);
			m_wndProps.AddItem(9200, 9221, "Shell", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond3 & 0x20);
			m_wndProps.AddItem(9200, 9222, "Safe", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond3 & 0x40);
			m_wndProps.AddItem(9200, 9223, "Reflect", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond3 & 0x80);
			m_wndProps.AddItem(9200, 9224, "Rage", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond4 & 0x01);
			m_wndProps.AddItem(9200, 9225, "Frozen", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond4 & 0x02);
			m_wndProps.AddItem(9200, 9226, "Prevent Death Once", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond4 & 0x04);
			m_wndProps.AddItem(9200, 9227, "Morph into Esper", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond4 & 0x08);
			m_wndProps.AddItem(9200, 9228, "Casting Spell Stance", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond4 & 0x10);
			m_wndProps.AddItem(9200, 9229, "Removed from Battle", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond4 & 0x20);
			m_wndProps.AddItem(9200, 9230, "Randomly defended by Interceptor", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond4 & 0x40);
			m_wndProps.AddItem(9200, 9231, "Floating", PROPT_BOOLEAN, 0, pDoc->magic[nSel].cond4 & 0x80);

		/*	m_wndProps.AddGroup(2000, "Spells");
		m_wndProps.AddItem(2000, 2001, "Spell 1", PROPT_FF3COMBO, 0x10000 | 7, (LPARAM)&m_magicArray, (LPARAM)pDoc->espers[nSel].spell1);
		m_wndProps.AddItem(2000, 2002, "Spell 1 Learn Rate", PROPT_NUMEDIT, 0, (LPARAM)pDoc->espers[nSel].spell1Rate);
		m_wndProps.AddItem(2000, 2003, "Spell 2", PROPT_FF3COMBO, 0x10000 | 7, (LPARAM)&m_magicArray, (LPARAM)pDoc->espers[nSel].spell2);
		m_wndProps.AddItem(2000, 2004, "Spell 2 Learn Rate", PROPT_NUMEDIT, 0, (LPARAM)pDoc->espers[nSel].spell2Rate);
		m_wndProps.AddItem(2000, 2005, "Spell 3", PROPT_FF3COMBO, 0x10000 | 7, (LPARAM)&m_magicArray, (LPARAM)pDoc->espers[nSel].spell3);
		m_wndProps.AddItem(2000, 2006, "Spell 3 Learn Rate", PROPT_NUMEDIT, 0, (LPARAM)pDoc->espers[nSel].spell3Rate);
		m_wndProps.AddItem(2000, 2007, "Spell 4", PROPT_FF3COMBO, 0x10000 | 7, (LPARAM)&m_magicArray, (LPARAM)pDoc->espers[nSel].spell4);
		m_wndProps.AddItem(2000, 2008, "Spell 4 Learn Rate", PROPT_NUMEDIT, 0, (LPARAM)pDoc->espers[nSel].spell4Rate);
		m_wndProps.AddItem(2000, 2009, "Spell 5", PROPT_FF3COMBO, 0x10000 | 7, (LPARAM)&m_magicArray, (LPARAM)pDoc->espers[nSel].spell5);
		m_wndProps.AddItem(2000, 2010, "Spell 5 Learn Rate", PROPT_NUMEDIT, 0, (LPARAM)pDoc->espers[nSel].spell5Rate);
*/
	// Re-expand the nodes by expanded them all and then closing the ones
	// that should remain closed.
	m_wndProps.ExpandAll();

	UINT nID;  BOOL bExpand;
	POSITION pos = m_expanded.GetStartPosition(); 
	while(pos != NULL)
	{
		m_expanded.GetNextAssoc(pos, nID, bExpand);

		if(bExpand == FALSE)
			m_wndProps.Expand(m_wndProps.IndexToItem(nID), TVE_COLLAPSE);
	}

	m_wndProps.SetRedraw(TRUE);
	m_wndProps.Invalidate();
}

void CMagicEditView::OnSelChangeItemList() 
{
	UpdateTree();
}

void CMagicEditView::OnChangeProperties(NMHDR* pNMHDR, LRESULT* pResult) 
{
	int nSel = m_wndList.GetCurSel();

	NMPROPTREE* pNMPropTree = (NMPROPTREE*)pNMHDR;
	LPPROPERTYNODE node = pNMPropTree->node;

	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();

/*	if(node->id == 1001)
	{
		CRect rect;
		m_wndList.GetItemRect(nSel, &rect);
		m_wndList.InvalidateRect(&rect);
	}
	else if(node->id == 1002) pDoc->espers[nSel].bonus = node->lData2;
	else if(node->id == 2001) pDoc->espers[nSel].spell1 = node->lData2;
	else if(node->id == 2002) pDoc->espers[nSel].spell1Rate = node->lData;
	else if(node->id == 2003) pDoc->espers[nSel].spell2 = node->lData2;
	else if(node->id == 2004) pDoc->espers[nSel].spell2Rate = node->lData;
	else if(node->id == 2005) pDoc->espers[nSel].spell3 = node->lData2;
	else if(node->id == 2006) pDoc->espers[nSel].spell3Rate = node->lData;
	else if(node->id == 2007) pDoc->espers[nSel].spell4 = node->lData2;
	else if(node->id == 2008) pDoc->espers[nSel].spell4Rate = node->lData;
	else if(node->id == 2009) pDoc->espers[nSel].spell5 = node->lData2;
	else if(node->id == 2010) pDoc->espers[nSel].spell5Rate = node->lData;
*/
	*pResult = 0;
}
