// MapView.cpp : implementation of the CMapView class
//

#include "stdafx.h"
#include "FF3Edit.h"

#include "FF3Doc.h"
#include "MapView.h"

#include "MainFrame.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMapView

IMPLEMENT_DYNCREATE(CMapView, CWorldBaseView)

BEGIN_MESSAGE_MAP(CMapView, CWorldBaseView)
	//{{AFX_MSG_MAP(CMapView)
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_TIMER()
	ON_COMMAND(ID_EDIT_UNDO, OnEditUndo)
	ON_UPDATE_COMMAND_UI(ID_EDIT_UNDO, OnUpdateEditUndo)
	ON_COMMAND(ID_VIEW_GRIDLINES, OnViewGridlines)
	ON_UPDATE_COMMAND_UI(ID_VIEW_GRIDLINES, OnUpdateViewGridlines)
	ON_WM_CONTEXTMENU()
	ON_WM_RBUTTONDOWN()
	ON_UPDATE_COMMAND_UI(ID_EDIT_PASTE, OnUpdateEditPaste)
	ON_COMMAND(ID_EDIT_CUT, OnEditCut)
	ON_COMMAND(ID_EDIT_COPY, OnEditCopy)
	ON_COMMAND(ID_EDIT_PASTE, OnEditPaste)
	ON_WM_SETCURSOR()
	ON_WM_CREATE()
	ON_WM_DESTROY()
	//}}AFX_MSG_MAP
	ON_COMMAND_RANGE(ID_TOOLS_POINTER, ID_TOOLS_FILL, OnChangeTool)
	ON_UPDATE_COMMAND_UI_RANGE(ID_TOOLS_POINTER, ID_TOOLS_FILL, OnUpdateTool)
	ON_MESSAGE(WM_MOUSELEAVE, OnMouseLeave)
	ON_MESSAGE(WM_INITVIEW, OnInitView)
	ON_UPDATE_COMMAND_UI(ID_EDIT_CUT, OnUpdateEditCutOrCopy)
	ON_UPDATE_COMMAND_UI(ID_EDIT_COPY, OnUpdateEditCutOrCopy)

	ON_COMMAND(ID_VIEW_PROPERTIES, OnViewTileDialog)
	ON_UPDATE_COMMAND_UI(ID_VIEW_PROPERTIES, OnUpdateViewTileDialog)
	ON_COMMAND(ID_VIEW_TILEWINDOW, OnViewTileWindow)
	ON_UPDATE_COMMAND_UI(ID_VIEW_TILEWINDOW, OnUpdateViewTileWindow)
	ON_COMMAND(ID_VIEW_TRIGGERS, OnViewTriggerWindow)
	ON_UPDATE_COMMAND_UI(ID_VIEW_TRIGGERS, OnUpdateViewTriggerWindow)

	ON_MESSAGE(WM_COMPAREVIEW, OnCompareView)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMapView construction/destruction

CMapView::CMapView()
{
	m_worldData = NULL;

	m_ptMouseTile = CPoint(0, 0);
	m_ptCurrentTrigger = CPoint(0, 0);

	m_nRoundDown = 16;

	m_nCurrentTool = ID_TOOLS_POINTER;

	m_rcTool.SetRectEmpty();
	m_rcMarquee.SetRectEmpty();

	m_bTimerSet = FALSE;
}

CMapView::~CMapView()
{
	if(m_pClipboard) delete m_pClipboard;
}

/////////////////////////////////////////////////////////////////////////////
// CMapView drawing

void CMapView::OnDraw(CDC* pDC)
{
	CClientDC dc(this);

	CFF3Doc* pDoc = GetDocument();
	CRect clientRect;  GetClientRect(&clientRect);

	BeginDraw();

	if(m_worldData == NULL) { EndDraw();  return; }
	if(!pDoc->m_bValid) { EndDraw(); return; }

	DrawMap(DM_HighlightTriggers, (m_nCurrentTool == ID_TOOLS_POINTER) ? DM_HT_ALL : 0,
		TAG_END, 0);

	DoTool(MTA_DRAWNOLOCK, NULL);

	if(Settings.bMapGridlines)
	{
		for(int x = 0; x < min(256, m_worldData->size.cx); x++)
			VerticalLine(x * m_nRoundDown, 0,
			min(clientRect.bottom, m_worldData->size.cy * m_nRoundDown),
			DDColor(24, 24, 24));

		for(int y = 0; y < min(256, m_worldData->size.cy); y++)
			HorizontalLine(0, y * m_nRoundDown,
			min(clientRect.right, m_worldData->size.cx * m_nRoundDown),
			DDColor(24, 24, 24));
	}

	EndDraw();

	if(!m_rcMarquee.IsRectEmpty())
	{
		m_rcTracker.m_rect.left = (m_rcMarquee.left - GetScrollPosition().x) * 16;
		m_rcTracker.m_rect.top = (m_rcMarquee.top - GetScrollPosition().y) * 16;
		m_rcTracker.m_rect.right = (m_rcMarquee.right - GetScrollPosition().x) * 16;
		m_rcTracker.m_rect.bottom = (m_rcMarquee.bottom - GetScrollPosition().y) * 16;
		m_rcTracker.Draw(&dc);
	}
	else
	{
		m_rcTracker.m_rect.SetRectEmpty();
	}

	TOOLPACKAGE tp;
	tp.pDC = &dc;
	DoTool(MTA_DRAWAFTERFLIP, &tp);
}

/////////////////////////////////////////////////////////////////////////////
// CMapView diagnostics

#ifdef _DEBUG
void CMapView::AssertValid() const
{
	CWorldBaseView::AssertValid();
}

void CMapView::Dump(CDumpContext& dc) const
{
	CWorldBaseView::Dump(dc);
}

CFF3Doc* CMapView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CFF3Doc)));
	return (CFF3Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMapView message handlers

void CMapView::OnInitialUpdate()
{
	OnUpdate(NULL, 0, 0);
}

void CMapView::OnViewGridlines()
{
	if(Settings.bMapGridlines == TRUE) Settings.bMapGridlines = FALSE;
	else Settings.bMapGridlines = TRUE;

	Invalidate();
}

void CMapView::OnUpdateViewGridlines(CCmdUI* pCmdUI)
{
	if(Settings.bMapGridlines == TRUE) pCmdUI->SetCheck(TRUE);
	else pCmdUI->SetCheck(FALSE);	
}

PROPERTYITEM g_strBattleBg[] =
{
	{ 0, "Grass", NULL },
	{ 1, "Brown Forest", NULL },
	{ 2, "Desert", NULL },
	{ 3, "Green Forest", NULL },
	{ 4, "Building", NULL },
	{ 5, "World of Ruin", NULL },
	{ 6, "The Veldt", NULL },
	{ 7, "Falling through Clouds", NULL },
	{ -1, NULL, NULL },
};

PROPERTYITEM g_strAirshipShadow[] =
{
	{ 0, "Small", NULL },
	{ 1, "Medium", NULL },
	{ 2, "Larger", NULL },
	{ 3, "Largest", NULL },
	{ -1, NULL, NULL },
};

LRESULT CMapView::OnInitView(WPARAM wParam, LPARAM lParam)
{
	m_worldData = (WorldData*)lParam;
	return 0;
}

void CMapView::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView)
{
	if(bActivate)
	{
	}
	else
	{
	}
}

void CMapView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
	CFF3Doc* pDoc = GetDocument();

	if(!m_worldData) return;

	// Update the tile list.
	m_wndTiles.m_wndTileList.ResetContent();
	for(int i = 0; i < 256; i++)
	{
		int nIndex = m_wndTiles.m_wndTileList.AddString(NULL);
		m_wndTiles.m_wndTileList.SetItemData(nIndex, (DWORD)m_worldData);
	}

	// Update the trigger list.
	CString s, t;
	m_wndTriggers.m_worldData = m_worldData;
	m_wndTriggers.m_wndTriggerList.DeleteAllItems();

	for(i = 0; i < m_worldData->triggers.GetSize(); i++)
	{
		m_wndTriggers.AddTrigger(m_worldData->triggers[i]);
	}

	// Update the tile editing dialog.
	m_dlgTiles.m_worldData = m_worldData;
	m_dlgTiles.m_wndCurrentTile.m_worldData = m_worldData;
	m_dlgTiles.m_wndTileList.ResetContent();
	m_dlgTiles.m_wndTileList.SetColumnWidth(22);
	m_dlgTiles.m_wndTileList.SetItemHeight(0, 22);
	for(i = 0; i < 256; i++)
	{
		int nIndex = m_dlgTiles.m_wndTileList.AddString(NULL);
		m_dlgTiles.m_wndTileList.SetItemData(nIndex, (DWORD)m_worldData);
	}

	m_dlgTiles.m_wndSmallTileList.ResetContent();
	m_dlgTiles.m_wndSmallTileList.SetColumnWidth(22);
	m_dlgTiles.m_wndSmallTileList.SetItemHeight(0, 22);
	for(i = 0; i < 256; i++)
	{
		int nIndex = m_dlgTiles.m_wndSmallTileList.AddString(NULL);
		m_dlgTiles.m_wndSmallTileList.SetItemData(nIndex, (DWORD)m_worldData);
	}

	// Update the palette combo.
	m_dlgTiles.m_wndPalettes.ResetContent();
	for(i = 0; i < 8; i++)
	{
		CString s;  s.Format(_T("Palette %d"), i);
		m_dlgTiles.m_wndPalettes.AddString(s);
	}

	// Update the property tree.
	m_dlgTiles.m_wndPropTree.DeleteAllItems();
	m_dlgTiles.m_wndPropTree.SetTrueFalseText(_T("Yes"), _T("No"));
	m_dlgTiles.m_wndPropTree.AddGroup(1000, "Travelling");
		m_dlgTiles.m_wndPropTree.AddItem(1000, WDF_WALKDISABLE, "Walking disabled", PROPT_BOOLEAN, NULL);
		m_dlgTiles.m_wndPropTree.AddItem(1000, WDF_CHOCODISABLE, "Chocobo disabled", PROPT_BOOLEAN, NULL);
		m_dlgTiles.m_wndPropTree.AddItem(1000, WDF_AIRSHIPDISABLE, "Airship landing disabled", PROPT_BOOLEAN, NULL);
		m_dlgTiles.m_wndPropTree.AddItem(1000, WDF_CHARACTERTRANS, "Bottom half of character is transparent", PROPT_BOOLEAN, NULL);
	m_dlgTiles.m_wndPropTree.AddGroup(2000, "Vehicles");
		m_dlgTiles.m_wndPropTree.AddItem(2000, WDF_AIRSHIPSHADOW, "Size of shadow cast by airship", PROPT_COMBO, 0, (LPARAM)&g_strAirshipShadow);
	m_dlgTiles.m_wndPropTree.AddGroup(3000, "Battles");
		m_dlgTiles.m_wndPropTree.AddItem(3000, WDF_BATTLEBACKGROUND, "Background image", PROPT_COMBO, 0, (LPARAM)&g_strBattleBg);
		m_dlgTiles.m_wndPropTree.AddItem(3000, WDF_ENEMYENCOUNTERS, "Activate enemy encounters", PROPT_BOOLEAN, NULL);
	m_dlgTiles.m_wndPropTree.AddGroup(4000, "Other");
		m_dlgTiles.m_wndPropTree.AddItem(4000, WDF_ALTERNATEMUSIC, "Alternate Music", PROPT_BOOLEAN, NULL);
		m_dlgTiles.m_wndPropTree.AddItem(4000, WDF_UNKNOWN5, "Trigger Event when Landed on (?)", PROPT_BOOLEAN, NULL);
		m_dlgTiles.m_wndPropTree.AddItem(4000, WDF_UNKNOWN6, "Land Airship, then Trigger Event (?)", PROPT_BOOLEAN, NULL);
//	m_dlgTiles.m_wndPropTree.AddGroup(5000, "Unknowns");
//		m_dlgTiles.m_wndPropTree.AddItem(5000, WDF_UNKNOWN1, "Unknown 1", PROPT_BOOLEAN, NULL);
//		m_dlgTiles.m_wndPropTree.AddItem(5000, WDF_UNKNOWN2, "Unknown 2", PROPT_BOOLEAN, NULL);
//		m_dlgTiles.m_wndPropTree.AddItem(5000, WDF_UNKNOWN3, "Unknown 3", PROPT_BOOLEAN, NULL);

	m_dlgTiles.m_wndPropTree.ExpandAll();

	m_dlgTiles.m_wndTileList.SetCurSel(0);
	m_dlgTiles.SendCommandMessage(&m_dlgTiles.m_wndTileList, LBN_SELCHANGE);

	CSize sizeTotal, sizePage;
	sizePage.cx = sizePage.cy = 16;
	SetScrollSizes(m_worldData->size, sizePage);

	Invalidate();
}

BOOL CMapView::OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll) 
{
	BOOL res = CWorldBaseView::OnScroll(nScrollCode, nPos, bDoScroll);

	m_worldData->ppdLastScrollPos = GetScrollPosition();

	return res;
}

void CMapView::SetCurrentTrigger(CPoint pt)
{
	CPoint oldTrigger = m_ptCurrentTrigger;
	m_ptCurrentTrigger = pt;

	CPoint ptOldScreen((oldTrigger.x - GetScrollPosition().x) * 16,
		(oldTrigger.y - GetScrollPosition().y) * 16);
	CPoint ptScreen((pt.x - GetScrollPosition().x) * 16,
		(pt.y - GetScrollPosition().y) * 16);

	CRect rect;  GetClientRect(rect);

	if(rect.PtInRect(ptOldScreen))
		InvalidateRect(CRect(ptOldScreen.x, ptOldScreen.y, ptOldScreen.x + 16, ptOldScreen.y + 16));
	if(rect.PtInRect(ptScreen))
		InvalidateRect(CRect(ptScreen.x, ptScreen.y, ptScreen.x + 16, ptScreen.y + 16));
}

void CMapView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if(!m_worldData) return;

	SetCapture();

	if(m_bPasting) return;

	CRect clientRect;  GetClientRect(clientRect);

	TOOLPACKAGE tp;
	tp.xTile = GetScrollPosition().x + point.x / 16;
	if(tp.xTile < 0) tp.xTile = 0;
	else if(tp.xTile > m_worldData->size.cx - 1) tp.xTile = m_worldData->size.cx - 1;
	tp.yTile = GetScrollPosition().y + point.y / 16;
	if(tp.yTile < 0) tp.yTile = 0;
	else if(tp.yTile > m_worldData->size.cy - 1) tp.yTile = m_worldData->size.cy - 1;
	tp.xScreenTile = point.x / 16;
	if(tp.xScreenTile < 0) tp.xScreenTile = 0;
	else if(tp.xScreenTile > clientRect.Width() / 16) tp.xScreenTile = clientRect.Width() / 16;
	tp.yScreenTile = point.y / 16;
	if(tp.yScreenTile < 0) tp.yScreenTile = 0;
	else if(tp.yScreenTile > clientRect.Height() / 16) tp.yScreenTile = clientRect.Height() / 16;
	DoTool(MTA_ONBUTTONDOWN, &tp);

	CWorldBaseView::OnLButtonDown(nFlags, point);
}

void CMapView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(!m_worldData) return;

	CRect clientRect;  GetClientRect(clientRect);

	TOOLPACKAGE tp;
	tp.xTile = GetScrollPosition().x + point.x / 16;
	if(tp.xTile < 0) tp.xTile = 0;
	else if(tp.xTile > m_worldData->size.cx - 1) tp.xTile = m_worldData->size.cx - 1;
	tp.yTile = GetScrollPosition().y + point.y / 16;
	if(tp.yTile < 0) tp.yTile = 0;
	else if(tp.yTile > m_worldData->size.cy - 1) tp.yTile = m_worldData->size.cy - 1;
	tp.xScreenTile = point.x / 16;
	if(tp.xScreenTile < 0) tp.xScreenTile = 0;
	else if(tp.xScreenTile > clientRect.Width() / 16) tp.xScreenTile = clientRect.Width() / 16;
	tp.yScreenTile = point.y / 16;
	if(tp.yScreenTile < 0) tp.yScreenTile = 0;
	else if(tp.yScreenTile > clientRect.Height() / 16) tp.yScreenTile = clientRect.Height() / 16;

	DoTool(MTA_MOVENOCAPTURE, &tp);

	CString s;  s.Format("%d, %d", tp.xTile, tp.yTile);
	((CFrameWndEx*)GetParentFrame())->m_cBarStandard.SetControlText(
		((CFrameWndEx*)GetParentFrame())->m_cBarStandard.CommandToIndex(ID_CURSOR_POSITION), s);

	if(GetCapture())
	{
		DoTool(MTA_ONMOUSEMOVE, &tp);

		CPoint ptLastVisible(clientRect.Width() / 16, clientRect.Height() / 16);

		m_ptScrollOffset = CPoint(0, 0);
		if(point.x < 24) m_ptScrollOffset.x = -1;
		if(point.y < 24) m_ptScrollOffset.y = -1;
		if(point.x > clientRect.right - 24) m_ptScrollOffset.x = 1;
		if(point.y > clientRect.bottom - 24) m_ptScrollOffset.y = 1;

		if(m_ptScrollOffset != CPoint(0, 0))
		{
			if(!m_bTimerSet)
			{
				SetTimer(31337, 10, NULL);
				m_bTimerSet = TRUE;
			}
		}
		else
		{
			KillTimer(31337);
			m_bTimerSet = FALSE;
		}
	}

	m_ptLastMouse = point;

	CWorldBaseView::OnMouseMove(nFlags, point);

	TRACKMOUSEEVENT tme;
	tme.cbSize = sizeof(TRACKMOUSEEVENT);
	tme.dwFlags = TME_LEAVE;
	tme.hwndTrack = GetSafeHwnd();
	_TrackMouseEvent(&tme);
}

void CMapView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	CRect clientRect;  GetClientRect(clientRect);

	TOOLPACKAGE tp;
	tp.xTile = GetScrollPosition().x + point.x / 16;
	if(tp.xTile < 0) tp.xTile = 0;
	else if(tp.xTile > m_worldData->size.cx - 1) tp.xTile = m_worldData->size.cx - 1;
	tp.yTile = GetScrollPosition().y + point.y / 16;
	if(tp.yTile < 0) tp.yTile = 0;
	else if(tp.yTile > m_worldData->size.cy - 1) tp.yTile = m_worldData->size.cy - 1;
	tp.xScreenTile = point.x / 16;
	if(tp.xScreenTile < 0) tp.xScreenTile = 0;
	else if(tp.xScreenTile > clientRect.Width() / 16) tp.xScreenTile = clientRect.Width() / 16;
	tp.yScreenTile = point.y / 16;
	if(tp.yScreenTile < 0) tp.yScreenTile = 0;
	else if(tp.yScreenTile > clientRect.Height() / 16) tp.yScreenTile = clientRect.Height() / 16;

	if(m_bPasting)
	{
		COleDataObject obj;

		if(obj.AttachClipboard())
		{
			if(obj.IsDataAvailable(App.clipboardWorldMap))
			{
				HGLOBAL hmem = obj.GetGlobalData(App.clipboardWorldMap);
				CMemFile sf((BYTE*)::GlobalLock(hmem), ::GlobalSize(hmem));

				sf.Read(&m_bClipWidth, 1);
				sf.Read(&m_bClipHeight, 1);

				if(m_pClipboard) delete m_pClipboard;
				m_pClipboard = new BYTE[m_bClipWidth * m_bClipHeight];
				sf.Read(m_pClipboard, m_bClipWidth * m_bClipHeight);

				::GlobalUnlock(hmem);

				BOOL bFirst = TRUE;
				for(int y = tp.yTile; y < min(tp.yTile + m_bClipHeight, m_worldData->size.cy); y++)
				{
					for(int x = tp.xTile; x < min(tp.xTile + m_bClipWidth, m_worldData->size.cx); x++)
					{
						AddUndoAction(bFirst, x, y, m_worldData->GetMapTile(x, y));
						bFirst = FALSE;
						m_worldData->SetMapTile(x, y,
							m_pClipboard[(y - tp.yTile) * m_bClipWidth + (x - tp.xTile)]);
					}
				}

				m_ptMarqueeDrag = CPoint(-1, -1);
				m_rcTool.SetRectEmpty();
				m_rcMarquee.SetRectEmpty();
				Invalidate();
				m_bPasting = FALSE;

				ReleaseCapture();
				KillTimer(31337);
				m_bTimerSet = FALSE;
				m_ptScrollOffset = CPoint(0, 0);

				CWorldBaseView::OnLButtonUp(nFlags, point);
				return;
			}
		}
	}

	DoTool(MTA_ONBUTTONUP, &tp);

	ReleaseCapture();
	KillTimer(31337);
	m_bTimerSet = FALSE;
	m_ptScrollOffset = CPoint(0, 0);

	CWorldBaseView::OnLButtonUp(nFlags, point);
}

void CMapView::DoTool(int nContext, TOOLPACKAGE* tp)
{
	if(m_nCurrentTool == ID_TOOLS_POINTER) DoToolPointer(nContext, tp);
	else if(m_nCurrentTool == ID_TOOLS_PICKUP) DoToolPickup(nContext, tp);
	else if(m_nCurrentTool == ID_TOOLS_MARQUEE) DoToolMarquee(nContext, tp);
	else if(m_nCurrentTool == ID_TOOLS_PENCIL) DoToolPencil(nContext, tp);
	else if(m_nCurrentTool == ID_TOOLS_RECTFILL) DoToolRectFill(nContext, tp);
	else if(m_nCurrentTool == ID_TOOLS_FILL) DoToolFill(nContext, tp);
}

void CMapView::DoToolPencil(int nContext, TOOLPACKAGE* tp)
{
	CFF3Doc* pDoc = GetDocument();
	int bTile = m_wndTiles.m_wndTileList.GetCurSel();

	static BOOL bFirst;

	if(nContext == MTA_ONBUTTONDOWN)
	{
		if((m_worldData->GetMapTile(tp->xTile, tp->yTile) != bTile) &&
			(m_rcMarquee.IsRectEmpty() || m_rcMarquee.PtInRect(CPoint(tp->xTile, tp->yTile))))
		{
			AddUndoAction(TRUE, tp->xTile, tp->yTile,
				m_worldData->GetMapTile(tp->xTile, tp->yTile));

			bFirst = TRUE;
			m_worldData->SetMapTile(tp->xTile, tp->yTile, bTile);
			CRect rect(tp->xScreenTile * 16, tp->yScreenTile * 16,
				(tp->xScreenTile + 1) * 16, (tp->yScreenTile + 1) * 16);
			InvalidateRect(&rect);
			bFirst = FALSE;
		}
	}
	else if(nContext == MTA_ONMOUSEMOVE)
	{
		if((m_worldData->GetMapTile(tp->xTile, tp->yTile) != bTile) &&
			(m_rcMarquee.IsRectEmpty() || m_rcMarquee.PtInRect(CPoint(tp->xTile, tp->yTile))))
		{
			AddUndoAction(FALSE, tp->xTile, tp->yTile,
				m_worldData->GetMapTile(tp->xTile, tp->yTile));

			m_worldData->SetMapTile(tp->xTile, tp->yTile, bTile);
			CRect rect(tp->xScreenTile * 16, tp->yScreenTile * 16,
				(tp->xScreenTile + 1) * 16, (tp->yScreenTile + 1) * 16);
			InvalidateRect(&rect);
		}
	}
	else if(nContext == MTA_MOVENOCAPTURE)
	{
		InvalidateRect(&m_rcTool);

		m_rcToolOld = m_rcTool;

		m_rcTool = CRect(tp->xScreenTile * 16, tp->yScreenTile * 16,
			(tp->xScreenTile + 1) * 16, (tp->yScreenTile + 1) * 16);

		InvalidateRect(&m_rcTool);
	}
	else if(nContext == MTA_DRAWAFTERFLIP)
	{
		if(bFirst)
			tp->pDC->DrawDragRect(&m_rcTool, CSize(2, 2), &m_rcToolOld, CSize(2, 2));
		else
			tp->pDC->DrawDragRect(&m_rcTool, CSize(2, 2), NULL, CSize(2, 2));
	}
}

void CMapView::DoToolRectFill(int nContext, TOOLPACKAGE* tp)
{
	CFF3Doc* pDoc = GetDocument();
	int bTile = m_wndTiles.m_wndTileList.GetCurSel();

	static BOOL bFirst;

	if(nContext == MTA_ONBUTTONDOWN)
	{
		m_rcTool.left = tp->xTile;
		m_rcTool.top = tp->yTile;
		m_rcTool.right = tp->xTile + 1;
		m_rcTool.bottom = tp->yTile + 1;
		m_ptTool = CPoint(tp->xTile, tp->yTile);

		bFirst = TRUE;
		CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
			(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
		InvalidateRect(&rect);
		bFirst = FALSE;
	}
	else if(nContext == MTA_ONMOUSEMOVE)
	{
		CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
			(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
		InvalidateRect(&rect);

		m_rcToolOld = m_rcTool;

		m_rcTool.left = min(m_ptTool.x, tp->xTile + 1);
		m_rcTool.top = min(m_ptTool.y, tp->yTile + 1);
		m_rcTool.right = max(m_ptTool.x, tp->xTile + 1);
		m_rcTool.bottom = max(m_ptTool.y, tp->yTile + 1);

		if(m_rcTool.right == m_ptTool.x)
		{
			m_rcTool.left--;
			m_rcTool.right++;
		}
		if(m_rcTool.bottom == m_ptTool.y)
		{
			m_rcTool.top--;
			m_rcTool.bottom++;
		}

		rect = CRect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
			(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
		InvalidateRect(&rect);
	}
	else if(nContext == MTA_ONBUTTONUP)
	{
		CWaitCursor wc;

		BOOL bFirst = TRUE;
		for(int y = min(m_rcTool.top, m_rcTool.bottom); y < max(m_rcTool.top, m_rcTool.bottom); y++)
			for(int x = min(m_rcTool.left, m_rcTool.right); x < max(m_rcTool.left, m_rcTool.right); x++)
			{
				AddUndoAction(bFirst, x, y, m_worldData->GetMapTile(x, y));
				bFirst = FALSE;
				m_worldData->SetMapTile(x, y, bTile);
			}

		m_rcTool.SetRectEmpty();
		Invalidate();
	}
	else if(nContext == MTA_DRAWAFTERFLIP)
	{
		if(!m_rcTool.IsRectEmpty() && GetCapture() == this)
		{
			CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
					(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
			CRect rectOld((m_rcToolOld.left - GetScrollPosition().x) * 16, (m_rcToolOld.top - GetScrollPosition().y) * 16,
					(m_rcToolOld.right - GetScrollPosition().x) * 16, (m_rcToolOld.bottom - GetScrollPosition().y) * 16);

			if(bFirst)
				tp->pDC->DrawDragRect(&rect, CSize(2, 2), &rectOld, CSize(2, 2));
			else
				tp->pDC->DrawDragRect(&rect, CSize(2, 2), NULL, CSize(2, 2));
		}
	}
	else if(nContext == MTA_ONRBUTTONDOWN)
	{
		if(GetCapture() == this)
		{
			m_rcTool.SetRectEmpty();
			Invalidate();
		}
	}
}

void CMapView::DoToolMarquee(int nContext, TOOLPACKAGE* tp)
{
	CFF3Doc* pDoc = GetDocument();
	int bTile = m_wndTiles.m_wndTileList.GetCurSel();

	static BOOL bFirst;
	static BOOL bValid;

	if(nContext == MTA_ONBUTTONDOWN)
	{
		m_rcTool.left = tp->xTile;
		m_rcTool.top = tp->yTile;
		m_rcTool.right = tp->xTile;
		m_rcTool.bottom = tp->yTile;
		m_ptTool = CPoint(tp->xTile, tp->yTile);

		bValid = FALSE;
		bFirst = TRUE;
		CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
			(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
		InvalidateRect(&rect);
		bFirst = FALSE;

		if(!m_rcMarquee.IsRectEmpty() && m_pClipboard)
		{
			for(int y = 0; y < m_bClipHeight; y++)
			{
				for(int x = 0; x < m_bClipWidth; x++)
				{
					m_worldData->SetMapTile(x + m_rcMarquee.left, y + m_rcMarquee.top,
						m_pClipboard[y * m_bClipWidth + x]);
				}
			}

			delete m_pClipboard;
			m_pClipboard = NULL;
			m_bClipWidth = m_bClipHeight = 0;
		}
	}
	else if(nContext == MTA_ONMOUSEMOVE)
	{
		bValid = TRUE;

		CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
			(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
		InvalidateRect(&rect);

		if(!m_bPasting)
		{
			m_rcToolOld = m_rcTool;

			m_rcTool.left = min(m_ptTool.x, tp->xTile + 1);
			m_rcTool.top = min(m_ptTool.y, tp->yTile + 1);
			m_rcTool.right = max(m_ptTool.x, tp->xTile + 1);
			m_rcTool.bottom = max(m_ptTool.y, tp->yTile + 1);

			if(m_rcTool.right == m_ptTool.x)
			{
				m_rcTool.left--;
				m_rcTool.right++;
			}
			if(m_rcTool.bottom == m_ptTool.y)
			{
				m_rcTool.top--;
				m_rcTool.bottom++;
			}
		}

		rect = CRect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
			(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
		InvalidateRect(&rect);

		CString s;  s.Format("%d, %d", m_rcTool.Width(), m_rcTool.Height());
		((CFrameWndEx*)GetParentFrame())->m_cBarStandard.SetControlText(
			((CFrameWndEx*)GetParentFrame())->m_cBarStandard.CommandToIndex(ID_DRAG_SIZE), s);
		
	}
	else if(nContext == MTA_MOVENOCAPTURE)
	{
		if(m_bPasting)
		{
			CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
				(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
			InvalidateRect(&rect);

			m_rcToolOld = m_rcTool;

			m_rcTool.left = tp->xTile;
			m_rcTool.top = tp->yTile;
			m_rcTool.right = m_rcTool.left + m_bClipWidth;
			m_rcTool.bottom = m_rcTool.top + m_bClipHeight;

			rect = CRect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
				(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
			InvalidateRect(&rect);
		}
	}
	else if(nContext == MTA_ONBUTTONUP)
	{
		if(bValid)
		{
			// If it's valid, we just -completed- a marquee.  Cut the selection inside.
			m_rcMarquee = m_rcTool;

			m_bClipWidth = m_rcMarquee.Width();
			m_bClipHeight = m_rcMarquee.Height();

			if(m_pClipboard) delete m_pClipboard;
			m_pClipboard = new BYTE[m_bClipWidth * m_bClipHeight];

			for(int y = 0; y < m_bClipHeight; y++)
			{
				for(int x = 0; x < m_bClipWidth; x++)
				{
					m_pClipboard[y * m_bClipWidth + x] =
						m_worldData->GetMapTile(x + m_rcMarquee.left, y + m_rcMarquee.top);
					m_worldData->SetMapTile(x + m_rcMarquee.left, y + m_rcMarquee.top, -1);
				}
			}
		}
		else
		{
			// If it's valid, we just -removed- a marquee.  Paste the selection in the
			// destination.
			if(!m_rcMarquee.IsRectEmpty() && m_pClipboard)
			{
				for(int y = 0; y < m_bClipHeight; y++)
				{
					for(int x = 0; x < m_bClipWidth; x++)
					{
						m_worldData->SetMapTile(x + m_rcMarquee.left, y + m_rcMarquee.top,
							m_pClipboard[y * m_bClipWidth + x]);
					}
				}

				delete m_pClipboard;
				m_pClipboard = NULL;
				m_bClipWidth = m_bClipHeight = 0;
			}

			m_rcMarquee.SetRectEmpty();
		}

		m_rcTool.SetRectEmpty();
		m_ptMarqueeDrag = CPoint(-1, -1);
		Invalidate();
	}
	else if(nContext == MTA_DRAWAFTERFLIP)
	{
		if(!m_rcTool.IsRectEmpty() && (GetCapture() == this || m_bPasting))
		{
			CRect rect((m_rcTool.left - GetScrollPosition().x) * 16, (m_rcTool.top - GetScrollPosition().y) * 16,
					(m_rcTool.right - GetScrollPosition().x) * 16, (m_rcTool.bottom - GetScrollPosition().y) * 16);
			CRect rectOld((m_rcToolOld.left - GetScrollPosition().x) * 16, (m_rcToolOld.top - GetScrollPosition().y) * 16 + 2,
					(m_rcToolOld.right - GetScrollPosition().x) * 16, (m_rcToolOld.bottom - GetScrollPosition().y) * 16 + 2);

			if(bFirst)
				tp->pDC->DrawDragRect(&rect, CSize(2, 2), &rectOld, CSize(2, 2));
			else
				tp->pDC->DrawDragRect(&rect, CSize(2, 2), NULL, CSize(2, 2));
		}
	}
	else if(nContext == MTA_ONRBUTTONDOWN)
	{
		if(GetCapture() == this)
		{
			m_rcTool.SetRectEmpty();
			bValid = FALSE;
			Invalidate();
		}
	}
}

void CMapView::DoToolPickup(int nContext, TOOLPACKAGE* tp)
{
	CFF3Doc* pDoc = GetDocument();
	int bTile = m_wndTiles.m_wndTileList.GetCurSel();
	static BOOL bFirst = TRUE;

	if(nContext == MTA_ONBUTTONDOWN || nContext == MTA_ONMOUSEMOVE)
	{
		bTile = m_worldData->GetMapTile(tp->xTile, tp->yTile);
		m_wndTiles.m_wndTileList.SetCurSel(bTile);
	}
	else if(nContext == MTA_MOVENOCAPTURE)
	{
		InvalidateRect(&m_rcTool);
		bFirst = FALSE;

		m_rcToolOld = m_rcTool;

		m_rcTool = CRect(tp->xScreenTile * 16, tp->yScreenTile * 16,
			(tp->xScreenTile + 1) * 16, (tp->yScreenTile + 1) * 16);

		InvalidateRect(&m_rcTool);
	}
	else if(nContext == MTA_DRAWAFTERFLIP)
	{
		if(bFirst)
			tp->pDC->DrawDragRect(&m_rcTool, CSize(2, 2), &m_rcToolOld, CSize(2, 2));
		else
			tp->pDC->DrawDragRect(&m_rcTool, CSize(2, 2), NULL, CSize(2, 2));
	}
}

void CMapView::DoToolPointer(int nContext, TOOLPACKAGE* tp)
{
	CFF3Doc* pDoc = GetDocument();
	int bTile = m_wndTiles.m_wndTileList.GetCurSel();
	static BOOL bFirst = TRUE;

	if(nContext == MTA_ONBUTTONDOWN || nContext == MTA_ONMOUSEMOVE)
	{
		POSITION pos = m_wndTriggers.m_wndTriggerList.GetFirstSelectedItemPosition();
		if(!pos) return;
		int nOldSel = m_wndTriggers.m_wndTriggerList.GetNextSelectedItem(pos);

		Trigger* pTrigger = m_worldData->GetTrigger(tp->xTile, tp->yTile);

		for(int i = 0; i < m_worldData->triggers.GetSize(); i++)
		{
			if(pTrigger == m_worldData->triggers[i] && nOldSel != i)
			{
				m_wndTriggers.m_wndTriggerList.SetItemState(nOldSel, LVIS_SELECTED | LVIS_FOCUSED, 0);
				m_wndTriggers.m_wndTriggerList.SetItemState(i, LVIS_SELECTED | LVIS_FOCUSED, LVIS_SELECTED | LVIS_FOCUSED);
				m_wndTriggers.m_wndTriggerList.EnsureVisible(i, FALSE);
				SetCurrentTrigger(CPoint(pTrigger->srcX, pTrigger->srcY));

				break;
			}
		}
	}
	else if(nContext == MTA_MOVENOCAPTURE)
	{
		InvalidateRect(&m_rcTool);
		bFirst = FALSE;

		m_rcToolOld = m_rcTool;

		m_rcTool = CRect(tp->xScreenTile * 16, tp->yScreenTile * 16,
			(tp->xScreenTile + 1) * 16, (tp->yScreenTile + 1) * 16);

		InvalidateRect(&m_rcTool);
	}
	else if(nContext == MTA_DRAWAFTERFLIP)
	{
		if(bFirst)
			tp->pDC->DrawDragRect(&m_rcTool, CSize(2, 2), &m_rcToolOld, CSize(2, 2));
		else
			tp->pDC->DrawDragRect(&m_rcTool, CSize(2, 2), NULL, CSize(2, 2));
	}
}

void CMapView::DoToolFill(int nContext, TOOLPACKAGE* tp)
{
	CFF3Doc* pDoc = GetDocument();
	int bTile = m_wndTiles.m_wndTileList.GetCurSel();

	if(nContext == MTA_ONBUTTONDOWN)
	{
		m_fillSeed = CPoint(tp->xTile, tp->yTile);
		m_seedTile = m_worldData->GetMapTile(tp->xTile, tp->yTile);

		// Replace this ingenious hack with a real algorithm later on.
//		HackFill(tp->xTile, tp->yTile, bTile, m_worldData->GetMapTile(tp->xTile, tp->yTile));
//		RecursiveFill(tp->xTile, tp->yTile, bTile, m_worldData->GetMapTile(tp->xTile, tp->yTile), TRUE);
		Invalidate();
	}
}

void CMapView::HackFill(int x, int y, BYTE bFillTile, BYTE bOverTile)
{
	CWaitCursor waitCursor;

	int mx, my;

	// Create a memory DC.
	CWindowDC dcWindow(this);

	CDC dc;
	dc.CreateCompatibleDC(&dcWindow);
	CBitmap* pBitmap = new CBitmap;
	pBitmap->CreateCompatibleBitmap(&dcWindow, m_worldData->size.cx, m_worldData->size.cy);
	CBitmap* pOldBitmap = dc.SelectObject(pBitmap);

	for(my = 0; my < m_worldData->size.cy; my++)
	{
		for(mx = 0; mx < m_worldData->size.cx; mx++)
		{
			if(m_worldData->GetMapTile(mx, my) == bOverTile)
				dc.SetPixelV(mx, my, RGB(255, 255, 255));
			else
				dc.SetPixelV(mx, my, RGB(0, 0, 0));
		}
	}

	CBrush fillBrush(RGB(255, 0, 0));
	CBrush* pOldBrush = dc.SelectObject(&fillBrush);
	dc.ExtFloodFill(x, y, RGB(255, 255, 255), FLOODFILLSURFACE);
	dc.SelectObject(pOldBrush);

	int nChanges = 0;
	BOOL bFirst = TRUE;

	for(my = 0; my < m_worldData->size.cy; my++)
	{
		for(mx = 0; mx < m_worldData->size.cx; mx++)
		{
			if(dc.GetPixel(mx, my) == RGB(255, 0, 0))
			{
				nChanges++;
			}
		}
	}

	m_undoActions.SetSize(m_undoActions.GetSize() + nChanges);

	for(my = 0; my < m_worldData->size.cy; my++)
	{
		for(mx = 0; mx < m_worldData->size.cx; mx++)
		{
			if(dc.GetPixel(mx, my) == RGB(255, 0, 0))
			{
				if(::GetAsyncKeyState(VK_ESCAPE) & 0x8000)
				{
					MessageBeep(MB_OK);
					break;
				}

				AddUndoAction(bFirst, mx, my, m_worldData->GetMapTile(mx, my));
				bFirst = FALSE;

				m_worldData->SetMapTile(mx, my, bFillTile);

				if(IsTileVisible(mx, my))
				{
					CRect tileRect = GetTileScreenRect(x, y);
					InvalidateRect(&tileRect);
				}
			}
		}
	}

	dc.SelectObject(pOldBitmap);
	if(pBitmap) delete pBitmap;
}

void CMapView::RecursiveFill(int x, int y, BYTE bFillTile, BYTE bOverTile, BOOL bFirst)
{
	int bThisTile = m_worldData->GetMapTile(x, y);

/*	if(bThisTile == bOverTile)
	{
//		AddUndoAction(bFirst, x, y, m_worldData->map[y * m_worldData->size.cx + x]);

		m_worldData->map[y * m_worldData->size.cx + x] = bFillTile;

//		if(IsTileVisible(x, y))
//		{
//			CRect tileRect = GetTileScreenRect(x, y);
//			InvalidateRect(&tileRect);
//		}

		if(x != m_worldData->size.cx)
			RecursiveFill(x + 1, y, bFillTile, bOverTile, FALSE);
		if(x != 0)
			RecursiveFill(x - 1, y, bFillTile, bOverTile, FALSE);
		if(y != m_worldData->size.cy)
			RecursiveFill(x, y + 1, bFillTile, bOverTile, FALSE);
		if(y != 0)
			RecursiveFill(x, y - 1, bFillTile, bOverTile, FALSE);
	}
*/

	short left, right;
	BOOL leftFound = FALSE, rightFound = FALSE;
	
	left = right = x;
//	left = x - 1;
//	right = x;
	
	while(!leftFound)
	{
		if(m_worldData->GetMapTile(left, y) == bOverTile && left >= 0)
		{
			m_worldData->SetMapTile(left, y, bFillTile);
	       
			if(y > 0)
				if(bOverTile == m_worldData->GetMapTile(left, y - 1))
					RecursiveFill(left, y - 1, bFillTile, bOverTile, FALSE);
	           
			--left;
		}
		else
			leftFound = TRUE;
	}

	m_worldData->SetMapTile(m_fillSeed.x, m_fillSeed.y, m_seedTile);
	
    while(!rightFound)
	{
		if(m_worldData->GetMapTile(right, y) == bOverTile && right <= m_worldData->size.cx)
		{
			m_worldData->SetMapTile(right, y, bFillTile);
	       
			if(y < m_worldData->size.cy)
				if(bOverTile == m_worldData->GetMapTile(right, y + 1))
					RecursiveFill(right, y + 1, bFillTile, bOverTile, FALSE);
           
			++right;
		}
		else
			rightFound = TRUE;
	}

	/*	while(!rightFound)
	{
		if(!rightFound = GetPixel (right,y))
        {
			SetPixel (right,y);
	       
			if(!GetPixel (right,y+1))
				MyFloodFillPlus (right,y+1);
	           
			++right;
		}
	}*/
}

void CMapView::AddUndoAction(BOOL bFirst, int x, int y, BYTE bPrevTile)
{
	UNDOACTION ua;
	ua.bFirstAction = bFirst;
	ua.x = x;
	ua.y = y;
	ua.bPrevTile = bPrevTile;

	if(GetUndoActionCount() == 200)
	{
		m_undoActions.RemoveAt(0);
	}

	m_undoActions.Add(ua);
}

void CMapView::OnTimer(UINT nIDEvent) 
{
	if(nIDEvent == 31337)
	{
		CPoint point;  GetCursorPos(&point);  ScreenToClient(&point);
		CRect clientRect;  GetClientRect(&clientRect);

		ScrollToPosition(GetScrollPosition() + m_ptScrollOffset);

		TOOLPACKAGE tp;
		tp.xTile = GetScrollPosition().x + point.x / 16;
		if(tp.xTile < 0) tp.xTile = 0;
		else if(tp.xTile > m_worldData->size.cx - 1) tp.xTile = m_worldData->size.cx - 1;
		tp.yTile = GetScrollPosition().y + point.y / 16;
		if(tp.yTile < 0) tp.yTile = 0;
		else if(tp.yTile > m_worldData->size.cy - 1) tp.yTile = m_worldData->size.cy - 1;
		tp.xScreenTile = point.x / 16;
		if(tp.xScreenTile < 0) tp.xScreenTile = 0;
		else if(tp.xScreenTile > clientRect.Width() / 16) tp.xScreenTile = clientRect.Width() / 16;
		tp.yScreenTile = point.y / 16;
		if(tp.yScreenTile < 0) tp.yScreenTile = 0;
		else if(tp.yScreenTile > clientRect.Height() / 16) tp.yScreenTile = clientRect.Height() / 16;

		DoTool(MTA_ONMOUSEMOVE, &tp);
	}
	else
		CWorldBaseView::OnTimer(nIDEvent);
}

void CMapView::OnChangeTool(UINT nID)
{
	ASSERT(nID >= ID_TOOLS_POINTER && nID <= ID_TOOLS_FILL);

	int nOldTool = m_nCurrentTool;
	m_nCurrentTool = nID;

	if(nOldTool == ID_TOOLS_POINTER || m_nCurrentTool == ID_TOOLS_POINTER)
		Invalidate();
}

void CMapView::OnUpdateTool(CCmdUI* pCmdUI)
{
	if(pCmdUI->m_nID == m_nCurrentTool)
	{
		pCmdUI->SetCheck(TRUE);
		return;
	}

	// default to not checked
	pCmdUI->SetCheck(FALSE);

	if(pCmdUI->m_nID == ID_TOOLS_FILL || pCmdUI->m_nID == ID_TOOLS_LINE || pCmdUI->m_nID == ID_TOOLS_RECT)
		pCmdUI->Enable(FALSE);
}

void CMapView::OnEditUndo() 
{
	BOOL bDone = FALSE;

	while(!bDone)
	{
		UNDOACTION ua = m_undoActions[m_undoActions.GetUpperBound()];

		bDone = ua.bFirstAction;

		m_worldData->SetMapTile(ua.x, ua.y, ua.bPrevTile);

		if(IsTileVisible(ua.x, ua.y))
		{
			CRect tileRect = GetTileScreenRect(ua.x, ua.y);
			InvalidateRect(&tileRect);
		}

		m_undoActions.RemoveAt(m_undoActions.GetUpperBound());
		
		if(m_undoActions.GetSize() == 0) bDone = TRUE;
	}
}

void CMapView::OnUpdateEditUndo(CCmdUI* pCmdUI) 
{
	if(m_undoActions.GetSize())
		pCmdUI->Enable(TRUE);
	else
		pCmdUI->Enable(FALSE);
}

int CMapView::GetUndoActionCount()
{
	int nActions = 0;

	for(int i = 0; i < m_undoActions.GetSize(); i++)
	{
		UNDOACTION ua = m_undoActions[i];
		if(ua.bFirstAction) nActions++;
	}

	return nActions;
}

void CMapView::OnContextMenu(CWnd* pWnd, CPoint point) 
{
	TOOLPACKAGE tp;
	tp.xTile = GetScrollPosition().x + point.x / 16;
	tp.yTile = GetScrollPosition().y + point.y / 16;
	tp.xScreenTile = point.x / 16;
	tp.yScreenTile = point.y / 16;

	DoTool(MTA_ONCONTEXTMENU, &tp);
	ReleaseCapture();
}

void CMapView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	TOOLPACKAGE tp;
	tp.xTile = GetScrollPosition().x + point.x / 16;
	tp.yTile = GetScrollPosition().y + point.y / 16;
	tp.xScreenTile = point.x / 16;
	tp.yScreenTile = point.y / 16;

	DoTool(MTA_ONRBUTTONDOWN, &tp);
	ReleaseCapture();
	
	CWorldBaseView::OnRButtonDown(nFlags, point);
}

LRESULT CMapView::OnMouseLeave(WPARAM wParam, LPARAM lParam)
{
	CClientDC dc(this);
	if(!m_rcTool.IsRectEmpty())
	{
		dc.DrawDragRect(CRect(0, 0, 0, 0), CSize(2, 2), &m_rcTool, CSize(2, 2));
		m_rcTool.SetRectEmpty();
	}

	return 0;
}

void CMapView::OnUpdateEditCutOrCopy(CCmdUI* pCmdUI) 
{
	if(m_rcMarquee.IsRectEmpty()) pCmdUI->Enable(FALSE);
	else pCmdUI->Enable(TRUE);
}

void CMapView::OnUpdateEditPaste(CCmdUI* pCmdUI) 
{
	COleDataObject obj;

	if(obj.AttachClipboard())
	{
		if(obj.IsDataAvailable(App.clipboardWorldMap))
		{
			pCmdUI->Enable(TRUE);
			if(m_bPasting) pCmdUI->SetCheck(TRUE);
			else pCmdUI->SetCheck(FALSE);
			return;
		}
	}

	pCmdUI->SetCheck(FALSE);
	pCmdUI->Enable(FALSE);
}

void CMapView::OnEditCut() 
{
	COleDataSource*	pSource = new COleDataSource();
	CSharedFile	sf(GMEM_MOVEABLE | GMEM_DDESHARE | GMEM_ZEROINIT);

	sf.Write(&m_bClipWidth, 1);
	sf.Write(&m_bClipHeight, 1);
	sf.Write(m_pClipboard, m_bClipWidth * m_bClipHeight);

	delete m_pClipboard;
	m_pClipboard = NULL;
	m_bClipWidth = m_bClipHeight = 0;
	Invalidate();

	HGLOBAL hMem = sf.Detach();
	if(!hMem) return;
	
	pSource->CacheGlobalData(App.clipboardWorldMap, hMem);
	pSource->SetClipboard();

//#if _MFC_VER <= 0x0600
//	::GlobalUnlock(hMem);
//#endif 
}

void CMapView::OnEditCopy() 
{
	COleDataSource*	pSource = new COleDataSource();
	CSharedFile	sf(GMEM_MOVEABLE | GMEM_DDESHARE | GMEM_ZEROINIT);

	for(int y = 0; y < m_bClipHeight; y++)
	{
		for(int x = 0; x < m_bClipWidth; x++)
		{
			m_worldData->SetMapTile(x + m_rcMarquee.left, y + m_rcMarquee.top,
				m_pClipboard[y * m_bClipWidth + x]);
		}
	}

//	delete m_pClipboard;
//	m_pClipboard = NULL;
//	m_bClipWidth = m_bClipHeight = 0;

	sf.Write(&m_bClipWidth, 1);
	sf.Write(&m_bClipHeight, 1);
	sf.Write(m_pClipboard, m_bClipWidth * m_bClipHeight);

	HGLOBAL hMem = sf.Detach();
	if(!hMem) return;
	
	pSource->CacheGlobalData(App.clipboardWorldMap, hMem);
	pSource->SetClipboard();

#if _MFC_VER <= 0x0600
	::GlobalUnlock(hMem);
#endif 
}

void CMapView::OnEditPaste() 
{
	COleDataObject obj;

	if(obj.AttachClipboard())
	{
		if(obj.IsDataAvailable(App.clipboardWorldMap))
		{
			m_bPasting = TRUE;
		}
	}
}

BOOL CMapView::OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message) 
{
	CPoint point;  ::GetCursorPos(&point);  ScreenToClient(&point);
	int nHit = m_rcTracker.HitTest(point);

	if(nHit == CRectTracker::hitMiddle && m_nCurrentTool == ID_TOOLS_MARQUEE)
	{
//		::SetCursor(AfxGetApp()->LoadStandardCursor(IDC_SIZEALL));

		if(m_ptMarqueeDrag == CPoint(-1, -1))
		{
			m_ptMarqueeDrag.x = (point.x - m_rcTracker.m_rect.left) / 16;
			m_ptMarqueeDrag.y = (point.y - m_rcTracker.m_rect.top) / 16;
		}
		return TRUE;
	}
	else
	{
		// Change the cursor based on which tool is selected.
		int nCursor = 0;

		if(m_nCurrentTool == ID_TOOLS_PENCIL) nCursor = IDC_TOOLS_PENCIL;
		else if(m_nCurrentTool == ID_TOOLS_PICKUP) nCursor = IDC_TOOLS_PICKUP;
		else if(m_nCurrentTool == ID_TOOLS_MARQUEE) nCursor = IDC_TOOLS_MARQUEE;
		else if(m_nCurrentTool == ID_TOOLS_RECT || m_nCurrentTool == ID_TOOLS_RECTFILL)
			nCursor = IDC_TOOLS_RECT;
		else if(m_nCurrentTool == ID_TOOLS_FILL) nCursor = IDC_TOOLS_FILL;
		else if(m_nCurrentTool == ID_TOOLS_LINE) nCursor = IDC_TOOLS_LINE;

		if(nCursor && nHitTest == HTCLIENT)
		{
			::SetCursor(AfxGetApp()->LoadCursor(MAKEINTRESOURCE(nCursor)));
			return TRUE;
		}
		else
			return CWorldBaseView::OnSetCursor(pWnd, nHitTest, message);
	}
}

int CMapView::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CWorldBaseView::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_wndTiles.Create(this);
	m_wndTriggers.Create(this);
	m_dlgTiles.Create(this);

	return 0;
}

void CMapView::OnDestroy()
{
	m_dlgTiles.DestroyWindow();
	m_wndTriggers.DestroyWindow();
	m_wndTiles.DestroyWindow();

	CWorldBaseView::OnDestroy();
}

void CMapView::OnViewTileDialog() 
{
	if(!m_dlgTiles.IsWindowVisible())
		m_dlgTiles.ShowWindow(SW_SHOW);
	else
		m_dlgTiles.ShowWindow(SW_HIDE);
}

void CMapView::OnUpdateViewTileDialog(CCmdUI* pCmdUI) 
{
	if(!m_dlgTiles.IsWindowVisible())
		pCmdUI->SetCheck(FALSE);
	else
		pCmdUI->SetCheck(TRUE);
}

void CMapView::OnViewTileWindow() 
{
	if(!m_wndTiles.IsWindowVisible())
		m_wndTiles.ShowWindow(SW_SHOW);
	else
		m_wndTiles.ShowWindow(SW_HIDE);
}

void CMapView::OnUpdateViewTileWindow(CCmdUI* pCmdUI) 
{
	if(!m_wndTiles.IsWindowVisible())
		pCmdUI->SetCheck(FALSE);
	else
		pCmdUI->SetCheck(TRUE);
}

void CMapView::OnViewTriggerWindow() 
{
	if(!m_wndTriggers.IsWindowVisible())
		m_wndTriggers.ShowWindow(SW_SHOW);
	else
		m_wndTriggers.ShowWindow(SW_HIDE);
}

void CMapView::OnUpdateViewTriggerWindow(CCmdUI* pCmdUI) 
{
	if(!m_wndTriggers.IsWindowVisible())
		pCmdUI->SetCheck(FALSE);
	else
		pCmdUI->SetCheck(TRUE);
}

LRESULT CMapView::OnCompareView(WPARAM wParam, LPARAM lParam)
{
	if((WorldData*)lParam == m_worldData) return TRUE;
	return FALSE;
}
