// MapView.h : interface of the CMapView class
//
/////////////////////////////////////////////////////////////////////////////

#include "WorldBaseView.h"

#include "MapViewDlg.h"
#include "MapTileDlg.h"
#include "MapTriggerDlg.h"

class CMapView : public CWorldBaseView
{
protected: // create from serialization only
	CMapView();
	DECLARE_DYNCREATE(CMapView)

	typedef struct
	{
		int xTile;
		int yTile;
		int xScreenTile;
		int yScreenTile;
		LPDDSURFACEDESC2 lpDest;
		CDC* pDC;
	} TOOLPACKAGE;

	typedef struct
	{
		BOOL bFirstAction;
		int x;
		int y;
		BYTE bPrevTile;
	} UNDOACTION;

	enum ToolContexts
	{
		MTA_ONBUTTONDOWN = 1,
		MTA_ONMOUSEMOVE,
		MTA_ONBUTTONUP,
		MTA_DRAW,
		MTA_DRAWNOLOCK,
		MTA_SETCURSOR,
		MTA_MOVENOCAPTURE,
		MTA_ONDOUBLECLICK,
		MTA_ONCONTEXTMENU,
		MTA_ONRBUTTONDOWN,
		MTA_DRAWAFTERFLIP
	};

	enum OverScrollDirections
	{
		OSD_LEFT = 1,
		OSD_RIGHT,
		OSD_UP,
		OSD_DOWN
	};

// Attributes
public:
	CFF3Doc* GetDocument();

protected:
	UINT m_nCurrentTool;
	CRect m_rcTool, m_rcToolOld;
	CPoint m_ptTool;
	CPoint m_ptMarqueeDrag;

	CArray<UNDOACTION, UNDOACTION> m_undoActions;
	CPoint m_fillSeed;
	BYTE m_seedTile;

	CMapViewDlg m_dlgTiles;
	CMapTileDlg m_wndTiles;
	CMapTriggerDlg m_wndTriggers;

// Operations
public:

protected:
	void Put8x8ChunkyTile(DDSURFACEDESC2* ddsd, int dx, int dy, BYTE bTile);
	void Put8x8ChunkyTileT(DDSURFACEDESC2* ddsd, int dx, int dy, BYTE bTile, float opacity);
	void Put16x16Tile(DDSURFACEDESC2* ddsd, int dx, int dy, BYTE bTile);
	void Put16x16TileT(DDSURFACEDESC2* ddsd, int dx, int dy, BYTE bTile, float opacity);

	void DoTool(int nContext, TOOLPACKAGE* tp);
	void DoToolPencil(int nContext, TOOLPACKAGE* tp);
	void DoToolPickup(int nContext, TOOLPACKAGE* tp);
	void DoToolMarquee(int nContext, TOOLPACKAGE* tp);
	void DoToolPointer(int nContext, TOOLPACKAGE* tp);
	void DoToolRectFill(int nContext, TOOLPACKAGE* tp);
	void DoToolFill(int nContext, TOOLPACKAGE* tp);

	void RecursiveFill(int x, int y, BYTE bFillTile, BYTE bOverTile, BOOL bFirst);
	void HackFill(int x, int y, BYTE bFillTile, BYTE bOverTile);

	void AddUndoAction(BOOL bFirst, int x, int y, BYTE bPrevTile);

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CMapView)
	public:
	virtual void OnDraw(CDC* pDC);  // overridden to draw this view
	virtual void OnInitialUpdate();
	virtual BOOL OnScroll(UINT nScrollCode, UINT nPos, BOOL bDoScroll = TRUE);
	protected:
	virtual void OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint);
	virtual void OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView);
	//}}AFX_VIRTUAL

// Implementation
public:
	void SetCurrentTrigger(CPoint pt);
	virtual ~CMapView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	int GetUndoActionCount();
	//{{AFX_MSG(CMapView)
	afx_msg void OnMouseMove(UINT nFlags, CPoint point);
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnLButtonUp(UINT nFlags, CPoint point);
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnEditUndo();
	afx_msg void OnUpdateEditUndo(CCmdUI* pCmdUI);
	afx_msg void OnViewGridlines();
	afx_msg void OnUpdateViewGridlines(CCmdUI* pCmdUI);
	afx_msg void OnContextMenu(CWnd* pWnd, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnUpdateEditPaste(CCmdUI* pCmdUI);
	afx_msg void OnEditCut();
	afx_msg void OnEditCopy();
	afx_msg void OnEditPaste();
	afx_msg BOOL OnSetCursor(CWnd* pWnd, UINT nHitTest, UINT message);
	afx_msg int OnCreate(LPCREATESTRUCT lpCreateStruct);
	afx_msg void OnDestroy();
	//}}AFX_MSG
	afx_msg void OnChangeTool(UINT nID);
	afx_msg void OnUpdateTool(CCmdUI* pCmdUI);
	afx_msg void OnUpdateEditCutOrCopy(CCmdUI* pCmdUI);
	afx_msg LRESULT OnMouseLeave(WPARAM wParam, LPARAM lParam);
	afx_msg LRESULT OnInitView(WPARAM wParam, LPARAM lParam);

	afx_msg void OnViewTileDialog();
	afx_msg void OnUpdateViewTileDialog(CCmdUI* pCmdUI);
	afx_msg void OnViewTileWindow();
	afx_msg void OnUpdateViewTileWindow(CCmdUI* pCmdUI);
	afx_msg void OnViewTriggerWindow();
	afx_msg void OnUpdateViewTriggerWindow(CCmdUI* pCmdUI);

	afx_msg LRESULT OnCompareView(WPARAM wParam, LPARAM lParam);

	DECLARE_MESSAGE_MAP()
};

#ifndef _DEBUG  // debug version in MapView.cpp
inline CFF3Doc* CMapView::GetDocument()
   { return (CFF3Doc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////
