// MapViewDlg.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "MapViewDlg.h"

#include "DIBSectionLite.h"
#include "PaletteEditDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMapViewDlg dialog


CMapViewDlg::CMapViewDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CMapViewDlg::IDD, pParent)
{
	//{{AFX_DATA_INIT(CMapViewDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CMapViewDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CMapViewDlg)
	DDX_Control(pDX, IDC_TFP_PROPTREE, m_wndPropTree);
	DDX_Control(pDX, IDC_PAL_PALETTES, m_wndPalettes);
	DDX_Control(pDX, IDC_PAL_COLORS, m_wndColors);
	DDX_Control(pDX, IDC_TFP_TILEEDIT, m_wndTileEdit);
	DDX_Control(pDX, IDC_TFP_CURRENTTILE, m_wndCurrentTile);
	DDX_Control(pDX, IDC_TFP_SMALLTILELIST, m_wndSmallTileList);
	DDX_Control(pDX, IDC_TFP_TILELIST, m_wndTileList);
	DDX_Control(pDX, IDC_IMPORT, m_wndImport);
	DDX_Control(pDX, IDC_EXPORT, m_wndExport);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CMapViewDlg, CDialog)
	//{{AFX_MSG_MAP(CMapViewDlg)
	ON_WM_CLOSE()
	ON_WM_DESTROY()
	ON_LBN_SELCHANGE(IDC_TFP_TILELIST, OnSelchangeTileList)
	ON_LBN_SELCHANGE(IDC_TFP_SMALLTILELIST, OnSelchangeSmallTileList)
	ON_CBN_SELENDOK(IDC_PAL_PALETTES, OnSelendokPalPalettes)
	ON_NOTIFY(CSN_SELCHANGED, IDC_PAL_COLORS, OnSelchangePalColors)
	ON_NOTIFY(CSN_COLOREDITED, IDC_PAL_COLORS, OnEditPalColors)
	ON_CONTROL(TFW_TILEPARTCHANGE, IDC_TFP_CURRENTTILE, OnPartChangeCurrentTile)
	ON_CONTROL(TEW_TILEEDIT, IDC_TFP_TILEEDIT, OnEditCurrentTile)
	ON_NOTIFY(PROPTN_PROPCHANGED, IDC_TFP_PROPTREE, OnChangeProperties)
	ON_COMMAND(ID_IMPORT_TILEPIECES, OnImportTilePieces)
	ON_COMMAND(ID_EXPORT_WHOLETILES, OnExportWholeTiles)
	ON_COMMAND(ID_EXPORT_TILEPIECES, OnExportTilePieces)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CMapViewDlg message handlers

BOOL CMapViewDlg::OnInitDialog() 
{
	CDialog::OnInitDialog();

	m_wndImport.SetMenuID(IDR_BUTTONMENUS, 0);
	m_wndExport.SetMenuID(IDR_BUTTONMENUS, 1);
	m_wndTileEdit.SetSizes(8, 8, 0);

	SetWindowPos(NULL, Settings.wsTileDlg.rect.left, Settings.wsTileDlg.rect.top, 0, 0,
		(Settings.wsTileDlg.bVisible ? SWP_SHOWWINDOW : SWP_HIDEWINDOW) | SWP_NOSIZE | SWP_NOZORDER);

	return TRUE;  // return TRUE unless you set the focus to a control
	              // EXCEPTION: OCX Property Pages should return FALSE
}

void CMapViewDlg::OnClose() 
{
	ShowWindow(SW_HIDE);
}

void CMapViewDlg::OnDestroy() 
{
	GetWindowRect(&Settings.wsTileDlg.rect);
	Settings.wsTileDlg.bVisible = (IsWindowVisible()) ? TRUE : FALSE;

	CDialog::OnDestroy();
}

BOOL CMapViewDlg::Create(CView* pView) 
{
	m_pView = pView;
	return CDialog::Create(IDD, pView->GetParentFrame());
}

void CMapViewDlg::OnSelchangeTileList() 
{
	int sel = m_wndTileList.GetCurSel();
	m_wndCurrentTile.SetCurrentTile(sel);

	UpdateTileProperties(sel);
}

void CMapViewDlg::OnSelchangeSmallTileList() 
{
	int sel = m_wndSmallTileList.GetCurSel();

	m_worldData->tileFormations[m_wndCurrentTile.m_bCurrentTile * 4 + m_wndCurrentTile.m_bCurrentTilePart] = sel;

	m_wndCurrentTile.Invalidate();

	UpdatePalette(m_worldData->tilePalettes[sel] / 16);
	UpdateTileEdit(sel);

	CRect itemRect;  m_wndTileList.GetItemRect(m_wndCurrentTile.m_bCurrentTile, &itemRect);
	m_wndTileList.InvalidateRect(&itemRect);

	m_pView->Invalidate();
}

void CMapViewDlg::OnPartChangeCurrentTile()
{
	int sel = m_worldData->tileFormations[m_wndCurrentTile.m_bCurrentTile * 4 + m_wndCurrentTile.m_bCurrentTilePart];
	UpdateSmallTileList(sel);
}

void CMapViewDlg::OnEditCurrentTile()
{
	m_wndTileList.Invalidate();
	m_wndSmallTileList.Invalidate();
	m_wndCurrentTile.Invalidate();
	m_pView->Invalidate();
}

void CMapViewDlg::OnSelendokPalPalettes() 
{
	int sel = m_wndPalettes.GetCurSel();

	m_worldData->tilePalettes[m_wndSmallTileList.GetCurSel()] = sel * 16;

	m_wndTileList.Invalidate();
	m_wndSmallTileList.Invalidate();
	m_wndCurrentTile.Invalidate();
	m_pView->Invalidate();

	UpdateColors(sel);
}

void CMapViewDlg::OnSelchangePalColors(NMHDR* pNotifyStruct, LRESULT* result)
{
	LPNMCOLORSELECTOR lpNMCS = (LPNMCOLORSELECTOR)pNotifyStruct;

	if(lpNMCS->nColorChanged == CSNC_FOREGROUND)
		m_wndTileEdit.SetCurrentColors(lpNMCS->nIndex, -1);
	else if(lpNMCS->nColorChanged == CSNC_BACKGROUND)
		m_wndTileEdit.SetCurrentColors(-1, lpNMCS->nIndex);
}

void CMapViewDlg::OnEditPalColors(NMHDR* pNotifyStruct, LRESULT* result)
{
	LPNMCOLORSELECTOR lpNMCS = (LPNMCOLORSELECTOR)pNotifyStruct;

	CString s;  s.Format("%d", m_wndPalettes.GetCurSel() * 16 + lpNMCS->nIndex);
	MessageBox(s);

	m_worldData->palette[m_wndPalettes.GetCurSel() * 16 + lpNMCS->nIndex] =
		lpNMCS->wRGB;
	m_wndTileEdit.SetColor(lpNMCS->nIndex, lpNMCS->wRGB);

	m_wndCurrentTile.Invalidate();
	m_wndTileList.Invalidate();
	m_wndSmallTileList.Invalidate();
	m_wndTileEdit.Invalidate();
	m_pView->Invalidate();
}

void CMapViewDlg::OnChangeProperties(NMHDR* pNMHDR, LRESULT* pResult) 
{
	int sel = m_wndTileList.GetCurSel();

	NMPROPTREE* pNMPropTree = (NMPROPTREE*)pNMHDR;

	if(pNMPropTree->node->id != WDF_BATTLEBACKGROUND)
	{
		m_worldData->tileProperties[sel] = (m_worldData->tileProperties[sel] & ~pNMPropTree->node->id) |
			((pNMPropTree->node->lData) ? pNMPropTree->node->id : 0);
	}
	else
	{
		m_worldData->tileProperties[sel] = (m_worldData->tileProperties[sel] & ~pNMPropTree->node->id) |
			(pNMPropTree->node->lData2 << 8);
	}

	*pResult = 0;
}

void CMapViewDlg::UpdateSmallTileList(int sel)
{
	m_wndSmallTileList.SetCurSel(sel);
	UpdatePalette(m_worldData->tilePalettes[sel] / 16);
	UpdateTileEdit(sel);
}

void CMapViewDlg::UpdateTileEdit(int sel)
{
	BYTE bAll, b1, b2;

	for(int y = 0; y < 8; y++)
	{
		for(int x = 0; x < 8; x += 2)
		{
			bAll = m_worldData->tiles[sel * 0x20 + (y * 4 + x / 2)];
			b1 = (bAll >> 4) & 0xF;
			b2 = (bAll) & 0xF;

			m_wndTileEdit.SetPixel(x + 1, y, b1);
			m_wndTileEdit.SetPixel(x, y, b2);
		}
	}
}

void CMapViewDlg::UpdatePalette(int sel)
{
	m_wndPalettes.SetCurSel(sel);
	UpdateColors(sel);
}

void CMapViewDlg::UpdateColors(int sel)
{
	m_wndColors.ResetContent();

	WORD colors[16];
	for(int i = 0; i < 16; i++)
		colors[i] = m_worldData->palette[sel * 16 + i];

	m_wndColors.AddColors(colors, 16);
	m_wndTileEdit.SetColors(colors, 16);
}

void CMapViewDlg::UpdateTileProperties(int sel)
{
	WORD wProp = m_worldData->tileProperties[sel];

	m_wndPropTree.SetItem(WDF_WALKDISABLE, ((wProp & WDF_WALKDISABLE) ? TRUE : FALSE));
	m_wndPropTree.SetItem(WDF_CHOCODISABLE, ((wProp & WDF_CHOCODISABLE) ? TRUE : FALSE));
	m_wndPropTree.SetItem(WDF_AIRSHIPDISABLE, ((wProp & WDF_AIRSHIPDISABLE) ? TRUE : FALSE));
	m_wndPropTree.SetItem(WDF_CHARACTERTRANS, ((wProp & WDF_CHARACTERTRANS) ? TRUE : FALSE));
	m_wndPropTree.SetItem(WDF_AIRSHIPSHADOW, PROP_DONOTCHANGE, (wProp & WDF_AIRSHIPSHADOW) >> 2);
	m_wndPropTree.SetItem(WDF_BATTLEBACKGROUND, PROP_DONOTCHANGE, HIBYTE(wProp & WDF_BATTLEBACKGROUND));
	m_wndPropTree.SetItem(WDF_ENEMYENCOUNTERS, ((wProp & WDF_ENEMYENCOUNTERS) ? TRUE : FALSE));
	m_wndPropTree.SetItem(WDF_ALTERNATEMUSIC, ((wProp & WDF_ALTERNATEMUSIC) ? TRUE : FALSE));
	m_wndPropTree.SetItem(WDF_UNKNOWN1, ((wProp & WDF_UNKNOWN1) ? TRUE : FALSE));
	m_wndPropTree.SetItem(WDF_UNKNOWN2, ((wProp & WDF_UNKNOWN2) ? TRUE : FALSE));
	m_wndPropTree.SetItem(WDF_UNKNOWN3, ((wProp & WDF_UNKNOWN3) ? TRUE : FALSE));
	m_wndPropTree.SetItem(WDF_UNKNOWN5, ((wProp & WDF_UNKNOWN5) ? TRUE : FALSE));
	m_wndPropTree.SetItem(WDF_UNKNOWN6, ((wProp & WDF_UNKNOWN6) ? TRUE : FALSE));
}

void CMapViewDlg::OnExportWholeTiles()
{
	static char szFilter[] = "Bitmap Files (*.bmp)|*.bmp|All Files (*.*)|*.*||";
	CFileDialog fdlg(FALSE, _T("bmp"), NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, szFilter, this);

	if(fdlg.DoModal() != IDOK)
		return;

	CDC dc;
	CBitmap bitmap;
	CPalette pal;

	// allocate memory block for logical palette
	HANDLE hBI = ::GlobalAlloc(GHND, sizeof(BITMAPINFO) + sizeof(RGBQUAD) * 256);
	if(hBI == 0) return;
	BITMAPINFO* lpBI = (BITMAPINFO*)::GlobalLock((HGLOBAL)hBI);

	ZeroMemory(lpBI, sizeof(BITMAPINFO));
	lpBI->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	lpBI->bmiHeader.biWidth = 256;
	lpBI->bmiHeader.biHeight = 256;
	lpBI->bmiHeader.biPlanes = 1;		// Must always be 1 according to docs
	lpBI->bmiHeader.biBitCount = 8;
	lpBI->bmiHeader.biCompression = BI_RGB;
	lpBI->bmiHeader.biSizeImage = 0;
	lpBI->bmiHeader.biXPelsPerMeter = 0;
	lpBI->bmiHeader.biYPelsPerMeter = 0;
	lpBI->bmiHeader.biClrUsed       = 0;
	lpBI->bmiHeader.biClrImportant  = 0;

	for(int i = 0; i < 128; i++)
	{
		lpBI->bmiColors[i].rgbRed = R555(m_worldData->palette[i]) * 8;
		lpBI->bmiColors[i].rgbGreen = G555(m_worldData->palette[i]) * 8;
		lpBI->bmiColors[i].rgbBlue = B555(m_worldData->palette[i]) * 8;
		lpBI->bmiColors[i].rgbReserved = 0;
	}

	for(i = 128; i < 256; i++)
	{
		lpBI->bmiColors[i].rgbRed = 0;
		lpBI->bmiColors[i].rgbGreen = 0;
		lpBI->bmiColors[i].rgbBlue = 0;
		lpBI->bmiColors[i].rgbReserved = 0;
	}

	BYTE* pImage = new BYTE[256 * 256];

	int x = 0, y = 0;
	for(i = 0; i < 256; i++)
	{
		Put16x16Tile(pImage, 256, 256, x, y, i);

		x += 16;

		if(x == 256)
		{
			x = 0;
			y += 16;
		}

	}

	CDIBSectionLite* pDIB = new CDIBSectionLite;
	pDIB->SetBitmap(lpBI, pImage);
	pDIB->Save(fdlg.GetPathName());

	if(pDIB) delete pDIB;
	if(pImage) delete pImage;

	::GlobalUnlock((HGLOBAL)hBI);
	::GlobalFree((HGLOBAL)hBI);
}

void CMapViewDlg::OnExportTilePieces()
{
	static char szFilter[] = "Bitmap Files (*.bmp)|*.bmp|All Files (*.*)|*.*||";
	CFileDialog fdlg(FALSE, _T("bmp"), NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, szFilter, this);

	if(fdlg.DoModal() != IDOK)
		return;

	// allocate memory block for logical palette
	HANDLE hBI = ::GlobalAlloc(GHND, sizeof(BITMAPINFO) + sizeof(RGBQUAD) * 256);
	if(hBI == 0) return;
	BITMAPINFO* lpBI = (BITMAPINFO*)::GlobalLock((HGLOBAL)hBI);

	ZeroMemory(lpBI, sizeof(BITMAPINFO));
	lpBI->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
	lpBI->bmiHeader.biWidth = 128;
	lpBI->bmiHeader.biHeight = 128;
	lpBI->bmiHeader.biPlanes = 1;		// Must always be 1 according to docs
	lpBI->bmiHeader.biBitCount = 8;
	lpBI->bmiHeader.biCompression = BI_RGB;
	lpBI->bmiHeader.biSizeImage = 0;
	lpBI->bmiHeader.biXPelsPerMeter = 0;
	lpBI->bmiHeader.biYPelsPerMeter = 0;
	lpBI->bmiHeader.biClrUsed       = 0;
	lpBI->bmiHeader.biClrImportant  = 0;

	for(int i = 0; i < 128; i++)
	{
		lpBI->bmiColors[i].rgbRed = R555(m_worldData->palette[i]) * 8;
		lpBI->bmiColors[i].rgbGreen = G555(m_worldData->palette[i]) * 8;
		lpBI->bmiColors[i].rgbBlue = B555(m_worldData->palette[i]) * 8;
		lpBI->bmiColors[i].rgbReserved = 0;
	}

	for(i = 128; i < 256; i++)
	{
		lpBI->bmiColors[i].rgbRed = 0;
		lpBI->bmiColors[i].rgbGreen = 0;
		lpBI->bmiColors[i].rgbBlue = 0;
		lpBI->bmiColors[i].rgbReserved = 0;
	}

	BYTE* pImage = new BYTE[128 * 128];

	int x = 0, y = 0;
	for(i = 0; i < 256; i++)
	{
		Put8x8ChunkyTile(pImage, 128, 128, x, y, i);

		x += 8;

		if(x == 128)
		{
			x = 0;
			y += 8;
		}

	}

	CDIBSectionLite* pDIB = new CDIBSectionLite;
	pDIB->SetBitmap(lpBI, pImage);
	pDIB->Save(fdlg.GetPathName());

	if(pDIB) delete pDIB;
	if(pImage) delete pImage;

	::GlobalUnlock((HGLOBAL)hBI);
	::GlobalFree((HGLOBAL)hBI);
}

void CMapViewDlg::OnImportTilePieces()
{
	static char szFilter[] = "Bitmap Files (*.bmp)|*.bmp|All Files (*.*)|*.*||";
	CFileDialog fdlg(TRUE, _T("bmp"), NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, szFilter, this);

	if(fdlg.DoModal() != IDOK)
		return;

	CDIBSectionLite* pDIB = new CDIBSectionLite;
	pDIB->Load(fdlg.GetPathName());
	BITMAPINFO* lpBI = pDIB->GetBitmapInfo();

	if((lpBI->bmiHeader.biWidth != 128) ||
		(lpBI->bmiHeader.biHeight != 128) || 
		(lpBI->bmiHeader.biBitCount != 8))
	{
		MessageBox("The bitmap selected is invalid.\nPlease select a bitmap 128 pixels wide\nby 128 pixels high, and with\n256 colors.", "Bitmap Invalid", MB_OK);
		if(pDIB) delete pDIB;
		return;
	}

	// Import color table.
	for(int i = 0; i < 128; i++)
	{
		m_worldData->palette[i] = RGB555(lpBI->bmiColors[i].rgbRed / 8,
			lpBI->bmiColors[i].rgbGreen / 8, lpBI->bmiColors[i].rgbBlue / 8);
	}

	BYTE* pImage = (BYTE*)pDIB->GetDIBits();

	int x = 0, y = 0;
	for(i = 0; i < 256; i++)
	{
		Get8x8ChunkyTile(pImage, 128, 128, x, y, i);

		x += 8;

		if(x == 128)
		{
			x = 0;
			y += 8;
		}

	}

	if(pDIB) delete pDIB;
}

void CMapViewDlg::Put8x8ChunkyTile(BYTE* pBitmap, int width, int height, int dx, int dy, BYTE bTile)
{
	int src = 0;
	BYTE bAll, b1, b2;

	for(int y = dy; y < dy + 8; y++)
	{
		for(int x = dx; x < dx + 8; x += 2)
		{
			bAll = m_worldData->tiles[bTile * 0x20 + src];

			b1 = (bAll >> 4) & 0xF;
			b2 = (bAll) & 0xF;

			pBitmap[(height - 1 - y) * width + x + 1] = m_worldData->tilePalettes[bTile] + b1;
			pBitmap[(height - 1 - y) * width + x] = m_worldData->tilePalettes[bTile] + b2;

			src += 1;
		}
	}
}

void CMapViewDlg::Get8x8ChunkyTile(BYTE* pBitmap, int width, int height, int dx, int dy, BYTE bTile)
{
	int src = 0;
	BYTE b1, b2;

	m_worldData->tilePalettes[bTile] = pBitmap[(height - 1 - dy) * width + dx + 1] & 0xF0;

	for(int y = dy; y < dy + 8; y++)
	{
		for(int x = dx; x < dx + 8; x += 2)
		{
			b1 = pBitmap[(height - 1 - y) * width + x + 1] & 0x0F;
			b2 = pBitmap[(height - 1 - y) * width + x] & 0x0F;

			m_worldData->tiles[bTile * 0x20 + src] = (b1 << 4) | b2;

			src += 1;
		}
	}
}

void CMapViewDlg::Put16x16Tile(BYTE* pBitmap, int width, int height, int dx, int dy, BYTE bTile)
{
	Put8x8ChunkyTile(pBitmap, width, height, dx, dy, m_worldData->tileFormations[bTile * 4 + 0]);
	Put8x8ChunkyTile(pBitmap, width, height, dx + 8, dy, m_worldData->tileFormations[bTile * 4 + 1]);
	Put8x8ChunkyTile(pBitmap, width, height, dx, dy + 8, m_worldData->tileFormations[bTile * 4 + 2]);
	Put8x8ChunkyTile(pBitmap, width, height, dx + 8, dy + 8, m_worldData->tileFormations[bTile * 4 + 3]);
}