// MonsterWnd.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "FF3Doc.h"
#include "MonsterWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CMonsterWnd

CMonsterWnd::CMonsterWnd()
{
	m_nMonst = 0;
}

CMonsterWnd::~CMonsterWnd()
{
}


BEGIN_MESSAGE_MAP(CMonsterWnd, CWnd)
	//{{AFX_MSG_MAP(CMonsterWnd)
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CMonsterWnd message handlers

void CMonsterWnd::OnPaint() 
{
	CPaintDC dc(this);

	CRect rect;  GetClientRect(&rect);
	dc.FillSolidRect(&rect, ::GetSysColor(COLOR_BTNFACE));

	DrawMonster(&dc);
}

void CMonsterWnd::DrawMonster(CDC* pDC)
{
	MonsterVisuals* pVis = &m_pDoc->monsterVisuals[m_nMonst];

	MonsterComposition* pComp;

	if(pVis->extra & 0x80)
		pComp = &m_pDoc->monsterCompositions[pVis->composition + 128];
	else
		pComp = &m_pDoc->monsterCompositions[pVis->composition];

	int offset = 0;

	if(pComp->length == 8)
	{
		for(int y = 0; y < 8; y++)
		{
			int x = 0;

			for(int bit = 7; bit >= 0; bit--, x += 8)
			{
				if(pComp->mask[y] & (1 << bit))
				{
					if(pVis->Is3Bitplane())
					{
						Put8x8x3Tile(pDC, x, y * 8, pVis, offset);
						offset += 0x18;
					}
					else
					{
						Put8x8x4Tile(pDC, x, y * 8, pVis, offset);
						offset += 0x20;
					}
				}
			}
		}
	}
	else if(pComp->length == 32)
	{
		for(int y = 0; y < 16; y++)
		{
			int x = 0;

			for(int bit = 15; bit >= 0; bit--, x += 8)
			{
				WORD wMask = MAKEWORD(pComp->mask[y * 2 + 1], pComp->mask[y * 2]);

				if(wMask & (1 << bit))
				{
					if(pVis->Is3Bitplane())
					{
						Put8x8x3Tile(pDC, x, y * 8, pVis, offset);
						offset += 0x18;
					}
					else
					{
						Put8x8x4Tile(pDC, x, y * 8, pVis, offset);
						offset += 0x20;
					}
				}
			}
		}
	}
}

void CMonsterWnd::Put8x8x4Tile(CDC *pDC, int dx, int dy, MonsterVisuals* pVis, int offset)
{
	BYTE aTile[64];
	BYTE bPlanes[4];

	BYTE* pTile = &m_pDoc->monsterGraphics[(pVis->gfx & 0x7FFF) * 8 + offset];

	for(int row = 0; row < 8; row++)
	{
		bPlanes[0] = pTile[row * 2];
		bPlanes[1] = pTile[row * 2 + 1];
		bPlanes[2] = pTile[row * 2 + 16];
		bPlanes[3] = pTile[row * 2 + 17];

		for(int bit = 7; bit >= 0; bit--)
		{
			BYTE mask = 1 << bit;

			// Get a color index by separating the bits on each plane,
			// shifting them the correct amount depending on the plane,
			// and ORing them together.
			aTile[row * 8 + (7 - bit)] = ((bPlanes[0] & mask) >> bit) |
								   (((bPlanes[1] & mask) >> bit) << 1) |
								   (((bPlanes[2] & mask) >> bit) << 2) |
								   (((bPlanes[3] & mask) >> bit) << 3);
								   
		}
	}

	for(int y = 0; y < 8; y++)
	{
		for(int x = 0; x < 8; x++)
		{
			BYTE bCol = aTile[y * 8 + x];
			
			if(bCol != 0)
			{
				WORD wPal = MAKEWORD(pVis->palette, pVis->extra) & 0x03FF;
				WORD wCol = m_pDoc->monsterPalettes[wPal * 8 + bCol];
				COLORREF cCol = RGB(R555(wCol) * 8, G555(wCol) * 8, B555(wCol) * 8);
				pDC->SetPixel(dx + x, dy + y, cCol);
			}
		}
	}
}

void CMonsterWnd::Put8x8x3Tile(CDC *pDC, int dx, int dy, MonsterVisuals* pVis, int offset)
{
	BYTE aTile[64];
	BYTE bPlanes[3];

	BYTE* pTile = &m_pDoc->monsterGraphics[(pVis->gfx & 0x7FFF) * 8 + offset];

	for(int row = 0; row < 8; row++)
	{
		bPlanes[0] = pTile[row * 2];
		bPlanes[1] = pTile[row * 2 + 1];
		bPlanes[2] = pTile[row + 16];

		for(int bit = 7; bit >= 0; bit--)
		{
			BYTE mask = 1 << bit;

			// Get a color index by separating the bits on each plane,
			// shifting them the correct amount depending on the plane,
			// and ORing them together.
			aTile[row * 8 + (7 - bit)] = ((bPlanes[0] & mask) >> bit) |
								   (((bPlanes[1] & mask) >> bit) << 1) |
								   (((bPlanes[2] & mask) >> bit) << 2);
								   
		}
	}

	for(int y = 0; y < 8; y++)
	{
		for(int x = 0; x < 8; x++)
		{
			BYTE bCol = aTile[y * 8 + x];
			
			if(bCol != 0)
			{
				WORD wPal = MAKEWORD(pVis->palette, pVis->extra) & 0x03FF;
				WORD wCol = m_pDoc->monsterPalettes[wPal * 8 + bCol];
				COLORREF cCol = RGB(R555(wCol) * 8, G555(wCol) * 8, B555(wCol) * 8);
				pDC->SetPixel(dx + x, dy + y, cCol);
			}
		}
	}
}
