// PixelEditor.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "PixelEditor.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPixelEditor

CPixelEditor::CPixelEditor()
{
	m_pixelArray = NULL;
	m_colorArray = NULL;
	m_size = CSize(0, 0);
	m_nPixelSize = 16;
	m_nColors = 0;

	m_bLeftDown = FALSE;
	m_bRightDown = FALSE;

	m_nFGColor = 0;
	m_nBGColor = 0;
}

CPixelEditor::~CPixelEditor()
{
	if(m_colorArray) delete m_colorArray;
	if(m_pixelArray) delete m_pixelArray;
}


BEGIN_MESSAGE_MAP(CPixelEditor, CWnd)
	//{{AFX_MSG_MAP(CPixelEditor)
	ON_WM_PAINT()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_LBUTTONUP()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CPixelEditor message handlers

void CPixelEditor::OnPaint() 
{
	if(!m_pixelArray || !m_colorArray) return;

	CPaintDC dc(this); // device context for painting

	int x, y;

	for(y = 0; y < m_size.cy; y++)
	{
		for(x = 0; x < m_size.cx; x++)
		{
			WORD wColor = m_colorArray[m_pixelArray[y * m_size.cx + x]];
			BYTE r = R555(wColor);
			BYTE g = G555(wColor);
			BYTE b = B555(wColor);

			dc.FillSolidRect(x * m_nPixelSize, y * m_nPixelSize,
				m_nPixelSize, m_nPixelSize,
				RGB(r * 8, g * 8, b * 8));
		}
	}

	CPen linePen(PS_SOLID, 1, RGB(192, 192, 192));
	CPen* oldPen = dc.SelectObject(&linePen);

	for(y = 0; y < m_size.cy * m_nPixelSize; y += m_nPixelSize)
	{
		dc.MoveTo(0, y);
		dc.LineTo(m_size.cx * m_nPixelSize, y);
	}

	for(x = 0; x < m_size.cx * m_nPixelSize; x += m_nPixelSize)
	{
		dc.MoveTo(x, 0);
		dc.LineTo(x, m_size.cy * m_nPixelSize);
	}
	
	dc.SelectObject(oldPen);
}

void CPixelEditor::OnLButtonDown(UINT nFlags, CPoint point) 
{
	m_bLeftDown = TRUE;
	SetCapture();

	int xTile = point.x / m_nPixelSize;
	int yTile = point.y / m_nPixelSize;

	SetPixel(xTile, yTile, m_nFGColor);
	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), PED_IMAGECHANGED),
		(LPARAM)GetSafeHwnd());
}

void CPixelEditor::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(m_bLeftDown)
	{
		int xTile = point.x / m_nPixelSize;
		int yTile = point.y / m_nPixelSize;

		SetPixel(xTile, yTile, m_nFGColor);
		GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), PED_IMAGECHANGED),
			(LPARAM)GetSafeHwnd());
	}
	else if(m_bRightDown)
	{
		int xTile = point.x / m_nPixelSize;
		int yTile = point.y / m_nPixelSize;

		SetPixel(xTile, yTile, m_nBGColor);
		GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), PED_IMAGECHANGED),
			(LPARAM)GetSafeHwnd());
	}
}

void CPixelEditor::OnLButtonUp(UINT nFlags, CPoint point) 
{
	m_bLeftDown = FALSE;
	ReleaseCapture();
}

void CPixelEditor::OnRButtonDown(UINT nFlags, CPoint point) 
{
	m_bRightDown = TRUE;
	SetCapture();

	int xTile = point.x / m_nPixelSize;
	int yTile = point.y / m_nPixelSize;

	SetPixel(xTile, yTile, m_nBGColor);
	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), PED_IMAGECHANGED),
		(LPARAM)GetSafeHwnd());
}

void CPixelEditor::OnRButtonUp(UINT nFlags, CPoint point) 
{
	m_bRightDown = FALSE;
	ReleaseCapture();
}

void CPixelEditor::SetSizes(int x, int y, int pixelSize)
{
	if(m_pixelArray) delete m_pixelArray;

	m_pixelArray = new BYTE[x * y];
	ZeroMemory(m_pixelArray, x * y);

	m_size = CSize(x, y);

	if(pixelSize == 0)
	{
		CRect rect;  GetClientRect(rect);
		m_nPixelSize = rect.Width() / m_size.cx;
	}
	else
	{
		m_nPixelSize = pixelSize;
	}

	// Size to content.
	SetWindowPos(NULL, 0, 0, x * m_nPixelSize + 4, y * m_nPixelSize + 4,
		SWP_NOMOVE | SWP_NOZORDER);
}

void CPixelEditor::SetColors(WORD *aColors, int nColors)
{
	if(m_colorArray) delete m_colorArray;

	m_colorArray = new WORD[nColors];
	CopyMemory(m_colorArray, aColors, nColors * sizeof(WORD));
	Invalidate();
}

void CPixelEditor::SetPixel(int x, int y, BYTE bColor)
{
	m_pixelArray[y * m_size.cx + x] = bColor;

	CRect rect(x * m_nPixelSize, y * m_nPixelSize,
		(x + 1) * m_nPixelSize, (y + 1) * m_nPixelSize);
	InvalidateRect(&rect);
}

BYTE CPixelEditor::GetPixel(int x, int y)
{
	return m_pixelArray[y * m_size.cx + x];
}

void CPixelEditor::SetCurrentColors(int bFG, int bBG)
{
	if(bFG != -1)
		m_nFGColor = (BYTE)bFG;

	if(bBG != -1)
		m_nBGColor = (BYTE)bBG;
}

BOOL CPixelEditor::OnEraseBkgnd(CDC* pDC) 
{
	return TRUE;
}

void CPixelEditor::ShiftUp()
{
	for(int y = 1; y < m_size.cy; y++)
		for(int x = 0; x < m_size.cx; x++)
			m_pixelArray[(y - 1) * m_size.cx + x] = m_pixelArray[y * m_size.cx + x];

	for(int x = 0; x < m_size.cx; x++)
		m_pixelArray[(m_size.cy - 1) * m_size.cx + x] = 0;

	Invalidate();
	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), PED_IMAGECHANGED),
		(LPARAM)GetSafeHwnd());
}

void CPixelEditor::ShiftDown()
{
	for(int y = m_size.cy - 2; y >= 0; y--)
		for(int x = 0; x < m_size.cx; x++)
			m_pixelArray[(y + 1) * m_size.cx + x] = m_pixelArray[y * m_size.cx + x];

	for(int x = 0; x < m_size.cx; x++)
		m_pixelArray[0 * m_size.cx + x] = 0;

	Invalidate();
	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), PED_IMAGECHANGED),
		(LPARAM)GetSafeHwnd());
}

void CPixelEditor::ShiftLeft()
{
	for(int x = 1; x < m_size.cx; x++)
		for(int y = 0; y < m_size.cy; y++)
			m_pixelArray[y * m_size.cx + (x - 1)] = m_pixelArray[y * m_size.cx + x];

	for(int y = 0; y < m_size.cy; y++)
		m_pixelArray[y * m_size.cx + (m_size.cx - 1)] = 0;

	Invalidate();
	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), PED_IMAGECHANGED),
		(LPARAM)GetSafeHwnd());
}

void CPixelEditor::ShiftRight()
{
	for(int x = m_size.cx - 2; x >= 0; x--)
		for(int y = 0; y < m_size.cy; y++)
			m_pixelArray[y * m_size.cx + (x + 1)] = m_pixelArray[y * m_size.cx + x];

	for(int y = 0; y < m_size.cy; y++)
		m_pixelArray[y * m_size.cx + 0] = 0;

	Invalidate();
	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), PED_IMAGECHANGED),
		(LPARAM)GetSafeHwnd());
}

void CPixelEditor::Fill(BYTE bColor)
{
	for(int i = 0; i < m_size.cx * m_size.cy; i++)
		m_pixelArray[i] = bColor;

	Invalidate();
	GetOwner()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), PED_IMAGECHANGED),
		(LPARAM)GetSafeHwnd());
}

void CPixelEditor::SetColor(int index, WORD wColor)
{
	m_colorArray[index] = wColor;
	Invalidate();
}
