// PropertyTree.cpp : implementation file
//

#include "stdafx.h"
#include "PropertyTree.h"
#include "FF3ListBox.h"

#include "memdc.h"
#include "resource.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CPropertyTree

CPropertyTree::CPropertyTree()
{
	m_clr3DFace			= ::GetSysColor(COLOR_3DFACE);
	m_clr3DShadow		= ::GetSysColor(COLOR_3DSHADOW);
	m_clr3DDkShadow		= ::GetSysColor(COLOR_3DDKSHADOW);
	m_clr3DLight		= ::GetSysColor(COLOR_3DLIGHT);
	m_clr3DHilight		= ::GetSysColor(COLOR_3DHILIGHT);
	m_clrSelTextColor	= ::GetSysColor(COLOR_BTNTEXT);
	m_clrSelBkColor		= m_clr3DHilight;

	m_nNameWidth = 0;

	m_imgButtons.Create(IDB_TREE_PLUSMINUS, 15, 1, RGB(255, 0, 255));

	m_strTrue = "True";
	m_strFalse = "False";

	m_pDoc = FALSE;
	m_bKeyDown = FALSE;
}

CPropertyTree::~CPropertyTree()
{
}


BEGIN_MESSAGE_MAP(CPropertyTree, CTreeCtrl)
	//{{AFX_MSG_MAP(CPropertyTree)
	ON_NOTIFY_REFLECT(TVN_SELCHANGED, OnSelChanged)
	ON_WM_CREATE()
	ON_WM_LBUTTONDOWN()
//	ON_WM_LBUTTONDBLCLK()
	ON_WM_DESTROY()
	ON_WM_KEYDOWN()
	ON_WM_KEYUP()
	//}}AFX_MSG_MAP
	ON_NOTIFY_REFLECT(NM_CUSTOMDRAW, OnCustomDraw)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CPropertyTree message handlers

HTREEITEM CPropertyTree::GetNextItem(HTREEITEM hItem)
{
	HTREEITEM hti;

	if(ItemHasChildren(hItem))
		return GetChildItem(hItem);		// return first child
	else
	{
		// return next sibling item
		// Go up the tree to find a parent's sibling if needed.
		while((hti = GetNextSiblingItem(hItem)) == NULL)
		{
			if((hItem = GetParentItem(hItem)) == NULL)
				return NULL;
		}
	}

	return hti;
}

LPPROPERTYNODE CPropertyTree::GetNode(UINT id)
{
	HTREEITEM hItem = IndexToItem(id);
	return (LPPROPERTYNODE)GetItemData(hItem);
}

HTREEITEM CPropertyTree::IndexToItem(UINT id)
{
	HTREEITEM htiCur = GetRootItem();
	
	while(htiCur)
	{
		LPPROPERTYNODE sItemData = (LPPROPERTYNODE)GetItemData(htiCur);

		if (sItemData->id == id)
			return htiCur;

		htiCur = GetNextItem(htiCur);
	}
	
	return NULL;
}

LRESULT CPropertyTree::IsItemExpanded(UINT id)
{
	HTREEITEM htiCur = GetRootItem();
	
	while(htiCur)
	{
		LPPROPERTYNODE sItemData = (LPPROPERTYNODE)GetItemData(htiCur);

		if (sItemData->id == id)
		{
			if(GetItemState(htiCur, TVIS_EXPANDED) & TVIS_EXPANDED)
				return TRUE;
			else
				return FALSE;
		}

		htiCur = GetNextItem(htiCur);
	}
	
	return 0x80000001;
}

void CPropertyTree::AddGroup(UINT id, LPCTSTR name)
{
	LPPROPERTYNODE node = new PROPERTYNODE;
	node->id = id;
	node->type = PROPT_GROUP;
	node->name = name;
	node->flags = 0;
	node->lData = NULL;
	node->lEmpty = NULL;

	InsertItem(TVIF_TEXT | TVIF_PARAM, name, 0, 0, 0, 0, (LPARAM)node, TVI_ROOT, TVI_LAST);

	CClientDC dc(this);
	CFont* oldFont = dc.SelectObject(&boldFont);
	CSize extent = dc.GetTextExtent(name, lstrlen(name));
	if(extent.cx > m_nNameWidth) m_nNameWidth = extent.cx + 4;
	dc.SelectObject(oldFont);
}

void CPropertyTree::AddItem(UINT parent, UINT id, LPCTSTR name, UINT type, DWORD flags,
							LPARAM lData, LPARAM lData2, LPARAM lEmpty) 
{
	HTREEITEM realParent;

	if(parent) realParent = IndexToItem(parent);
	else return;

	LPPROPERTYNODE node = new PROPERTYNODE;
	node->id = id;
	node->type = type;
	node->flags = flags;
	node->name = name;
	node->lData = lData;
	node->lData2 = lData2;
	node->lEmpty = lEmpty;

	if(node->type == PROPT_STATIC || node->type == PROPT_EDIT || node->type == PROPT_FILE ||
		node->type == PROPT_FONT)
		node->strText = (LPCTSTR)lData;
	else
		node->strText.Empty();

	InsertItem(TVIF_TEXT | TVIF_PARAM, name, 0, 0, 0, 0, (LPARAM)node, realParent, TVI_LAST);

	CClientDC dc(this);
	CFont* oldFont = dc.SelectObject(&regFont);
	CSize extent = dc.GetTextExtent(name, lstrlen(name));
	if(extent.cx > m_nNameWidth) m_nNameWidth = extent.cx + 4;
	dc.SelectObject(oldFont);
}

void CPropertyTree::ExpandBranch(HTREEITEM hti)
{
	if(ItemHasChildren(hti))
	{
		Expand(hti, TVE_EXPAND);
		hti = GetChildItem(hti);
		
		do ExpandBranch(hti); while((hti = GetNextSiblingItem(hti)) != NULL);
	}
    
	EnsureVisible(GetSelectedItem());
}

void CPropertyTree::ExpandAll()
{
	HTREEITEM htiCur = GetRootItem();

	while(htiCur)
	{
		ExpandBranch(htiCur);
		htiCur = GetNextItem(htiCur);
	}

	SelectSetFirstVisible(GetRootItem());
}

void CPropertyTree::OnSelChanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	
	HTREEITEM hti = GetSelectedItem();
	m_htiSel = hti;

	*pResult = 0;
}

HTREEITEM CPropertyTree::GetFirstSiblingItem(HTREEITEM hti)
{
	HTREEITEM htiPrev;

	for(;;)
	{
		htiPrev = GetPrevSiblingItem(hti);
		if(htiPrev == NULL) return hti;
		hti = htiPrev;
	}
}

void CPropertyTree::OnCustomDraw(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NMTVCUSTOMDRAW *pTVCD = (NMTVCUSTOMDRAW*)pNMHDR;
	
	// The rect for the cell gives correct left and right values.
	CRect rect = pTVCD->nmcd.rc;
	CRect client; GetClientRect(client);

	CDC* pDC = CDC::FromHandle(pTVCD->nmcd.hdc);

	// By default set the return value to do the default behavior.
	*pResult = 0;
	
	switch( pTVCD->nmcd.dwDrawStage )
	{
		// First stage (for the whole control)
		case CDDS_PREPAINT: 
		{		
			*pResult = CDRF_NOTIFYITEMDRAW;
		}
		break;

		// Stage three (called for each subitem of the focused item)
		case CDDS_ITEMPREPAINT | CDDS_SUBITEM: 
		{
			*pResult = CDRF_NOTIFYSUBITEMDRAW | CDRF_NOTIFYPOSTPAINT;
		}
		break;

		// Stage four (called for each subitem of the focused item)
		case CDDS_ITEMPOSTPAINT | CDDS_SUBITEM: 
		{
			*pResult = CDRF_SKIPDEFAULT;	
		}
		break;

		default: 
			// Stage two handled here. (called for each item)
			if(GetSelectedItem() == (HTREEITEM)pTVCD->nmcd.dwItemSpec)
			{
				LPPROPERTYNODE node = (LPPROPERTYNODE)GetItemData((HTREEITEM)pTVCD->nmcd.dwItemSpec);

				CPen penBtnFace(PS_SOLID, 1, m_clr3DFace);
				CPen* oldPen = pDC->SelectObject(&penBtnFace);
				pDC->MoveTo(16, 0); pDC->LineTo(rect.right, 0);
				pDC->MoveTo(16, rect.bottom); pDC->LineTo(rect.right, rect.bottom);
				pDC->MoveTo(16, 0); pDC->LineTo(16, rect.bottom);
				if(node->type != PROPT_GROUP) pDC->MoveTo(16 + m_nNameWidth, rect.top); pDC->LineTo(16 + m_nNameWidth, rect.bottom);
				pDC->SelectObject(oldPen);

				// Draw a plus or minus.
				if(node->type == PROPT_GROUP)
					m_imgButtons.Draw(pDC,
						GetItemState((HTREEITEM)pTVCD->nmcd.dwItemSpec, TVIS_EXPANDED) & TVIS_EXPANDED ? 1 : 0,
						CPoint(rect.left, rect.top + 1), ILD_NORMAL);

				// Highlight or erase the row.
				CRect nameSpace; nameSpace.CopyRect(rect);
				nameSpace.left += 17;
				if(node->type != PROPT_GROUP) nameSpace.right = nameSpace.left + m_nNameWidth - 1;
				nameSpace.top++;
				pDC->FillSolidRect(nameSpace, ::GetSysColor(COLOR_HIGHLIGHT));

				CRect propSpace; propSpace.CopyRect(rect);
				propSpace.left += m_nNameWidth + 17;
				propSpace.top++;
				if(node->type != PROPT_GROUP)
					if(!(node->type & PROPT_FF3))
						pDC->FillSolidRect(propSpace, ::GetSysColor(COLOR_WINDOW));
					else
						if((node->flags & 0x10000) && (node->lData2 == 0xFF))
							pDC->FillSolidRect(propSpace, ::GetSysColor(COLOR_WINDOW));
						else
							pDC->FillSolidRect(propSpace, RGB(112, 120, 192));

				pDC->SetBkColor(::GetSysColor(COLOR_HIGHLIGHT));

				CFont* pOldFont;
				if(node->type == PROPT_GROUP) pOldFont = pDC->SelectObject(&boldFont);
				else pOldFont = pDC->SelectObject(&regFont);

				// Draw the property name.
				CRect nameRect; nameRect.CopyRect(rect);
				nameRect.left += 18;
				nameRect.right = nameRect.left + m_nNameWidth - 4;
				pDC->DrawText(GetItemText((HTREEITEM)pTVCD->nmcd.dwItemSpec), nameRect, DT_VCENTER | DT_SINGLELINE | DT_LEFT);

				// Draw the property.
				pDC->SetBkColor(::GetSysColor(COLOR_WINDOW));
				CRect propRect; propRect.CopyRect(rect);
				propRect.left += 19 + m_nNameWidth;

				CString propVal;
				BYTE* propFF3 = NULL;
				int propFF3Len = 0;

				if(node->type == PROPT_STATIC || node->type == PROPT_EDIT)
				{
					propVal = (LPCTSTR)node->strText;
					if(propVal.IsEmpty()) propVal = (LPCTSTR)node->lEmpty;
				}
				if(node->type == PROPT_FILE)
				{
					if(node->flags & PROPF_SHOWICON && !node->strText.IsEmpty())
					{
						propRect.left += 18;

						SHFILEINFO sfi;
						SHGetFileInfo(node->strText, NULL, &sfi, sizeof(sfi),
							SHGFI_ICON | SHGFI_SMALLICON);
						::DrawIconEx(pDC->GetSafeHdc(), rect.left + m_nNameWidth + 18,
							rect.top, sfi.hIcon, 16, 16, NULL, NULL, DI_NORMAL);
					}
					
					propVal = (LPCTSTR)node->strText;
					if(propVal.IsEmpty()) propVal = (LPCTSTR)node->lEmpty;
				}
				if(node->type == PROPT_FONT)
				{
					propVal = (LPCTSTR)node->strText;
					if(propVal.IsEmpty()) propVal = (LPCTSTR)node->lEmpty;
				}
				else if(node->type == PROPT_COMBO)
				{
					LPPROPERTYITEM props = (LPPROPERTYITEM)node->lData;
					if(!node->lData) propVal = (LPCTSTR)node->lEmpty;
					else
					{
						while((*props).value != -1)
						{
							if((*props).value == node->lData2)
							{
								if(node->flags & PROPF_PREFIXVALUE) propVal.Format("%d - ", (*props).value);
								propVal += (*props).desc;
								break;
							}

							props++;
						}
					}
				}
				else if(node->type == PROPT_FF3COMBO)
				{
					if((node->flags & 0x10000) && (node->lData2 == 0xFF))
					{
						propVal = "None";
					}
					else
					{
						CPtrArray* pArray = (CPtrArray*)node->lData;
						propFF3 = (BYTE*)pArray->GetAt(node->lData2);
						propFF3Len = node->flags & 0xFFFF;
					}
				}
				else if(node->type == PROPT_FF3EDIT)
				{
					propFF3 = (BYTE*)node->lData;
					propFF3Len = node->flags & 0xFFFF;
				}
				else if(node->type == PROPT_NUMEDIT)
				{
					propVal.Format("%lu", node->lData);
				}
				if(node->type == PROPT_COLOR)
				{
					CRect colorRect; colorRect.CopyRect(rect);
					colorRect.left += m_nNameWidth + 18;  colorRect.right = colorRect.left + 12;
					colorRect.top += 2;  colorRect.bottom--;
					pDC->Draw3dRect(colorRect, ::GetSysColor(COLOR_WINDOWTEXT), ::GetSysColor(COLOR_WINDOWTEXT));
					colorRect.DeflateRect(1, 1);
					pDC->FillSolidRect(colorRect, node->lData);
					pDC->SetBkColor(::GetSysColor(COLOR_WINDOW));

					propRect.left += 18;
					propVal.Format("0x%08X", node->lData);
				}
				else if(node->type == PROPT_BOOLEAN)
				{
					if(node->lData) propVal = m_strTrue;
					else propVal = m_strFalse;
				}

				if(!propFF3)
					pDC->DrawText(propVal, propRect, DT_VCENTER | DT_SINGLELINE | DT_LEFT);
				else
				{
					CPoint ptText(propRect.left, propRect.top + 4);

					for(int i = 0; i < propFF3Len; i++)
					{
						OutputFixedFontChar(pDC, ptText, m_pDoc, propFF3[i]);
						ptText.x += 8;
					}
				}

				// If it's a type that requires a button, draw it.
				CRect buttonRect; buttonRect.CopyRect(rect);
				buttonRect.left = rect.right - 17;
				if(node->type == PROPT_COMBO || node->type == PROPT_FF3COMBO || node->type == PROPT_SYSCOLOR ||
					node->type == PROPT_BOOLEAN)
				{
					if(pTVCD->nmcd.uItemState & CDIS_SELECTED)
					{
						pDC->DrawFrameControl(buttonRect, DFC_BUTTON, DFCS_BUTTONPUSH);
						m_imgButtons.Draw(pDC, 2, CPoint(buttonRect.left + 1, buttonRect.top), ILD_NORMAL);
					}
					else
					{
						pDC->DrawFrameControl(buttonRect, DFC_BUTTON, DFCS_BUTTONPUSH | DFCS_PUSHED);
						m_imgButtons.Draw(pDC, 2, CPoint(buttonRect.left + 2, buttonRect.top + 1), ILD_NORMAL);
					}
				}
				else if(node->type == PROPT_FILE || node->type == PROPT_FONT ||
					node->type == PROPT_COLOR)
				{
					pDC->DrawFrameControl(buttonRect, DFC_BUTTON, DFCS_BUTTONPUSH);
					m_imgButtons.Draw(pDC, 3, CPoint(buttonRect.left, buttonRect.top), ILD_NORMAL);
				}

				pDC->SelectObject(pOldFont);

				*pResult = CDRF_SKIPDEFAULT | CDRF_NEWFONT;
			}
			else
			{
				LPPROPERTYNODE node = (LPPROPERTYNODE)GetItemData((HTREEITEM)pTVCD->nmcd.dwItemSpec);

				CPen penBtnFace(PS_SOLID, 1, m_clr3DFace);
				CPen* oldPen = pDC->SelectObject(&penBtnFace);
				pDC->MoveTo(16, 0); pDC->LineTo(rect.right, 0);
				pDC->MoveTo(16, rect.bottom); pDC->LineTo(rect.right, rect.bottom);
				pDC->MoveTo(16, 0); pDC->LineTo(16, rect.bottom);
				if(node->type != PROPT_GROUP) pDC->MoveTo(16 + m_nNameWidth, rect.top); pDC->LineTo(16 + m_nNameWidth, rect.bottom);
				pDC->SelectObject(oldPen);

				// Draw a plus or minus.
				if(node->type == PROPT_GROUP)
					m_imgButtons.Draw(pDC,
						GetItemState((HTREEITEM)pTVCD->nmcd.dwItemSpec, TVIS_EXPANDED) & TVIS_EXPANDED ? 1 : 0,
						CPoint(rect.left, rect.top + 1), ILD_NORMAL);

				CRect nameSpace; nameSpace.CopyRect(rect);
				nameSpace.left += 17;
				if(node->type != PROPT_GROUP) nameSpace.right = nameSpace.left + m_nNameWidth - 1;
				nameSpace.top++;
				pDC->FillSolidRect(nameSpace, ::GetSysColor(COLOR_WINDOW));

				CRect propSpace; propSpace.CopyRect(rect);
				propSpace.left += m_nNameWidth + 17;
				propSpace.top++;
				if(node->type != PROPT_GROUP)
					if(!(node->type & PROPT_FF3))
						pDC->FillSolidRect(propSpace, ::GetSysColor(COLOR_WINDOW));
					else
						if((node->flags & 0x10000) && (node->lData2 == 0xFF))
							pDC->FillSolidRect(propSpace, ::GetSysColor(COLOR_WINDOW));
						else
							pDC->FillSolidRect(propSpace, RGB(112, 120, 192));

				pDC->SetBkColor(::GetSysColor(COLOR_WINDOW));

				CFont* pOldFont;
				if(node->type == PROPT_GROUP) pOldFont = pDC->SelectObject(&boldFont);
				else pOldFont = pDC->SelectObject(&regFont);

				// Draw the property name.
				CRect nameRect; nameRect.CopyRect(rect);
				nameRect.left += 18;
				nameRect.right = nameRect.left + m_nNameWidth - 4;
				pDC->DrawText(GetItemText((HTREEITEM)pTVCD->nmcd.dwItemSpec), nameRect, DT_VCENTER | DT_SINGLELINE | DT_LEFT);

				// There may have been a button there previously, get rid of it.
				CRect buttonRect; buttonRect.CopyRect(rect);
				buttonRect.left = rect.right - 17;
				buttonRect.top++;
				if(!(node->type & PROPT_FF3))
					pDC->FillSolidRect(buttonRect, ::GetSysColor(COLOR_WINDOW));
				else
					if((node->flags & 0x10000) && (node->lData2 == 0xFF))
						pDC->FillSolidRect(propSpace, ::GetSysColor(COLOR_WINDOW));
					else
						pDC->FillSolidRect(buttonRect, RGB(112, 120, 192));

				// Draw the property.
				pDC->SetBkColor(::GetSysColor(COLOR_WINDOW));
				CRect propRect; propRect.CopyRect(rect);
				propRect.left += 19 + m_nNameWidth;

				CString propVal;
				BYTE* propFF3 = NULL;
				int propFF3Len = 0;
				
				if(node->type == PROPT_STATIC || node->type == PROPT_EDIT)
				{
					propVal = (LPCTSTR)node->lData;
					if(propVal.IsEmpty()) propVal = (LPCTSTR)node->lEmpty;
				}
				if(node->type == PROPT_FILE)
				{
					if(node->flags & PROPF_SHOWICON && !node->strText.IsEmpty())
					{
						propRect.left += 18;

						SHFILEINFO sfi;
						SHGetFileInfo(node->strText, NULL, &sfi, sizeof(sfi),
							SHGFI_ICON | SHGFI_SMALLICON);
						::DrawIconEx(pDC->GetSafeHdc(), rect.left + m_nNameWidth + 18,
							rect.top, sfi.hIcon, 16, 16, NULL, NULL, DI_NORMAL);
					}

					propVal = (LPCTSTR)node->strText;
					if(propVal.IsEmpty()) propVal = (LPCTSTR)node->lEmpty;
				}
				if(node->type == PROPT_FONT)
				{
					propVal = (LPCTSTR)node->strText;
					if(propVal.IsEmpty()) propVal = (LPCTSTR)node->lEmpty;
				}
				if(node->type == PROPT_COMBO)
				{
					LPPROPERTYITEM props = (LPPROPERTYITEM)node->lData;
					if(!node->lData) propVal = (LPCTSTR)node->lEmpty;
					else
					{
//						if(node->flags & PROPF_PREFIXVALUE) propVal.Format("%d - ", node->lData2);
//						propVal += props[node->lData2].desc;

						while((*props).value != -1)
						{
							if((*props).value == node->lData2)
							{
								if(node->flags & PROPF_PREFIXVALUE) propVal.Format("%d - ", (*props).value);
								propVal += (*props).desc;
								break;
							}

							props++;
						}
					}
				}
				else if(node->type == PROPT_FF3COMBO)
				{
					if((node->flags & 0x10000) && (node->lData2 == 0xFF))
					{
						propVal = "None";
					}
					else
					{
						CPtrArray* pArray = (CPtrArray*)node->lData;
						propFF3 = (BYTE*)pArray->GetAt(node->lData2);
						propFF3Len = node->flags & 0xFFFF;
					}
				}
				else if(node->type == PROPT_FF3EDIT)
				{
					propFF3 = (BYTE*)node->lData;
					propFF3Len = node->flags & 0xFFFF;
				}
				if(node->type == PROPT_COLOR)
				{
					CRect colorRect; colorRect.CopyRect(rect);
					colorRect.left += m_nNameWidth + 18;  colorRect.right = colorRect.left + 12;
					colorRect.top += 2;  colorRect.bottom--;
					pDC->Draw3dRect(colorRect, ::GetSysColor(COLOR_WINDOWTEXT), ::GetSysColor(COLOR_WINDOWTEXT));
					colorRect.DeflateRect(1, 1);
					pDC->FillSolidRect(colorRect, node->lData);
					pDC->SetBkColor(::GetSysColor(COLOR_WINDOW));

					propRect.left += 18;
					propVal.Format("0x%08X", node->lData);
				}
				if(node->type == PROPT_NUMEDIT)
				{
					propVal.Format("%lu", node->lData);
				}
				if(node->type == PROPT_BOOLEAN)
				{
					if(node->lData) propVal = m_strTrue;
					else propVal = m_strFalse;
				}

				if(!propFF3)
					pDC->DrawText(propVal, propRect, DT_VCENTER | DT_SINGLELINE | DT_LEFT);
				else
				{
					CPoint ptText(propRect.left, propRect.top + 4);

					for(int i = 0; i < propFF3Len; i++)
					{
						OutputFixedFontChar(pDC, ptText, m_pDoc, propFF3[i]);
						ptText.x += 8;
					}
				}

				pDC->SelectObject(pOldFont);

				*pResult = CDRF_SKIPDEFAULT | CDRF_NEWFONT;
			}
	
			break;
	}
}

int CPropertyTree::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CTreeCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;
		
	return 0;
}

void CPropertyTree::OnLButtonDown(UINT nFlags, CPoint point) 
{
	UINT flags;
	HTREEITEM tiSelected = HitTest(point, &flags);

	if((flags & TVHT_ONITEMRIGHT) || (flags & TVHT_ONITEMINDENT) || 
		(flags & TVHT_ONITEM) || (flags & TVHT_ONITEMBUTTON))
	{
		SelectItem(tiSelected);
	}

	CTreeCtrl::OnLButtonDown(nFlags, point);

	if(point.x > m_nNameWidth + 16)
		PerformPopupAction(tiSelected);
}

BOOL CPropertyTree::PreTranslateMessage(MSG* pMsg) 
{
	switch(pMsg->message)
	{
		case WM_KEYDOWN:
		case WM_KEYUP:

		switch(pMsg->wParam)
		{
			case VK_RETURN:

			// Bad Windows!  Don't eat my messages before I get them.
			SendMessage(pMsg->message, pMsg->wParam, pMsg->lParam);
			return TRUE;
		}

		break;
	}	

	return CTreeCtrl::PreTranslateMessage(pMsg);
}

void CPropertyTree::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if(nChar == VK_RETURN)
	{
		m_bKeyDown = TRUE;
	}

	CTreeCtrl::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CPropertyTree::OnKeyUp(UINT nChar, UINT nRepCnt, UINT nFlags) 
{
	if(nChar == VK_RETURN && m_bKeyDown == TRUE)
	{
		m_bKeyDown = FALSE;
		HTREEITEM tiSelected = GetSelectedItem();
		PerformPopupAction(tiSelected);
	}

	if(::IsWindow(GetSafeHwnd()))
		CTreeCtrl::OnKeyUp(nChar, nRepCnt, nFlags);
}

void CPropertyTree::PerformPopupAction(HTREEITEM tiSelected)
{
	if(tiSelected)
	{
		LPPROPERTYNODE node = (LPPROPERTYNODE)GetItemData(tiSelected);
		
		if(node->type == PROPT_BOOLEAN || node->type == PROPT_COMBO)
		{
			CRect rect; GetItemRect(tiSelected, rect, FALSE);
			CRect buttonRect; buttonRect.CopyRect(rect);
			buttonRect.left = rect.right - 17;
	
			CPropTreeInLineLB* wndLB = new CPropTreeInLineLB;
			wndLB->Create((node->flags & PROPF_OWNERDRAW) ? LBS_OWNERDRAWFIXED : 0, this,
				IDC_TREE_INPLACELIST);
			wndLB->SetFont(&regFont);

			if(node->type == PROPT_COMBO)
			{
				LPPROPERTYITEM props = (LPPROPERTYITEM)node->lData;

				if(props)
				{
					wndLB->ResetContent();

					while((*props).value != -1)
					{
						CString propVal;

						if(node->flags & PROPF_PREFIXVALUE) propVal.Format("%d - ", (*props).value);
						propVal += (*props).desc;

						int item = wndLB->AddString(propVal);
						wndLB->SetItemData(item, (LPARAM)props);

						props++;
					}
		
					wndLB->SetCurSel(node->lData2);
				}
			}
			if(node->type == PROPT_BOOLEAN)
			{
				wndLB->ResetContent();

				int item = wndLB->AddString(m_strTrue);
				wndLB->SetItemData(item, TRUE);
				item = wndLB->AddString(m_strFalse);
				wndLB->SetItemData(item, FALSE);

				if(node->lData) wndLB->SetCurSel(0);
				else wndLB->SetCurSel(1);
			}

			wndLB->RecalcHeight();
			CRect windowRect; GetWindowRect(windowRect);
			wndLB->MoveWindow(windowRect.left + rect.left + m_nNameWidth + 18,
				windowRect.top + rect.bottom + 1, rect.Width() - (m_nNameWidth + 16),
				wndLB->m_nExplicitHeight);
			
			wndLB->DoModal();

			if(node->type == PROPT_COMBO)
			{
				if(wndLB->m_nSelection != LB_ERR)
				{
					LPPROPERTYITEM props = (LPPROPERTYITEM)node->lData;
					
					node->lData2 = props[wndLB->m_nSelection].value; //wndLB->m_nSelection;
					SendNotification(PROPTN_PROPCHANGED, node);
				}
			}
			if(node->type == PROPT_BOOLEAN)
			{
				if(wndLB->m_nSelection != LB_ERR)
				{
					if(wndLB->m_nSelection == 0) node->lData = TRUE;
					if(wndLB->m_nSelection == 1) node->lData = FALSE;
					SendNotification(PROPTN_PROPCHANGED, node);
				}
			}

			if(wndLB) wndLB->DestroyWindow();
		}
		if(node->type == PROPT_FF3COMBO)
		{
			CRect rect; GetItemRect(tiSelected, rect, FALSE);
			CRect buttonRect; buttonRect.CopyRect(rect);
			buttonRect.left = rect.right - 17;
	
			CFF3ListBoxPopup* wndLB = new CFF3ListBoxPopup;
			wndLB->Create(this);

			CPtrArray* pArray = (CPtrArray*)node->lData;

			CRect windowRect; GetWindowRect(windowRect);
			CPoint ptPopup(windowRect.left + rect.left + m_nNameWidth + 18,
				windowRect.top + rect.bottom + 1);
				
			wndLB->Invoke(NULL, m_pDoc, ptPopup, rect.Width() - (m_nNameWidth + 16), pArray, node->flags, node->lData2);

			if(wndLB->m_nSel != LB_ERR)
			{
				node->lData2 = wndLB->m_nSel;
				SendNotification(PROPTN_PROPCHANGED, node);
			}

			if(wndLB) wndLB->DestroyWindow();
		}
		if(node->type == PROPT_EDIT || node->type == PROPT_NUMEDIT)
		{
			CRect rect; GetItemRect(tiSelected, rect, FALSE);

			CRect editRect; editRect.CopyRect(rect);
			editRect.top++;
			editRect.left += 18 + m_nNameWidth;

//			if(editRect.PtInRect(point))
			{
				CPropTreeInLineEdit* wndEdit = new CPropTreeInLineEdit;
				wndEdit->Create(0, this, IDC_TREE_INPLACEEDIT);
				wndEdit->SetFont(&regFont);

				CString editStr;

				if(node->type == PROPT_NUMEDIT)
				{
					editStr.Format("%lu", node->lData); 
				}
				if(node->type == PROPT_EDIT)
				{
					editStr.Format("%s", node->strText);
				}

				wndEdit->SetWindowText(editStr);
				wndEdit->SetSel(0, -1);
				wndEdit->MoveWindow(editRect.left, editRect.top, editRect.Width(), editRect.Height());
				wndEdit->DoModal();

				if(node->type == PROPT_EDIT)
				{
					node->strText = wndEdit->m_strText;
					SendNotification(PROPTN_PROPCHANGED, node);
				}
				if(node->type == PROPT_NUMEDIT)
				{
					node->lData = atol(wndEdit->m_strText);
					SendNotification(PROPTN_PROPCHANGED, node);
				}

				if(wndEdit) delete wndEdit;
			}
		}
		if(node->type == PROPT_FF3EDIT)
		{
			CRect rect; GetItemRect(tiSelected, rect, FALSE);

			CRect editRect; editRect.CopyRect(rect);
			editRect.top++;
			editRect.left += 18 + m_nNameWidth;

			CPropTreeInLineFF3Edit* wndEdit = new CPropTreeInLineFF3Edit;
			wndEdit->Create(0, this, IDC_TREE_INPLACEEDIT);
			wndEdit->m_pDoc = m_pDoc;
			wndEdit->m_nMaxLength = node->flags & 0xFFFF;
			wndEdit->SizeToContent();
			wndEdit->SetText((BYTE*)node->lData, node->flags & 0xFFFF);

			wndEdit->MoveWindow(editRect.left, editRect.top, editRect.Width(), editRect.Height());
			wndEdit->DoModal();

			if(!::IsWindow(wndEdit->GetSafeHwnd())) return;

			if(wndEdit->m_bValid)
			{
				for(int i = 0; i < (node->flags & 0xFFFF); i++)
					((BYTE*)node->lData)[i] = wndEdit->m_pText[i];
				SendNotification(PROPTN_PROPCHANGED, node);
			}

			if(wndEdit) wndEdit->DestroyWindow();
		}
	}
}

void CPropertyTree::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	UINT flags;
	HTREEITEM tiSelected = HitTest(point, &flags);

	if( (flags & TVHT_ONITEMRIGHT)  || (flags & TVHT_ONITEMINDENT) ||
		(flags & TVHT_ONITEM)		|| (flags & TVHT_ONITEMBUTTON) ) 
	{
		SelectItem(tiSelected);
	}

	CTreeCtrl::OnLButtonDblClk(nFlags, point);

	if(tiSelected)
	{
		LPPROPERTYNODE node = (LPPROPERTYNODE)GetItemData(tiSelected);
		
		if(node->type == PROPT_COMBO)
		{
			LPPROPERTYITEM props = (LPPROPERTYITEM)node->lData;
			node->lData2++;
			if(props[node->lData2].value == -1) node->lData2 = 0;

			CRect rect; GetItemRect(tiSelected, rect, FALSE);
			rect.left += 19 + m_nNameWidth; rect.right -= 17;
			rect.top++;
			InvalidateRect(rect);

			SendNotification(PROPTN_PROPCHANGED, node);
		}
		if(node->type == PROPT_FF3COMBO)
		{
			CPtrArray* pArray = (CPtrArray*)node->lData;
			node->lData2++;
			if(node->lData2 >= pArray->GetSize()) node->lData2 = 0;

			CRect rect; GetItemRect(tiSelected, rect, FALSE);
			rect.left += 19 + m_nNameWidth; rect.right -= 17;
			rect.top++;
			InvalidateRect(rect);

			SendNotification(PROPTN_PROPCHANGED, node);
		}
		if(node->type == PROPT_BOOLEAN)
		{
			if(node->lData == FALSE) node->lData = TRUE;
			else node->lData = FALSE;
			
			CRect rect; GetItemRect(tiSelected, rect, FALSE);
			rect.left += 19 + m_nNameWidth; rect.right -= 17;
			rect.top++;
			InvalidateRect(rect);

			SendNotification(PROPTN_PROPCHANGED, node);
		}
		if(node->type == PROPT_EDIT || node->type == PROPT_NUMEDIT)
		{
			CRect rect; GetItemRect(tiSelected, rect, FALSE);

			CRect editRect; editRect.CopyRect(rect);
			editRect.top++;
			editRect.left += 18 + m_nNameWidth;

			CPropTreeInLineEdit* wndEdit = new CPropTreeInLineEdit;
			wndEdit->Create((node->type == PROPT_NUMEDIT) ? ES_NUMBER : 0, this, IDC_TREE_INPLACEEDIT);
			wndEdit->SetFont(&regFont);

			CString editStr;

			if(node->type == PROPT_NUMEDIT)
			{
				editStr.Format("%lu", node->lData); 
			}
			if(node->type == PROPT_EDIT)
			{
				editStr.Format("%s", node->strText);
			}

			wndEdit->SetWindowText(editStr);
			wndEdit->SetSel(0, -1);
			wndEdit->MoveWindow(editRect.left, editRect.top, editRect.Width(), editRect.Height());
			wndEdit->DoModal();

			if(node->type == PROPT_EDIT)
			{
				node->strText = wndEdit->m_strText;
			}
			if(node->type == PROPT_NUMEDIT)
			{
				node->lData = atol(wndEdit->m_strText);
			}

			if(wndEdit) delete wndEdit;

			SendNotification(PROPTN_PROPCHANGED, node);
		}
		if(node->type == PROPT_COLOR)
		{
			CColorDialog cdlg(node->lData, CC_FULLOPEN, this);
				
			if(cdlg.DoModal() == IDOK)
			{
				node->lData = cdlg.GetColor();
				SendNotification(PROPTN_PROPCHANGED, node);
			}
		}
		if(node->type == PROPT_FILE)
		{
			CFileDialog fdlg(TRUE, NULL, node->strText, NULL,
				(LPCTSTR)node->lData2, this);
				
			if(fdlg.DoModal() == IDOK)
			{
				node->strText = fdlg.GetPathName();
				SendNotification(PROPTN_PROPCHANGED, node);
			}
		}
		if(node->type == PROPT_FONT)
		{
			CFontDialog fdlg((LPLOGFONT)node->lData2, CF_EFFECTS | CF_SCREENFONTS, NULL, this);
				
			if(fdlg.DoModal() == IDOK)
			{
				node->strText = fdlg.GetFaceName();
				SendNotification(PROPTN_PROPCHANGED, node);
			}
		}
	}
}

void CPropertyTree::PreSubclassWindow()
{
	CClientDC dc(this);
	int nHeight = -((dc.GetDeviceCaps(LOGPIXELSY) * 8.5) / 72);

	if(!(HFONT)regFont)
		regFont.CreateFont(nHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0, DEFAULT_CHARSET, OUT_CHARACTER_PRECIS,
			CLIP_CHARACTER_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, "Tahoma");
	
	if(!(HFONT)boldFont)
		boldFont.CreateFont(nHeight, 0, 0, 0, FW_BOLD, 0, 0, 0, DEFAULT_CHARSET, OUT_CHARACTER_PRECIS,
			CLIP_CHARACTER_PRECIS, DEFAULT_QUALITY, DEFAULT_PITCH | FF_DONTCARE, "Tahoma");
}

void CPropertyTree::SendNotification(UINT code, LPPROPERTYNODE node)
{
	NMPROPTREE nmpt;
	nmpt.hdr.hwndFrom = GetSafeHwnd();
	nmpt.hdr.idFrom = ::GetWindowLong(GetSafeHwnd(), GWL_ID);
	nmpt.hdr.code = code;
	nmpt.node = node;
	GetOwner()->SendMessage(WM_NOTIFY, ::GetWindowLong(GetSafeHwnd(), GWL_ID), (LPARAM)&nmpt);
}

void CPropertyTree::OnDestroy() 
{
	DeleteAllItems();

	CTreeCtrl::OnDestroy();
}

BOOL CPropertyTree::DeleteAllItems()
{
	HTREEITEM hItem = GetRootItem();

	while(hItem != NULL)
	{
		LPPROPERTYNODE lpNode = (LPPROPERTYNODE)GetItemData(hItem);
		if(lpNode) delete lpNode;

		hItem = GetNextItem(hItem);
	}

	return CTreeCtrl::DeleteAllItems();
}

void CPropertyTree::SetItem(UINT id, LPARAM lData, LPARAM lData2) 
{
	HTREEITEM hItem = IndexToItem(id);
	
	if(!hItem) return;

	LPPROPERTYNODE node = (LPPROPERTYNODE)GetItemData(hItem);
	if(lData != PROP_DONOTCHANGE) node->lData = lData;
	if(lData2 != PROP_DONOTCHANGE) node->lData2 = lData2;
	
	if(node->type == PROPT_STATIC || node->type == PROPT_EDIT || node->type == PROPT_FILE ||
		node->type == PROPT_FONT)
		node->strText = (LPCTSTR)lData;
	else
		node->strText.Empty();

	CRect rect;  GetItemRect(hItem, &rect, FALSE);
	InvalidateRect(rect);
}

void CPropertyTree::SetTrueFalseText(CString strTrue, CString strFalse)
{
	m_strTrue = strTrue;
	m_strFalse = strFalse;
}
