// ImageCollection.cpp : implementation file
//
// You are free to use, modify and distribute this source, as long as
// it is not sold for profit, and this HEADER stays intact. This source is
// supplied "AS-IS", and without WARRANTY OF ANY KIND. The user
// agrees to hold the author(s) blameless from any or all problems that
// may arise from the use or distribution of this code.
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include <..\src\afximpl.h>
#include "resource.h"

#include "ResourceCollection.h"
#include "CommandBar.h"

// Structure local to this function.
struct CToolBarData
{
	WORD wVersion;
	WORD wWidth;
	WORD wHeight;
	WORD wItemCount;
	WORD* items() { return (WORD*)(this + 1); }	// WORD aItems[wItemCount]
};

CResourceCollection::CResourceCollection(UINT nID, CFrameWnd* pWnd)
{
	m_nID = nID;
//	m_pFrameWnd = pWnd;
	m_images.Create(16, 16, ILC_MASK, 0, 10);
	m_bLocked = FALSE;
}

BOOL CResourceCollection::LoadResources(UINT nID)
{
	// Load the toolbar images.
	LPTSTR lpResName = MAKEINTRESOURCE(nID);

	HINSTANCE hInst = AfxFindResourceHandle(lpResName, RT_BITMAP);
	HRSRC hRsrc = ::FindResource(hInst, lpResName, RT_BITMAP);
	if(hRsrc == NULL) return NULL;
	HBITMAP hbmToolbar = AfxLoadSysColorBitmap(hInst, hRsrc, FALSE);

	CToolBarData* ptbd;
	CBitmap bmToolbar;
	bmToolbar.Attach(hbmToolbar); // destructor will detach & destroy

	if((hInst = AfxFindResourceHandle(lpResName, RT_TOOLBAR)) == NULL ||
	   (hRsrc = FindResource(hInst, lpResName, RT_TOOLBAR)) == NULL ||
	   (ptbd = (CToolBarData*)LoadResource(hInst, hRsrc)) == NULL)
	{
		TRACE(_T("Can't load toolbar %d\n"), nID);
		return FALSE;
	}

	ASSERT(ptbd->wVersion == 1);

	CSize sz(ptbd->wWidth, ptbd->wHeight);
	int image = m_images.Add(&bmToolbar, GetSysColor(COLOR_3DFACE));

	// Add a separator to the list.
	CResourceItem* ri = CreateNewResource(ID_SEPARATOR);
	ri->dwType = cBarControlSeparator;
	ri->dwStyle = 0;
	ri->nImage = -1;
	ri->size = CSize(6, 0);
	ri->strText.Empty();

	WORD i;
	for(i = 0; i < ptbd->wItemCount; i++)
	{
		if(ptbd->items()[i] != ID_SEPARATOR)
		{
			UINT nCmd = ptbd->items()[i];

			CResourceItem* ri = CreateNewResource(nCmd);
			ri->dwType = cBarControlButton;
			ri->dwStyle = cBarControlStyleIcon;
			ri->nImage = image;
			ri->size = CSize(23, 22);

			CString string;
			BOOL bLoaded = string.LoadString(nCmd);
			if(bLoaded && !string.IsEmpty())
			{
				int ttpos = string.Find('\n');
				if(ttpos != -1) ri->strText = string.Mid(ttpos + 1);
			}
			
			image++;
		}
	}

	UpdateAcceleratorTable();
	return TRUE;
}

CResourceItem* CResourceCollection::GetResourceItem(UINT nCmd)
{
	CResourceItem* ri;

	if(m_mapCmdToResource.Lookup(nCmd, (CObject*&)ri)) return ri;
	else return NULL;
}

void CResourceCollection::AddResource(UINT nCmd, DWORD dwType, DWORD dwStyle,
									  CString strText, int nImage, CSize size)
{
	CResourceItem* ri = CreateNewResource(nCmd);
	ri->dwType = dwType;
	ri->dwStyle = dwStyle;
	ri->nImage = nImage;
	ri->size = size;
	ri->strText = strText;

	UpdateAcceleratorTable();
}

CResourceItem* CResourceCollection::CreateNewResource(UINT nCmd)
{
	CResourceItem* ri = new CResourceItem();
	m_mapCmdToResource.SetAt(nCmd, ri);
	return ri;
}

void CResourceCollection::SetControlType(UINT nCmd, DWORD dwType)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return;
	ri->dwType = dwType;
}

DWORD CResourceCollection::GetControlType(UINT nCmd)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return 0;
	else return ri->dwType;
}

void CResourceCollection::SetControlStyle(UINT nCmd, DWORD dwStyle)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return;
	ri->dwStyle = dwStyle;
}

DWORD CResourceCollection::GetControlStyle(UINT nCmd)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return 0;
	else return ri->dwStyle;
}

void CResourceCollection::SetControlState(UINT nCmd, WORD wState)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return;
	ri->wState = wState;

	for(int i = 0; i < m_aCommandBars.GetSize(); i++)
	{
		int nIndex = -1;
		for(;;)
		{
			nIndex = m_aCommandBars[i]->CommandToIndex(nCmd, nIndex + 1);
			if(nIndex == -1) break;

			m_aCommandBars[i]->RedrawControl(nIndex);
		}
	}
}

WORD CResourceCollection::GetControlState(UINT nCmd)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return 0;
	else return ri->wState;
}

int CResourceCollection::GetControlImage(UINT nCmd)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return -1;
	else return ri->nImage;
}

void CResourceCollection::SetControlImage(UINT nCmd, int nImage)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return;
	ri->nImage = nImage;
}

CString CResourceCollection::GetControlText(UINT nCmd)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return CString();
	else return ri->strText;
}

void CResourceCollection::SetControlText(UINT nCmd, LPCTSTR lpszText)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return;
	ri->strText = lpszText;
}

int CResourceCollection::GetControlSelection(UINT nCmd)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return -1;
	else return ri->nSelection;
}

void CResourceCollection::SetControlSelection(UINT nCmd, int nSel)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return;
	ri->nSelection = nSel;

	for(int i = 0; i < m_aCommandBars.GetSize(); i++)
	{
		int nIndex = -1;
		for(;;)
		{
			nIndex = m_aCommandBars[i]->CommandToIndex(nCmd, nIndex + 1);
			if(nIndex == -1) break;

			m_aCommandBars[i]->RedrawControl(nIndex);
		}
	}
}

CString CResourceCollection::GetControlEntry(UINT nCmd)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return CString();
	else return ri->strEntry;
}

void CResourceCollection::SetControlEntry(UINT nCmd, LPCTSTR lpszText)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return;
	ri->strEntry = lpszText;

	for(int i = 0; i < m_aCommandBars.GetSize(); i++)
	{
		int nIndex = -1;
		for(;;)
		{
			nIndex = m_aCommandBars[i]->CommandToIndex(nCmd, nIndex + 1);
			if(nIndex == -1) break;

			m_aCommandBars[i]->RedrawControl(nIndex);

			CEdit* pEdit = m_aCommandBars[i]->GetEditControl(nIndex);
			if(pEdit) pEdit->SetWindowText(lpszText);
		}
	}
}

void CResourceCollection::SetControlSize(UINT nCmd, CSize size)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return;
	ri->size = size;
}

CSize CResourceCollection::GetControlSize(UINT nCmd)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return CSize(0, 0);
	else return ri->size;
}

void CResourceCollection::UpdateAcceleratorTable()
{
	HACCEL hAccel = ::LoadAccelerators(AfxGetInstanceHandle(), MAKEINTRESOURCE(m_nID));
	if(!hAccel) return;

	int nAccelSize = ::CopyAcceleratorTable(hAccel, NULL, 0);
	LPACCEL lpAccel = new ACCEL[nAccelSize];
	::CopyAcceleratorTable (hAccel, lpAccel, nAccelSize);

	// Update all the resource items with their accelerators.
	for(int i = 0; i < nAccelSize; i++)
	{
		CResourceItem* ri = GetResourceItem(lpAccel[i].cmd);
		if(ri) ::CopyMemory(&ri->accel, &lpAccel[i], sizeof(ACCEL));
	}

	// Resource-based accelerator tables are usually freed automatically
	// when a program exits, but since this function may be called multiple times,
	// I delete it here so memory is not wasted.
	::DestroyAcceleratorTable(hAccel);
}

CString CResourceCollection::GetControlAcceleratorString(UINT nCmd)
{
	CResourceItem* ri;
	if(!(ri = GetResourceItem(nCmd))) return CString();
	
	LPACCEL lpAccel = &ri->accel;

	// If the three fields in the ACCEL struct are 0, that means an
	// accelerator was never set for that command.  Return an empty string.
	if(!lpAccel->fVirt && !lpAccel->key && !lpAccel->cmd) return CString();

	CString strAccel;  strAccel.Empty();
	if(lpAccel->fVirt & FALT) ConvertKeyToString(strAccel, VK_MENU);
	if(lpAccel->fVirt & FCONTROL) ConvertKeyToString(strAccel, VK_CONTROL);
	if(lpAccel->fVirt & FSHIFT) ConvertKeyToString(strAccel, VK_SHIFT);
	if(lpAccel->fVirt & FVIRTKEY) ConvertKeyToString(strAccel, lpAccel->key, TRUE);
	else strAccel += (char)lpAccel->key;

	return strAccel;
}

void CResourceCollection::ConvertKeyToString(CString& strAccel, UINT uiVirtKey, BOOL bLast)
{
	char szBuffer[48];

	long lParam = ::MapVirtualKey(uiVirtKey, 0) << 16;
	::GetKeyNameText(lParam, szBuffer, 48);
	strAccel += szBuffer;

	if(!bLast) strAccel += '+';
}

void CResourceCollection::DrawImage(CDC* pDC, int nImage, CPoint pt, int nStyle)
{
	switch(nStyle)
	{
		case imageNormal:
			m_images.Draw(pDC, nImage, pt, ILD_NORMAL);
			break;

		case imageDisabled:
			// create memory dc
			CDC memDC;
			memDC.CreateCompatibleDC(pDC);

			// create mono or color bitmap
			CBitmap bm;
			bm.CreateBitmap(16, 16, 1, 1, NULL);

			// draw image into memory DC--fill BG white first
			CBitmap* pOldBitmap = memDC.SelectObject(&bm);
			memDC.PatBlt(0, 0, 16, 16, WHITENESS);
			m_images.Draw(&memDC, nImage, CPoint(0, 0), ILD_NORMAL); 

			// This seems to be required. Why, I don't know. ???
			COLORREF colorOldBG = pDC->SetBkColor(RGB(255, 255, 255));

			// Draw using hilite offset by (1,1), then shadow
			CBrush brShadow(GetSysColor(COLOR_3DSHADOW));
			CBrush brHilite(GetSysColor(COLOR_3DHIGHLIGHT));
			CBrush* pOldBrush = pDC->SelectObject(&brHilite);
			pDC->BitBlt(pt.x + 1, pt.y + 1, 16, 16, &memDC, 0, 0, 0xb8074a);
			pDC->SelectObject(&brShadow);
			pDC->BitBlt(pt.x, pt.y, 16, 16, &memDC, 0, 0, 0xb8074a);
			pDC->SelectObject(pOldBrush);
			pDC->SetBkColor(colorOldBG);				 // restore
			memDC.SelectObject(pOldBitmap);				 // ...
			
			break;
	}
}

void CResourceCollection::AddCommandBar(CCommandBar* pCommandBar)
{
	for(int i = 0; i < m_aCommandBars.GetSize(); i++)
		if(m_aCommandBars[i] == pCommandBar) return;

	m_aCommandBars.Add(pCommandBar);
}

void CResourceCollection::RemoveCommandBar(CCommandBar* pCommandBar)
{
	for(int i = 0; i < m_aCommandBars.GetSize(); i++)
		if(m_aCommandBars[i] == pCommandBar) m_aCommandBars.RemoveAt(i);
}

void CResourceCollection::RemoveAllCommandBars()
{
	m_aCommandBars.RemoveAll();
}

void CResourceCollection::ShowControlBar(LPCTSTR lpszName, BOOL bShow)
{
	for(int i = 0; i < m_aCommandBars.GetSize(); i++)
	{
		CString strTitle;
		m_aCommandBars[i]->GetWindowText(strTitle);

		if(!strTitle.CompareNoCase(lpszName))
		{
			((CFrameWnd*)m_aCommandBars[i]->GetOwner())->ShowControlBar(m_aCommandBars[i],
				bShow, FALSE);
		}
	}
}

void CResourceCollection::ShowControlBar(CCommandBar* pBar, BOOL bShow)
{
	((CFrameWnd*)pBar->GetOwner())->ShowControlBar(pBar, bShow, FALSE);
}

void CResourceCollection::PopupControlBarMenu(CFrameWnd* pWnd, CPoint pt)
{
	// Not implemented.
	return;

	CMenu barMenu;  barMenu.LoadMenu(IDR_COMMANDBAR_MENU);
	CMenu* pMenu = barMenu.GetSubMenu(0);

	// Walk the array backwards, adding toolbars to the _beginning_ of the menu.
	for(int i = m_aCommandBars.GetSize() - 1; i >= 0; i--)
	{
		if(m_aCommandBars[i]->m_dwStyle & CBCS_MENUBAR) continue;

		CString strTitle;
		m_aCommandBars[i]->GetWindowText(strTitle);

		pMenu->InsertMenu(0, MF_BYPOSITION | MF_STRING,
			m_aCommandBars[i]->GetDlgCtrlID(), strTitle);
	}

	pMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, pt.x, pt.y, pWnd);

	barMenu.DestroyMenu();
}
