// ResourceCollection.h : header file
//
// You are free to use, modify and distribute this source, as long as
// it is not sold for profit, and this HEADER stays intact. This source is
// supplied "AS-IS", and without WARRANTY OF ANY KIND. The user
// agrees to hold the author(s) blameless from any or all problems that
// may arise from the use or distribution of this code.
//
/////////////////////////////////////////////////////////////////////////////

#ifndef __RESOURCECOLLECTION_H__
#define __RESOURCECOLLECTION_H__

class CCommandBar;
class CResourceCollection;
class CResourceItem;

// Control types
typedef enum
{
	cBarControlSeparator = 1,

	cBarControlButton = 2,
	cBarControlButtonPopup = 3,
	cBarControlButtonDropDown = 4,
	cBarControlPopup = 5,
	cBarControlComboBox = 6,
	cBarControlDropDown = 7,
	cBarControlGraphicCombo = 8,
	cBarControlGraphicDropDown = 9,
	cBarControlEdit = 10,
	cBarControlLabel = 11,

	cBarControlExpandingGrid = 12,
	cBarControlGrid = 13,
	cBarControlSplitButtonPopup = 14,
	cBarControlSplitDropDown = 15,

} CmdBarControlType;

// Control display styles
typedef enum
{
	cBarControlStyleCaption = 1,
	cBarControlStyleIcon = 2,
	cBarControlStyleIconAndCaption = 3,

} CmdBarControlStyle;


// Control states
#define TBES_DISABLED		0x00000001	// Disabled
#define TBES_CHECKED		0x00000002	// Checked
#define TBES_INDETERMINATE	0x00000004	// Indeterminate


// Styles for CResourceCollection::DrawImage()
enum imageStyle
{
	imageNormal, imageDisabled
};

class CResourceItem : public CObject
{
public:
	CResourceItem() { dwType = 0; dwStyle = 0; nImage = -1; wState = 0;
		::ZeroMemory(&accel, sizeof(ACCEL)); }

	DWORD dwType;
	DWORD dwStyle;
	int nImage;
	CString strText;
	CString strEntry;
	CSize size;
	WORD wState;
	int nSelection;

	ACCEL accel;
};

class CResourceCollection : public CObject
{
public:
	CResourceCollection(UINT nID, CFrameWnd* pWnd);

public:
	BOOL LoadResources(UINT nID);
	void AddResource(UINT nCmd, DWORD dwType, DWORD dwStyle = cBarControlStyleIcon,
		CString strText = CString(), int nImage = -1, CSize size = CSize(23, 22));

	CImageList* GetImageList() { return &m_images; }

	void SetControlType(UINT nCmd, DWORD dwType);
	DWORD GetControlType(UINT nCmd);

	void SetControlStyle(UINT nCmd, DWORD dwStyle);
	DWORD GetControlStyle(UINT nCmd);

	void SetControlState(UINT nCmd, WORD wState);
	WORD GetControlState(UINT nCmd);

	int GetControlImage(UINT nCmd);
	void SetControlImage(UINT nCmd, int nImage);

	CString GetControlText(UINT nCmd);
	void SetControlText(UINT nCmd, LPCTSTR lpszText);

	CString GetControlEntry(UINT nCmd);
	void SetControlEntry(UINT nCmd, LPCTSTR lpszText);

	int GetControlSelection(UINT nCmd);
	void SetControlSelection(UINT nCmd, int nSel);

	void SetControlSize(UINT nCmd, CSize size);
	CSize GetControlSize(UINT nCmd);

	void UpdateAcceleratorTable();
	CString GetControlAcceleratorString(UINT nCmd);
	void ConvertKeyToString(CString& strAccel, UINT uiVirtKey, BOOL bLast = FALSE);

	void SetLockState(BOOL bLocked) { m_bLocked = bLocked; }
	BOOL GetLockState() { return m_bLocked; }

	void DrawImage(CDC* pDC, int nImage, CPoint pt, int nStyle);

	void AddCommandBar(CCommandBar* pCommandBar);
	void RemoveCommandBar(CCommandBar* pCommandBar);
	void RemoveAllCommandBars();

	void ShowControlBar(LPCTSTR lpszName, BOOL bShow);
	void ShowControlBar(CCommandBar* pBar, BOOL bShow);
	void PopupControlBarMenu(CFrameWnd* pWnd, CPoint pt);

	int GetCommandBarArraySize() { return m_aCommandBars.GetSize(); }
	CCommandBar* GetCommandBarFromIndex(int nIndex) { return m_aCommandBars[nIndex]; }

protected:
	CResourceItem* GetResourceItem(UINT nCmd);
	CResourceItem* CreateNewResource(UINT nCmd);

	UINT m_nID;
//	CFrameWnd* m_pFrameWnd;
	CTypedPtrArray<CPtrArray, CCommandBar*> m_aCommandBars;
	CImageList m_images;

	CMapWordToOb m_mapCmdToResource;

	BOOL m_bLocked;

//	CMap<UINT, UINT, DWORD, DWORD> m_mapCmdToType;
//	CMap<UINT, UINT, DWORD, DWORD> m_mapCmdToStyle;
//	CMap<UINT, UINT, int, int> m_mapCmdToImage;
//	CMap<UINT, UINT, CString, CString> m_mapCmdToText;
//	CMap<UINT, UINT, CSize, CSize> m_mapCmdToSize;		// useful only for combos, edits, and statics
//	CMap<UINT, UINT, DWORD, DWORD> m_mapCmdToState;
};

#endif