// SizingFormView.cpp : implementation file
//

#include "stdafx.h"
#include "SizingFormView.h"

#include <..\src\afximpl.h>
//#include <..\src\occimpl.h>
#include <afxpriv.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CSizingFormView

IMPLEMENT_DYNAMIC(CSizingFormView, CView)

CSizingFormView::CSizingFormView(LPCTSTR lpszTemplateName)
{
	m_lpszTemplateName = lpszTemplateName;
	m_pCreateContext = NULL;
	m_hWndFocus = NULL;     // focus window is unknown
	m_mruSize.cx = m_mruSize.cy = 0;
	m_nMinBoundID = 0;
}

CSizingFormView::CSizingFormView(UINT nIDTemplate)
{
	ASSERT_VALID_IDR(nIDTemplate);
	m_lpszTemplateName = MAKEINTRESOURCE(nIDTemplate);
	m_pCreateContext = NULL;
	m_hWndFocus = NULL;     // focus window is unknown
	m_mruSize.cx = m_mruSize.cy = 0;
	m_nMinBoundID = 0;
}

CSizingFormView::~CSizingFormView()
{
}

BEGIN_MESSAGE_MAP(CSizingFormView, CView)
	//{{AFX_MSG_MAP(CSizingFormView)
	ON_WM_SIZE()
	ON_WM_SETFOCUS()
	ON_MESSAGE(WM_QUERY3DCONTROLS, OnQuery3dControls)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
#ifndef _AFX_NO_OCC_SUPPORT
	ON_MESSAGE(WM_INITDIALOG, HandleInitDialog)
#endif
END_MESSAGE_MAP()

BOOL CSizingFormView::Create(LPCTSTR /*lpszClassName*/, LPCTSTR /*lpszWindowName*/,
							 DWORD dwRequestedStyle, const RECT& rect, CWnd* pParentWnd, UINT nID,
							 CCreateContext* pContext) 
{
	ASSERT(pParentWnd != NULL);
	ASSERT(m_lpszTemplateName != NULL);

	m_pCreateContext = pContext;    // save state for later OnCreate

	// initialize common controls
	VERIFY(AfxDeferRegisterClass(AFX_WNDCOMMCTLS_REG));

	// call PreCreateWindow to get prefered extended style
	CREATESTRUCT cs; memset(&cs, 0, sizeof(CREATESTRUCT));
	if (dwRequestedStyle == 0)
		dwRequestedStyle = WS_CHILD | WS_VISIBLE;
	cs.style = dwRequestedStyle;
	if (!PreCreateWindow(cs))
		return FALSE;

	// create a modeless dialog
	if (!CreateDlg(m_lpszTemplateName, pParentWnd))
		return FALSE;

	m_pCreateContext = NULL;

	SetDlgCtrlID(nID);

	// initialize controls etc
	if (!ExecuteDlgInit(m_lpszTemplateName))
		return FALSE;

	LPCDLGTEMPLATE lpDialogTemplate = NULL;
	HGLOBAL hDialogTemplate = NULL;
	HINSTANCE hInst = AfxFindResourceHandle(m_lpszTemplateName, RT_DIALOG);
	HRSRC hResource = ::FindResource(hInst, m_lpszTemplateName, RT_DIALOG);
	hDialogTemplate = LoadResource(hInst, hResource);
	if (hDialogTemplate != NULL)
		lpDialogTemplate = (LPCDLGTEMPLATE)LockResource(hDialogTemplate);
	ASSERT(lpDialogTemplate != NULL);

	CDialogTemplate dlgTemp(lpDialogTemplate);
	CSize size;
	dlgTemp.GetSizeInPixels(&size);
	dlgTemp.Detach();

	// free resource
	UnlockResource(hDialogTemplate);
	FreeResource(hDialogTemplate);

	// force the size in the template.
	SetWindowPos(NULL, rect.left, rect.top, size.cx, size.cy, SWP_NOZORDER | SWP_NOACTIVATE);

	// make visible if requested
	if (dwRequestedStyle & WS_VISIBLE)
		ShowWindow(SW_NORMAL);

	return TRUE;
}

void CSizingFormView::SetControlInfo(WORD nID, WORD nAnchor)
{
	if(nAnchor == ANCHOR_LEFT /* || nAnchor == ANCHOR_TOP*/) return;

	if(nAnchor == MINIMUM_BOUND)
	{
		m_nMinBoundID = nID;
		return;
	}

	// Add resizing behaviour for the control
	DWORD dwControlInfo = nID | (nAnchor << 16);
	m_aControlInfo.Add(dwControlInfo);
}

void CSizingFormView::OnInitialUpdate() 
{
	ASSERT_VALID(this);

	if(!UpdateData(FALSE))
		TRACE0("UpdateData failed during formview initial update.\n");

	CView::OnInitialUpdate();
}

int CSizingFormView::OnCreate(LPCREATESTRUCT lpcs) 
{
	// since we can't get the create context parameter passed in
	//  through CreateDialog, we use a temporary member variable
	lpcs->lpCreateParams = (LPVOID)m_pCreateContext;
	return CView::OnCreate(lpcs);
}

void CSizingFormView::OnActivateView(BOOL bActivate, CView* pActivateView,
									 CView* pDeactiveView)
{
	if (SaveFocusControl())
		return;     // don't call base class when focus is already set

	CView::OnActivateView(bActivate, pActivateView, pDeactiveView);
}

void CSizingFormView::OnActivateFrame(UINT nState, CFrameWnd* /*pFrameWnd*/)
{
	if (nState == WA_INACTIVE)
		SaveFocusControl();     // save focus when frame loses activation
}

BOOL CSizingFormView::SaveFocusControl()
{
	// save focus window if focus is on this window's controls
	HWND hWndFocus = ::GetFocus();
	if (hWndFocus != NULL && ::IsChild(m_hWnd, hWndFocus))
	{
		m_hWndFocus = hWndFocus;
		return TRUE;
	}
	return FALSE;
}

void CSizingFormView::OnSetFocus(CWnd*)
{
	if (!::IsWindow(m_hWndFocus))
	{
		// invalid or unknown focus window... let windows handle it
		m_hWndFocus = NULL;
		Default();
		return;
	}
	// otherwise, set focus to the last known focus window
	::SetFocus(m_hWndFocus);
}

/////////////////////////////////////////////////////////////////////////////
// CSizingFormView message handlers

void CSizingFormView::OnSize(UINT nType, int cx, int cy) 
{
	CView::OnSize(nType, cx, cy);

	// Do nothing if minimized.
	if(nType == SIZE_MINIMIZED) return;

	// Save some time if the old size and new size are the same.
	if(m_mruSize == CSize(cx, cy)) return;

	// Don't do anything if we've hit the minimum constraint.
	if(m_nMinBoundID)
	{
		CRect rect;
		CWnd* pBound = GetDlgItem(m_nMinBoundID);
		
		if(pBound)
		{
			pBound->GetClientRect(&rect);

			if(cx <= rect.Width()) cx = rect.Width();
			if(cy <= rect.Height()) cy = rect.Height();
		}
	}

	int dx = cx - m_mruSize.cx;
	int dy = cy - m_mruSize.cy;

	if(m_mruSize.cx)
	{
		// Move and Size the controls using the information
		// we got in SetControlInfo()

		CRect wndRect;
		CWnd* pWnd;
		DWORD dwControlInfo;
		short nAnchor;
		
		for(int i = 0; i < m_aControlInfo.GetSize(); i++)
		{
			dwControlInfo = m_aControlInfo[i];
			pWnd = GetDlgItem(LOWORD(dwControlInfo));

			// do nothing if anchored to top and or left
			if(!HIWORD(dwControlInfo)) continue;

			nAnchor = HIWORD(dwControlInfo);
			pWnd->GetWindowRect(&wndRect);  ScreenToClient(&wndRect);
			
			if(nAnchor & RESIZE_HORZ) wndRect.right += dx;
			else if(nAnchor & ANCHOR_RIGHT) wndRect.OffsetRect(dx, 0);

			if(nAnchor & RESIZE_VERT) wndRect.bottom += dy;
			else if(nAnchor & ANCHOR_BOTTOM) wndRect.OffsetRect(0, dy);

			pWnd->MoveWindow(&wndRect);
		}
	}

	m_mruSize.cx = cx;
	m_mruSize.cy = cy;
}

BOOL CSizingFormView::PreCreateWindow(CREATESTRUCT& cs) 
{
	cs.style &= ~WS_BORDER;
	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;

	BOOL res = CView::PreCreateWindow(cs);
	
	return res;
}

BOOL CSizingFormView::PreTranslateMessage(MSG* pMsg) 
{
	ASSERT(pMsg != NULL);
	ASSERT_VALID(this);
	ASSERT(m_hWnd != NULL);

	// allow tooltip messages to be filtered
	if(CView::PreTranslateMessage(pMsg))
		return TRUE;

	// don't translate dialog messages when in Shift+F1 help mode
	CFrameWnd* pFrameWnd = GetTopLevelFrame();
	if (pFrameWnd != NULL && pFrameWnd->m_bHelpMode)
		return FALSE;

	// since 'IsDialogMessage' will eat frame window accelerators,
	//   we call all frame windows' PreTranslateMessage first
	pFrameWnd = GetParentFrame();   // start with first parent frame
	while (pFrameWnd != NULL)
	{
		// allow owner & frames to translate before IsDialogMessage does
		if (pFrameWnd->PreTranslateMessage(pMsg))
			return TRUE;

		// try parent frames until there are no parent frames
		pFrameWnd = pFrameWnd->GetParentFrame();
	}

	// filter both messages to dialog and from children
	return PreTranslateInput(pMsg);
}

void CSizingFormView::OnDraw(CDC* pDC)
{
	ASSERT_VALID(this);

	// do nothing - dialog controls will paint themselves,
	//   and Windows dialog controls do not support printing
#ifdef _DEBUG
	if (pDC->IsPrinting())
		TRACE0("Warning: CSizingFormView does not support printing.\n");
#endif

	UNUSED(pDC);     // unused in release build
}

#ifndef _AFX_NO_OCC_SUPPORT

LRESULT CSizingFormView::HandleInitDialog(WPARAM, LPARAM)
{
	Default();  // allow default to initialize first (common dialogs/etc)

	// create OLE controls
/*	COccManager* pOccManager = afxOccManager;
	if ((pOccManager != NULL) && (m_pOccDialogInfo != NULL))
	{
		if (!pOccManager->CreateDlgControls(this, m_lpszTemplateName,
			m_pOccDialogInfo))
		{
			TRACE0("Warning: CreateDlgControls failed during form view init.\n");
			return FALSE;
		}
	}*/

	return TRUE;
}

BOOL CSizingFormView::SetOccDialogInfo(_AFX_OCC_DIALOG_INFO* pOccDialogInfo)
{
	m_pOccDialogInfo = pOccDialogInfo;
	return TRUE;
}

#endif //!_AFX_NO_OCC_SUPPORT
