// TabbedFrame.cpp : implementation file
//

#include "stdafx.h"
#include "TabbedFrame.h"
#include "FF3Edit.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(CTabbedViewGroup, CObject)

CTabbedViewGroup::CTabbedViewGroup()
{
	// User will call AddViewClass() here as many times as
	// necessary to add tabs to the view.  The user can
	// also set m_nActiveView to make the active tab
	// something other than the first one (default).

	m_nActiveView = 0;
}

CTabbedViewGroup::~CTabbedViewGroup()
{
	// There's really no good reason to overload this if
	// the user doesn't do anything special in the
	// constructor.

	for(int i = 0; i < m_aViews.GetSize(); i++)
		delete m_aViews[i];
}

void CTabbedViewGroup::AddViewClass(LPCTSTR lpszTab, CRuntimeClass* pViewClass)
{
	TabbedView* pTV = new TabbedView;

	pTV->strLabel = lpszTab;
	pTV->pClass = pViewClass;

	m_aViews.Add(pTV);
}


/////////////////////////////////////////////////////////////////////////////
// CFormFrame

IMPLEMENT_DYNCREATE(CTabbedFrame, CFrameWndEx)

CTabbedFrame::CTabbedFrame()
{
}

CTabbedFrame::~CTabbedFrame()
{
}

BEGIN_MESSAGE_MAP(CTabbedFrame, CFrameWndEx)
	//{{AFX_MSG_MAP(CTabbedFrame)
	ON_WM_CREATE()
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_GETTABCTRL, OnGetTabCtrl)
	ON_NOTIFY(TCN_SELCHANGE, 1000, OnTabSelChanged)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTabbedFrame message handlers

BOOL CTabbedFrame::PreCreateWindow(CREATESTRUCT& cs) 
{
	BOOL res = CFrameWndEx::PreCreateWindow(cs);
	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;

	return res;
}

int CTabbedFrame::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if(CFrameWndEx::OnCreate(lpCreateStruct) == -1)
		return -1;

	if(!m_wndTab.Create(TCS_FOCUSNEVER | WS_CHILD | WS_CLIPSIBLINGS | WS_VISIBLE,
		CRect(0, 0, 0, 0), this, 1000))
		return -1;

	NONCLIENTMETRICS ncm;
	ncm.cbSize = sizeof(ncm);
	::SystemParametersInfo(SPI_GETNONCLIENTMETRICS, NULL, (PVOID)&ncm, NULL);
	m_fontTab.CreateFont(ncm.lfMenuFont.lfHeight, 0, 0, 0, FW_NORMAL, 0, 0, 0,
		DEFAULT_CHARSET, OUT_CHARACTER_PRECIS, CLIP_CHARACTER_PRECIS, DEFAULT_QUALITY,
		DEFAULT_PITCH | FF_DONTCARE, ncm.lfMenuFont.lfFaceName);
	m_wndTab.SetFont(&m_fontTab);

	TC_ITEM tc;
	tc.mask = TCIF_TEXT;

	for(int i = 0; i < m_pViewGroup->m_aViews.GetSize(); i++)
	{
		tc.pszText = (char*)(LPCTSTR)m_pViewGroup->m_aViews[i]->strLabel;
		tc.cchTextMax = m_pViewGroup->m_aViews[i]->strLabel.GetLength();
		TabCtrl_InsertItem(m_wndTab.GetSafeHwnd(), i, &tc);
	}

	m_wndTab.SetCurSel(m_pViewGroup->m_nActiveView);

	return 0;
}

LRESULT CTabbedFrame::OnGetTabCtrl(WPARAM, LPARAM)
{
	return (LRESULT)&m_wndTab;
}

CView* CTabbedFrame::GetViewByName(LPCTSTR lpszName)
{
	for(int i = 0; i < m_pViewGroup->m_aViews.GetSize(); i++)
	{
		if(!(m_pViewGroup->m_aViews[i]->strLabel.Compare(lpszName)))
			return m_pViewGroup->m_aViews[i]->pView;
	}

	return NULL;
}

BOOL CTabbedFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext) 
{
	// The new view class is actually a collection of view classes
	// derived from CTabbedViewGroup.
	m_pViewGroup = (CTabbedViewGroup*)pContext->m_pNewViewClass->CreateObject();
	
	// Now is the time to notify the document if RECEIVE_INIT_MESSAGE is present.
	if(m_pViewGroup->m_pfnInit)
	{
		(m_pViewGroup->m_pfnInit)(m_pViewGroup);
	}

	for(int i = 0; i < m_pViewGroup->m_aViews.GetSize(); i++)
	{
		CCreateContext context;
		context.m_pNewViewClass = m_pViewGroup->m_aViews[i]->pClass;
		context.m_pCurrentDoc = pContext->m_pCurrentDoc;
		
		UINT nID;
		if(i == m_pViewGroup->m_nActiveView) nID = AFX_IDW_PANE_FIRST;
		else nID = 0;

		CView* pNewView = STATIC_DOWNCAST(CView, context.m_pNewViewClass->CreateObject());

		// views are always created with a border!
		pNewView->Create(NULL, NULL, WS_CHILD | WS_BORDER, CRect(0, 0, 0, 0),
			this, nID, &context);
		
		if(i == m_pViewGroup->m_nActiveView)
		{
			pNewView->ShowWindow(SW_SHOW);
			SetActiveView(pNewView);
		}

		m_pViewGroup->m_aViews[i]->pView = pNewView;
	}

	return TRUE;
}

void CTabbedFrame::OnTabSelChanged(NMHDR* pNotifyStruct, LRESULT* result)
{
	CView* pOldActiveView = GetActiveView();
	pOldActiveView->SetDlgCtrlID(0);
	pOldActiveView->ShowWindow(SW_HIDE);

	CView* pNewView = m_pViewGroup->m_aViews[m_wndTab.GetCurSel()]->pView;
	
	if(pNewView != NULL)
	{
		// The new view is there, but invisible and not active.
		pNewView->SetDlgCtrlID(AFX_IDW_PANE_FIRST);
		pNewView->ShowWindow(SW_SHOW);
		SetActiveView(pNewView);
		RecalcLayout();
	}

	*result = 0;
	return;
}
