// TileEditWnd.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "TileEditWnd.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTileEditWnd

CTileEditWnd::CTileEditWnd()
{
	m_bCurrentTile = 0;
	m_worldData = NULL;
	m_bColor = 0;
}

CTileEditWnd::~CTileEditWnd()
{
}


BEGIN_MESSAGE_MAP(CTileEditWnd, CButton)
	//{{AFX_MSG_MAP(CTileEditWnd)
	ON_WM_LBUTTONUP()
	ON_WM_ERASEBKGND()
	ON_WM_LBUTTONDOWN()
	ON_WM_MOUSEMOVE()
	ON_WM_PAINT()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTileEditWnd message handlers

void CTileEditWnd::DrawItem(LPDRAWITEMSTRUCT lpdis) 
{
}

void CTileEditWnd::PreSubclassWindow() 
{
	CButton::PreSubclassWindow();

	CRect rect;  GetClientRect(&rect);
	rect.bottom -= rect.Height() % 16 - 4;
	SetWindowPos(NULL, 0, 0, rect.Height(), rect.Height(), SWP_NOMOVE | SWP_NOZORDER);
}

void CTileEditWnd::Put8x8ChunkyTile(DDSURFACEDESC2* ddsd, int dx, int dy, BYTE bTile)
{
	int src = 0;
	DWORD c1f, c2f;

	CClientDC dc(this);

	for(int y = dy; y < dy + 8; y++)
	{
		for(int x = dx; x < dx + 8; x += 2)
		{
			BYTE r, g, b;

			BYTE bAll = m_worldData->tiles[bTile * 0x20 + src];

			BYTE b1 = (bAll >> 4) & 0xF;
			BYTE b2 = (bAll) & 0xF;

			WORD c1555 = m_worldData->palette[m_worldData->tilePalettes[bTile] + b1];
			WORD c2555 = m_worldData->palette[m_worldData->tilePalettes[bTile] + b2];

			int pixelWidth = m_rectOffset.Width() / 8;
			int pixelHeight = m_rectOffset.Height() / 8;
			int cx = x * pixelWidth;
			int cy = y * pixelHeight;

			r = R555(c1555) * 8;
			g = G555(c1555) * 8;
			b = B555(c1555) * 8;
			c1f = RGB(b, g, r);

			r = R555(c2555) * 8;
			g = G555(c2555) * 8;
			b = B555(c2555) * 8;
			c2f = RGB(b, g, r);

			dc.FillSolidRect(cx, cy, pixelWidth, pixelHeight, c2f);
			dc.FillSolidRect(cx + pixelWidth, cy, pixelWidth, pixelHeight, c1f);

			src += 1;
		}
	}
}

void CTileEditWnd::SetEditColor(BYTE bColor)
{
	m_bColor = bColor;
}

void CTileEditWnd::SetCurrentTile(BYTE bTile)
{
	m_bCurrentTile = bTile;

	Invalidate();
}

void CTileEditWnd::OnLButtonDown(UINT nFlags, CPoint point) 
{
	SetCapture();
	
	int pixelWidth = m_rectOffset.Width() / 8;
	int pixelHeight = m_rectOffset.Height() / 8;

	CPoint ptPixel((point.x - 3) / pixelWidth, (point.y - 3) / pixelHeight);

	if(ptPixel.x > 7 || ptPixel.y > 7)
	{
		CButton::OnLButtonDown(nFlags, point);
		return;
	}

	BYTE src = ptPixel.y * 4 + (ptPixel.x / 2);
	BOOL bSecondPixel = (ptPixel.x & 0x1) ? TRUE : FALSE;
	BYTE bAll = m_worldData->tiles[m_bCurrentTile * 0x20 + src];

	// These are actually backwards from what you'd think.
	BYTE bPixel;
	if(bSecondPixel) bPixel = (m_bColor << 4) | (bAll & 0xF);
	else bPixel = (bAll & 0xF0) | (m_bColor);

	if(bPixel != bAll)
	{
		m_worldData->tiles[m_bCurrentTile * 0x20 + src] = bPixel;
		Invalidate();
		GetParent()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), TEW_TILEEDIT), (LPARAM)GetSafeHwnd());
	}

	CButton::OnLButtonDown(nFlags, point);
}

void CTileEditWnd::OnMouseMove(UINT nFlags, CPoint point) 
{
	if(GetCapture() != this) return;

	int pixelWidth = m_rectOffset.Width() / 8;
	int pixelHeight = m_rectOffset.Height() / 8;

	CPoint ptPixel((point.x - 3) / pixelWidth, (point.y - 3) / pixelHeight);

	if(ptPixel.x > 7 || ptPixel.y > 7)
	{
		CButton::OnMouseMove(nFlags, point);
		return;
	}

	BYTE src = ptPixel.y * 4 + (ptPixel.x / 2);
	BOOL bSecondPixel = (ptPixel.x & 0x1) ? TRUE : FALSE;
	BYTE bAll = m_worldData->tiles[m_bCurrentTile * 0x20 + src];

	// These are actually backwards from what you'd think.
	BYTE bPixel;
	if(bSecondPixel) bPixel = (m_bColor << 4) | (bAll & 0xF);
	else bPixel = (bAll & 0xF0) | (m_bColor);

	if(bPixel != bAll)
	{
		m_worldData->tiles[m_bCurrentTile * 0x20 + src] = bPixel;
		Invalidate();
		GetParent()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), TEW_TILEEDIT), (LPARAM)GetSafeHwnd());
	}
	
	CButton::OnMouseMove(nFlags, point);
}

void CTileEditWnd::OnLButtonUp(UINT nFlags, CPoint pt) 
{
	ReleaseCapture();

	CButton::OnLButtonUp(nFlags, pt);
}

BOOL CTileEditWnd::OnEraseBkgnd(CDC* pDC) 
{
	return FALSE;
}

void CTileEditWnd::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
	CRect rect;  GetClientRect(&rect);
	m_rectOffset = CRect(0, 0, rect.Width(), rect.Height());

	if(m_worldData)
	{
		Put8x8ChunkyTile(NULL, 0, 0, m_bCurrentTile);
	}

	CPen pen(PS_SOLID, 1, RGB(192, 192, 192));
	CPen* oldPen = dc.SelectObject(&pen);

	for(int x = rect.left; x <= rect.right; x += rect.Width() / 8)
	{
		dc.MoveTo(x, rect.top);
		dc.LineTo(x, rect.bottom);
	}

	for(int y = rect.top; y <= rect.bottom; y += rect.Height() / 8)
	{
		dc.MoveTo(rect.left, y);
		dc.LineTo(rect.right, y);
	}

	dc.SelectObject(oldPen);
}
