// TileFormationWnd.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "TileFormationWnd.h"

#include "MapViewDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTileFormationWnd

CTileFormationWnd::CTileFormationWnd()
{
	m_bCurrentTile = 0;
	m_bCurrentTilePart = 0;
	m_worldData = NULL;
}

CTileFormationWnd::~CTileFormationWnd()
{
}


BEGIN_MESSAGE_MAP(CTileFormationWnd, CWnd)
	//{{AFX_MSG_MAP(CTileFormationWnd)
	ON_WM_PAINT()
	ON_WM_LBUTTONUP()
	ON_WM_ERASEBKGND()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTileFormationWnd message handlers

void CTileFormationWnd::OnPaint() 
{
	CPaintDC dc(this);

	CRect rect;  GetClientRect(rect);

	if(m_worldData)
	{
		BYTE bTile = m_bCurrentTile;

		Put8x8ChunkyTile(&dc, 0, 0, m_worldData->tileFormations[bTile * 4]);
		Put8x8ChunkyTile(&dc, 8, 0, m_worldData->tileFormations[bTile * 4 + 1]);
		Put8x8ChunkyTile(&dc, 0, 8, m_worldData->tileFormations[bTile * 4 + 2]);
		Put8x8ChunkyTile(&dc, 8, 8, m_worldData->tileFormations[bTile * 4 + 3]);
	}

	CPen linePen(PS_SOLID, 1, RGB(192, 192, 192));
	CPen boxPen(PS_SOLID | PS_INSIDEFRAME, 2, RGB(255, 0, 0));

	CPen* oldPen = dc.SelectObject(&linePen);
	dc.MoveTo(0, rect.Height() / 2);
	dc.LineTo(rect.Width(), rect.Height() / 2);
	dc.MoveTo(rect.Width() / 2, 0);
	dc.LineTo(rect.Width() / 2, rect.Height());
	dc.SelectObject(oldPen);
	
	CRect selRect;

	if(m_worldData)
	{
		if(m_bCurrentTilePart == 0)
			selRect = CRect(0, 0, rect.Width() / 2, rect.Height() / 2);
		else if(m_bCurrentTilePart == 1)
			selRect = CRect(rect.Width() / 2, 0, rect.Width(), rect.Height() / 2);
		else if(m_bCurrentTilePart == 2)
			selRect = CRect(0, rect.Height() / 2, rect.Width() / 2, rect.Height());
		else if(m_bCurrentTilePart == 3)
			selRect = CRect(rect.Width() / 2, rect.Height() / 2, rect.Width(), rect.Height());

		oldPen = dc.SelectObject(&boxPen);
		CBrush* oldBrush = (CBrush*)dc.SelectObject(::GetStockObject(NULL_BRUSH));
		dc.Rectangle(&selRect);
		dc.SelectObject(oldPen);
	}
}

void CTileFormationWnd::PreSubclassWindow() 
{
	CWnd::PreSubclassWindow();

	CRect rect;  GetClientRect(&rect);
	rect.bottom -= rect.Height() % 16 - 4;
	SetWindowPos(NULL, 0, 0, rect.Height(), rect.Height(), SWP_NOMOVE | SWP_NOZORDER);
}

void CTileFormationWnd::Put8x8ChunkyTile(CDC* pDC, int dx, int dy, BYTE bTile)
{
	int src = 0;
	BYTE r, g, b, bAll, b1, b2;
	WORD c1555, c2555;
	DWORD c1f, c2f;

	CRect rect;  GetClientRect(rect);

	for(int y = dy; y < dy + 8; y++)
	{
		for(int x = dx; x < dx + 8; x += 2)
		{
			bAll = m_worldData->tiles[bTile * 0x20 + src];

			b1 = (bAll >> 4) & 0xF;
			b2 = (bAll) & 0xF;

			c1555 = m_worldData->palette[m_worldData->tilePalettes[bTile] + b1];
			c2555 = m_worldData->palette[m_worldData->tilePalettes[bTile] + b2];

			r = R555(c1555) * 8;
			g = G555(c1555) * 8;
			b = B555(c1555) * 8;
			c1f = RGB(r, g, b);

			r = R555(c2555) * 8;
			g = G555(c2555) * 8;
			b = B555(c2555) * 8;
			c2f = RGB(r, g, b);

			int pixelWidth = rect.Width() / 16;
			int pixelHeight = rect.Height() / 16;
			int cx = x * pixelWidth;
			int cy = y * pixelHeight;

			pDC->FillSolidRect(cx, cy, pixelWidth, pixelHeight, c2f);
			pDC->FillSolidRect(cx + pixelWidth, cy, pixelWidth, pixelHeight, c1f);

			src += 1;
		}
	}
}

void CTileFormationWnd::SetCurrentTile(BYTE bTile)
{
	m_bCurrentTile = bTile;

	Invalidate();

	GetParent()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), TFW_TILEPARTCHANGE), (LPARAM)GetSafeHwnd());
}

void CTileFormationWnd::OnLButtonUp(UINT nFlags, CPoint pt) 
{
	CRect rect;  GetClientRect(&rect);

	int nHalfWidth = rect.Width() / 2;
	int nHalfHeight = rect.Height() / 2;

	if(pt.x < nHalfWidth && pt.y < nHalfHeight) m_bCurrentTilePart = 0;
	if(pt.x > nHalfWidth && pt.y < nHalfHeight) m_bCurrentTilePart = 1;
	if(pt.x < nHalfWidth && pt.y > nHalfHeight) m_bCurrentTilePart = 2;
	if(pt.x > nHalfWidth && pt.y > nHalfHeight) m_bCurrentTilePart = 3;
	
	Invalidate();

	GetParent()->SendMessage(WM_COMMAND, MAKELONG(GetDlgCtrlID(), TFW_TILEPARTCHANGE), (LPARAM)GetSafeHwnd());

	CWnd::OnLButtonUp(nFlags, pt);
}

BOOL CTileFormationWnd::OnEraseBkgnd(CDC* pDC) 
{
	return FALSE;
}
