// WorkspaceView.cpp : implementation of the CWorkspaceView class
//

#include "stdafx.h"
#include "FF3Edit.h"

#include "FF3Doc.h"
#include "ViewGroups.h"
#include "WorkspaceView.h"
#include "MapFrame.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWorkspaceView

IMPLEMENT_DYNCREATE(CWorkspaceView, CTreeView)

BEGIN_MESSAGE_MAP(CWorkspaceView, CTreeView)
	//{{AFX_MSG_MAP(CWorkspaceView)
	ON_WM_CREATE()
	ON_NOTIFY_REFLECT(TVN_DELETEITEM, OnDeleteItem)
	ON_WM_LBUTTONDBLCLK()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWorkspaceView construction/destruction

CWorkspaceView::CWorkspaceView()
{
	m_imgIcons.Create(IDB_WORKSPACEICONS, 16, 0, RGB(255, 0, 255));
}

CWorkspaceView::~CWorkspaceView()
{
}

BOOL CWorkspaceView::PreCreateWindow(CREATESTRUCT& cs)
{
	cs.style |= TVS_HASLINES | TVS_HASBUTTONS | TVS_LINESATROOT | TVS_SHOWSELALWAYS;

	return CTreeView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CWorkspaceView drawing

void CWorkspaceView::OnDraw(CDC* pDC)
{
	CFF3Doc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
	// TODO: add draw code for native data here
}

void CWorkspaceView::OnInitialUpdate()
{
	CTreeView::OnInitialUpdate();

	// TODO: You may populate your TreeView with items by directly accessing
	//  its tree control through a call to GetTreeCtrl().
}

int CWorkspaceView::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CTreeView::OnCreate(lpCreateStruct) == -1)
		return -1;
	
//	GetTreeCtrl().SetImageList(&m_imgIcons, TVSIL_NORMAL);

	return 0;
}

/////////////////////////////////////////////////////////////////////////////
// CWorkspaceView diagnostics

#ifdef _DEBUG
void CWorkspaceView::AssertValid() const
{
	CTreeView::AssertValid();
}

void CWorkspaceView::Dump(CDumpContext& dc) const
{
	CTreeView::Dump(dc);
}

CFF3Doc* CWorkspaceView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CFF3Doc)));
	return (CFF3Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CWorkspaceView message handlers

HTREEITEM CWorkspaceView::GetNextItem(HTREEITEM hItem)
{
	HTREEITEM hti;

	if(GetTreeCtrl().ItemHasChildren(hItem))
		return GetTreeCtrl().GetChildItem(hItem);		// return first child
	else
	{
		// return next sibling item
		// Go up the tree to find a parent's sibling if needed.
		while((hti = GetTreeCtrl().GetNextSiblingItem(hItem)) == NULL)
		{
			if((hItem = GetTreeCtrl().GetParentItem(hItem)) == NULL)
				return NULL;
		}
	}

	return hti;
}

void CWorkspaceView::ExpandBranch(HTREEITEM hti)
{
	if(GetTreeCtrl().ItemHasChildren(hti))
	{
		GetTreeCtrl().Expand(hti, TVE_EXPAND);
		hti = GetTreeCtrl().GetChildItem(hti);
		
		do
			ExpandBranch(hti);
		while((hti = GetTreeCtrl().GetNextSiblingItem(hti)) != NULL);
	}
    
	GetTreeCtrl().EnsureVisible(GetTreeCtrl().GetSelectedItem());
}

void CWorkspaceView::ExpandAll()
{
	HTREEITEM htiCur = GetTreeCtrl().GetRootItem();

	while(htiCur)
	{
		ExpandBranch(htiCur);
		htiCur = GetNextItem(htiCur);
	}
}

HTREEITEM CWorkspaceView::InsertItem(HTREEITEM hParent, LPCTSTR lpszText, int nImage,
									 CRuntimeClass* pFrameClass, CRuntimeClass* pViewClass,
									 UINT nID, DWORD dwData)
{
	WorkspaceItem* pItem = new WorkspaceItem;
	pItem->pFrameClass = pFrameClass;
	pItem->pViewClass = pViewClass;
	pItem->nID = nID;
	pItem->dwData = dwData;

	return GetTreeCtrl().InsertItem(TVIF_TEXT | TVIF_IMAGE | TVIF_SELECTEDIMAGE | TVIF_PARAM,
		lpszText, nImage, nImage, 0, 0, (LPARAM)pItem, hParent, TVI_LAST);
}

void CWorkspaceView::OnDeleteItem(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	WorkspaceItem* pItem = (WorkspaceItem*)pNMTreeView->itemOld.lParam;
	if(pItem) delete pItem;

	*pResult = 0;
}

void CWorkspaceView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CFF3Doc* pDoc = GetDocument();

	HTREEITEM hTemp;

	if(!pDoc->m_bValid)
	{
		GetTreeCtrl().SelectItem(NULL);

		GetTreeCtrl().DeleteAllItems();
		InsertItem(TVI_ROOT, "No file loaded", -1, NULL, NULL, 0, NULL);

		return;
	}

	HTREEITEM sel = GetTreeCtrl().GetSelectedItem();
	GetTreeCtrl().DeleteAllItems();

	hTemp = InsertItem(TVI_ROOT, "World Maps", -1, NULL, NULL, 0, NULL);
	for(int i = 0; i < 2; i++)
		InsertItem(hTemp, pDoc->worldData[i].name, -1, RUNTIME_CLASS(CTabbedFrame),
			RUNTIME_VIEWGROUP(WorldMapGroup), IDR_WORLDMAP, (DWORD)&pDoc->worldData[i]);

	InsertItem(TVI_ROOT, "Dialogue and Fonts", -1, RUNTIME_CLASS(CTabbedFrame),
		RUNTIME_VIEWGROUP(DialogueGroup), IDR_DIALOGUE, 'DLG ');

	InsertItem(TVI_ROOT, "Items and Magic", -1, RUNTIME_CLASS(CTabbedFrame),
		RUNTIME_VIEWGROUP(ItemGroup), IDR_ITEMMAGIC, 'ITMG');

	InsertItem(TVI_ROOT, "Monsters", -1, RUNTIME_CLASS(CTabbedFrame),
		RUNTIME_VIEWGROUP(MonsterGroup), IDR_MONSTER, 'MNST');

	InsertItem(TVI_ROOT, "Characters", -1, RUNTIME_CLASS(CTabbedFrame),
		RUNTIME_VIEWGROUP(CharacterGroup), IDR_CHARACTER, 'CHAR');

	ExpandAll();
}

void CWorkspaceView::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
	CTreeView::OnLButtonDblClk(nFlags, point);

	HTREEITEM hSel = GetTreeCtrl().GetSelectedItem();
	CFF3Doc* pDoc = GetDocument();

	if(!hSel) return;
	if(!pDoc->m_bValid) return;

	WorkspaceItem* pItem = (WorkspaceItem*)GetTreeCtrl().GetItemData(hSel);

	if(!pItem || !pItem->pFrameClass || !pItem->pViewClass) return;

	CFrameWnd* pFrame = pDoc->CreateFrame(pItem->pFrameClass, pItem->pViewClass, pItem->nID, (LPVOID)pItem->dwData);

	if(pFrame)
	{
		pFrame->SendMessageToDescendants(WM_INITVIEW, NULL, pItem->dwData, FALSE, TRUE);
		pFrame->InitialUpdateFrame(GetDocument(), TRUE);
	}
}
