// MapView.cpp : implementation of the CWorldBaseView class
//

#include "stdafx.h"
#include "FF3Edit.h"

#include "FF3Doc.h"
#include "WorldBaseView.h"

#include "MainFrame.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CWorldBaseView

IMPLEMENT_DYNCREATE(CWorldBaseView, CDirectDrawView)

BEGIN_MESSAGE_MAP(CWorldBaseView, CDirectDrawView)
	//{{AFX_MSG_MAP(CWorldBaseView)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CWorldBaseView construction/destruction

CWorldBaseView::CWorldBaseView()
{
	m_worldData = NULL;

	m_ptMouseTile = CPoint(0, 0);
	m_ptCurrentTrigger = CPoint(0, 0);

	m_nRoundDown = 16;

	m_bTimerSet = FALSE;

	m_rcMarquee.SetRectEmpty();
	m_rcTracker.m_nStyle = CRectTracker::solidLine | CRectTracker::hatchedBorder;
	m_rcTracker.m_rect.SetRectEmpty();

	m_bClipWidth = m_bClipHeight = 0;
	m_pClipboard = NULL;

	m_bPasting = FALSE;
}

CWorldBaseView::~CWorldBaseView()
{
}

/////////////////////////////////////////////////////////////////////////////
// CWorldBaseView drawing

void CWorldBaseView::OnDraw(CDC* pDC)
{
	CClientDC dc(this);

	BeginDraw();
	DrawMap(DM_HighlightTriggers, DM_HT_ALL, TAG_END);
	EndDraw();
}

void CWorldBaseView::Put8x8ChunkyTile(DDSURFACEDESC2* ddsd, int dx, int dy, BYTE bTile, DWORD dwColor, BYTE bOpac)
{
	CFF3Doc* pDoc = GetDocument();

	int src = 0;
	BYTE r, g, b, bAll, b1, b2;
	WORD c1555, c2555;
	DWORD c1f, c2f;
	BOOL bTrans = FALSE;

	if(DDraw.ddpfPrimary.dwRGBBitCount == 16)
	{
		WORD* lpDest = (WORD*)ddsd->lpSurface + (ddsd->lPitch / 2) * dy;

		for(int y = dy; y < dy + 8; y++)
		{
			for(int x = dx; x < dx + 8; x += 2)
			{
				bAll = m_worldData->tiles[bTile * 0x20 + src];

				b1 = (bAll >> 4) & 0x0F;
				b2 = (bAll) & 0x0F;

				c1555 = m_worldData->palette[m_worldData->tilePalettes[bTile] + b1];
				c2555 = m_worldData->palette[m_worldData->tilePalettes[bTile] + b2];

				r = R555(c1555);
				g = G555(c1555);
				b = B555(c1555);
				c1f = (b) | (g << 6) | (r << 11);

				r = R555(c2555);
				g = G555(c2555);
				b = B555(c2555);
				c2f = (b) | (g << 6) | (r << 11);

				if(dwColor && bOpac)
				{
					c1f = SubtractDDColor(c1f, dwColor, bOpac);
					c2f = SubtractDDColor(c2f, dwColor, bOpac);
				}

				lpDest[x + 1] = (WORD)c1f;
				lpDest[x] = (WORD)c2f;

				src += 1;
			}

			lpDest += ddsd->lPitch / 2;
		}
	}
	else if(DDraw.ddpfPrimary.dwRGBBitCount == 24)
	{
		BYTE* lpDest = (BYTE*)ddsd->lpSurface + (ddsd->lPitch) * dy;

		for(int y = dy; y < dy + 8; y++)
		{
			int sx = dx * 3;

			for(int x = dx; x < dx + 8; x += 2)
			{
				bAll = m_worldData->tiles[bTile * 0x20 + src];

				b1 = (bAll >> 4) & 0xF;
				b2 = (bAll) & 0xF;

				c1555 = m_worldData->palette[m_worldData->tilePalettes[bTile] + b1];
				c2555 = m_worldData->palette[m_worldData->tilePalettes[bTile] + b2];

				r = R555(c1555) * 8;
				g = G555(c1555) * 8;
				b = B555(c1555) * 8;
				c1f = (b) | (g << 8) | (r << 16);

				r = R555(c2555) * 8;
				g = G555(c2555) * 8;
				b = B555(c2555) * 8;
				c2f = (b) | (g << 8) | (r << 16);

				if(dwColor && bOpac)
				{
					c1f = SubtractDDColor(c1f, dwColor, bOpac);
					c2f = SubtractDDColor(c2f, dwColor, bOpac);
				}

				lpDest[sx] = (BYTE)(c1f & 0xFF);
				lpDest[sx + 1] = (BYTE)((c1f >> 8) & 0xFF);
				lpDest[sx + 2] = (BYTE)((c1f >> 16) & 0xFF);

				lpDest[sx + 3] = (BYTE)(c2f & 0xFF);
				lpDest[sx + 4] = (BYTE)((c2f >> 8) & 0xFF);
				lpDest[sx + 5] = (BYTE)((c2f >> 16) & 0xFF);

				src += 1;
				sx += 6;
			}

			lpDest += ddsd->lPitch;
		}
	}
	else if(DDraw.ddpfPrimary.dwRGBBitCount == 32)
	{
		DWORD* lpDest = (DWORD*)ddsd->lpSurface + (ddsd->lPitch / 4) * dy;

		for(int y = dy; y < dy + 8; y++)
		{
			for(int x = dx; x < dx + 8; x += 2)
			{
				bAll = m_worldData->tiles[bTile * 0x20 + src];

				b1 = (bAll >> 4) & 0xF;
				b2 = (bAll) & 0xF;

				c1555 = m_worldData->palette[m_worldData->tilePalettes[bTile] + b1];
				c2555 = m_worldData->palette[m_worldData->tilePalettes[bTile] + b2];

				r = R555(c1555) * 8;
				g = G555(c1555) * 8;
				b = B555(c1555) * 8;
				c1f = (b) | (g << 8) | (r << 16);

				r = R555(c2555) * 8;
				g = G555(c2555) * 8;
				b = B555(c2555) * 8;
				c2f = (b) | (g << 8) | (r << 16);

				if(dwColor && bOpac)
				{
					c1f = SubtractDDColor(c1f, dwColor, bOpac);
					c2f = SubtractDDColor(c2f, dwColor, bOpac);
				}

				lpDest[x + 1] = c1f;
				lpDest[x] = c2f;

				src += 1;
			}

			lpDest += ddsd->lPitch / 4;
		}
	}
}

void CWorldBaseView::Put16x16Tile(DDSURFACEDESC2* ddsd, int dx, int dy, BYTE bTile, DWORD dwColor, BYTE bOpac)
{
	Put8x8ChunkyTile(ddsd, dx, dy, m_worldData->tileFormations[bTile * 4 + 0], dwColor, bOpac);
	Put8x8ChunkyTile(ddsd, dx + 8, dy, m_worldData->tileFormations[bTile * 4 + 1], dwColor, bOpac);
	Put8x8ChunkyTile(ddsd, dx, dy + 8, m_worldData->tileFormations[bTile * 4 + 2], dwColor, bOpac);
	Put8x8ChunkyTile(ddsd, dx + 8, dy + 8, m_worldData->tileFormations[bTile * 4 + 3], dwColor, bOpac);
}

/////////////////////////////////////////////////////////////////////////////
// CWorldBaseView diagnostics

#ifdef _DEBUG
void CWorldBaseView::AssertValid() const
{
	CDirectDrawView::AssertValid();
}

void CWorldBaseView::Dump(CDumpContext& dc) const
{
	CDirectDrawView::Dump(dc);
}

CFF3Doc* CWorldBaseView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CFF3Doc)));
	return (CFF3Doc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CWorldBaseView message handlers

void CWorldBaseView::OnInitialUpdate()
{
	CDirectDrawView::OnInitialUpdate();
	
	CSize sizeTotal, sizePage;
	sizeTotal.cx = sizeTotal.cy = 0;
	sizePage.cx = sizePage.cy = 16;
	SetScrollSizes(sizeTotal, sizePage);
}

CRect CWorldBaseView::GetVisibleTileBounds()
{
	CRect bounds;

	CPoint ptScroll = GetScrollPosition();
	CRect rect;  GetClientRect(&rect);

	bounds.left = ptScroll.x;
	bounds.top = ptScroll.y;
	bounds.right = ptScroll.x + rect.Width() / 16;
	bounds.bottom = ptScroll.y + rect.Height() / 16;

	return bounds;
}

CRect CWorldBaseView::GetTileScreenRect(int x, int y)
{
	CPoint ptScroll = GetScrollPosition();

	CRect screenRect((x - ptScroll.x) * 16, (y - ptScroll.y) * 16,
		(x - ptScroll.x + 1) * 16, (y - ptScroll.y + 1) * 16);

	return screenRect;
}

BOOL CWorldBaseView::IsTileVisible(int x, int y)
{
	CRect bounds = GetVisibleTileBounds();

	if((x >= bounds.left && x <= bounds.right) && (y >= bounds.top && y <= bounds.bottom))
		return TRUE;
	
	return FALSE;
}

void CWorldBaseView::DrawMap(DWORD tag, DWORD value, ... )
{
	if(m_worldData == NULL) return;

	DWORD propTag = tag;
	DWORD propValue = value;
	CMap<DWORD, DWORD, DWORD, DWORD> tagColl;

	va_list marker;
	va_start(marker, value);
	while(propTag != TAG_END)
	{
		tagColl[propTag] = propValue;
		propTag = va_arg(marker, DWORD);
		propValue = va_arg(marker, DWORD);
	}
	va_end(marker);

	BOOL bTagFound = FALSE;
	DWORD dwValue = 0;

	CRect clientRect;  GetClientRect(&clientRect);
	int width = m_rcUpdate.Width();
	int height = m_rcUpdate.Height();

	int xBegin = 0, xEnd = 0, yBegin = 0, yEnd = 0, xTileBegin = 0, yTileBegin = 0;

	xTileBegin = GetScrollPosition().x;
	yTileBegin = GetScrollPosition().y;
	xBegin = m_rcUpdate.left / 16 + GetScrollPosition().x;
	yBegin = m_rcUpdate.top / 16 + GetScrollPosition().y;
	xEnd = min(xBegin + width / 16 + 1, m_worldData->size.cx);
	yEnd = min(yBegin + height / 16 + 1, m_worldData->size.cy);

	DDSURFACEDESC2 ddsd;
	ZeroMemory(&ddsd, sizeof(ddsd));
	ddsd.dwSize = sizeof(ddsd);

    HRESULT hres;
	DMSHADERECT dmsr;

	while((hres = m_lpSurfaceBack->Lock(NULL, &ddsd, 0, NULL)) == DDERR_WASSTILLDRAWING);
	if(hres == DD_OK)
	{
		for(int y = yBegin; y < yEnd; y++)
		{
			for(int x = xBegin; x < xEnd; x++)
			{
				::SetRectEmpty(&dmsr.rect);
				DWORD dwColor = 0;
				BYTE bOpac = 0;

				bTagFound = tagColl.Lookup(DM_ShadeRects, dwValue);
				if(bTagFound && dwValue)
				{
					DMSHADERECT* pRects = (DMSHADERECT*)dwValue;

					while(!::IsRectEmpty(&(*pRects).rect))
					{
						if(::PtInRect(&(*pRects).rect, CPoint(x, y)))
						{
							dwColor = (*pRects).color;
							bOpac = (*pRects).opacity;
						}

						pRects++;
					}
				}

				bTagFound = tagColl.Lookup(DM_HighlightTriggers, dwValue);
				if(bTagFound && dwValue == DM_HT_ALL)
				{
					Trigger* pTrigger = m_worldData->GetTrigger(x, y);

					if(pTrigger)
					{
						if(pTrigger->type == triggerEvent)
						{
							dwColor = DDColor(31, 15, 15);
							bOpac = 2;
						}
						else
						{
							dwColor = DDColor(15, 15, 31);
							bOpac = 2;
						}
					}
				}

				if(x == m_ptCurrentTrigger.x && y == m_ptCurrentTrigger.y)
				{
					Trigger* pTrigger = m_worldData->GetTrigger(x, y);

					if(pTrigger)
					{
						dwColor = DDColor(31, 31, 15);
						bOpac = 2;
					}
				}

				Put16x16Tile(&ddsd, (x - xBegin) * 16, (y - yBegin) * 16,
					m_worldData->GetMapTile(x, y), dwColor, bOpac);
			}
		}

		if(m_pClipboard)
		{
			int xScreen = m_rcUpdate.left / 16 + GetScrollPosition().x;
			int yScreen = m_rcUpdate.top / 16 + GetScrollPosition().y;
			xBegin = max(xScreen, m_rcMarquee.left);
			yBegin = max(yScreen, m_rcMarquee.top);
			xEnd = min(xScreen + width / 16, m_rcMarquee.right);
			yEnd = min(yScreen + height / 16, m_rcMarquee.bottom);

			for(int y = yBegin; y < yEnd; y++)
			{
				for(int x = xBegin; x < xEnd; x++)
				{
					Put16x16Tile(&ddsd, (x - xScreen) * 16, (y - yScreen) * 16,
						m_pClipboard[(y - m_rcMarquee.top) * m_bClipWidth + x - m_rcMarquee.left], 0, 0);
				}
			}
		}

		m_lpSurfaceBack->Unlock(NULL);
	}
}

void CWorldBaseView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint)
{
	CFF3Doc* pDoc = GetDocument();

	if(lHint == NULL || (WorldData*)lHint == m_worldData) return;

	m_worldData = (WorldData*)lHint;

	CSize sizeTotal, sizePage;
	sizeTotal.cx = m_worldData->size.cx;
	sizeTotal.cy = m_worldData->size.cy;
	sizePage.cx = sizePage.cy = 16;
	SetScrollSizes(sizeTotal, sizePage);
	ScrollToPosition(m_worldData->ppdLastScrollPos);

	Invalidate();
}