
#if !defined(AFX_CDXCSIZINGDIALOG_H__1D0F68C2_BF90_11D1_A589_444553540000__INCLUDED_)
#define AFX_CDXCSIZINGDIALOG_H__1D0F68C2_BF90_11D1_A589_444553540000__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

//
// cdxCSizingDialog.h : header file
// -----------------------------------------------------------------------
// Author:  Hans Bhler (hans.buehler@student.hu-berlin.de)
//          codex design (http://www-pool.mathematik.hu-berlin.de/~codex
// Version: 1.5
// Release: 4 (Jan 1999 to www.codeguru.com)
// -----------------------------------------------------------------------
// Notes to changes for release 2 (V1.3):
// -	Now uses cdxCDynamicControlsManager which was
//		cdxCSizingDialog::ControlInfo::Header before and had been extended
//		to handle all kinds of windows with controls.
//	-	Since being derived from both CDialog and cdxCDynamicControlsManager
//		it now allows to set custom minimum and maximum sizes :)
//	-	ReorganizeControls() always returns true now.
// -	StretchWindow() can now smallen the window, too.
//		Returns an error if the window would be too small.
// -	You can now easily make several controls act equally (they remain at
//		the same position and size).
//		To implement this, add the first control using AddSzControlEx() or any
//		of its short-cuts, get the returned cdxCDynamicControlsManager::ControlPosition
//		object and use its Add() member function to add the other controls.
// -	cdxCSizingDialog is now DECLAREed_DYNAMIC.
// -	RestoreWindowPosition() now allows to restore the visible state
//		what was infuctional before.
//		Note that the calling convention changed !!!!
// Changes to Rel.3 (V1.4):
// - Destructor no longer calls a virtual function.
// Changes to Rel.4 (V1.5):
// - _bool_ ReorganizeControls() has been removed (it called cdxCDynamic
//   ControlsManager::ReorganizeControls() and returned true).
// - Create(CWnd *pParent) has been added.
// - Create(UINT, CWnd *) has been added (and is virtual).
// - OnInitDialogAutoPos() added.
// -----------------------------------------------------------------------
// Comments welcome.
//
//

#include	"cdxCDynamicControlsManager.h"

/*
 * cdxCSizingDialog[CDialog,cdxCDynamicControlsManager]
 * ====================================================
 * A CDialog dervied class that implements a sizable dialog class.
 *
 * (w)Mar.1998 Hans Bhler, codex design (release 1)
 * (w)Oct.1998 Hans Bhler, codex design (release 2)
 *    hans.buehler@student.hu-berlin.de;
 *    posted to www.codeguru.com
 *    This is freeware; you use this on your own risk.
 */

class cdxCSizingDialog : public CDialog, public cdxCDynamicControlsManager
{
	DECLARE_DYNAMIC(cdxCSizingDialog);

protected:
	enum		// kept here for compatibility
	{
		DEF_STRETCH	=	10,
		exIgnore		=	0,
		exMaximum	=	100
	};

	//
	// members
	//

private:
	CString			m_strAutoPos;		// for EnableAutoPosition()
public:
	const Freedom	m_InitFreedom;		// these are needed for compatibility to older
	const bool		m_bMkIcon;			// versions of cdxCSizingDialog
	const UINT		m_iddDlgID;

protected:
	cdxCSizingDialog(UINT idd, CWnd* pParent = NULL, Freedom fd = fdAll, bool bSizeIcon = true) : CDialog(idd, pParent), m_InitFreedom(fd), m_bMkIcon(bSizeIcon), m_iddDlgID(idd) { ASSERT(idd != 0); }
public:
	virtual ~cdxCSizingDialog() { cdxCSizingDialog::DestroyWindow(); }

	//
	// create it
	//
	BOOL Create(CWnd *pParent = NULL) { return CDialog::Create(m_iddDlgID,pParent); }
	virtual BOOL Create(UINT idd, CWnd *pParent) { if(! CDialog::Create(idd,pParent)) return FALSE; ReorganizeControls(); return TRUE; }

	//
	// various helpers
	//
protected:
	void PumpMessages(void);

	//
	// dialog stuff
	//
protected:
	BOOL OnInitDialog(UINT iAddSzPercent, Freedom fd, bool bMkIcon);
	BOOL OnInitDialogAutoPos(const CString & strAutoPos, UINT restoreFlags, UINT iAddSzPercent, Freedom fd, bool bMkIcon);
	BOOL OnInitDialogAutoPos(const CString & strAutoPos, UINT restoreFlags = rflg_none, UINT iAddSzPercent = 10) { return OnInitDialogAutoPos(strAutoPos,restoreFlags,iAddSzPercent,m_InitFreedom,m_bMkIcon); }
	BOOL OnInitDialogAutoPos(UINT restoreFlags, UINT iAddSzPercent, Freedom fd, bool bMkIcon);
	BOOL OnInitDialogAutoPos(UINT restoreFlags = rflg_none, UINT iAddSzPercent = 10) { return OnInitDialogAutoPos(restoreFlags,iAddSzPercent,m_InitFreedom,m_bMkIcon); }

	virtual BOOL OnInitDialog() { return OnInitDialog(10,m_InitFreedom,m_bMkIcon); }
	afx_msg void OnGetMinMaxInfo(MINMAXINFO FAR* lpMMI);
	afx_msg void OnSize(UINT nType, int cx, int cy);

public:
	virtual BOOL DestroyWindow(void);

protected:
	DECLARE_MESSAGE_MAP()

	//
	// obsolete - kept for being compatible
	//
protected:
	BOOL OnInitDialog(UINT iAddSzPercent) { return OnInitDialog(iAddSzPercent,m_InitFreedom,m_bMkIcon); }
public:
	bool RestoreWindowPosition(LPCTSTR lpszProfile, bool bWithState) { return cdxCDynamicControlsManager::RestoreWindowPosition(lpszProfile,bWithState ? rflg_visibility : rflg_none); }
	bool RestoreWindowPosition(UINT idsProfile, bool bWithState = false) { CString s;s.LoadString(idsProfile); return cdxCDynamicControlsManager::RestoreWindowPosition(s,bWithState ? rflg_visibility : rflg_none); }
	bool StoreWindowPosition(UINT idsProfile) { CString s;s.LoadString(idsProfile); return cdxCDynamicControlsManager::StoreWindowPosition(s); }
	bool RestoreWindowPosition(LPCTSTR lpszProfile, UINT restoreFlags = rflg_none) { return cdxCDynamicControlsManager::RestoreWindowPosition(lpszProfile,restoreFlags); }
	bool StoreWindowPosition(LPCTSTR lpszProfile) { return cdxCDynamicControlsManager::StoreWindowPosition(lpszProfile); }
};

/////////////////////////////////////////////////////////////////////////////
// inlines
/////////////////////////////////////////////////////////////////////////////

/*
 * An OnInitDialog() that auto-repositions the window and will
 * ensure its position will be stored if the window is been
 * closed later.
 */

inline BOOL cdxCSizingDialog::OnInitDialogAutoPos(const CString & strAutoPos, UINT restoreFlags, UINT iAddSzPercent, Freedom fd, bool bMkIcon)
{
	BOOL	b	=	OnInitDialog(iAddSzPercent,fd,bMkIcon);

	ASSERT(!strAutoPos.IsEmpty());
	if(!strAutoPos.IsEmpty())
	{
		m_strAutoPos	=	strAutoPos;
		RestoreWindowPosition(strAutoPos,restoreFlags);
	}
	return b;
}

/*
 * the same thing with auto-formatted registry key
 * this function is vier
 */

inline BOOL cdxCSizingDialog::OnInitDialogAutoPos(UINT restoreFlags, UINT iAddSzPercent, Freedom fd, bool bMkIcon)
{
	CString	s;
	s.Format(_T("WindowPositions\\Dialogs\\Dialog#%lx"),m_iddDlgID);
	return OnInitDialogAutoPos(s,restoreFlags,iAddSzPercent,fd,bMkIcon);
}


#endif // !defined(AFX_CDXCSIZINGDIALOG_H__1D0F68C2_BF90_11D1_A589_444553540000__INCLUDED_)
