// FontEditView.cpp : implementation file
//

#include "stdafx.h"
#include "FF3Edit.h"
#include "FontEditView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CVFontEditView

IMPLEMENT_DYNCREATE(CVFontEditView, CSizingFormView)

CVFontEditView::CVFontEditView()
	: CSizingFormView(CVFontEditView::IDD)
{
	//{{AFX_DATA_INIT(CVFontEditView)
	//}}AFX_DATA_INIT

	m_nVertShift = 2;
}

CVFontEditView::~CVFontEditView()
{
}

void CVFontEditView::DoDataExchange(CDataExchange* pDX)
{
	CSizingFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CVFontEditView)
	DDX_Control(pDX, IDC_CHARWIDTH, m_wndWidth);
	DDX_Control(pDX, IDC_CHARLIST, m_wndList);
	DDX_Control(pDX, IDC_COLORS, m_wndColors);
	DDX_Control(pDX, IDC_FONTEDIT, m_wndFontEdit);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CVFontEditView, CSizingFormView)
	//{{AFX_MSG_MAP(CVFontEditView)
	ON_LBN_SELCHANGE(IDC_CHARLIST, OnSelChangeCharList)
	ON_BN_CLICKED(IDC_IMPORTTTF, OnImportTTF)
	ON_EN_CHANGE(IDC_CHARWIDTH, OnChangeCharWidth)
	ON_BN_CLICKED(IDC_SHIFTFONTUP, OnShiftFontUp)
	ON_BN_CLICKED(IDC_SHIFTFONTDOWN, OnShiftFontDown)
	ON_NOTIFY(CSN_SELCHANGED, IDC_COLORS, OnSelChangeColors)
	ON_CONTROL(PED_IMAGECHANGED, IDC_FONTEDIT, OnChangeFontEdit)
	ON_BN_CLICKED(IDC_SHIFTUP, OnShiftUp)
	ON_BN_CLICKED(IDC_SHIFTDOWN, OnShiftDown)
	ON_BN_CLICKED(IDC_SHIFTLEFT, OnShiftLeft)
	ON_BN_CLICKED(IDC_SHIFTRIGHT, OnShiftRight)
	ON_BN_CLICKED(IDC_CLEAR, OnClear)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CVFontEditView diagnostics

#ifdef _DEBUG
void CVFontEditView::AssertValid() const
{
	CSizingFormView::AssertValid();
}

void CVFontEditView::Dump(CDumpContext& dc) const
{
	CSizingFormView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CVFontEditView message handlers

void CVFontEditView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();

	m_wndList.m_pDoc = pDoc;

	char ch;

	for(int i = 0x20; i < 0x80; i++)
	{
		ch = VariableCharToASCII(i);
		int index = m_wndList.AddString(NULL);
		m_wndList.SetItemData(index, MAKELONG(i, ch));
	}

	m_wndList.SetItemHeight(0, 28);
	m_wndList.SetColumnWidth(30);

	WORD colors[2] = {
		RGB555(14, 15, 24), RGB555(31, 31, 31) };

	m_wndColors.ResetContent();
	m_wndColors.AddColors(colors, 2);

	m_wndFontEdit.SetSizes(12, 11, 16);
	m_wndFontEdit.SetColors(colors, 2);

	m_wndList.SetCurSel(0);
	m_wndColors.SetForegroundColor(1);
	m_wndColors.SetBackgroundColor(0);

	OnSelChangeCharList();

	m_wndFontEdit.SetCurrentColors(1, 0);
}

void CVFontEditView::OnSelChangeCharList() 
{
	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();

	int nSel = m_wndList.GetCurSel();
	BYTE bChar = (BYTE)LOWORD(m_wndList.GetItemData(nSel));
	WORD* pChar = &pDoc->variableFont[(bChar - 0x20) * 11];

	CString s;  s.Format("%d", pDoc->charWidths[bChar]);
	m_wndWidth.SetWindowText(s);

	for(int y = 0; y < 11; y++)
		for(int x = 0; x < 12; x++)
			m_wndFontEdit.SetPixel(x, y, (pChar[y] & (1 << (11 - x)) ? 1 : 0));
}

void CVFontEditView::OnSelChangeColors(NMHDR* pNotifyStruct, LRESULT* result) 
{
	LPNMCOLORSELECTOR lpNMCS = (LPNMCOLORSELECTOR)pNotifyStruct;

	if(lpNMCS->nColorChanged == CSNC_FOREGROUND)
		m_wndFontEdit.SetCurrentColors(lpNMCS->nIndex, -1);
	else if(lpNMCS->nColorChanged == CSNC_BACKGROUND)
		m_wndFontEdit.SetCurrentColors(-1, lpNMCS->nIndex);
}

void CVFontEditView::OnChangeCharWidth() 
{
	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();

	CString s;  m_wndWidth.GetWindowText(s);
	int nWidth = atoi(s);

	int nSel = m_wndList.GetCurSel();
	BYTE bChar = (BYTE)LOWORD(m_wndList.GetItemData(nSel));

	if(nWidth > 12)
	{
		nWidth = 12;
		s.Format("%d", nWidth);
		m_wndWidth.SetWindowText(s);
		MessageBeep(-1);
	}

	pDoc->charWidths[bChar] = nWidth;

	CRect rect;
	m_wndList.GetItemRect(nSel, &rect);
	m_wndList.InvalidateRect(rect);
}

void CVFontEditView::OnChangeFontEdit()
{
	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();
	int nSel = m_wndList.GetCurSel();
	BYTE bChar = (BYTE)LOWORD(m_wndList.GetItemData(nSel));

	WORD* pChar = &pDoc->variableFont[(bChar - 0x20) * 11];

	for(int y = 0; y < 11; y++)
	{
		WORD w = 0;

		for(int x = 0; x < 12; x++)
		{
			if(m_wndFontEdit.GetPixel(x, y) == 1)
				w |= (1 << (11 - x));
		}

		pChar[y] = w;
	}

	CRect rect;
	m_wndList.GetItemRect(nSel, &rect);
	m_wndList.InvalidateRect(&rect);
}

void CVFontEditView::OnImportTTF() 
{
	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();

	CFontDialog fdlg(NULL, CF_EFFECTS | CF_SCREENFONTS,
		NULL, GetParentFrame());

	if(fdlg.DoModal() == IDOK)
	{
		fdlg.GetCurrentFont(&m_logFont);
		m_logFont.lfQuality |= NONANTIALIASED_QUALITY;
		
		ImportFont();
	}
}

void CVFontEditView::ImportFont()
{
	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();

	CDC dc;
	CBitmap bmp;
	CFont font;
	dc.CreateCompatibleDC(GetDC());
	bmp.CreateCompatibleBitmap(&dc, 16, 16);
	CBitmap* oldBitmap = dc.SelectObject(&bmp);
	font.CreateFontIndirect(&m_logFont);
	CFont* oldFont = dc.SelectObject(&font);

	dc.SetTextColor(RGB(0, 0, 0));
	dc.SetBkMode(TRANSPARENT);

	for(int n = 0; n < m_wndList.GetCount(); n++)
	{
		BYTE bChar = (BYTE)LOWORD(m_wndList.GetItemData(n));
		char bASCII = (char)HIWORD(m_wndList.GetItemData(n));

		if(bASCII != 0)
		{
			dc.FillSolidRect(0, 0, 16, 16, RGB(255, 255, 255));
			CString s;  s.Format("%c", bASCII);

			dc.TextOut(0, 0, s);
			pDoc->charWidths[bChar] = (BYTE)dc.GetTextExtent(s).cx;

			WORD* pChar = &pDoc->variableFont[(bChar - 0x20) * 11];

			for(int y = 0; y < 11; y++)
			{
				WORD w = 0;

				for(int x = 0; x < pDoc->charWidths[bChar]; x++)
				{
					if(dc.GetPixel(x, y + m_nVertShift) == RGB(0, 0, 0))
						w |= (1 << (11 - x));
				}

				pChar[y] = w;
			}
		}
	}

	dc.SelectObject(oldFont);
	dc.SelectObject(oldBitmap);

	m_wndList.Invalidate();
}

void CVFontEditView::OnShiftFontUp() 
{
	m_nVertShift++;
	ImportFont();
}

void CVFontEditView::OnShiftFontDown() 
{
	m_nVertShift--;
	ImportFont();
}


/////////////////////////////////////////////////////////////////////////////
// CFFontEditView

IMPLEMENT_DYNCREATE(CFFontEditView, CSizingFormView)

CFFontEditView::CFFontEditView()
	: CSizingFormView(CFFontEditView::IDD)
{
	//{{AFX_DATA_INIT(CFFontEditView)
	//}}AFX_DATA_INIT
}

CFFontEditView::~CFFontEditView()
{
}

void CFFontEditView::DoDataExchange(CDataExchange* pDX)
{
	CSizingFormView::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CFFontEditView)
	DDX_Control(pDX, IDC_CHARLIST, m_wndList);
	DDX_Control(pDX, IDC_COLORS, m_wndColors);
	DDX_Control(pDX, IDC_FONTEDIT, m_wndFontEdit);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CFFontEditView, CSizingFormView)
	//{{AFX_MSG_MAP(CFFontEditView)
	ON_LBN_SELCHANGE(IDC_CHARLIST, OnSelChangeCharList)
	ON_NOTIFY(CSN_SELCHANGED, IDC_COLORS, OnSelChangeColors)
	ON_CONTROL(PED_IMAGECHANGED, IDC_FONTEDIT, OnChangeFontEdit)
	ON_BN_CLICKED(IDC_SHIFTUP, OnShiftUp)
	ON_BN_CLICKED(IDC_SHIFTDOWN, OnShiftDown)
	ON_BN_CLICKED(IDC_SHIFTLEFT, OnShiftLeft)
	ON_BN_CLICKED(IDC_SHIFTRIGHT, OnShiftRight)
	ON_BN_CLICKED(IDC_CLEAR, OnClear)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFFontEditView diagnostics

#ifdef _DEBUG
void CFFontEditView::AssertValid() const
{
	CSizingFormView::AssertValid();
}

void CFFontEditView::Dump(CDumpContext& dc) const
{
	CSizingFormView::Dump(dc);
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CFFontEditView message handlers

void CFFontEditView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();

	m_wndList.ResetContent();
	m_wndList.m_pDoc = pDoc;

	char ch;

	for(int i = 0x80; i < 0x100; i++)
	{
		ch = FixedCharToASCII(i);
		int index = m_wndList.AddString(NULL);
		m_wndList.SetItemData(index, MAKELONG(i, ch));
	}

	m_wndList.SetItemHeight(0, 22);
	m_wndList.SetColumnWidth(22);

	WORD colors[4] = {
		RGB555(14, 15, 24), RGB555(0, 0, 0), RGB555(14, 14, 14), RGB555(31, 31, 31) };

	m_wndColors.ResetContent();
	m_wndColors.AddColors(colors, 4);

	m_wndFontEdit.SetSizes(8, 8, 16);
	m_wndFontEdit.SetColors(colors, 4);

	m_wndList.SetCurSel(0);
	m_wndColors.SetForegroundColor(3);
	m_wndColors.SetBackgroundColor(0);

	OnSelChangeCharList();

	m_wndFontEdit.SetCurrentColors(3, 0);
}

void CFFontEditView::OnSelChangeCharList() 
{
	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();

	int nSel = m_wndList.GetCurSel();
	BYTE bChar = (BYTE)LOWORD(m_wndList.GetItemData(nSel));

	BYTE* pChar = &pDoc->fixedFont[(bChar - 0x80) * 16];

	BYTE aTile[64];
	BYTE bPlanes[2];
	BYTE mask;

	for(int row = 0; row < 8; row++)
	{
		bPlanes[0] = pChar[row * 2];
		bPlanes[1] = pChar[row * 2 + 1];

		for(int bit = 7; bit >= 0; bit--)
		{
			mask = 1 << bit;
		
			aTile[row * 8 + (7 - bit)] = ((bPlanes[0] & mask) >> bit) |
								   (((bPlanes[1] & mask) >> bit) << 1);
		}
	}

	for(int y = 0; y < 8; y++)
		for(int x = 0; x < 8; x++)
			m_wndFontEdit.SetPixel(x, y, aTile[y * 8 + x]);
}

void CFFontEditView::OnSelChangeColors(NMHDR* pNotifyStruct, LRESULT* result) 
{
	LPNMCOLORSELECTOR lpNMCS = (LPNMCOLORSELECTOR)pNotifyStruct;

	if(lpNMCS->nColorChanged == CSNC_FOREGROUND)
		m_wndFontEdit.SetCurrentColors(lpNMCS->nIndex, -1);
	else if(lpNMCS->nColorChanged == CSNC_BACKGROUND)
		m_wndFontEdit.SetCurrentColors(-1, lpNMCS->nIndex);
}

void CFFontEditView::OnChangeFontEdit()
{
	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();
	int nSel = m_wndList.GetCurSel();
	BYTE bChar = (BYTE)LOWORD(m_wndList.GetItemData(nSel));
	BYTE* pChar = &pDoc->fixedFont[(bChar - 0x80) * 16];

	BYTE bPlanes[2];

	for(int row = 0; row < 8; row++)
	{
		bPlanes[0] = bPlanes[1] = 0;

		for(int col = 0; col < 8; col++)
		{
			BYTE bColor = m_wndFontEdit.GetPixel(col, row);
			if(bColor & 1) bPlanes[0] |= (1 << (7 - col));
			if(bColor & 2) bPlanes[1] |= (1 << (7 - col));
		}

		pChar[row * 2] = bPlanes[0];
		pChar[row * 2 + 1] = bPlanes[1];
	}

	CRect rect;
	m_wndList.GetItemRect(nSel, &rect);
	m_wndList.InvalidateRect(&rect);
}

/*void CFFontEditView::ImportFont()
{
	CFF3Doc* pDoc = (CFF3Doc*)GetDocument();

	CDC dc;
	CBitmap bmp;
	CFont font;
	dc.CreateCompatibleDC(GetDC());
	bmp.CreateCompatibleBitmap(&dc, 16, 16);
	CBitmap* oldBitmap = dc.SelectObject(&bmp);
	font.CreateFontIndirect(&m_logFont);
	CFont* oldFont = dc.SelectObject(&font);

	dc.SetBkMode(TRANSPARENT);

	for(int n = 0; n < m_wndList.GetCount(); n++)
	{
		BYTE bChar = (BYTE)LOWORD(m_wndList.GetItemData(n));
		char bASCII = (char)HIWORD(m_wndList.GetItemData(n));

		if(bASCII != 0)
		{
			COLORREF colors[4] = {
				RGB(112, 120, 192), RGB(0, 0, 0), RGB(112, 112, 112), RGB(255, 255, 255) };

			dc.FillSolidRect(0, 0, 16, 16, colors[2]);
			CString s;  s.Format("%c", bASCII);

			dc.SetTextColor(colors[1]);
			dc.TextOut(1, 1, s);
			dc.SetTextColor(colors[3]);
			dc.TextOut(0, 0, s);

			BYTE* pChar = &pDoc->fixedFont[(bChar - 0x80) * 16];

			BYTE aTile[64];
			BYTE bPlanes[2];

			for(int y = 0; y < 8; y++)
				for(int x = 0; x < 8; x++)
					if(dc.GetPixel(x, y + m_nVertShift) == colors[0]) aTile[y * 8 + x] = 0;
					else if(dc.GetPixel(x, y + m_nVertShift) == colors[1]) aTile[y * 8 + x] = 1;
					else if(dc.GetPixel(x, y + m_nVertShift) == colors[2]) aTile[y * 8 + x] = 2;
					else if(dc.GetPixel(x, y + m_nVertShift) == colors[3]) aTile[y * 8 + x] = 3;

			for(int row = 0; row < 8; row++)
			{
				bPlanes[0] = bPlanes[1] = 0;

				for(int col = 0; col < 8; col++)
				{
					if(aTile[row * 8 + col] & 1) bPlanes[0] |= (1 << (7 - col));
					if(aTile[row * 8 + col] & 2) bPlanes[1] |= (1 << (7 - col));
				}

				pChar[row * 2] = bPlanes[0];
				pChar[row * 2 + 1] = bPlanes[1];
			}
		}
	}

	dc.SelectObject(oldFont);
	dc.SelectObject(oldBitmap);

	m_wndList.Invalidate();
}*/


void CFFontEditView::OnShiftUp() 
{
	m_wndFontEdit.ShiftUp();
}

void CFFontEditView::OnShiftDown() 
{
	m_wndFontEdit.ShiftDown();
}

void CFFontEditView::OnShiftLeft() 
{
	m_wndFontEdit.ShiftLeft();
}

void CFFontEditView::OnShiftRight() 
{
	m_wndFontEdit.ShiftRight();
}

void CFFontEditView::OnClear() 
{
	m_wndFontEdit.Fill(0);
}

void CVFontEditView::OnShiftUp() 
{
	m_wndFontEdit.ShiftUp();
}

void CVFontEditView::OnShiftDown() 
{
	m_wndFontEdit.ShiftDown();
}

void CVFontEditView::OnShiftLeft() 
{
	m_wndFontEdit.ShiftLeft();
}

void CVFontEditView::OnShiftRight() 
{
	m_wndFontEdit.ShiftRight();
}

void CVFontEditView::OnClear() 
{
	m_wndFontEdit.Fill(0);
}
